# coding=utf-8
# Copyright © 2015 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

from ConfigParser import SafeConfigParser
from cStringIO import StringIO
import os
import sys

module_dir = os.path.dirname(os.path.abspath(sys.modules[__name__].__file__))

DEFAULT_CONFIG = """
[mws]
url = http://localhost:8080/mws
username = admin
password = secret
"""


def getACPythonHome(verbose=False):
    """Return the Adaptive Python API home directory.
    If the ACPYTHON_HOME environment variable is defined, return that value.
    If acpython was installed the normal way, return $MOABHOMEDIR,
    defaulting to /opt/moab .
    """
    # Override using ACPYTHON_HOME environment variable
    acpython_home = os.environ.get('ACPYTHON_HOME')
    if acpython_home:
        return acpython_home
    # Detect if we are running out of the source directory
    module_parent = os.path.dirname(module_dir)
    if os.path.basename(module_parent) not in (
                        'site-packages', 'dist-packages'):
        if os.path.isdir(os.path.join(module_parent, 'etc')):
            return module_parent
    # Default: use MOABHOMEDIR, fall back to /opt/moab
    return os.environ.get('MOABHOMEDIR', '/opt/moab')


def getConfigDir(verbose=False):
    """
    Return the Adaptive Python API configuration directory.
    By default this is /opt/moab/etc
    """
    acpython_home = getACPythonHome(verbose=verbose)
    if not acpython_home:
        raise EnvironmentError('Missing acpython home, not installed properly')
    config_dir = os.path.join(acpython_home, 'etc')
    return config_dir


def getLogDir(verbose=False):
    """
    Return the Adaptive Python API log file directory.
    By default this is /opt/moab/log
    """
    acpython_home = getACPythonHome(verbose=verbose)
    if not acpython_home:
        raise EnvironmentError('Missing acpython home, not installed properly')
    log_dir = os.path.join(acpython_home, 'log')
    return log_dir


def getConfig(filename='acpython.ini', verbose=False):
    """
    Return the settings from the main Adaptive Python API configuration
    file. By default the file read is acpython.ini in the configuration
    directory. The returned value is a ConfigParser.SafeConfigParser object.
    """
    config_dir = getConfigDir(verbose=verbose)
    config_filename = os.path.join(config_dir, filename)
    if not os.path.isfile(config_filename):
        msg = StringIO()
        print >> msg, "Missing configuration file:", config_filename
        print >> msg, "Example configuration file content:"
        print >> msg, DEFAULT_CONFIG
        raise EnvironmentError(msg.getvalue())
    if verbose:
        print 'Reading acpython config file:', config_filename
    config = SafeConfigParser()
    config.read(config_filename)
    return config
