/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

/**
 * Damn tiny wrapper for $.ajax.
 *
 * Any API method will return a promise.
 */

;(function($, window) {
  'use strict';

  var Viewpoint = window.Viewpoint || {};

  function ajax(url, method, data, type) {
    var deferred = $.Deferred();

    var options = {
      dataType: 'json',
      headers: {
        'Accept': 'application/x.viewpoint+json; version=2.0',
        'Authorization': 'Unobtrustive'
      },
      method: method,
      url: url,
      xhr: function() {
        var xhr = $.ajaxSettings.xhr();

        xhr.upload.addEventListener('progress', function(event) {
          if (event.lengthComputable)
            deferred.notify((event.loaded / event.total * 100).toFixed(2));
        });

        return xhr;
      }
    };

    if (data) {
      if (data.constructor === FormData) {
        $.extend(options, {
          cache: false,
          contentType: false,
          data: data,
          processData: false
        });
      } else {
        $.extend(options, {
          contentType: 'application/x.viewpoint+json; version=2.0',
          data: JSON.stringify(data)
        });
      }
    }

    if (type == 'text') {
      $.extend(true, options, {
        dataType: 'text',
        headers: {
          'Accept': 'text/plain'
        }
      });
    }

    // Hide $.ajax's promise with own one (for progress support)

    $.ajax(options)
        .done(deferred.resolve)
        .fail(deferred.reject);

    return deferred.promise();
  }

  var API = function() {};

  // XXX: 'use strict' forbids using of arguments.callee.name
  // but ES6 supports Function.name

  API.prototype.delete = function(url) {
    return ajax(url, 'DELETE');
  };

  API.prototype.get = function(url, type) {
    return ajax(url, 'GET', null, type);
  };

  API.prototype.head = function(url, type) {
    return ajax(url, 'HEAD', null, type);
  };

  API.prototype.options = function(url) {
    return ajax(url, 'OPTIONS');
  };

  API.prototype.patch = function(url, data) {
    return ajax(url, 'PATCH', data);
  };

  API.prototype.post = function(url, data) {
    return ajax(url, 'POST', data);
  };

  API.prototype.put = function(url, data) {
    return ajax(url, 'PUT', data);
  };

  Viewpoint.API = API;

  if (!$.viewpoint)
    $.viewpoint = {};

  $.viewpoint.api = new Viewpoint.API();

  window.Viewpoint = Viewpoint;
})(jQuery, window);
