/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

/**
 * Loader helper to be used with Viewpoint.API.
 */

;(function($) {
  'use strict';
  // http://www.w3.org/TR/html5/disabled-elements.html
  var DISABLED_ELEMENTS = 'button, fieldset, input, optgroup, option, select, textarea';

  $.fn.loader = function(api, errorHandlers) {
    var createdLoaders = [];
    var element = this,
        anchor = element;
    // show message which generate loader or not
    var showError = $(this).data('show-error');
    if (showError === undefined) {
      showError = true;
    }

    // Anchor is an element to attach a loader,
    // collect all extra anchors
    element.map(function() {
      anchor = anchor.add($(this).data('api-anchor'));
    });

    anchor.each(function() {
      $(this).data('loader',
        $('<div>')
          .addClass('loader')
          .appendTo(document.body));
    });

    // Wrap API
    $.each(api, function(k, v) {
      if (k == 'head')
        return;

      // Wrap the API method
      api[k] = function() {
        createdLoaders = [];
        anchor.each(function() {
          var anchor = $(this),
            loader = anchor.data('loader'),
            height = anchor.outerHeight(),
            width = anchor.outerWidth();

          // We will need to disable some elements before API request
          // and restore their state when API response received
          anchor
            .add(anchor.find(DISABLED_ELEMENTS))
            .filter(DISABLED_ELEMENTS)
            .filter(function() {
              return typeof $(this).data('api-disabled') === undefined;
            })
            .prop('disabled', function() {
              var el = $(this);
              el.data('api-disabled', el.prop('disabled'));
              return true;
            });
            
          if (loader !== undefined) { 
            var requests = loader.data('requests') || 0;
            createdLoaders.push(loader);
            loader
              [Math.min(height, width) < 31 ?
                'addClass' : 'removeClass']('small')
              .css({ left: '', top: '' })
              .offset(anchor.offset())
              .height(height)
              .width(width)
              .show()
              .data('requests', ++requests);
          }
        });

        // Call the wrapped method
        return v
          .apply(this, arguments)
          .always(function() {
            $(createdLoaders).each(function(i, loader) {
              var requests = loader.data('requests') || 0;
              loader.data('requests', --requests);
              if (requests === 0) {
                loader.hide();
              }
            });
            anchor.each(function() {
              var anchor = $(this);
              // Restore elements state
              anchor
                .add(anchor.find(DISABLED_ELEMENTS))
                .filter(function() {
                  return typeof $(this).data('api-disabled') !== undefined;
                })
                .prop('disabled', function() {
                  return $(this).data('api-disabled');
                })
                .removeData('api-disabled');
            });
          })
          .fail(function(jqXHR, textStatus, errorThrown) {
            if ($(window).data('unloading'))
              return;

            if (errorHandlers && errorHandlers[jqXHR.status]) {
              errorHandlers[jqXHR.status](jqXHR, textStatus, errorThrown);
              return;
            }


            if ($.notify) {
              var detail;
              if (jqXHR.responseJSON && jqXHR.responseJSON.detail) {
                var msg = jqXHR.responseJSON.detail;
                detail = (msg.indexOf('Could not an operation.') > -1) ? 'Could not perform requested operation.' + msg.substring(msg.indexOf(".") + 1) : msg;
              } else if (jqXHR.status == 400) {
                // XXX: assume that server returned object
                // which describes field errors

                // TODO: do this function more readable, additional checks for response format may be required
                var stringify = function(o, path) {
                  // Try to detect human readable field name
                  return _.isString(o) ? o : _(o)
                    .mapValues(function(v, k) {
                      var field = _([path, k]).filter().join('.');
                      var label = $('[data-api-field="' + field + '"]')
                        .data('api-label') || k;
                      return label + ':' +
                        '<ul>' +
                        _(v.text || v)
                          .reject(_.isEmpty)
                          .map(function(x) {
                            if (_.isObject(v) && !_.isArray(v))
                              return '<li>' + stringify(v, field) + '</li>';
                            else
                              return '<li>' + stringify(x, field) + '</li>';
                          })
                          .join('') +
                        '</ul>';
                    })
                    .map()
                    .join('');
                };

                detail = 'Validation error. See details below.<br><br>' +
                  stringify(jqXHR.responseJSON);
              } else if (jqXHR.status == 0) {
                detail = 'Could not connect';
              } else if (jqXHR.status == 406) {
                // XXX: we do not check Content-Type
                return;
              } else if (jqXHR.status == 500 && /file-manager/.test(window.location.href)) {
                detail = 'File Manager cannot perform the requested operation at this time.';
              } else if (jqXHR.responseText && !(/.*<!DOCTYPE html>.*/.test(jqXHR.responseText))) {
                detail = jqXHR.responseText;
              } else if (jqXHR.statusText) {
                detail = jqXHR.statusText;
              }

              var message = detail || textStatus || errorThrown;

              if (console && console.debug)
                console.debug(message);

              if (jqXHR.status === 0) {
                if (!notifyAlreadyShown(message))
                  $.notify('<p>' + message + '</p>',
                    { delay: 0, type: 'warning' });
              } else if (showError) {
                if (!notifyAlreadyShown(message))
                  $.notify('<p>' + message + '</p>',
                    { delay: 0, type: 'warning' });
              }
            }
          });
      };
    });

    function notifyAlreadyShown(message) {
      if ($('.alert-warning [data-notify="message"] p')
            .filter(function() {
              return _.escape($(this).text()) === message;}).length === 0 ) {
        return false;
      } else {
        return true;
      }
    }

    // XXX: all elements share API instance and thus anchors

    return element
      .data('api', api);
  };

  $(window).on('beforeunload', function() {
    $(window).data('unloading', true);
  });

  $(function() {
    $('.api').each(function() {
      $(this).loader(new Viewpoint.API());
    });
  });
})(jQuery);
