/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';
  var table = null;
  var api;
  var perms = $.viewpoint.user.permissions;
  var filter = '';

  $(function() {
    api = $('.datagrid-container').data('api');
    createDataTable();

    $('#elements-per-page-select').change(function() {
      loadData();
    });

    $('#refresh-btn').click(function() {
      loadData();
    });

    function formatJobType(jobType) {
      if (!jobType.id) {
        return jobType.text;
      }
      var typeIcon = window.getJobTypeIcon(jobType.id);
      return typeIcon + jobType.text;
    }

    api.get('/api/template-types/')
      .done(function(data) {
        $.each(data.results, function(_i, type) {
          if (type.id == 'nitro' && !perms.nitro_templates)
            return;
          if (type.id == 'remote_viz' && !perms.remote_visualization_templates)
            return;
          var typeIcon = window.getJobTypeIcon(type.id);
          $('#template-type').append(
            '<option value="' + type.id + '">' +
              typeIcon + type.name +
            '</option>'
          );
        });
        $('#template-type').prepend('<option value="all" selected>All</option>');

        $('#template-type').select2({
          width: '100%',
          minimumResultsForSearch: -1,
          escapeMarkup: function(m) {
            return m;
          },
          templateResult: formatJobType,
          templateSelection: formatJobType
        });

        $('#template-type').change(function() {
          var type = $('#template-type').val();
          if (type != 'all') {
            filter = '&type=' + type;
          } else {
            filter = '';
          }
          loadData();
        });
      });

    $('#rename-template').on('shown.bs.modal', function() {
      $('#template-name-input').focus();
    });

    $('#save-new-template-name').click(function() {
      $('#rename-template-form').submit();
    });

    $(document).on('click', '.popover-content .import', function(event) {
      $('.trigger').popover('hide');
      var id = $(event.target).data('id');

      var modalApi = $('#rename-template').find('.modal-content').data('api');
      modalApi.get('/api/templates/default/' + id + '/')
          .done(function(data) {
            var fieldToRemove = ['url', 'template_builder_version', 'id', 'use', 'history_url', 'version', 'changed_by' , 'date'];
            $.each(fieldToRemove, function(i, fieldName) {
              delete data[fieldName];
            });

            $('#template-name-input').val(data.name);
            $('#rename-template').modal('show');

            $('#rename-template-form').unbind('submit').submit(function(event) {
              event.preventDefault();
              data['name'] = $('#template-name-input').val();
              modalApi.post('/api/templates/', data).done(function(data) {
                $('#rename-template').modal('hide');
                $.notify('Template was successfully deployed',
                  { type: 'success' });
              }).fail(function(jqXHR, textStatus, errorThrown) {
                if (jqXHR.responseText.indexOf('duplicate key value violates unique constraint')) {
                  $.notify('We apologize but that name is already in use. Please pick a different name',
                  { type: 'warning' });
                } else {
                  $.notify(textStatus, { type: 'warning' });
                }
              });
            });
          })
      .fail(function(jqXHR) {
        if (jqXHR.status === 404) {
          $.notify('We detected unsynchronized content. Please try to refresh the page',
            { type: 'warning' });
        } else {
          $.notify(jqXHR.statusText,
            { type: 'warning' });
        }
      });
    });

    $(document).on('click', '.popover-content .export', function(event) {
      $('.trigger').popover('hide');
    });

  });

  function loadData(page) {
    page = page || 1;

    var pageSize = parseInt($('#elements-per-page-select').val());

    api.get('/api/templates/default/?page_size=' + pageSize + '&page=' + page + filter).done(function(data) {
      table.clear();
      table.rows.add(data.results);
      table.draw();
      updatePaginator(data.count, pageSize, page);
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    totalRows = parseInt(totalRows);
    rowsPerPage = parseInt(rowsPerPage);
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('.paginator-container').bootpag({
      total: total, // total pages
      page: currentRow,
      maxVisible: 7,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      loadData(num);
    }).show();
  }

  function createDataTable() {
    var opts = {
      data: [],
      dom: 't',
      bInfo: false,
      bDestroy: true,
      ordering: false,
      'columnDefs': [{
        'targets': 0,
        'data': null,
        'defaultContent': 'N/A',
        'className': 'name-cell',
        'render': function(data, type, full) {
          return createNameColumn(data, type, full);
        }
      }, {
        'targets': 1,
        'data': 'version_description',
        'defaultContent': 'N/A',
        'className': 'description-cell'
      }],
      'initComplete': function(settings, json) {
        createPopupMenu();
      },
      'fnCreatedRow': function(nRow, aData, iDataIndex) {
        $(nRow).attr('data-id', aData.id);
      }
    };

    loadData(1);
    var $table = $('#data-table');
    table = $table.DataTable(opts);
    $table.on('draw.dt', function() {
      createPopupMenu();
    });
  }

  function createNameColumn(data, type, full) {
    var links = '';
    links += '<a class="import" data-id=' + data.id + '>Deploy this template<a/>';
    links += '<a href="/api/templates/default/'+ data.id + '.template" class="export" data-id=' + data.id + '>Export</a>';
    var typeIcon = window.getJobTypeIcon(data.type);

    var block = $('<span>' + data.name + '</span>').add(
        $('<div class="menu-btn-container">')
          .append('<span class="glyphicon glyphicon-align-justify trigger">')
          .append(
            $('<div>')
              .addClass('content hide')
              .append(links)
          )
    );
    typeIcon = '<div class="template-icon">' + typeIcon + '</div>';
    return typeIcon + block[0].outerHTML + block[1].outerHTML;

  }

  function createPopupMenu() {
    $('.popover').remove();
    $('.trigger').popover('destroy');
    $('.trigger').popover({

      html: true,
      container: 'body',
      animation: false,
      content: function() {
        return $(this).parent().find('.content').html();
      },
      placement: 'right'
    });
    $('.trigger').off('click').on('click', function(e) {
      $('.trigger').popover('hide');
      $(this).popover('show');
      e.stopPropagation();
    }).popover();
    // Appending context-menu button
    $('.name-cell .menu-btn-container').hide();
    $(document)
      .off('mouseenter', '.name-cell')
      .on('mouseenter', '.name-cell', function() {
        $(this).children('.menu-btn-container').show();
      })
      .off('mouseleave', '.name-cell')
      .on('mouseleave', '.name-cell', function() {
        $('.name-cell .menu-btn-container').hide();
      });
    var mouseleave = function() {
      setTimeout(function() {
        if ($('.popover:hover').length === 0) {
          $('.trigger').popover('hide');
        }
      }, 250);
    };
    $(document)
      .off('mouseleave', '.popover')
      .on('mouseleave', '.popover', mouseleave)
      .off('mouseleave', '#data-table tr')
      .on('mouseleave', '#data-table tr', mouseleave);
  }

})(jQuery);
