/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

  var apiConfigurationUrl = '/api/configuration/main/';
  var nitroConfigNamesMap = {
    'URL': 'nitro-url',
    'USERNAME': 'nitro-username',
    'PASSWORD': 'nitro-password'
  };
  var isTest = false;

  $(function() {
    loadConfiguration();

    $('#reset-permissions').change(function() {
      $('#reset-permissions-alert-container')[$(this).prop("checked") ? 'show' : 'hide']();
    });

    $('#save-configuration-btn').click(function() {
      isTest = false;
      var api = $('#configuration-content').loader(new Viewpoint.API(), {400: fail400Callback}).data('api');
      var requestBody = {
        handler: 'api2.handlers.configuration.TestNitro',
        data: getNitroConfig()
      };

      api.post('/api/tasks/', requestBody).done(function() {
        requestBody.data.NITRO.IS_ENABLED = true;
        api.patch(apiConfigurationUrl, requestBody.data).done(function() {
          $.notify('Configuration has been saved',
            {
              type: 'success'
            });
        });
      });

    });

    $('#test-configuration-btn').click(function() {
      isTest = true;
      var api = $('#configuration-content').loader(new Viewpoint.API(), {400: fail400Callback}).data('api');
      var requestBody = {
        handler: 'api2.handlers.configuration.TestNitro',
        data: getNitroConfig()
      };

      api.post('/api/tasks/', requestBody).done(function() {
        $.notify('Communication test completed successfully!', {type: 'success'});
      });
    });
  });

  function fail400Callback(jqXHR) {
      var message = "Communication test failed. Please verify your configuration settings.";
      if (!isTest) {
        message = "Could not save the configuration. Please verify your settings.";
      }
      if (jqXHR.status == 400) {
        $.notify(message, {type: 'warning', delay: 0});
      }
    }

  function loadConfiguration() {
    var api = $('#configuration-content').data('api');

    api.get(apiConfigurationUrl).done(function(data) {
      $.each(nitroConfigNamesMap, function(paramName, paramId) {
        $('#' + paramId).val(data.NITRO[paramName]);
      });
      $('#nitro-trust-self-signed').prop("checked", !data.NITRO.CLIENT_VERIFY_SSL);
    });
  }

  function getNitroConfig() {
    var config = {NITRO: {}};
    $.each(nitroConfigNamesMap, function(paramName, paramId) {
      if (paramName == 'URL') {
        config.NITRO[paramName] = removeLastSlash($('#' + paramId).val());
      } else {
        config.NITRO[paramName] = $('#' + paramId).val();
      }
    });
    config.NITRO.CLIENT_VERIFY_SSL = !$('#nitro-trust-self-signed').prop("checked");
    return config;
  }

  function removeLastSlash(url) {
    if(url.substr(-1) === '/') {
        return removeLastSlash(url.substr(0, url.length - 1));
    }
    return url;
  }

})(jQuery);
