/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

  var principalId = null;
  var principalInitData = null;
  var availableEntities = {
    ldap: false,
    pam: false
  };
  var leaveFromPage = false;
  var tooltips = {
    'LDAP': 'e.g. username',
    'LDAPOU': 'e.g. ou=Engineering,ou=people,dc=testldap,dc=ac',
    'LDAPGROUP': 'e.g. cn=Engineering,ou=groups,dc=testldap,dc=ac'
  };

  $(function() {
    principalId = $.url('?id');
    if (principalId) {
      $('#page-header').text('Update Principal');
    }

    loadAvailableEntities();
    loadRoles();
    renderUsersTable();

    var validatorConfig = {
      submitButton: $('#apply-btn,#done-btn'),
      selector: ':input[name]:not([type="submit"], button, [formnovalidate]):enabled:visible, span[name], #principal-users'
    };
    $('form[data-toggle="validator2"]').validator(validatorConfig);
    $('body').on('validationCompleted', 'form[data-toggle="validator2"]', savingHandler);

    $('#apply-btn,#done-btn').click(function() {
      leaveFromPage = $(this).is('#done-btn');
      $('form[data-toggle="validator2"]').validator('validateAll');
    });

    $('#cancel-btn').click(function(e) {
      window.location.href = '/configuration/principals/';
    });

    $(window).on('beforeunload', function() {
      if (JSON.stringify(principalInitData) != JSON.stringify(serializePrincipal())) {
        return 'You have unsaved data!';
      }
    });

    $('#edit-users-btn').click(function() {
      $('#user-editing-modal').modal('show');
    });

    $('#add-entity-btn').click(function(e) {
      addEntityRow('', 'LDAPGROUP');
    });

    $('body').on('click', '.type-field', function(){
      $(this).parent().parent().find('td input').attr('title', $(this).find('option:selected').attr('title'));
    });

    $('#save-users-btn').click(function(e) {
      var entities = $('#user-editing-modal')
        .find('tbody')
        .find('.entity-row')
        .filter(function() {
          return !!$(this).find('.name-field').val();
        })
        .map(function(index, entityRow) {
          return {
            name: $(entityRow).find('.name-field').val(),
            type: $(entityRow).find('.type-field').val()
          };
        }).get();

      $('#principal-users').val(entities.length ? JSON.stringify(entities) : '').trigger('change');
      $('#user-editing-modal').modal('hide');
      renderUsersTable(entities);
    });

    $('#user-editing-modal').on('show.bs.modal', function() {
      var $modal = $('#user-editing-modal');
      var $tbody = $modal.find('tbody');
      var initData = JSON.parse($('#principal-users').val() || '[]');

      $tbody.empty().append(
        initData && initData.length ?
          $.map(initData, function(entity) {
            return buildTableRow(entity);
          }) :
          $tbody.append(buildTableRow({name: '', type: 'LDAPGROUP'}))
      );
    });
  });

  function loadAvailableEntities() {
    var api = $('#configuration-content').data('api');
    api.get('/api/mws-status').done(function(data) {
      availableEntities.ldap = data.ldap && data.ldap.connected;
      availableEntities.pam = data.pam && data.pam.connected;
    });
  }

  function addEntityRow(name, type) {
    var $modal = $('#user-editing-modal');
    var $tbody = $modal.find('tbody');

    $tbody.append(buildTableRow({name: name, type: type}));
  }

  function buildTableRow(entity) {
    return $('<tr></tr>').addClass('entity-row').append([
      $('<td></td>').append(
        $('<input>')
          .addClass('name-field form-control')
          .attr('type', 'text')
          .val(entity.name)
          .attr('title', tooltips[entity.type])
      ),
      $('<td></td>').append(buildEntityTypeSelect(entity.type)),
      $('<td></td>').append($('<span></span>').addClass('glyphicon glyphicon-remove').click(removeRowHandler))
    ]);

    function removeRowHandler() {
      $(this).closest('tr').remove();
    }
  }

  function buildEntityTypeSelect(val) {
    var ldapOptions = [
      {value: 'LDAPGROUP', text: 'LDAP Group', title: tooltips['LDAPGROUP']},
      {value: 'LDAPOU', text: 'LDAP Organizational Unit', title: tooltips['LDAPOU']},
      {value: 'LDAP', text: 'LDAP Username', title: tooltips['LDAP']}
    ];
    var pamOptions = [
      {value: 'PAMGROUP', text: 'PAM Group', title: ''},
      {value: 'PAM', text: 'PAM Username', title: ''}
    ];

    return $('<select></select>').append(
      $.map(availableEntities.ldap ? (ldapOptions) :
        (availableEntities.pam ? pamOptions : ldapOptions), function(option) {
        return $('<option></option>')
          .attr('value', option.value)
          .attr('title', option.title)
          .text(option.text)
          .prop('selected', option.value == val);
      })
    ).addClass('type-field form-control');
  }

  function savingHandler() {
    var api = $('#configuration-content').data('api');
    var requestUrl = principalId ? ('/api/principals/' + principalId + '/') : '/api/principals/';

    api[principalId ? 'put' : 'post'](requestUrl, serializePrincipal()).done(function(data) {
      principalId = data.id;
      principalInitData = serializePrincipal();
      $.notify(leaveFromPage ? 'The Principal has been saved. Redirecting back to principal list...' : 'The Principal has been saved', {
        type: 'success',
        onClose: function() {
          if (leaveFromPage) {
            window.location.href = '/configuration/principals/';
          }
        }
      });
    });
  }

  function loadRoles() {
    var api = $('#configuration-content').data('api');
    api.get('/api/roles/?page_size=1000').done(function(data) {

      var rolesList = $.map(data.results, function(role, index) {
        return $('<div></div>').addClass('checkbox').append(
          $('<label></label>').attr('title', role.description).append([
            $('<input>')
              .attr('name', 'checkbox')
              .attr('data-popover-anchor', '#roles-container')
              .attr('type', 'checkbox')
              // Add this attribute later when Roles list already filled with roles
              // This prevents showing validation message just after load in IE (IRIS-2565)
              //.attr('data-required-checkbox', true)
              .data('roleId', role.id)
              .data('roleName', role.name),
            role.name
          ])
        );
      });
      $('#roles-container').append(rolesList);

      if (principalId) {
        loadInitData();
      } else {
        principalInitData = serializePrincipal();
      }
    });
  }

  function serializePrincipal() {
    var principal = {};
    var usersGroupsJson = JSON.parse($('#principal-users').val() || '[]');
    principal.name = $('#principal-name').val();
    principal.description = $('#principal-description').val();
    principal.users = [];
    principal.groups = [];

    $.each(usersGroupsJson, function(i, e) {
      if (e.type == 'LDAP' || e.type == 'PAM') {
        principal.users.push(e);
      } else if ((e.type == 'LDAPGROUP') || (e.type == 'LDAPOU') || (e.type == 'PAMGROUP')) {
        principal.groups.push(e);
      }
    });

    principal.attachedRoles = $('#roles-container').find('input[type=checkbox]').filter(function() {
      return $(this).prop('checked');
    }).map(function() {
      return {id: $(this).data('roleId')};
    }).get();

    return principal;
  }

  function loadInitData() {
    var api = $('#configuration-content').data('api');
    api.get('/api/principals/' + principalId + '/').done(function(data) {
      $('#principal-name').val(data.name || '');
      $('#principal-description').val(data.description || '');

      var enabledCheckBoxesIds = $.map(data.attachedRoles || [], function(role, index) {
        return role.id;
      });
      $('#roles-container').find('input[type=checkbox]').filter(function() {
        return $.inArray($(this).data('roleId'), enabledCheckBoxesIds) > -1;
      }).prop('checked', true);

      $('#roles-container').find('input[type=checkbox]').attr('data-required-checkbox', true);

      renderUsersTable(data.users.concat(data.groups || []) || []);
      $('#principal-users').val(JSON.stringify(data.users.concat(data.groups || []) || []));

      principalInitData = serializePrincipal();
    });
  }

  function renderUsersTable(users) {
    var $table = $('#users-table');
    var typesMap = {
      LDAPGROUP: 'LDAP Group',
      LDAPOU: 'LDAP Organizational Unit',
      LDAP: 'LDAP Username',
      PAMGROUP: 'PAM Group',
      PAM: 'PAM Username'
    };

    if (!users || !users.length) {
      //render empty table
      $table.find('tbody').empty().append(
        $('<tr></tr>')
          .append($('<td></td>')
            .addClass('text-center')
            .attr('colspan', 3)
            .text('No users/groups set')
          )
      );
      return;
    }

    $table.find('tbody').empty().append(
      $.map(users, function(entity, index) {
        return $('<tr></tr>')
          .append([
            $('<td></td>').text(entity.name),
            $('<td></td>').text(typesMap[entity.type] || '')
          ]);
      })
    );

  }
})(jQuery);
