/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

  var apiConfigurationUrl = '/api/configuration/main/';
  var remoteVizConfigNamesMap = {
    'URL': 'remove-viz-url',
    'USERNAME': 'remote-viz-username',
    'PASSWORD': 'remote-viz-password',
  };
  var privateKeyFile = null;
  var isTest = false;

  $(function() {
    loadConfiguration();

    $('#save-configuration-btn').click(function() {
      isTest = false;
      if ($('#remote-viz-auth-method').val() === 'key-based' && !$('#private-key-file').val()  && !privateKeyFile) {
        $.notify('You must upload a private key', {type: 'warning'});
        return;
      }
      var api = $('#configuration-content').loader(new Viewpoint.API(), {400: fail400Callback}).data('api');
      var requestBody = {
        handler: 'api2.handlers.configuration.TestRemoteViz',
        data: getFastxConfig()
      };

      api.post('/api/tasks/', requestBody)
        .done(function() {
          requestBody.data.FASTX.IS_ENABLED = true;
          requestBody.data.FASTX.TOKEN = null;
          api.patch(apiConfigurationUrl, requestBody.data).done(function() {
            $.notify('Configuration has been saved',
            {
              type: 'success'
            });
            $('.key-auth-note').text('').hide();
          });
        });
    });

    $('#test-configuration-btn').click(function() {
      isTest = true;
      if ($('#remote-viz-auth-method').val() === 'key-based' && !$('#private-key-file').val() && !privateKeyFile) {
        $.notify('You must upload a private key', {type: 'warning'});
        return;
      }
      if (privateKeyFile && !privateKeyFile.length) {
        $.notify('You have uploaded an empty file', {type: 'warning'});
        return;
      }
      var api = $('#configuration-content').loader(new Viewpoint.API(), {400: fail400Callback}).data('api');
      var requestBody = {
        handler: 'api2.handlers.configuration.TestRemoteViz',
        data: getFastxConfig()
      };

      api.post('/api/tasks/', requestBody).done(function() {
        $.notify('Communication test completed successfully!', {type: 'success'});
      });
    });

    function fail400Callback(jqXHR) {
      var message = "Communication test failed. Please verify your configuration settings.";
      if (!isTest) {
        message = "Could not save the configuration. Please verify your settings.";
      }
      if (jqXHR.status == 400) {
        $.notify(message, {type: 'warning', delay: 0});
      }
    }

    $('#remote-viz-auth-method').change(function() {
      if ($(this).val() === 'key-based') {
        $('#remote-viz-password').closest('.form-group').hide();
        $('#private-key-file').parent().parent().show();
        $('.key-auth-note').show();
        if (!$('#private-key-file').val() && !privateKeyFile) {
          $('#test-configuration-btn').addClass('disabled').parent()
            .attr('title', 'You must upload a key in order to test your configuration');
          $('#save-configuration-btn').addClass('disabled').parent()
            .attr('title', 'You must upload a key in order to save your configuration');
        }
        if (privateKeyFile)
          $('#private-key-file').next().attr('title', 'Select a new Private Key');
      } else {
        $('#remote-viz-password').closest('.form-group').show();
        $('#private-key-file').parent().parent().hide();
        $('#test-configuration-btn, #save-configuration-btn').removeClass('disabled');
        $('#test-configuration-btn').removeClass('disabled').parent()
          .attr('title', 'Test your configuration');
        $('#save-configuration-btn').removeClass('disabled').parent()
          .attr('title', 'Save your configuration');
        $('.key-auth-note').hide();
      }
    });

    $('#private-key-file').val('');
    $('#private-key-file').on('change', handleFileSelect);
  });

  function handleFileSelect(evt) {
    var files = evt.target.files;
    var f = files[0];
    var reader = new FileReader();

    reader.onload = (function() {
      return function(e) {
        privateKeyFile = e.target.result;
      };
    })(f);

    reader.onloadstart = (function() {
      return function(e) {
        $('#test-configuration-btn, #save-configuration-btn').addClass('disabled');
      };
    })(f);

    reader.onloadend = (function() {
      return function(e) {
        $('#test-configuration-btn, #save-configuration-btn').removeClass('disabled');
        $('#private-key-file').next().attr('title', 'Select a new Private Key');
        $('.key-auth-note').text('You have selected the "' + f.name + '" file. ' +
          'Please press save button to upload this key and save configuration.');
        $('#private-key-file').next().text('Change Key');
      };
    })(f);

    reader.readAsText(f);
  }

  function loadConfiguration() {
    var api = $('#configuration-content').data('api');

    api.get(apiConfigurationUrl).done(function(data) {
      $.each(remoteVizConfigNamesMap, function(paramName, paramId) {
        $('#' + paramId).val(data.FASTX[paramName]);
      });
      $('#remote-viz-trust-self-signed').prop('checked', !data.FASTX.CLIENT_VERIFY_SSL);
      if (data.FASTX.USE_PK) {
        privateKeyFile = data.FASTX.PRIVATE_KEY;
        $('#remote-viz-auth-method').val('key-based').trigger('change');
        $('#remote-viz-password').val('');
        $('#private-key-file').next().text('Change Key');
      } else {
        $('#remote-viz-auth-method').val('password').trigger('change');
      }
    });
  }

  function getFastxConfig() {
    var config = {FASTX: {}};
    $.each(remoteVizConfigNamesMap, function(paramName, paramId) {
      config.FASTX[paramName] = $('#' + paramId).val();
    });
    config.FASTX.CLIENT_VERIFY_SSL = !$('#remote-viz-trust-self-signed').prop('checked');
    config.FASTX.USE_PK = $('#remote-viz-auth-method').val() === 'key-based';
    config.FASTX.PRIVATE_KEY = privateKeyFile;
    config.FASTX.URL = removeLastSlash(config.FASTX.URL);
    if (config.FASTX.USE_PK)
      $('#remote-viz-password').val('');
    return config;
  }

  function removeLastSlash(url) {
    if(url.substr(-1) === '/') {
        return removeLastSlash(url.substr(0, url.length - 1));
    }
    return url;
  }

})(jQuery);
