/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

  var roleId = null,
      roleInitData = null,
      leaveFromPage = false,
      mwsPermissions = {},
      domainPermissions = {},
      disabledPermissions = ['nodes', 'templates', 'file_manager', 'sessions', 'configuration', 'home', 'workload'],
      permissionsMap = {
        'home': ['read-jobs', 'read-insight-user'],
        'workload': ['read-jobs', 'read-insight-user'],
        'node_summary_widget': ['read-insight-privileged'],
        'system_resources_widget': ['read-insight-privileged'],
        'create_job': ['read-credentials', 'read-nodes', 'create-jobs', 'create-job-arrays', 'update-jobs', 'cancel-jobs', 'read-resource-types'],
        'job_details': ['read-nodes', 'read-resource-types'],
        'priority_analysis': ['update-jobs'],
        'job_view_all': ['read-insight-privileged', 'update-jobs-any'],
        'templates': ['read-credentials'],
        'nodes': ['read-insight-privileged', 'update-nodes-features', 'read-nodes'],
        'roles': ['create-roles', 'read-roles', 'update-roles', 'delete-roles', 'read-permissions'],
        'principals': ['create-principals', 'read-principals', 'update-principals', 'delete-principals', 'read-roles'],
        'licensed_features': ['read-diagnostics']
      };

  $(function() {
    roleId = $.url('?id');
    var api = $('#configuration-content').data('api');

    if (roleId) {
      $('#page-header').text('Update Role');
      $.when(
        api.get('/api/permissions/?page_size=1000'),
        api.get('/api/roles/' + roleId + '/')
      )
      .done(function(args1, args2) {
        loadPermissions(args1[0], args2[0]);
        loadInitData(args2[0]);
      });
    } else {
      api.get('/api/permissions/?page_size=1000').done(function(data) {
        loadPermissions(data);
      });
    }

    var validatorConfig = {
      submitButton: $('#apply-btn,#done-btn'),
      selector: ':input[name]:not([type="submit"], button, [formnovalidate]):enabled:visible, span[name], #principal-users'
    };
    $('form[data-toggle="validator2"]').validator(validatorConfig);
    $('body').on('validationCompleted', 'form[data-toggle="validator2"]', savingHandler);

    $('#apply-btn,#done-btn').click(function() {
      leaveFromPage = $(this).is('#done-btn');
      $('form[data-toggle="validator2"]').validator('validateAll');
    });

    $('#cancel-btn').click(function(e) {
      window.location.href = '/configuration/roles/';
    });

    $(document).on({
      mouseenter: function () {
          $('.permission-description .description').text($(this).data('description'));
          $('.permission-description').show();
      },
      mouseleave: function () {
        $('.permission-description').hide();
      }
    }, ".jstree-anchor");

    $(window).on('beforeunload', function() {
      if (JSON.stringify(roleInitData) != JSON.stringify(serializeRole())) {
        return 'You have unsaved data!';
      }
    });
  });

  function savingHandler() {
    var api = $('#configuration-content').data('api');
    var requestUrl = roleId ? ('/api/roles/' + roleId + '/') : '/api/roles/';

    api[roleId ? 'put' : 'post'](requestUrl, serializeRole()).done(function(data) {
      if ('HPCUser' == data.name || 'HPCAdmin' == data.name) {
        $('#reset-btn').unbind('click');
        $('#reset-btn').show().click(resetRole(data));
      } else {
        $('#reset-btn').hide();
      }
      roleInitData = serializeRole();
      roleId = data.id;
      $.notify(leaveFromPage ? 'The Role has been saved. Redirecting back to roles list...': 'The Role has been saved', {
        type: 'success',
        onClose: function() {
          if (leaveFromPage) {
            window.location.href = '/configuration/roles/';
          }
        }
      });
    });
  }

  function loadPermissions(permissionsData, roleData) {
    mwsPermissions = permissionsData;
    var permissionsSchema = [];

    if (!permissionsData || !permissionsData.results) {
      return;
    }

    var permissionsOfRole = [];
    if (roleData && roleData.permissions.length > 0) {
       permissionsOfRole = $.map(roleData.permissions, function(perm, index) {
         return perm.id;
       });
    }

    $.each(permissionsData.results, function(i, permission) {
      if (permission.isViewpointPermission) {
        var id_parent = permission.resource.split('.');
        permissionsSchema.push({
          'id': id_parent[0],
          'parent': id_parent[1],
          'text': permission.label,
          'state': {
            'disabled': $.inArray(id_parent[0], disabledPermissions) > -1,
            'checked': $.inArray(permission.id, permissionsOfRole) > -1,
          },
          'a_attr': {
            'data-description': permission.description,
            'data-permissionId': permission.id,
            'data-permissionName': permission.label
          }
        });
      }

      if (!permission.isViewpointPermission) {
        domainPermissions[permission.label] = permission.id;
      }
    });

    function getSelectAllCheckbox(containerSelector) {
      var selectAllHandler = function() {
        if ($(this).is(':checked'))
          $(containerSelector).jstree('check_all');
        else
          $(containerSelector).jstree('uncheck_all');
      };

      return $('<div></div>')
        .addClass('checkbox')
        .append(
          $('<label></label>').append([
            $('<input>').attr('type', 'checkbox').click(selectAllHandler),
            $('<strong></strong>').text('Select All')])
        );
    }

    $('#permissions-container')
      .empty()
      .append([
        $('<div></div>').addClass('col-xs-12').append($('<strong></strong>').text('Viewpoint Permissions:')),
        $('<div></div>').addClass('col-xs-12').append(getSelectAllCheckbox('#viewpoint-permissions-container')),
        $('<div></div>').addClass('col-xs-4').addClass('permission-description')
        .append([
          $('<div></div>').addClass('header').text('Permission description:'),
          $('<div></div>').addClass('description')
        ]),
        $('<div></div>')
          .addClass('col-xs-12 permissions-list')
          .attr('id', 'viewpoint-permissions-container')
      ]);
    $('#viewpoint-permissions-container')
      .jstree({
        'core' : {
          'check_callback': true,
          'animation' : 0,
          'themes' : { 'icons' : false },
          'data' : permissionsSchema
        },
        'plugins' : [
          'checkbox'
        ],
        'checkbox': {
          'three_state': false,
          'tie_selection': false
        }
      })
      .on('loaded.jstree', function() {
        $('#viewpoint-permissions-container').jstree('open_all');
        roleInitData = serializeRole();
      })
      .on('check_node.jstree', function(e, n) {
        var tree = $('#viewpoint-permissions-container').jstree(true);
        $.each(n.node.parents.slice(0, -1), function(k, p) {
          if (!tree.is_checked(p))
            tree.check_node(p);
        });
      })
      .on('uncheck_node.jstree', function(e, n) {
        var tree = $('#viewpoint-permissions-container').jstree(true);
        $.each(n.node.children, function(k, child) {
          tree.uncheck_node(child);
        });
        // check whether any children are checked
        var parent = tree.get_node(n.node.parent);
        var hasSelectedChildren = false;
        $.each(parent.children, function(i, c) {
          if (tree.is_checked(tree.get_node(c))) {
            hasSelectedChildren = true;
            return false;
          }
        });
        if (!hasSelectedChildren && n.node.parents.length == 2)
          tree.uncheck_node(n.node.parent);
      });
  }

  function serializeRole() {
    var role = {},
        permissions = $('#viewpoint-permissions-container').jstree().get_checked(true);

    role.name = $('#role-name').val();
    role.description = $('#role-description').val();
    role.permissions = permissions.map(function(el) {
      return el.a_attr['data-permissionId'];
    });

    // add domain permissions
    var jstreePermIds = permissions.map(function(el) {
      return el.id;
    });
    $.each(jstreePermIds, function(i, v) {
      if (v in permissionsMap) {
        $.each(permissionsMap[v], function() {
          var permId = domainPermissions[this];
          if (permId && $.inArray(permId, role.permissions) == -1) {
            role.permissions.push(permId);
          }
        });
      }
    });

    role.permissions = _.uniq(role.permissions)
      .map(function(permId){
        return {id: permId};
      });

    return role;
  }

  function resetRole(roleData) {
    var api = $('#configuration-content').data('api');
    return function() {
      $.viewpoint.confirm('Reset Operation', 'Are you sure you want to reset this role?').done(function(result) {
        if (result) {
          api.post('/api/tasks/', {
            handler: 'api2.handlers.roles.ResetRole',
            data: {
              roleName: roleData.name,
              prototypeName: roleData.name
            }
          }).done(function(roleData) {
            api.get('/api/permissions/?page_size=1000').done(function(permData) {
              loadPermissions(permData, roleData);
            });
            $.notify('The role was successfully reset', {
              type: 'success'
            });
          });
        }
      });
    };
  }

  function loadInitData(roleData) {
    $('#role-name').val(roleData.name || '');
    $('#role-description').val(roleData.description || '');
    if ('HPCUser' == roleData.name || 'HPCAdmin' == roleData.name) {
      $('#reset-btn').unbind('click');
      $('#reset-btn').show().click(resetRole(roleData));
    }
  }
})(jQuery);
