/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';
  var table = null;

  $(function() {
    createDataTable();

    $('#elements-per-page-select').change(function() {
      loadData();
    });

    $('#refresh-btn').click(function() {
      loadData();
    });

    $(document).on('click', '.popover-content .role-delete', function(event) {
      $('.trigger').popover('hide');
      var api = $('.datagrid-container').data('api');
      var url = $(event.target).data('url');

      $.viewpoint.confirm('Delete Role', 'Are you sure?').done(function(result) {
        if (result) {
          api.delete(url).done(function() {
            loadData();
            $.notify('The role was successfully deleted', {type: 'success'});
          });
        }
      });
    });
  });

  function loadData(page) {
    page = page || 1;

    var api = $('.datagrid-container').data('api');
    var pageSize = parseInt($('#elements-per-page-select').val());

    api.get('/api/roles/?page_size=' + pageSize + '&page=' + page).done(function(data) {
      table.clear();
      table.rows.add(data.results);
      table.draw();
      updatePaginator(data.count, pageSize, page);
    });

    checkBaseRolesExisting();
  }

  function checkBaseRolesExisting() {
    if ($.viewpoint.user.name != "moab-admin" && (!$.viewpoint.user.permissions.create_roles ||
          !$.viewpoint.user.permissions.update_roles || !$.viewpoint.user.permissions.read_permissions))
      return;
    async.series(getTasks(), function(err, results) {
      var $rolesRecreatingBtn = $('#recreate-basic-roles-btn');
      if (6 == _.filter(results, function(e) {
          return e;
        }).length) {
        $rolesRecreatingBtn.hide();
      } else {
        $rolesRecreatingBtn
          .show()
          .unbind('click')
          .click(function() {
            var api = $('.datagrid-container').data('api');
            var basicRoles = {
              HPCAdmin: {
                name: 'HPCAdmin',
                description: 'Administrative user, with privileges for all features and jobs',
                scope: 'GLOBAL',
                permissions: []
              },
              HPCUser: {
                name: 'HPCUser',
                description: 'Basic user, with permission to create and manage their own jobs',
                scope: 'GLOBAL',
                permissions: []
              },
              NitroUser: {
                name: 'NitroUser',
                description: 'Basic user, with permission to create and manage their own Nitro jobs',
                scope: 'GLOBAL',
                permissions: []
              },
              NitroAdmin: {
                name: 'NitroAdmin',
                description: 'Administrative user, with permission to create Nitro application templates and manage other user\'s Nitro jobs',
                scope: 'GLOBAL',
                permissions: []
              },
              RemoteVizUser: {
                name: 'RemoteVizUser',
                description: 'Basic user, with permission to create and manage their own remote visualization jobs',
                scope: 'GLOBAL',
                permissions: []
              },
              RemoteVizAdmin: {
                name: 'RemoteVizAdmin',
                description: 'Administrative user, with permission to create remote visualization application templates and manage other user\'s remote visualization jobs',
                scope: 'GLOBAL',
                permissions: []
              }
            };

            _.each(results, function(isExist, roleName) {
              if (!isExist) {
                // Create role
                api.post('/api/roles/', basicRoles[roleName]).done(function() {
                  // Set permissions
                  api.post('/api/tasks/', {
                    handler: 'api2.handlers.roles.ResetRole',
                    data: {
                      roleName: roleName,
                      prototypeName: roleName
                    }
                  }).done(function() {
                    $.notify('The role \'{role}\' was successfully recreated'.replace('{role}', roleName), {
                      type: 'success'
                    });
                    loadData();
                  });
                });
              }
            });

          });
      }
    });

    function getTasks() {
      var result = {};
      var basicRoles = ['HPCAdmin', 'HPCUser', 'NitroUser', 'NitroAdmin', 'RemoteVizUser', 'RemoteVizAdmin'];

      _.each(basicRoles, function(roleName) {
        result[roleName] = function(callback) {
          new Viewpoint.API().get('/api/roles/{role}/'.replace('{role}', roleName))
            .done(function(data) {
              callback(null, true);
            })
            .fail(function(jqXHR) {
              callback(404 == jqXHR.status ? null : jqXHR, 404 == jqXHR.status ? false : null)
            });
        };
      });

      return result;
    }
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    totalRows = parseInt(totalRows);
    rowsPerPage = parseInt(rowsPerPage);
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('.paginator-container').bootpag({
      total: total, // total pages
      page: currentRow,
      maxVisible: 7,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      loadData(num);
    }).show();
  }

  function createDataTable() {
    var opts = {
      data: [],
      dom: 't',
      bInfo: false,
      bDestroy: true,
      ordering: false,
      'columnDefs': [{
        'targets': 0,
        'data': null,
        'defaultContent': 'N/A',
        'className': 'name-cell',
        'render': function(data, type, full) {
          return createNameColumn(data, type, full);
        }
      }, {
        'targets': 1,
        'data': 'description',
        'defaultContent': 'N/A',
        'className': 'description-cell'
      }],
      'initComplete': function(settings, json) {
        createPopupMenu();
      },
      'fnCreatedRow': function(nRow, aData, iDataIndex) {
        $(nRow).attr('data-id', aData.id);
      }
    };

    loadData(1);
    var $table = $('#data-table');
    table = $table.DataTable(opts);
    $table.on('draw.dt', function() {
      createPopupMenu();
    });
  }

  function createNameColumn(data, type, full) {
    var links = '';
    links += $.viewpoint.user.permissions.update_roles ?
    '<a href="/configuration/role/?id=' + data.id + '">Modify<a/>' : '';
    links += $.viewpoint.user.permissions.delete_roles ?
    '<a href="#" class="role-delete" data-url="' + data.url + '">Delete</a>' : '';

    var block = $('<a ' + ($.viewpoint.user.permissions.update_roles ?
      'href="/configuration/role/?id=' + data.id + '"': '') + '>' + data.name + '</a>').add(
      $.viewpoint.user.permissions.update_roles || $.viewpoint.user.permissions.delete_roles ?
        $('<div class="menu-btn-container">')
          .append('<span class="glyphicon glyphicon-align-justify trigger">')
          .append(
            $('<div>')
              .addClass('content hide')
              .append(links)
          ) : ''
    );

    return _.reduce(block, function(result, $str) {
      return result + $str.outerHTML;
    }, '');
  }

  function createPopupMenu() {
    $('.popover').remove();
    $('.trigger').popover('destroy');
    $('.trigger').popover({

      html: true,
      container: 'body',
      animation: false,
      content: function() {
        return $(this).parent().find('.content').html();
      },
      placement: 'right'
    });
    $('.trigger').off('click').on('click', function(e) {
      $('.trigger').popover('hide');
      $(this).popover('show');
      e.stopPropagation();
    }).popover();
    // Appending context-menu button
    $('.name-cell div').hide();
    $(document)
      .off('mouseenter', '.name-cell')
      .on('mouseenter', '.name-cell', function() {
        $(this).children('div').show();
      })
      .off('mouseleave', '.name-cell')
      .on('mouseleave', '.name-cell', function() {
        $('.name-cell div').hide();
      });
    var mouseleave = function() {
      setTimeout(function() {
        if ($('.popover:hover').length === 0) {
          $('.trigger').popover('hide');
        }
      }, 250);
    };
    $(document)
      .off('mouseleave', '.popover')
      .on('mouseleave', '.popover', mouseleave)
      .off('mouseleave', '#data-table tr')
      .on('mouseleave', '#data-table tr', mouseleave);
    $('.trigger').on('show.bs.popover', popoverShow);
    $('.trigger').on('shown.bs.popover', popoverShown);
  }

  function popoverShow(event) {
    var count = $('#data-table tbody tr.active-session').length;
    if (count == 1) {
      $('#data-table tbody tr.active-session').removeClass('active-session');
    }
  }

  function popoverShown(event) {
    var item = $(event.target).closest('tr')[0];
    if (!$(item).hasClass('active-session')) {
      $('.popover-content').removeClass('active');
    } else {
      $('.popover-content').addClass('active');
    }
  }
})(jQuery);
