/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

(function($, window, document, undefined) {
  'use strict';

  // DURATIONPICKER PUBLIC CLASS DEFINITION
  var Durationpicker = function(element, options) {
    this.widget = '';
    this.$element = $(element);
    this.isOpen = options.isOpen;
    this.modalBackdrop = options.modalBackdrop;
    this.orientation = options.orientation;
    this.showSeconds = options.showSeconds;
    this.template = options.template;
    this.appendWidgetTo = options.appendWidgetTo;
    this.showWidgetOnAddonClick = options.showWidgetOnAddonClick;
    this.readonly = options.readonly;

    this._init();
  };

  Durationpicker.prototype = {

    constructor: Durationpicker,
    _init: function() {
      var self = this;

      if (this.showWidgetOnAddonClick && (this.$element.parent().hasClass('input-append') || this.$element.parent().hasClass('input-prepend'))) {
        this.$element.parent('.input-append, .input-prepend').find('.add-on').on({
          'click.timepicker': $.proxy(this.showWidget, this)
        });
        if (this.readonly) {
          this.$element.on({
            'click.timepicker': $.proxy(this.showWidget, this)
          });
        }
      } else {
        if (this.template) {
          this.$element.on({
            'focus.timepicker': $.proxy(this.showWidget, this),
            'click.timepicker': $.proxy(this.showWidget, this)
          });
        }
      }

      if (this.template !== false) {
        this.$widget = $(this.getTemplate())
      .on('click', $.proxy(this.widgetClick, this))
      .on('change', $.proxy(this.widgetInputChanged, this));
      } else {
        this.$widget = false;
      }

      if (this.$widget !== false) {
        this.$widget.find('input').each(function() {
          $(this).on({
            'click.timepicker': function() { $(this).select(); },
          });
        });
      }

      if (this.readonly) {
        this.$element.prop('readonly', true);
      }
    },

  decrementWeek: function() {
    this.setValue('bootstrap-timepicker-week', -1);
  },

  decrementDay: function() {
    this.setValue('bootstrap-timepicker-day', -1);
  },

  decrementHour: function() {
    this.setValue('bootstrap-timepicker-hour', -1);
  },

  decrementMinute: function(step) {
    this.setValue('bootstrap-timepicker-minute', -1);
  },

  decrementSecond: function() {
    this.setValue('bootstrap-timepicker-second', -1);
  },

  setValue: function(selector, step) {
    var input = this.$widget.find('input.' + selector)[0];
    var val = $(input).val();
    if (val) {
      var iVal = parseInt(val);
      if (isNaN(iVal)) {
        return;
      }
      iVal += step;
      if (iVal < 0) {
        $(input).val(0);
      } else {
        $(input).val(iVal);
      }
    } else {
      if (step == 1) {
        $(input).val(1);
      } else {
        $(input).val(0);
      }
    }
  },

  getTemplate: function() {
    var template,
    weekTemplate,
    dayTemplate,
    hourTemplate,
    minuteTemplate,
    secondTemplate,
    templateContent;

    weekTemplate = '<input type="text" class="bootstrap-timepicker-week" />';
    dayTemplate = '<input type="text" class="bootstrap-timepicker-day" />';
    hourTemplate = '<input type="text" class="bootstrap-timepicker-hour" />';
    minuteTemplate = '<input type="text" class="bootstrap-timepicker-minute" />';
    secondTemplate = '<input type="text" class="bootstrap-timepicker-second" />';

    templateContent = '<table>' +
         '<tr>'+
           '<td><a href="#" data-action="incrementWeek"><span class="glyphicon glyphicon-chevron-up"></span></a></td>'+
           '<td class="separator">&nbsp;</td>'+
           '<td><a href="#" data-action="incrementDay"><span class="glyphicon glyphicon-chevron-up"></span></a></td>'+
           '<td class="separator">&nbsp;</td>'+
           '<td><a href="#" data-action="incrementHour"><span class="glyphicon glyphicon-chevron-up"></span></a></td>'+
           '<td class="separator">&nbsp;</td>'+
           '<td><a href="#" data-action="incrementMinute"><span class="glyphicon glyphicon-chevron-up"></span></a></td>'+
           (this.showSeconds ?
             '<td class="separator">&nbsp;</td>'+
             '<td><a href="#" data-action="incrementSecond"><span class="glyphicon glyphicon-chevron-up"></span></a></td>'
           : '') +
         '</tr>'+
         '<tr>'+
       '<td>'+ weekTemplate +'</td> '+
       '<td class="separator">w</td>'+
       '<td>'+ dayTemplate +'</td> '+
       '<td class="separator">d&nbsp;</td>'+
           '<td>'+ hourTemplate +'</td> '+
           '<td class="separator">:</td>'+
           '<td>'+ minuteTemplate +'</td> '+
           (this.showSeconds ?
            '<td class="separator">:</td>'+
            '<td>'+ secondTemplate +'</td>'
           : '') +
         '</tr>'+
         '<tr>'+
           '<td><a href="#" data-action="decrementWeek"><span class="glyphicon glyphicon-chevron-down"></span></a></td>'+
           '<td class="separator"></td>'+
           '<td><a href="#" data-action="decrementDay"><span class="glyphicon glyphicon-chevron-down"></span></a></td>'+
           '<td class="separator"></td>'+
           '<td><a href="#" data-action="decrementHour"><span class="glyphicon glyphicon-chevron-down"></span></a></td>'+
           '<td class="separator"></td>'+
           '<td><a href="#" data-action="decrementMinute"><span class="glyphicon glyphicon-chevron-down"></span></a></td>'+
           (this.showSeconds ?
            '<td class="separator">&nbsp;</td>'+
            '<td><a href="#" data-action="decrementSecond"><span class="glyphicon glyphicon-chevron-down"></span></a></td>'
           : '') +
         '</tr>'+
       '</table>';

      switch(this.template) {
      case 'modal':
        template = '<div class="bootstrap-timepicker-widget modal hide fade in" data-backdrop="'+ (this.modalBackdrop ? 'true' : 'false') +'">'+
          '<div class="modal-header">'+
            '<a href="#" class="close" data-dismiss="modal">×</a>'+
            '<h3>Pick a Time</h3>'+
          '</div>'+
          '<div class="modal-content">'+
            templateContent +
          '</div>'+
          '<div class="modal-footer">'+
            '<a href="#" class="btn btn-primary" data-dismiss="modal">OK</a>'+
          '</div>'+
        '</div>';
        break;
      case 'dropdown':
        template = '<div class="bootstrap-timepicker-widget dropdown-menu">'+ templateContent +'</div>';
        break;
      }

    return template;
  },

  setInput: function(durationObj) {
    var v = '';
    if (durationObj.weeks) {
      v += durationObj.weeks + 'w';
    }
    if (durationObj.days) {
      if (v) {
        v += ' ';
      }
      v += durationObj.days + "d";
    }
    if (!durationObj.hours && !durationObj.minutes && !durationObj.seconds) {
      this.$element.val(v);
      return;
    }
    if (v) {
      v += ' ';
    }
    var hours = durationObj.hours || '0';
    var minutes = durationObj.minutes || '00';
    var seconds = durationObj.seconds || '00';
    minutes += '';
    seconds += '';
    if (minutes.length == 1) {
      minutes = '0' + minutes;
    }
    if (seconds.length == 1) {
      seconds = '0' + seconds;
    }
    v += hours + ':' + minutes + ':' + seconds;
    this.$element.val(v);
  },

  validatePickers: function(durationObj) {
    var isValid = true;
    this.$widget.find('input.bootstrap-timepicker-week').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-day').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-hour').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-minute').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-second').removeClass('error');
    if (isNaN(durationObj.weeks) || durationObj.weeks < 0) {
      this.$widget.find('input.bootstrap-timepicker-week').addClass('error');
      isValid = false;
    }
    if (isNaN(durationObj.days) || durationObj.days < 0) {
      this.$widget.find('input.bootstrap-timepicker-day').addClass('error');
      isValid = false;
    }
    if (isNaN(durationObj.hours) || durationObj.hours < 0) {
      this.$widget.find('input.bootstrap-timepicker-hour').addClass('error');
      isValid = false;
    }
    if (isNaN(durationObj.minutes) || durationObj.minutes < 0) {
      this.$widget.find('input.bootstrap-timepicker-minute').addClass('error');
      isValid = false;
    }
    if (isNaN(durationObj.seconds) || durationObj.seconds < 0) {
      this.$widget.find('input.bootstrap-timepicker-second').addClass('error');
      isValid = false;
    }
    return isValid;
  },

  prepareDuration: function(durationObj) {
    var minutes = Math.floor(durationObj.seconds / 60);
    if (minutes > 0) {
      durationObj.seconds = durationObj.seconds % 60;
      durationObj.minutes += minutes;
    }

    var hours = Math.floor(durationObj.minutes / 60);
    if (hours > 0) {
      durationObj.minutes = durationObj.minutes % 60;
      durationObj.hours += hours;
    }

    var days = Math.floor(durationObj.hours / 24);
    if (days > 0) {
      durationObj.hours = durationObj.hours % 24;
      durationObj.days += days;
    }

    var weeks = Math.floor(durationObj.days / 7);
    if (weeks > 0) {
      durationObj.days = durationObj.days % 7;
      durationObj.weeks += weeks;
    }

    var fullSeconds = durationObj.seconds +
      60*durationObj.minutes +
      3600*durationObj.hours +
      86400*durationObj.days +
      604800*durationObj.weeks;
    durationObj.fullSeconds = fullSeconds;
    return durationObj;
  },

  parseDuration: function() {
    var durationObj = this.getDurationFromInput();
    if (!durationObj.valid) {
      return 0;
    }
    if (this.validatePickers(durationObj)) {
      return this.prepareDuration(durationObj);
    } else {
      return 0;
    }
  },

  setFullSeconds: function(fullSeconds) {
    var seconds = parseInt(fullSeconds, 10);
    if (isNaN(seconds)) {
      return;
    }
    var durationObj = {
      seconds: seconds,
      minutes: 0,
      hours: 0,
      days: 0,
      weeks: 0
    };
    durationObj = this.prepareDuration(durationObj);
    this.setInput(durationObj);
  },

  getDurationFromPickers: function() {
    var durationObj = {};
    var weeks = this.$widget.find('input.bootstrap-timepicker-week').val() || '0';
    var days = this.$widget.find('input.bootstrap-timepicker-day').val() || '0';
        var hours = this.$widget.find('input.bootstrap-timepicker-hour').val() || '0';
        var minutes = this.$widget.find('input.bootstrap-timepicker-minute').val() || '0';
        var seconds = this.$widget.find('input.bootstrap-timepicker-second').val() || '0';
    durationObj.weeks = parseInt(weeks);
    durationObj.days = parseInt(days);
    durationObj.hours = parseInt(hours);
    durationObj.minutes = parseInt(minutes);
    durationObj.seconds = parseInt(seconds);
    return durationObj;
  },

  clear: function() {
    this.setPickers({
      weeks: 0,
      days: 0,
      hours: 0,
      minutes: 0,
      seconds: 0
    });
  },

  setPickers: function(durationObj) {
    durationObj = durationObj || {};

    this.$widget.find('input.bootstrap-timepicker-week').val('0').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-day').val('0').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-hour').val('0').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-minute').val('0').removeClass('error');
    this.$widget.find('input.bootstrap-timepicker-second').val('0').removeClass('error');

    if (durationObj.weeks) {
      this.$widget.find('input.bootstrap-timepicker-week').val(durationObj.weeks);
    }
    if (durationObj.days) {
      this.$widget.find('input.bootstrap-timepicker-day').val(durationObj.days);
    }
    if (durationObj.hours) {
      this.$widget.find('input.bootstrap-timepicker-hour').val(durationObj.hours);
    }
    if (durationObj.minutes) {
      this.$widget.find('input.bootstrap-timepicker-minute').val(durationObj.minutes);
    }
    if (durationObj.seconds) {
      this.$widget.find('input.bootstrap-timepicker-second').val(durationObj.seconds);
    }
  },

  getDurationFromInput: function() {
    var val = this.$element.val();
    var elms = val.split(' ');
    var durationObj = {
      valid: true
    };
    $.each(elms, function(i, elm) {
      if (elm[elm.length - 1] == 'w') {
        var weeks = elm.substr(0, elm.length - 1);
        durationObj.weeks = parseInt(weeks, 10);
        if (isNaN(durationObj.weeks)) {
          durationObj.valid = false;
        }
      } else if (elm[elm.length - 1] == 'd') {
        var days = elm.substr(0, elm.length - 1);
        durationObj.days = parseInt(days, 10);
        if (isNaN(durationObj.days)) {
          durationObj.valid = false;
        }
      } else {
        var time_elms = elm.split(':');
        if (time_elms.length == 3) {
          durationObj.hours = parseInt(time_elms[0], 10);
          durationObj.minutes = parseInt(time_elms[1], 10);
          durationObj.seconds = parseInt(time_elms[2], 10);
        } else if (time_elms.length == 2) {
          durationObj.hours = parseInt(time_elms[0], 10);
          durationObj.minutes = parseInt(time_elms[1], 10);
          durationObj.seconds = 0;
        } else {
         durationObj.valid = false;
        }
      }
    });
    durationObj.hours = durationObj.hours ? durationObj.hours : 0;
    durationObj.minutes = durationObj.minutes ? durationObj.minutes : 0;
    durationObj.seconds = durationObj.seconds ? durationObj.seconds : 0;
    durationObj.weeks = durationObj.weeks ? durationObj.weeks : 0;
    durationObj.days = durationObj.days ? durationObj.days : 0;
    return durationObj;
  },

  hideWidget: function() {
    if (this.isOpen === false) {
      return;
    }

    var durationObj = this.getDurationFromPickers();

    if (this.validatePickers(durationObj)) {
      durationObj = this.prepareDuration(durationObj);
      this.setInput(durationObj);

      this.$element.trigger({
        'type': 'hide.durationpicker',
        'duration': {
          'value': durationObj
        }
      });
    }

    if (this.template === 'modal' && this.$widget.modal) {
      this.$widget.modal('hide');
    } else {
      this.$widget.removeClass('open');
    }

    $(document).off('mousedown.timepicker, touchend.timepicker');

    this.isOpen = false;
    // show/hide approach taken by datepicker
    this.$widget.detach();
    },

  incrementWeek: function() {
    this.setValue('bootstrap-timepicker-week', 1);
  },

  incrementDay: function() {
    this.setValue('bootstrap-timepicker-day', 1);
  },

  incrementHour: function() {
    this.setValue('bootstrap-timepicker-hour', 1);
  },

  incrementMinute: function(step) {
    this.setValue('bootstrap-timepicker-minute', 1);
  },

  incrementSecond: function() {
    this.setValue('bootstrap-timepicker-second', 1);
  },

    // This method was adapted from bootstrap-datepicker.
  place: function() {
      if (this.isInline) {
        return;
      }
      var widgetWidth = this.$widget.outerWidth(), widgetHeight = this.$widget.outerHeight(), visualPadding = 10, windowWidth =
        $(window).width(), windowHeight = $(window).height(), scrollTop = $(window).scrollTop();

      var zIndex = parseInt(this.$element.parents().filter(function() {}).first().css('z-index'), 10) + 10;
      var offset = this.component ? this.component.parent().offset() : this.$element.offset();
      var height = this.component ? this.component.outerHeight(true) : this.$element.outerHeight(false);
      var width = this.component ? this.component.outerWidth(true) : this.$element.outerWidth(false);
      var left = offset.left, top = offset.top;

      this.$widget.removeClass('timepicker-orient-top timepicker-orient-bottom timepicker-orient-right timepicker-orient-left');

      if (this.orientation.x !== 'auto') {
        this.picker.addClass('datepicker-orient-' + this.orientation.x);
        if (this.orientation.x === 'right') {
          left -= widgetWidth - width;
        }
      } else{
        // auto x orientation is best-placement: if it crosses a window edge, fudge it sideways
        // Default to left
        this.$widget.addClass('timepicker-orient-left');
        if (offset.left < 0) {
          left -= offset.left - visualPadding;
        } else if (offset.left + widgetWidth > windowWidth) {
          left = windowWidth - widgetWidth - visualPadding;
        }
      }
      // auto y orientation is best-situation: top or bottom, no fudging, decision based on which shows more of the widget
      var yorient = this.orientation.y, topOverflow, bottomOverflow;
      if (yorient === 'auto') {
        topOverflow = -scrollTop + offset.top - widgetHeight;
        bottomOverflow = scrollTop + windowHeight - (offset.top + height + widgetHeight);
        if (Math.max(topOverflow, bottomOverflow) === bottomOverflow) {
          yorient = 'top';
        } else {
          yorient = 'bottom';
        }
      }
      this.$widget.addClass('timepicker-orient-' + yorient);
      if (yorient === 'top'){
        top += height;
      } else{
        top -= widgetHeight + parseInt(this.$widget.css('padding-top'), 10);
      }

      this.$widget.css({
        top : top,
        left : left,
        zIndex : zIndex
      });
    },

    remove: function() {
      $('document').off('.timepicker');
      if (this.$widget) {
        this.$widget.remove();
      }
      delete this.$element.data().timepicker;
    },

    showWidget: function() {
      if (this.isOpen) {
        return;
      }

      if (this.$element.is(':disabled')) {
        return;
      }

      // show/hide approach taken by datepicker
      this.$widget.appendTo(this.appendWidgetTo);
      var self = this;
      $(document).on('mousedown.timepicker, touchend.timepicker', function (e) {
        // This condition was inspired by bootstrap-datepicker.
        // The element the timepicker is invoked on is the input but it has a sibling for addon/button.
        if (!(self.$element.parent().find(e.target).length ||
            self.$widget.is(e.target) ||
            self.$widget.find(e.target).length)) {
          self.hideWidget();
        }
      });

      var durationObj = this.getDurationFromInput();
      if (durationObj.valid) {
        this.setPickers(durationObj);
      } else {
        this.clear();
      }

      this.$element.trigger({
        'type': 'show.durationpicker',
        'duration': {
          'value': durationObj
        }
      });

      this.place();

      if (this.template === 'modal' && this.$widget.modal) {
        this.$widget.modal('show').on('hidden', $.proxy(this.hideWidget, this));
      } else {
        if (this.isOpen === false) {
          this.$widget.addClass('open');
        }
      }

      this.isOpen = true;
    },

    widgetClick: function(e) {
    e.stopPropagation();
    e.preventDefault();

    var $input = $(e.target),
    action = $input.closest('a').data('action');

    if (action) {
      this[action]();
    }
  },

  widgetInputChanged: function(e) {
    var $input = $(e.target);
    var iVal = parseInt($input.val(), 10);
    if (isNaN(iVal) || iVal < 0) {
      $input.addClass('error');
    } else {
      $input.removeClass('error');
    }
  }
};

  // DURATIONPICKER PLUGIN DEFINITION
  $.fn.durationpicker = function(option) {
    var args = Array.apply(null, arguments);
    args.shift();

    var methodResult;
    var result = this.each(function() {
      var $this = $(this);
      var data = $this.data('durationpicker');
      var options = typeof option === 'object' && option;

      if (!data) {
        $this.data('durationpicker', (data = new Durationpicker(this, $.extend({}, $.fn.durationpicker.defaults, options, $(this).data()))));
      }

      if (typeof option === 'string') {
        methodResult = data[option].apply(data, args);
      }
    });
    if (typeof methodResult != 'undefined') {
      return methodResult;
    } else {
      return result;
    }
  };

  $.fn.durationpicker.defaults = {
    isOpen: false,
    modalBackdrop: false,
    orientation: { x: 'auto', y: 'auto'},
    showSeconds: true,
    template: 'dropdown',
    appendWidgetTo: 'body',
    showWidgetOnAddonClick: true,
    readonly: false
  };

  $.fn.durationpicker.Constructor = Durationpicker;

})(jQuery, window, document);
