/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  var apiDirectory = '/api/directory/';
  var apiFile = '/api/file/';
  var apiFSObject = '/api/fsobject/';
  var apiFileBulk = '/api/remote-fs-command/';
  var currentFolderPath = '';
  var startPath = '~';
  // initinalDir may be in url parameter
  var initialDir = $.url('?path') ? decodeURI($.url('?path')) : startPath;
  var isInitialization = true;
  var filePreviewSettings = {
    startPosition: 'footer',
    lines: 10
  };

  var renderDate = window.renderDate;

  var currentPage = 1;
  var currentOrder = 'display_name';
  var currentOrderDirection = '';
  var currentFilterField = 'display_name';
  var currentFilterValue = '';

  var bytesToSize = window.renderSize;
  var folders = [];
  var files = [];

  $(function() {
    // Initial Grid View
    $('#grid-view-Btn').addClass('selected-view');
    $('#list-view-Btn').click(switchFileViewToList);
    $('#grid-view-Btn').click(switchFileViewToGrid);
    $('#new-folder-Btn').click(function() {
      $('#item-name').val('');
      $('#new-item-dialog .modal-title').text('Create new folder');
      $('#new-item-dialog #select-item-btn').text('Create');
      $('#new-item-dialog #select-item-btn').attr('data-target', 'folder');
      $('#new-item-dialog').modal('show');
    });
    $('#new-item-dialog').on('shown.bs.modal', function () {
      $('#item-name').focus();
    });

    $('#upload-Btn').click(function() {
      $('#upload-file-form')[0].reset();
      $('#upload-file-dialog .selected-file').text('Browse');
      $('#upload-file-dialog #start-uploading-btn').addClass('disabled');
      $('#upload-file-dialog').modal({ backdrop: 'static', keyboard: false });
    });

    $('.refresh-current-folder').click(function() {
      loadData(renderData, currentFolderPath);
    });

    $('.clear-filter').click(function() {
      $('#filter-by-input').val('');
      setFilter();
    });

    $('#start-uploading-btn').click(function() {
      // check the file size to upload
      var size = $('#upload-file-input')[0].files[0].size;
      if (maxFileSize > 0 && size > maxFileSize) {
        $.notify('The given file is too large for upload', { type: 'warning' });
        return false;
      }
      var filesCount = $('#upload-file-input')[0].files.length;
      var isAlreadyExist = false;
      // check that selected files are missing
      $.each($('#upload-file-input')[0].files, function(index, file) {
        $('.file-manager-container.api').data('api').head(apiFile + currentFolderPath + file.name)
        .done(function(data, textStatus, jqXHR) {
            isAlreadyExist = true;
            filesCount--;            
          }
        )
        .fail(function(jqXHR) {
            filesCount--;                
          }
        );
      });
      
      checkCallbacks();
      
      // If all selected files are missing, start the upload process
      function checkCallbacks() {
        if(filesCount > 0) {            
          setTimeout(checkCallbacks, 500);
        } else {
          if (isAlreadyExist) {
            $.notify('File or directory already exists', { type: 'warning' });
            return false;
          } else {
            var data = new FormData($('#upload-file-form')[0]);
            var progressBar = $('#upload-file-dialog').find('.progress');
            
            progressBar.show();
            setProgressBarTo(0);
            
            function setProgressBarTo(value) {
              progressBar.find('.progress-bar').css('width', value + '%');
            }
            
            $('#upload-file-dialog .api').data('api').post(encodeURI(apiDirectory + currentFolderPath), data)
              .always(function(data) {
                progressBar.hide();
                setProgressBarTo(0);
              })
              .done(function(data) {
                $.notify('File has been successfully uploaded.', { type: 'success' });
                $('#upload-file-dialog').modal('hide');
                loadData(renderData, currentFolderPath);
                $('.select-all-container').show();
              })
              .progress(function(progress) {
                setProgressBarTo(Math.ceil(progress));
              });
          }
        }
      }        
    });

    $('#preview-settings-btn').popover({
      html: true,
      placement: 'left',
      content: function() {
        return $('#preview-settings-popover-content').html();
      }
    });

    $('#preview-settings-btn').on('shown.bs.popover', function () {
      $('.popover-content .preview-settings-start-position').val(filePreviewSettings.startPosition);
      $('.popover-content .preview-settings-lines').val(filePreviewSettings.lines);
    });

    $(document).click(function (e) {
      if (typeof $(e.target).data('original-title') == 'undefined' &&
       !$(e.target).parents().is('.popover.in')) {
        $('[data-original-title]').popover('hide');
      }
    });

    $('body').on('click', '.popover-content #apply-preview-settings-btn', function() {
      filePreviewSettings.startPosition = $('.popover-content .preview-settings-start-position').val() || 'footer';
      filePreviewSettings.lines = $('.popover-content .preview-settings-lines').val() || 10;

      $('#preview-settings-btn').popover('hide');
      loadPreview();
    });

    $('#select-item-btn').click(function() {
      $('#new-item-dialog').find('.form-horizontal').submit();
    });
    $('#new-item-dialog').find('.form-horizontal').submit(selectItemBtnCallback);

    function selectItemBtnCallback(event) {
      event.preventDefault();

      var target = $('#select-item-btn').attr('data-target');
      var api = $('#new-item-dialog .api').data('api');

      if (target == 'folder') {
        var newFolderName = $('#item-name').val();

        if (newFolderName.trim() === '') {
          $.notify('Directory name must not be empty', { type: 'warning' });
          return;
        }

        if (newFolderName.length > 255) {
          $.notify('Directory name can be no longer than 255 characters', { type: 'warning' });
          return;
        }

        api.post(encodeURI(apiDirectory + currentFolderPath + newFolderName + '/'), {})
            .done(function(data) {
              $('#new-item-dialog').modal('hide');
              $('.select-all-container').show();
              loadData(renderData, currentFolderPath);
            });

        return;
      }

      if (target == 'fileRenamedOnDetails' || target == 'fileRenamed') {
        var url = $('#new-item-dialog .dialog-data').val();
        var newFileName = $('#item-name').val();

        if (newFileName.trim() === '') {
          return;
        }

        var fn = target == 'fileRenamedOnDetails' ?
            fileRenamedOnDetails : fileRenamed;
        $('#new-item-dialog').modal('hide');
        renameFile(url, newFileName, fn);

        return;
      }
    }

    $('#upload-file-input').on('change', function(event) {
      var files = event.target.files;
      if (files.length > 1) {
        $('#upload-file-dialog .selected-file').text(files.length + ' files selected');
      } else {
        var f = files[0];
        $('#upload-file-dialog .selected-file').text(f.name);
      }

      $('#upload-file-dialog #start-uploading-btn').removeClass('disabled');
    });

    $('.order-by-name, .order-by-date, .order-by-type, .order-by-size').click(changeOrder);
    $('#filter-btn').click(setFilter);
    $('#filter-by-input').on('keypress', function(event) {
      if (event.which === 13)
        setFilter();
    });

    $(document).on('click', '#file-Manager .folder-item', folderClicked);
    $(document).on('click', '#file-Manager .folder-holder .glyphicon-folder-open', folderClicked);
    $(document).on('click', '#file-Manager .folder-up', folderUpClicked);
    $(document).on('click', '#file-Manager .folder-up .glyph-up', folderUpClicked);
    $(document).on('click', '#file-Manager .bc-item', breadcrumbFolderClicked);
    $(document).on('click', '#file-Manager .file-name', fileClicked);
    $(document).on('click', '#file-Manager #files .glyphicon-folder-open', fileClicked);
    $(document).on('mouseover', '#file-Manager .file-name', fileHover);
    $(document).on('click', '#file-Manager .file-details .close-details', closeDetails);
    $(document).on('click', '#file-Manager #files .item .folder-container', clickItemHandler);
    $(document).on('click', '#file-Manager #files .item .file-container', clickItemHandler);
    $(document).on('click', 'html', clickScreenHandler);
    $(document).on('click', '#file-Manager .select-all', selectDeselectAll);
    $(document).on('click', '#file-Manager .home-link-container', homeClicked);

    $(document).on('click',
      '.file-action-bar .download', detailsDownloadClicked);
    $(document).on('click',
      '.popover-content .download', downloadClicked);
    $(document).on('click',
      '.popover-content .compress', compressClicked);
    $(document).on('click',
      '.popover-content .compress-all', compressClicked);
    $(document).on('click',
      '.file-action-bar .compress', detailsCompressClicked);
    $(document).on('click',
      '.popover-content .uncompress', uncompressClicked);
    $(document).on('click',
      '.file-action-bar .uncompress', detailsUncompressClicked);
    $(document).on('click',
      '.popover-content .delete', deleteClicked);
    $(document).on('click',
      '.file-action-bar .delete', detailsDeleteClicked);
    $(document).on('click',
      '.popover-content .move', moveClicked);
    $(document).on('click',
      '.file-action-bar .move', detailsMoveClicked);
    $(document).on('click',
      '.popover-content .copy', copyClicked);
    $(document).on('click',
      '.file-action-bar .copy', detailsCopyClicked);
    $(document).on('click',
      '.popover-content .rename', renameClicked);
    $(document).on('click',
      '.file-action-bar .rename', detailsRenameClicked);
    $(document).on('click',
      '.popover-content .permissions', permissionsClicked);
    $(document).on('click',
      '.file-action-bar .permissions', detailsPermissionsClicked);

    $(document).keydown(keyPressTriggered);

    $('.breadcrumb').hide();
    checkPermissions(function () {
      loadData(renderData, initialDir);
    });

    bindTriggerEvents();
    setPageSize();
    createPaginator();

    $('.fm-space #viewPerPage').change(function() {
      currentPage = 1;
      localStorage.setItem("fmPageSize", $(this).val());
      loadData(renderData, currentFolderPath);
    });
  });

  function setPageSize() {
    var pageSize = localStorage.getItem("fmPageSize");
    if (pageSize) {
      $('#viewPerPage').val(pageSize);
    }
  }

  function checkPermissions(fn) {
    var api = $('.file-manager-container.api').data('api');
    api.head(apiDirectory)
      .done(fn)
      .fail(function(data) {
        var status = data.status;
        if (status === 403) {
          $('.file-grid-space').css('height', '200px');
          $('.breadcrumb').hide();
          $('#upload-Btn').hide();
          $('#new-folder-Btn').hide();
          var msg = '<br>' +
            'Access denied: We\'re sorry but you cannot access this area of the ' +
            'portal because your Viewpoint user does not have a system account' +
            '<br>&nbsp;';
          showMessage(msg, 'warning');
          $('.alert-warning .close-alert').hide();
          $('#grid-view-Btn').hide();
          $('#list-view-Btn').hide();
        } else {
          fn();
        }
      });
  }

  function keyPressTriggered(event) {
    if ($('.file-details').is(":visible")) {
      if (event.keyCode == 8 || event.altKey && event.keyCode == 37) {
        if (!$('.preview-settings-lines').is(':focus') && !modalWindowIsOpen()) {
          closeDetails();
          event.preventDefault();
        }
      }
    } else {
      var isMac = (window.navigator.platform.indexOf('Mac') > -1);
      var commandKeyClicked = isMac && event.metaKey || !isMac && event.ctrlKey;
      if (event.keyCode == 65 && commandKeyClicked) {
        event.preventDefault();
        selectDeselectAll();
      }
    }

    function modalWindowIsOpen() {
      var windowIsOpen = false;
      $.each($('.modal'), function(k, v) {
        if ($(v).is(':visible')) {
          windowIsOpen = true;
          return false;
        }
      });
      return windowIsOpen;
    }
  }

  function setFilter() {
    currentFilterValue = $('#filter-by-input').val();
    currentPage = 1;
    loadData(renderData, currentFolderPath);
  }

  function changeOrder(event) {
    var orderField = '';
    if ($(event.target).hasClass('order-by-name')) {
      orderField = 'display_name';
    }
    if ($(event.target).hasClass('order-by-date')) {
      orderField = 'date_modified';
    }
    if ($(event.target).hasClass('order-by-type')) {
      orderField = 'extension';
    }
    if ($(event.target).hasClass('order-by-size')) {
      orderField = 'size_bytes';
    }
    if (!orderField) {
      return;
    }

    if (currentOrder == orderField) {
      currentOrderDirection = (currentOrderDirection == '-') ? '': '-';
    } else {
      currentOrderDirection = '';
      currentOrder = orderField;
    }

    $('.order-marker-name').hide();
    $('.order-marker-date').hide();
    $('.order-marker-type').hide();
    $('.order-marker-size').hide();
    switch (orderField) {
      case 'display_name':
        $('.order-field-displayed').text('Name');
        $('.order-marker-name').show();
        break;
      case 'date_modified':
        $('.order-field-displayed').text('Date');
        $('.order-marker-date').show();
        break;
      case 'extension':
        $('.order-field-displayed').text('Type');
        $('.order-marker-type').show();
        break;
      case 'size_bytes':
        $('.order-field-displayed').text('Size');
        $('.order-marker-size').show();
        break;
    }

    if (currentOrderDirection == '-') {
      $('.order-marker')
        .removeClass('glyphicon-arrow-up')
        .addClass('glyphicon-arrow-down');
    } else {
      $('.order-marker')
        .removeClass('glyphicon-arrow-down')
        .addClass('glyphicon-arrow-up');
    }
    currentPage = 1;
    loadData(renderData, currentFolderPath);
  }

  function switchFileViewToList(event) {
    event.preventDefault();
    if ($('#list-view-Btn').hasClass('selected-view')) {
      return;
    }
    $('#grid-view-Btn').toggleClass('selected-view');
    $('#list-view-Btn').toggleClass('selected-view');

    $('#files .item').removeClass('grid-group-item');
    $('#files .item').addClass('list-group-item');
    setView('list');
  }

  function switchFileViewToGrid(event) {
    event.preventDefault();
    if ($('#grid-view-Btn').hasClass('selected-view')) {
      return;
    }
    $('#grid-view-Btn').toggleClass('selected-view');
    $('#list-view-Btn').toggleClass('selected-view');

    $('#files .item').removeClass('list-group-item');
    $('#files .item').addClass('grid-group-item');
    setView('grid');
  }

  function setView(view) {
    switch (view) {
      case 'list':
        $('.file-details').hide();
        $('.files-area').show();
        $('.paginator-space').show();
        $('.files-grid-header').show();
        break;
      case 'grid':
        $('.file-details').hide();
        $('.files-area').show();
        $('.paginator-space').show();
        $('.files-grid-header').hide();
        break;
      case 'details':
        $('.files-area').hide();
        $('.paginator-space').hide();
        $('.file-details').show();
        $('.selected-items-status').hide();
        $('.select-all-container').hide();
        $('.files-grid-header').hide();
        break;
    }
  }

  function loadData(fn, path, fileName, renderLeftPanelOnly) {
    renderLeftPanelOnly = typeof renderLeftPanelOnly !== 'undefined' ? renderLeftPanelOnly : false;
    $('.paginator-space').hide();
    var api = $('.file-manager-container.api').data('api');
    if (currentPage != 1) {
      api = $('.files-holder.api').data('api');
    }
    path = path || '';
    if (path) {
      if (path.substr(path.length - 1) !== '/') {
        path += '/';
      }
    }
    var encodedPath = encodeURI(path);
    $('#file-manager').data('have-previos-page', false);
    if (isInitialization) {
      api.get(apiFSObject + encodedPath).done(function(data) {
          isInitialization = false;
          var objectType = data.object_type;
          if (objectType == 'DIRECTORY') {
            loadData(renderData, path);
          } else if (objectType == 'FILE') {
            var fileName = path.match(/([^\/]*)\/*$/)[1];
            var previousPath = path.replace(fileName, '').replace('//', '/');
            loadData(renderDataAndShowDetails, previousPath, fileName);
          }
        }
      ).fail(function() {
          isInitialization = false;
          loadData(renderData, startPath);
        });
    }

    if (!isInitialization) {
      var pageSize = $('.fm-space #viewPerPage').val();
      pageSize = pageSize || 100;
      var url = apiDirectory + encodedPath +
        '?page=' + currentPage +
        '&page_size=' + pageSize +
        '&ordering=' + currentOrderDirection + currentOrder;
      if (currentFilterValue && !renderLeftPanelOnly) {
        url += '&filter-by=' + currentFilterField +
          '&filter-value=' + currentFilterValue;
      }
      api.get(url)
        .done(function(data, textStatus, jqXHR) {
          if (path.match(/~\/$/)) {
            var i = data.current_directory.url.indexOf(apiDirectory);
            path = data.current_directory.url.substr(i + apiDirectory.length);
          }
          currentFolderPath = path;
          folders = [];
          files = [];
          if (path) {
            folders.push({
              object_type: 'DIRECTORY',
              root: true
            });
          }

          $.each(data.contents, function(_i, item) {
            if (item.object_type == 'DIRECTORY') {
              folders.push(item);
            }

            if (item.object_type == 'FILE') {
              files.push(item);
            }
          });

          if (renderLeftPanelOnly) {
            fn(folders);
            return;
          }
          fn(path, files, folders, fileName);

          if (data.next || data.previous) {
            var pageSize = $('.fm-space #viewPerPage').val();
            pageSize = pageSize || 100;
            var pagesCount = Math.ceil(data.count / pageSize);
            if (pagesCount == 0) { pagesCount = 1; }
            $('.paginator-container').bootpag({
              total: pagesCount, // total pages
              page: currentPage,  // default page
              next: 'next',
              prev: 'prev'
            }).show();
            $('.paginator-container').show();
          } else {
            $('.paginator-container').hide();
          }
          if (!$('.file-details').is(':visible'))
            $('.paginator-space').show();
          if (!data.count) {
            $('.select-all-container').hide();
          } else {
            $('.select-all-container').show();
          }
          var $filterInput = $('#filter-by-input').val();
          if ($filterInput != '') {
            $('.filter-message-container').show();
            $('.filter-message').text('Filtering by "' + $filterInput + '" returned ' + data.count + ' results');
          } else {
            $('.filter-message-container').hide();
          }
        });
    }
  }

  function createPaginator() {
    $('.paginator-container').bootpag({
      total: 10,
      maxVisible: 7,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      // reload
      currentPage = num;
      loadData(renderData, currentFolderPath);
    });
  }

  function renderBreadcrumbs(path) {
    var bc = $('#file-manager .breadcrumb');
    bc.empty();
    if (path[path.length - 1] == '/') {
      path = path.substring(0, path.length - 1);
    }

    var locked = true;
    while (path) {
      bc.prepend(getBreadcrumbItem(path, locked));
      var index = path.lastIndexOf('/');
      path = path.substring(0, index);
      locked = false;
    }

    bc.prepend(getBreadcrumbItem('', locked));
    $('.breadcrumb').show();

    var elm = $('.breadcrumb-bar')[0];
    bc.css('right', '');
    bc.css('left', '');
    $('.left-ellipsis').hide();
    // check content is clipped (overflow: hidden)
    if (elm.scrollWidth > $(elm).innerWidth()) {
      bc.css('right', '0');
      $('.left-ellipsis').show();
    } else {
      bc.css('left', '0');
    }
  }

  function getBreadcrumbItem(path, locked) {
    var li;
    if (path) {
      var index = path.lastIndexOf('/');
      var folder = path.substring(index + 1);
      if (folder.trim() === '') {
        folder = '"' + folder + '"';
      }
      li = $('<li><a class="bc-item" href="#" data-url="' + path + '">' +
        folder + '</a></li>');
    } else {
      li = $('<li><a class="bc-item" href="#" data-url="">Remote File System</a></li>');
    }

    if (locked) {
      li.find('a').addClass('disabled');
    }

    return li;
  }

  function folderClicked(event) {
    event.preventDefault();
    event.stopPropagation();
    if ($(event.target).hasClass('folder-up')) { return; }
    if ($(event.target).hasClass('glyph-up')) { return; }
    $('.tooltip').hide();
    var folder = $(event.target).closest('a').text();
    var path = currentFolderPath + folder;
    currentPage = 1;
    loadData(renderData, path);
    closeDetails();
    updateSelectedItemsStatus('clear');
  }

  function fileHover(event) {
    var elm = $(event.target)[0];
    // check content is clipped (overflow: hidden)
    if (elm.scrollWidth > $(elm).innerWidth()) {
      $(elm).attr('title', $(elm).html());
    }
  }

  function breadcrumbFolderClicked(event) {
    event.preventDefault();
    var path = $(event.target).attr('data-url');
    currentPage = 1;
    loadData(renderData, path);
    closeDetails();
    updateSelectedItemsStatus('clear');
  }

  function clickScreenHandler(event) {
    var target = event.target;
    if ($(target).is('.select-all') ||
      // Click on Context menu items doesn't deselect all
      $(target).is('.popover-content') ||
      $(target).is('.popover-content a')) {
      return;
    }
    // Click on File & Folder items doesn't deselect all
    if ($(target).is('.item .folder-container') ||
      $(target).is('.item .file-container') ||
      $(target).parents().is('.item .folder-container') ||
      $(target).parents().is('.item .file-container')) {
        return;
    }
    clearMultiSelection();
    updateSelectedItemsStatus('clear');
  }

  function clickItemHandler(event) {
    var item = $(event.target).closest('.item');
    var isMac = (window.navigator.platform.indexOf('Mac') > -1);
    var commandKeyClicked = isMac && event.metaKey || !isMac && event.ctrlKey;
    if (event.shiftKey) {
      selectGroup(item);
    } else if (commandKeyClicked) {
      toggleItem(item);
    } else {
      // Clear all selected items before selecting current one
      $('#file-Manager #files .item').removeClass('active');
      selectItem(item);
    }
    updateSelectedItemsStatus();
  }

  function toggleItem(item) {
    $('#files .item').removeClass('lastClicked');
    item.addClass('lastClicked');
    item.toggleClass('active');
  }

  function selectItem(item) {
    $('#files .item').removeClass('lastClicked');
    item.addClass('lastClicked');
    item.addClass('active');
  }

  function selectGroup(item) {
    if ($('.lastClicked').length == 0) {
      selectItem(item);
      return;
    }
    if ($('.lastClicked').is($(item))) {
      selectItem(item);
      return;
    }
    // Select Group of elements
    var inGroup = false;
    $('#files .item').each(function(_i, element) {
      if ($(element).is($(item)) || $(element).is($('.lastClicked'))) {
        inGroup = !inGroup;
        if (!inGroup) {
          $(element).addClass('active');
        }
      }
      if (inGroup) {
        $(element).addClass('active');
      }
    });

    // Clear text selection which appears after using shift+click
    clearPageSelection();
  }

  function clearPageSelection() {
    if (window.getSelection) {
      if (window.getSelection().empty) {  // Chrome
        window.getSelection().empty();
      } else if (window.getSelection().removeAllRanges) {  // Firefox
        window.getSelection().removeAllRanges();
      }
    } else if (document.selection) {  // IE
      document.selection.empty();
    }
  }

  function fileClicked(event) {
    event.preventDefault();
    event.stopPropagation();
    if ($(event.target).hasClass('trigger')) { return; }

    var span;
    if ($(event.target).hasClass('glyphicon-folder-open')) {
      span = $(event.target).siblings('.file-name')[0];
    } else {
      span = $(event.target).closest('.file-name')[0];
    }
    $('.tooltip').hide();    
    // For Folder item GoTo folder
    if ($(span).hasClass('folder')) {
      var folder = $(span).attr('data-folder');
      var path = currentFolderPath + folder;
      clearMultiSelection();
      loadData(renderData, path);
      return;
    }
    // For File item show Details pane
    var apiUrl = $($(event.target).siblings('.popup-wrapper').find('.data-file-url')).text() +
        '?check_perms=read';

    var navigateFn = function () {
      if (passDataToDetails(event.target)) {
        setView('details');
        loadPreview();
      }
    };

    $('.file-sign-box.api').data('api').head(apiUrl)
      .done(navigateFn)
      .fail(function(jqXHR) {
        var status = jqXHR.status;
        if (status === 403) {
          $.notify('Permissions denied', { type: 'warning' });
        }
        else {
          navigateFn();
        }
      });
  }

  function passDataToDetails(fileSpan) {    
    var fileUrl = $(fileSpan).siblings('.popup-wrapper').find('.data-file-url');
    if (fileUrl.length === 0) {
      return false;
    }     
    var name = $(fileSpan).contents().filter(function() {
      return this.nodeType == Node.TEXT_NODE;
    }).text();
    var url = $(fileUrl).text();
    var size = $(fileUrl).attr('data-size'),
        archived = $(fileUrl).attr('data-archived'),
        modified = $(fileUrl).attr('data-modified');
        permissions = $(fileUrl).attr('data-permissions');
    
    $('.details-file-name').text(name);
    $('.details-data-file-url').val(url);
    $('.download-link').attr('href', url);
    $('.details-data-file-archived').val(archived);
    $('.details-data-file-permissions').val(permissions);
    $('.details-file-size').text('Size: ' + size);
    $('.details-file-modified').text('Modified: ' + modified);
    
    $('.file-details .compress').show();
    $('.file-details .uncompress').show();
    if (archived == 1) {
      $('.file-details .compress').hide();
      $('#preview-settings-btn').hide();
    } else {
      $('.file-details .uncompress').hide();
      $('#preview-settings-btn').show();
    }
    $('#preview-settings-btn').popover('hide');
    return true;    
  }

  function updateSelectedItemsStatus(clear) {
    var count = 0;
    if (!clear) {
      count = $('.files-holder .files .item.active').length;
    }
    if (count > 0) {
      $('.selected-items-status').show();
      var status = '';
      if (count > 1) {
        status = count + ' items selected';
      } else {
        status = '1 item selected';
      }
      $('.selected-items-status').html(status);
    } else {
      $('.selected-items-status').hide();
    }
    if (isAllSelected()) {
      $('.select-all').text('Deselect All');
    } else {
      $('.select-all').text('Select All');
    }
  }

  function loadPreview() {
    var api = $('.file-sign-box.api').data('api');
    var url = $('.details-data-file-url').val() +
        '?format=preview&start_position=' + filePreviewSettings.startPosition +
        '&lines=' + filePreviewSettings.lines;
    api.head(url, 'text')
      .done(function(data, textStatus, jqXHR) {
        api.get(url, 'text')
          .done(function(data, textStatus, jqXHR) {
            setTimeout(function () {
              $('.preview-box').resizable({
                handleSelector: ".resize-handler"
              });
              $('.preview-box').show();
              $('.resize-handler').show();
              $('.preview-box').animate({ width: "350px" }, 1000, function () {
                $('.preview-box').text(data);
              });
            }, 600);
          })
          .fail(function(jqXHR, textStatus, errorThrown) {
          });
      });
  }

  function closeDetails() {
    if ($('#file-manager').data('have-previos-page')) {
      window.history.back();
    } else {
      if ($('#list-view-Btn').hasClass('selected-view')) {
        setView('list');
      } else {
        setView('grid');
      }
      $('.preview-box').text('');
      $('.preview-box').hide();
      $('.resize-handler').hide();
      $('.preview-box').css("width", 0);
      updateSelectedItemsStatus();
      $('.select-all-container').show();
    }
  }

  function folderUpClicked(event) {
    event.preventDefault();
    if (currentFolderPath && currentFolderPath != '/') {
      var path = currentFolderPath.substring(0, currentFolderPath.length - 1);
      var index = path.lastIndexOf('/');
      path = path.substring(0, index);
      currentPage = 1;
      loadData(renderData, path);
      closeDetails();
      updateSelectedItemsStatus('clear');
    }
  }

  function renderData(path, files, folders) {
    if (currentPage == 1) {
    if (currentFilterValue == '') {
      renderFolders(folders);
    } else {
      loadData(renderFolders, currentFolderPath, '', true);
    }
      renderBreadcrumbs(path);
    }
    renderFilesAndFolders(files, folders);
  }

  function renderDataAndShowDetails(path, files, folders, fileName) {
    // This attribute is needed to understand what the user has come from another page (for example from job details)
    $('#file-manager').data('have-previos-page', true);
    renderData(path, files, folders);

    var $fileTitle = $('span.file-name').filter(function() {
      return $(this).text() == fileName;
    });
    $fileTitle.trigger('click');
  }

  function renderFolders(folders) {
    $('.folder-list').empty();

    var strings = [];
    if (folders[0] && folders[0].root) {
      strings.push(
        '<li>',
          '<a href="#" class="folder-up">',
            '<img src="/static/iris/img/folder.png" class="small-icon">',
            '<div>. . /</div>',
          '</a>',
        '</li>'
      );
      folders.shift();
    }

    for (var i = 0; i < folders.length; ++i) {
      var icon = '<img src="/static/iris/img/folder.png" class="small-icon">';
      if (folders[i].is_symlink) {
        if (folders[i].is_symlink_ok) {
          icon = '<img src="/static/iris/img/folder_link.png" class="small-icon">';
        } else {
          icon = '<img src="/static/iris/img/folder_broken_link.png" class="small-icon">';
        }
      }
      strings.push(
        '<li>',
          '<a href="#" class="folder-item">',
            icon,
            '<div title="' + folders[i].display_name + '">',
               folders[i].display_name,
            '</div>',
          '</a>',
        '</li>'
      );
    }

    $('.folder-list').append(strings.join(''));
  }

  function renderFolder(folderData) {
    var itemClass = $('#list-view-Btn').hasClass('selected-view') ?
      'list-group-item' : 'grid-group-item';
    var name = folderData.display_name,
        size = '&nbsp;',
        date = renderDate(folderData.date_modified),
        permissions = preparePermissionsView(folderData.permission_mask),
        isSymlink = folderData.is_symlink,
        isSymlinkOk = folderData.is_symlink_ok;

    var dataLink = '<a href="#" class="data-file-url" ' +
      'data-permissions="' + permissions + '" ' +
      '>' + folderData.url + '</a>';

    var icon = '<img src="/static/iris/img/folder.png" class="big-icon">';
    if (isSymlink) {
      if (isSymlinkOk) {
        icon = '<img src="/static/iris/img/folder_link.png" class="big-icon">';
      } else {
        icon = '<img src="/static/iris/img/folder_broken_link.png" class="big-icon">';
      }
    }

    var strings = [];
    strings.push(
     '<div class="item col-xs-4 col-lg-4 ' + itemClass + '">',
     '<div class="folder-container">',
     '<div class="caption">',
       icon,
       '<span class="file-name folder" data-folder="' + name + '">',
         name + '/',
       '</span>',
       '<span class="popup-wrapper">',
         '<div class="popup-container">',
           '<span class="glyphicon glyphicon-menu-hamburger trigger"></span>',
           '<div class="content hide">',
              dataLink,
              '<a href="#" class="compress">Compress</a>',
              '<a href="#" class="compress-all">Compress</a>',
              '<a href="#" class="copy">Copy</a>',
              '<a href="#" class="move">Move</a>',
              '<a href="#" class="rename">Rename</a>',
              '<a href="#" class="delete">Delete</a>',
              '<a href="#" class="permissions">Permissions</a>',
           '</div>',
         '</div>',
       '</span>',
       '<span class="ftype">&nbsp;</span>',
       '<span class="fsize">' + size + '</span>',
       '<span class="fdate">' + date + '</span>',
     '</div>',
     '</div>',
     '</div>');
    return strings.join('');
  }

  function renderFile(fileData) {
    var itemClass = $('#list-view-Btn').hasClass('selected-view') ?
      'list-group-item' : 'grid-group-item';

    var name = fileData.display_name,
        date = renderDate(fileData.date_modified),
        url = fileData.url,
        size = bytesToSize(fileData.size_bytes),
        permissions = preparePermissionsView(fileData.permission_mask),
        isSymlink = fileData.is_symlink,
        isSymlinkOk = fileData.is_symlink_ok,
        extension = '&nbsp;';

    if (/.*\.(.*)/.test(fileData.display_name))
      extension = fileData.display_name.match(/.*\.(.*)/)[1];

    var archived, archiveLink;
    if (fileData.is_archive) {
      archiveLink = '<a href="#" class="uncompress">Uncompress</a>';
      archived = 1;
    } else {
      archiveLink = '<a href="#" class="compress">Compress</a>';
      archived = 0;
    }

    var strings = [];
    var dataLink = '<a href="#" class="data-file-url" ' +
      'data-archived="' + archived + '" ' +
      'data-size="' + size + '" ' +
      'data-permissions="' + permissions + '" ' +
      'data-modified="' + date + '" ' +
      '>' + url + '</a>';

    var icon = '<img src="/static/iris/img/file.png" class="big-icon">';
    if (isSymlink) {
      if (isSymlinkOk) {
        icon = '<img src="/static/iris/img/file_link.png" class="big-icon">';
      } else {
        icon = '<img src="/static/iris/img/file_broken_link.png" class="big-icon">';
      }
    }

    strings.push(
     '<div class="item col-xs-4 col-lg-4 ' + itemClass + '">',
     '<div class="file-container">',
     '<div class="caption">',
       icon,
       '<span class="file-name">',
         name,
       '</span>',
       '<a class="download-file-link" id="'+ name.replace('.', '-') +'" download href="' + url + '">Download File Link</a>',
       '<span class="popup-wrapper">',
         '<div class="popup-container">',
           '<span class="glyphicon glyphicon-menu-hamburger trigger"></span>',
           '<div class="content hide">',
              dataLink,              
              '<a href="#" class="download">Download</a>',
              archiveLink,
              '<a href="#" class="compress-all">Compress</a>',
              '<a href="#" class="copy">Copy</a>',
              '<a href="#" class="move">Move</a>',
              '<a href="#" class="rename">Rename</a>',
              '<a href="#" class="delete">Delete</a>',
              '<a href="#" class="permissions">Permissions</a>',
           '</div>',
         '</div>',
       '</span>',
       '<span class="ftype">' + extension + '</span>',
       '<span class="fsize">' + size + '</span>',
       '<span class="fdate">' + date + '</span>',
     '</div>',
     '</div>',
     '</div>');
    return strings.join('');
  }

  function renderFilesAndFolders(files, folders) {
    $('#files').empty();
    var strings = [];
    for (var i = 0; i < folders.length; ++i) {
      if (folders[i].root) { continue; }
      strings.push(renderFolder(folders[i]));
    }
    for (var i = 0; i < files.length; ++i) {
      strings.push(renderFile(files[i]));
    }
    $('#files').append(strings.join(''));
    createPopupMenu();
  }

  function createPopupMenu() {

    $('.trigger').popover({
      html: true,
      container: 'body',
      animation: false,
      content: function() {
        return $(this).parent().find('.content').html();
      },
      placement: 'right'
    });
    $('.trigger').on('click', function(e) {
      $('.trigger').popover('hide');
      $(this).popover('show');
      e.stopPropagation();
    }).popover();
    $('.trigger').on('show.bs.popover', popoverShow);
    $('.trigger').on('shown.bs.popover', popoverShown);
  }


  function bindTriggerEvents() {

    $(document).on('mouseleave', '.popover', function () {
      setTimeout(function () {
        if ($('.popover:hover').length === 0) {
          $('.trigger').popover('hide');
        }
      }, 250);
    });
    $('body').on('click', function(e) {
      if (typeof $(e.target).data('original-title') == 'undefined' && !$(e.target).parents().is('.popover.in')) {
        $('.trigger').popover('hide');
      }
    });
  }

  function getActionFileUrl(actionLink) {
    var dataContainer = $(actionLink).siblings('.data-file-url')[0];
    return $(dataContainer).text();
  }

  function showMessage(msg, type, lifePeriod) {
    var notifyType = type || 'info';
    var period = lifePeriod || 0;
    $.notify('<div style="overflow-x: hidden; text-overflow: ellipsis">' + msg + '</div>', {
      type: notifyType,
      delay: period
    });
  }

  function getSelectedUrls() {
    var urls = $('#file-Manager #files .item.active').map(function() {
      return $(this).find('.data-file-url').text();
    });
    return urls;
  }

  function popoverShow(event) {
    var count = $('.files-holder .files .item.active').length;
    if (count == 1) {
      $('.files-holder .files .item.active').removeClass('active');
    }
  }

  function popoverShown(event) {
    var item = $(event.target).closest('.item')[0];
    if (!$(item).hasClass('active')) {
      clearMultiSelection();
      $('.popover-content').removeClass('active');
    } else {
      $('.popover-content').addClass('active');
    }
  }

  /////////////////////////////////////////////////////////////////////////
  // Action Click Handlers

  function detailsCompressClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    compressFile(url, fileCompressed);
  }

  function compressClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    var urls = getSelectedUrls();
    $('.popover').remove();
    if ($.inArray(url, urls) > -1) {
      compressFiles(urls, fileCompressed);
    } else {
      compressFile(url, fileCompressed);
    }
  }

  function detailsUncompressClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    uncompressFile(url, fileUncompressedOnDetails);
  }

  function uncompressClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    $('.popover').remove();
    uncompressFile(url, fileUncompressed);
  }

  function deleteClicked(event) {
    $('.trigger').popover('hide');
    $.viewpoint.confirm('Delete Operation', 'Are you sure?').done(function(result) {
      if (result) {
        event.preventDefault();
        var url = getActionFileUrl(event.target);
        var urls = getSelectedUrls();
        $('.popover').remove();
        if ($.inArray(url, urls) > -1) {
          deleteFiles(urls, fileListDeleted);
        } else {
          deleteFile(url, fileDeleted);
        }
      }
    });
  }

  function detailsDeleteClicked(event) {
    $.viewpoint.confirm('Delete Operation', 'Are you sure?').done(function(result) {
      if (result) {
        event.preventDefault();
        var url = $('.details-data-file-url').val();
        deleteFile(url, fileDeletedOnDetails);
      }
    });
  }

  function moveClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    var urls = getSelectedUrls();
    var dir = url.substring(0, url.lastIndexOf('/'));
    $('.popover').remove();
    if ($.inArray(url, urls) > -1) {
      showSelectDestinationDialogForMove(urls, dir, fileMoved);
    } else {
      showSelectDestinationDialogForMove([url], dir, fileMoved);
    }
  }

  function detailsMoveClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    var dir = url.substring(0, url.lastIndexOf('/'));
    showSelectDestinationDialogForMove([url], dir, fileMovedOnDetails);
  }

  function copyClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    var urls = getSelectedUrls();
    var dir = url.substring(0, url.lastIndexOf('/'));
    $('.popover').remove();
    if ($.inArray(url, urls) > -1) {
      showSelectDestinationDialogForCopy(urls, dir, fileCopied);
    } else {
      showSelectDestinationDialogForCopy([url], dir, fileCopied);
    }
  }

  function detailsCopyClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    var dir = url.substring(0, url.lastIndexOf('/'));
    showSelectDestinationDialogForCopy([url], dir, fileCopiedOnDetails);
  }

  function detailsDownloadClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    $('.file-manager-container.api').data('api').head(url + '?check_perms=read')
      .done(function() {
        $('.download-link')[0].click();
      })
      .fail(function() {
        $.notify('Permissions denied', { type: 'warning' });
        return;
      });
  }
  
  function downloadClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    $('.file-manager-container.api').data('api').head(url + '?check_perms=read')
      .done(function() {
        window.location = url;
        $('#files .item').removeClass('active');
      })
      .fail(function() {
        $.notify('Permissions denied', { type: 'warning' });
        return;
      });
  }

  function renameClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    $('.popover').remove();
    showRenameDialog(url, 'fileRenamed');
  }

  function detailsRenameClicked(event) {
    event.preventDefault();
    var url = $('.details-data-file-url').val();
    showRenameDialog(url, 'fileRenamedOnDetails');
  }

  function permissionsClicked(event) {
    event.preventDefault();
    var url = getActionFileUrl(event.target);
    var dataContainer = $(event.target).siblings('.data-file-url')[0];
    var permissions = $(dataContainer).attr('data-permissions');    
    $('.popover').remove();
    var itemName = currentFolderPath + getItemName(url);
    showPermissionsDialog(url, permissions, itemName);
  }

  function detailsPermissionsClicked(event) {
    event.preventDefault();
    var permissions = $('.details-data-file-permissions').val();
    var url = $('.details-data-file-url').val();
    var itemName = currentFolderPath + getItemName(url);
    showPermissionsDialog(url, permissions, itemName);
  }

  function showRenameDialog(url, fname) {
    if (url.substr(url.length - 1) == '/') { // isFolder
      $('#new-item-dialog .modal-title').text('New Folder Name');
    } else {
      $('#new-item-dialog .modal-title').text('New File Name');
    }

    $('#new-item-dialog .dialog-data').val(url);
    $('#new-item-dialog #item-name').val('');
    $('#new-item-dialog #select-item-btn').text('Set');
    $('#new-item-dialog #select-item-btn').attr('data-target', fname);
    $('#new-item-dialog').modal('show');
  }

  function showPermissionsDialog(url, permissions_mask, itemName) {
    var config = {
      name: url,
      permissions: permissions_mask,
      itemName: itemName,
      result: false
    };
    $.viewpoint.modal.show('/file-permissions', config)
      .done(function(data) {
        if (data.result) {
          changePermissions(url, data.oct, permissionsChanged);
        }
      });
  }

  function showSelectDestinationDialogForCopy(urls, dir, fn) {
    // dir is for possible start point of dialog
    var config = {
      path: '',
      startPath: currentFolderPath,
      title: 'Select Destination Folder',
      showFiles: false,
      showDirectories: true,
      forbidEmptyPath: true
    };
    $.viewpoint.modal.show('/open-file', config)
      .done(function(targetDir) {
        if (targetDir.fileSelected && targetDir.result) {
          copyFile(urls, targetDir.result, fn);
          loadData(renderData, currentFolderPath);
        }
      });
  }

  function showSelectDestinationDialogForMove(urls, dir, fn) {
    // dir is for possible start point of dialog
    var config = {
      path: '',
      startPath: currentFolderPath,
      title: 'Select Destination Folder',
      showFiles: false,
      showDirectories: true,
      forbidEmptyPath: true
    };
    $.viewpoint.modal.show('/open-file', config)
      .done(function(targetDir) {
        if (targetDir.fileSelected && targetDir.result) {
          moveFile(urls, targetDir.result, fn);
          currentPage = 1;
          loadData(renderData, currentFolderPath);
        }
      });
  }

  function getItemName(fileUrl) {
    if (fileUrl.substr(fileUrl.length - 1) == '/') { // Dir
      fileUrl = fileUrl.substring(0, fileUrl.length - 1);
    }
    return fileUrl.split('/').pop();
  }

  function getItemFolderPath(fileUrl) {
    if (fileUrl.substr(fileUrl.length - 1) == '/') { // Dir
      fileUrl = fileUrl.substring(0, fileUrl.length - 1);
    }
    return fileUrl.substring(0, fileUrl.lastIndexOf('/'));
  }

  function extractItemName(fileSpan) {
    var name = '';
    if ($(fileSpan).hasClass('folder')) {
      name = $(fileSpan).attr('data-folder');
    } else {
      name = $(fileSpan).contents().filter(function() {
        return this.nodeType == Node.TEXT_NODE;
      }).text();
    }
    return name;
  }

  /////////////////////////////////////////////////////////////////////////
  // Action Callbacks

  function fileCompressed(data) {
    if (data && data.display_name) {
      showMessage('Archive "' + data.display_name + '" has been created', '', 5000);
      loadData(function (path, files, folders) {
        renderFilesAndFolders(files, folders);
      }, currentFolderPath);
    }
  }

  function fileUncompressed(data) {
    if (data && data.display_name) {
      loadData(function (path, files, folders) {
        renderFolders(folders);
        renderFilesAndFolders(files, folders);
      }, currentFolderPath);
    }
  }

  function fileUncompressedOnDetails(data) {
    if (data && data.display_name) {
      fileUncompressed(data);
    }
    showMessage(data.display_name + ' file has uncompressed.', '', 5000);
  }

  function fileDeleted(data) {
    currentPage = 1;
    if (data && data.display_name) {
      hideFileRowAfterAction(data.display_name);
      if (data.object_type == 'DIRECTORY') {
        hideFolderRowAfterAction(data.display_name);
      }
    }
  }

  function fileListDeleted(data) {
    currentPage = 1;
    if (data && data.objects) {
      var urlList = _.map(data.objects, function(item) { return item.url; });
      clearMultiSelection();

      var cb = function () {
        if ($deletedElements.last().is(this)) {
          loadData(renderData, currentFolderPath);
        }
      };

      var $deletedElements = $('.files-holder .files .item').filter(function() {
        var url = decodeURI($(this).find('.data-file-url').text());
        return $.inArray(url, urlList) > -1;
      });

      $deletedElements.addClass('deleted').slideUp(600, cb);
    }
    else {
      loadData(renderData, currentFolderPath);
    }
  }

  function fileDeletedOnDetails(data) {
    if (data && data.display_name) {
      fileDeleted(data);
      closeDetails();
    }
  }

  function fileMoved(data) {
    if (data && data.target) {
      var fileUrl = data.objects[0].url;
      var lastSymbol = fileUrl.substring(fileUrl.length - 1);
      var path = fileUrl.split("/");
      var fileName = (lastSymbol == "/") ? path[path.length-2] : path[path.length-1];
      var msg = (lastSymbol == "/") ? 'Directory ' : 'File '; 
      //hideFileRowAfterAction(decodeURI(data.fileName));      
      //if (data.object_type == 'DIRECTORY') {        
      //  hideFolderRowAfterAction(data.display_name);
      //}      
      showMessage(msg + fileName + ' was moved to ' + data.target, '', 4000);
    }
  }

  function fileMovedOnDetails(data) {
    if (data && data.target) {
      var deprecatedUrl = $('.details-data-file-url').val();
      var newUrl = deprecatedUrl.substring(0, deprecatedUrl.indexOf(apiFile)) + apiFile.substring(0, apiFile.length-1) + data.target + "/" + $('.details-file-name').text();
      $('.details-data-file-url').val(newUrl);
      $('.download-link').attr("href", newUrl);
      currentFolderPath = data.target.substring(1);
      loadData(renderData, currentFolderPath);
      
      //hideFileRowAfterAction(decodeURI(newUrl)); 
      //if (data.object_type == 'DIRECTORY') {
      //  hideFolderRowAfterAction(data.display_name);
      //}
      var msg = 'File ' + $('.details-file-name').text() + ' was moved to ' +
        data.target;
      showMessage(msg, '', 4000);
      //closeDetails();
    }
  }

  function fileCopied(data) {   
    if (data && data.target) {
      var fileUrl = data.objects[0].url;
      var lastSymbol = fileUrl.substring(fileUrl.length - 1);
      if (!data.display_name) {
        var path = fileUrl.split("/");
        data.display_name = (lastSymbol == "/") ? path[path.length-2] : path[path.length-1];
      }      
      var msg = (lastSymbol == "/") ? 'Directory ' : 'File ';       
      showMessage(msg + data.display_name + ' was copied to ' + data.target, '', 4000);
    }
  }

  function fileCopiedOnDetails(data) {
    data.display_name = $('.details-file-name').text();
    fileCopied(data); // nothing special
  }

  function fileRenamed(data) {
    if (data && data.display_name) {
      var oldName = data.oldName;
      $('.files-holder .files .item').each(function() {
        var fileSpan = $(this).find('.file-name');
        var name = extractItemName(fileSpan);
        if (name == decodeURI(oldName)) {
          var newRow;
          if (data.object_type == 'DIRECTORY') {
            newRow = renderFolder(data);
          } else {
            newRow = renderFile(data);
          }
          $(fileSpan).closest('.item').replaceWith(newRow);
          createPopupMenu();
          return false;
        }
      });
      var deprecatedUrl = $('.details-data-file-url').val();
      var newUrl = deprecatedUrl.substring(0, deprecatedUrl.lastIndexOf("/") + 1) + data.display_name;
      $('.details-data-file-url').val(newUrl);
      $('.download-link').attr("href", newUrl);
      if (data.object_type == 'DIRECTORY') {
        folderRenamed(oldName, data.display_name);
      }
    }
  }

  function fileRenamedOnDetails(data) {
    if (data && data.display_name) {
      $('.details-file-name').text(data.display_name);
      fileRenamed(data);
      //closeDetails();
    }
  }

  function permissionsChanged(data) {
    if (data && data.display_name) {
      $('.files-holder .files .item').each(function() {
        var fileSpan = $(this).find('.file-name');
        var name = extractItemName(fileSpan);
        if (name == data.display_name) {
          var newRow;
          if (data.object_type == 'DIRECTORY') {
            newRow = renderFolder(data);
          } else {
            newRow = renderFile(data);
          }
          $(fileSpan).closest('.item').replaceWith(newRow);
          var permission_mask = preparePermissionsView(data.permission_mask);
          $('.details-data-file-permissions').val(permission_mask);
          createPopupMenu();
        }
      });
    }
  }

  function hideFileRowAfterAction(nameToDelete) {
    $('.files-holder .files .item').each(function() {
      var fileSpan = $(this).find('.file-name');
      var name = extractItemName(fileSpan);
      if (name == nameToDelete) {
        $(this).addClass('deleted');
        $(this).slideUp(600, function() { loadData(renderData, currentFolderPath); });
        return false;
      }
    });
  }

  function hideFolderRowAfterAction(nameToDelete) {
    var itemToRemove;
    $('.folder-holder .folder-list li').each(function() {
      var name = $(this).find('a div').text();
      if (name == nameToDelete) {
        itemToRemove = $(this);
        return false;
      }
    });
    if (itemToRemove) {
      itemToRemove.remove();
    }
  }

  function folderRenamed(oldName, name) {
    var itemToRename;
    $('.folder-holder .folder-list li').each(function() {
      var name = $(this).find('a div').text();
      if (name == decodeURI(oldName)) {
        itemToRename = $(this);
        return false;
      }
    });
    itemToRename.find('a div').text(name);
  }

  function clearMultiSelection() {
    $('#file-Manager #files .item').removeClass('active');
    updateSelectedItemsStatus('clear');
  }

  function isAllSelected() {
    var allSelected = true;
    if ($('#files .item').length) {
      $('#files .item').each(function(_i) {
        if (!$(this).hasClass('active')) {
          allSelected = false;
          return false;
        }
      });
    } else {
      allSelected = false;
      return false;
    }
    return allSelected;
  }

  function selectDeselectAll() {
    var allSelected = isAllSelected();
    if (allSelected) {
      $('#files .item').removeClass('active');
    } else {
      $('#files .item').addClass('active');
    }
    $('#files .item').removeClass('lastClicked');
    updateSelectedItemsStatus();
  }

  function homeClicked() {
    closeDetails();
    loadData(renderData, startPath);
  }

  /////////////////////////////////////////////////////////////////////////
  // Actions (Multiple selection)

  function compressFiles(fileUrls, fn) {
    var date = new Date();
    var archiveName = 'compressed-archive-' + date.getFullYear() + "-" +
      (date.getMonth() + 1) + "-" + date.getDate() + "-" +
      date.getHours() + "." + date.getMinutes() + "." + date.getSeconds();
    var api = $('.file-manager-container.api').data('api');
    var actionUrl = '';
    var opts = {
      objects: []
    };
    $.each(fileUrls, function(_i, url) {
      if (!actionUrl) {
        actionUrl = apiFile + currentFolderPath + '/' + archiveName + '.zip';
      }
      opts.objects.push({
        'url': decodeURI(url)
      });
    });

    api.put(encodeURI(actionUrl), opts).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
    });
    clearMultiSelection();
  }

  function deleteFiles(fileUrls, fn) {
    var api = $('.file-manager-container.api').data('api');

    var opts = {
      objects: [],
      action: 'delete'
    };
    $.each(fileUrls, function(_i, url) {
      opts.objects.push({
        'url': decodeURI(url)
      });
    });

    api.post(encodeURI(apiFileBulk), opts).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
    });

    $('.selected-items-status').hide();
  }

  function getAllFileNames() {
    return $('#files .item .file-name').map(function() {
      return $(this).html();
    }).get();
  }

  function incrementFileName(file, extension) {
    var justfile = file.substr(0, file.length - extension.length);
    var newFileName;
    if (justfile[justfile.length - 1] == ')') {
      var i = justfile.lastIndexOf('(');
      if (i > -1) {
        var num = justfile.substr(i).replace('(', '').replace(')', '');
        var inum = parseInt(num, 10);
        if (!isNaN(inum)) {
          inum++;
          newFileName = justfile.substr(0, i) + '(' + inum + ')';
        }
      }
    }
    if (newFileName) {
      return newFileName + extension;
    } else {
      return justfile + '(1)' + extension;
    }
  }

  /////////////////////////////////////////////////////////////////////////
  // Actions

  // Action: compress
  function compressFile(fileUrl, fn) {
    var api = $('.file-manager-container.api').data('api');
    api.head(apiFile + currentFolderPath + getItemName(fileUrl) + '?check_perms=read')
      .done(function() {
        var archiveName = getItemName(fileUrl) + '.zip';
        var files = getAllFileNames();
        while (files.indexOf(archiveName) > -1) {
          archiveName = incrementFileName(archiveName, '.zip');
        }
        var actionUrl = apiFile + currentFolderPath + archiveName;
        var opts = {
          objects: [{
            'url': decodeURI(fileUrl)
          }]
        };
        
        api.put(actionUrl, opts).done(function(data, textStatus, jqXHR) {
          if (fn) {
            fn(data);
          }
        });
      })
      .fail(function() {
        $.notify('Permissions denied', { type: 'warning' });
        return;
      });
    
  }

  // Action: uncompress
  function uncompressFile(fileUrl, fn) {
    actionPatch(fileUrl, 'uncompress', fn);
  }

  // Action: delete
  function deleteFile(fileUrl, fn) {
    var api = $('.file-manager-container.api').data('api');
    api.delete(fileUrl).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
      var url = apiDirectory + encodeURI(currentFolderPath);
      api.get(url).done(function(data){
        if (!data.count) {
          $('.select-all-container').hide();
        }
      });
    });
  }

  function sendRemoteFsCommand(commandName, fileUrls, targetPath, fn) {
    var api = $('.file-manager-container.api').data('api');
    var opts = {
      objects: [],
      action: commandName,
      target: targetPath
    };

    $.each(fileUrls, function(_i, url) {
      opts.objects.push({
        'url': decodeURI(url)
      });
    });

    api.post(encodeURI(apiFileBulk), opts).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
    });
  }

  // Action: move
  function moveFile(fileUrls, newFolder, fn) {
    sendRemoteFsCommand('move', fileUrls, newFolder, fn);
  }

  // Action: copy
  function copyFile(fileUrls, newFolder, fn) {
    sendRemoteFsCommand('copy', fileUrls, newFolder, fn);
  }

  // Action: rename
  function renameFile(fileUrl, newName, fn) {
    var api = $('.file-manager-container.api').data('api');
    var actionData = {
      'action': 'rename',
      'new_name': newName
    };
    api.patch(fileUrl, actionData).done(function(data, textStatus, jqXHR) {
      data.oldName = getItemName(fileUrl);
      if (fn) {
        fn(data);
      }
    });
  }

  // Action: set permissions
  function changePermissions(fileUrl, permissionsMask, fn) {
    var api = $('.file-manager-container.api').data('api');
    var actionData = {
      'action': 'chmod',
      'permission_mask': permissionsMask
    };
    api.patch(fileUrl, actionData).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
    });
  }
  
  function preparePermissionsView(permissions) {      
    while (permissions.length < 4) {
      permissions = '0' + permissions;
    }
    return permissions;        
  }

  function actionPatch(fileUrl, action, fn) {
    var api = $('.file-manager-container.api').data('api');
    var actionData = {
      'action': action
    };
    api.patch(fileUrl, actionData).done(function(data, textStatus, jqXHR) {
      if (fn) {
        fn(data);
      }
    });
  }

  function actionPatchWithTarget(fileUrl, action, targetFolder, fn) {
    var api = $('.file-manager-container.api').data('api');
    var actionData = {
      'action': action,
      'target_path': targetFolder
    };
    api.patch(fileUrl, actionData).done(function(data, textStatus, jqXHR) {
      data.fileName = getItemName(fileUrl);
      data.targetFolder = targetFolder;
      if (fn) {
        fn(data);
      }
    });
  }
})(jQuery);
