/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($, window) {
  'use strict';

  if (typeof $ === 'undefined')
    throw new Error('This script requires jQuery');

  if (typeof async === 'undefined')
    throw new Error('This script requires Async.js');

  var Viewpoint = window.Viewpoint || {};

  if (typeof Viewpoint.API === 'undefined')
    throw new Error('This script requires Viewpoint API');

  var JobWatcher = function(options) {
    var DEFAULTS = {
        api: new Viewpoint.API()
    };

    this.addTask = function(handler) {
      this.q.push({ job: this.options.job, api: this.options.api }, handler);
    };

    this.options = $.extend({}, DEFAULTS, options);

    this.q = async.queue(function(task, callback) {
      setTimeout(function() {
          $.when(
            task.api.get('/api/jobs/' + task.job + '/'),
            task.api.get('/api/jobs/' + task.job + '/requirements/')
          )
          .done(function(res1, res2) {
            var data = res1[0], reqsData = res2[0];
            // run function from jw.watch
            callback({
              state: data.state,
              queue_status: data.queue_status,
              holds: data.holds,
              dedicate_all_processors: data.dedicate_all_processors,
              requirements: reqsData.results[0]
            });
          })
          .fail(function(error) {
            console.error(error);
          });
      }, 1000);
    });
  };

  // handler is function which updates page and add tasks to async.queue
  JobWatcher.prototype.watch = function(handler) {
    if (!this.q.started) {
      this.q.push({ job: this.options.job, api: this.options.api }, handler);
    }
  };

  Viewpoint.JobWatcher = JobWatcher;

  window.Viewpoint = Viewpoint;
})(jQuery, window);
