/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($, window) {
  'use strict';

  var dashboardChartDefaultColor = '#737373';

  var queueStatusColorScheme = {
    ACTIVE: '#53c35b',
    COMPLETED: '#0784cc',
    ELIGIBLE: '#808080',
    BLOCKED: '#b70e09',
    NONE: '#99958C',
    DEFAULT: dashboardChartDefaultColor
  };

  window.getColorByQueueStatus = function(status) {
    var item = queueStatusColorScheme[status.toUpperCase()];
    return item || queueStatusColorScheme['DEFAULT'];
  };

  var nodeStatusColorScheme = {
    OFFLINE: '',
    BUSY: '#ec971f',
    DOWN: '#c9302c',
    IDLE: '#808080',
    UNKNOWN: '#91a3b3',
    RUNNING: '#53c35b',
    DRAINED: '#daa520',
    FLUSH: '',
    NONE: '#f1bf83',
    RESERVED: '',
    UP: '',
    DEFAULT: dashboardChartDefaultColor
  };

  var nodeStatusColorSchemeDark = {
    OFFLINE: '',
    BUSY: '#D37E06',
    DOWN: '#B01713',
    IDLE: '#676767',
    UNKNOWN: '#788A9A',
    RUNNING: '#3AAA42',
    DRAINED: '#C18C07',
    FLUSH: '',
    NONE: '#D8A66A',
    RESERVED: '',
    UP: '',
    DEFAULT: '5A5A5A'
  };

  window.getColorByNodeState = function(state, dark) {
    if (dark) {
      var item = nodeStatusColorSchemeDark[state.toUpperCase()];
      return item || nodeStatusColorSchemeDark['DEFAULT'];
    } else {
      var item = nodeStatusColorScheme[state.toUpperCase()];
      return item || nodeStatusColorScheme['DEFAULT'];
    }
  };

  var VariablesTooltips = function() {
    return {
      name: 'Allows you to specify a more friendly name (note: name must be alphanumeric)',
      duration: 'The amount of time the job is expected to run for',
      eligibledate: 'Declares the time after which the job is eligible for execution',
      account: 'Defines the account associated with the job',
      destinationQueue: 'Defines the destination queue / class of the job',
      qos: 'Defines the desired QoS for the job',
      memorypernode: 'Defines the resources that are required by the job and establishes a limit to the amount of resource that can be consumed',
      architecture: 'Defines the resources that are required by the job and establishes a limit to the amount of resource that can be consumed',
      exedir: 'Your job’s execution directory',
      errorpath: 'Defines the path to be used for the standard error stream of the job',
      outputpath: 'Defines the path to be used for the standard output stream of the job',
      join: 'Merge stderr and stdout streams?',
      hold: 'Specifies that a user hold be applied to the job at submission time',
      shell: 'Declares the shell that interprets the job script, e.g. /bin/bash',
      genericresources: 'Allows for specification of additional job attributes',
      moabenvironmentvariables: 'Push Moab environment variables to job?',
      maillist: 'Specifies the list of users to whom mail is sent by the execution server. Overrides the EMAILADDRESS specified on the USERCFG [credential]',
      featuresRequested: 'Contains a list of features that are required for the job to run',
      featuresExcluded: 'Contains a list of features that can not be present on hardware the job needs to run',
      priority: 'Defines the priority of the job',
      emailoptions: 'A list of requested email options',
      startindex: 'The start index of the job',
      endindex: 'The end index of the job',
      moabtemplates: 'Defines set of MOAB job templates',
      cpuNodes: 'Defines the resources that are required by the job and establishes a limit to the amount of resource that can be consumed'
    };
  }();

  window.VariablesTooltips = VariablesTooltips;

  var QueueStatusTooltips = function() {
    return {
      ACTIVE: 'Job is currently executing the user application',
      ELIGIBLE: 'Job is eligible to run and will start as soon as resources become available',
      BLOCKED: 'Job is currently blocked',
      COMPLETED: 'Job has completed'
    };
  }();

  window.QueueStatusTooltips = QueueStatusTooltips;

  var StatusTooltips = function() {
    return {
      RUNNING: 'Job is currently executing the user application',
      SUSPENDED: 'Job was running but has been suspended by the scheduler or an administrator; user application is still in place on the allocated compute resources, but it is not executing',
      BLOCKED: 'Job is currently blocked',
      DEFERRED: 'Job that has been held by Moab due to an inability to schedule the job under current conditions. Deferred jobs are held for DEFERTIME before being placed in the idle queue. This process is repeated DEFERCOUNT times before the job is placed in batch hold',
      VACATED: 'Job canceled after partial execution due to a system failure or, more commonly, because user cancelled the job while it was running',
      REMOVED: 'Job has run to its requested walltime successfully but has been canceled by the scheduler or resource manager due to exceeding its walltime or violating another policy; includes jobs canceled by users or administrators either before or after a job has started',
      COMPLETED: 'Job has completed running without failure',
      STAGING: 'Job is currently being staged',
      NOTQUEUED: 'Job has not been queued',
      IDLE: 'Job is currently queued and eligible to run but is not executing',
      STARTING: 'Batch system has attempted to start the job and the job is currently performing pre-start tasks that may include provisioning resources, staging data, or executing system pre-launch scripts',
      UNKNOWN: 'Moab cannot determine the state of the job',
      HOLD: 'Job is idle and is not eligible to run due to a user, (system) administrator, or batch system hold'
    };
  }();

  window.StatusTooltips = StatusTooltips;

  // make sure that ckeditor will attempt to load its dynamic resources from a proper location.
  window.CKEDITOR_BASEPATH = '/static/ckeditor/';

  function getJobTypeIcon(type) {
    var typeIcon = '';
    switch (type) {
      case 'regular':
        typeIcon = '<span class="job-type-icon icon-hpc"></span>';
        break;
      case 'array':
        typeIcon = '<span class="job-type-icon icon-array"></span>';
        break;
      case 'nitro':
        typeIcon = '<span class="job-type-icon icon-nitro"></span>';
        break;
      case 'remote_viz':
        typeIcon = '<span class="job-type-icon icon-remote"></span>';
        break;
    }
    return typeIcon;
  }

  window.getJobTypeIcon = getJobTypeIcon;

  function renderDate(dateField) {
    if (dateField) {
      var time = moment.utc(new Date(dateField));
      return moment(time).format('YYYY/MM/DD');
    } else {
      return '';
    }
  }

  function renderSize(bytes) {
    if (bytes) {
      var k = 1024;
      var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
      var i = Math.floor(Math.log(bytes) / Math.log(k));
      if (i === 0) {
        // prevent fractions when pure Bytes are shown
        return [bytes, sizes[i]].join(' ');
      }
      return (parseFloat((bytes / Math.pow(k, i)).toFixed(2))) + ' ' + sizes[i];
    } else {
      return '0 Bytes';
    }
  }

  window.renderDate = renderDate;
  window.renderSize = renderSize;
})(jQuery, window);
