/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

(function($) {
  'use strict';

  var api;
  var table;
  var Repeater;
  var totalCount;
  var refreshTimer;
  var pageNumber = 1;
  var sort = {
    'name': 1
  };

  var searchParams = {
    'NodeID': '',
    'Class': '',
    'Features': '',
    'resourceState': '',
    'procsFrom': '',
    'procsTo': '',
    'jobsFrom': '',
    'jobsTo': '',
    'proc_utilFrom': '',
    'proc_utilTo': '',
    'memory_utilFrom': '',
    'memory_utilTo': ''
  };
  var sortParams = {
    'name': 1,
    'state': -1,
    'available_processors': -1,
    'jobs': -1,
    'processor_utilization_percentage': -1,
    'memory_utilization_percentage': -1
  };
  var dataTableOpts = {
    dom: 'tlp',
    bFilter: false,
    bInfo: false,
    bPaginate: false,
    order: [
      [0, 'asc']
    ],
    'columnDefs': [{
      'width': '14%',
      'targets': 0,
      'data': 'name',
      'sType': 'sortOff',
      'defaultContent': 'N/A',
      'className': 'node-id-cell',
      'render': function(data, type, full) {
        var params = jQuery.extend(true, {}, searchParams);
        // Collect and store to href all filter parameters
        for (var i in params) {
          if (!params[i] || params[i] == '-1' || params[i] == '0') {
            delete params[i];
          }
        }
        return '<a href="/nodedetails/?node_name=' + data + '&' +
          $.param(params) + '" title="' + data + '">' + data + '</a>';
      }
    }, {
      'width': '10%',
      'targets': 1,
      'data': 'state',
      'sType': 'sortOff',
      'defaultContent': 'N/A',
      'className': 'node-state',
      'render': function(data, type, full) {
        return '<span title="' + data + '">' + data + '</span>';
      }
    }, {
      'width': '10%',
      'targets': 2,
      'bSortable': false,
      'data': 'classes',
      'sType': 'sortOff',
      'className': 'node-class',
      'defaultContent': 'N/A',
      'render': function(data, type, full) {
        if (data.length > 1) {
          var dataContent = '';
          for (var i in data) {
            dataContent += "<span class='filter_classes'>" + data[i] + "</span></br>";
          }
          return '<span title="' + data + '" data-content="' + dataContent + '" data-toggle="popover_filter"\
                     style="text-decoration:none;cursor:auto;" data-placement="right">' + data + '</span>';
        } else if (data.length == 1) {
          return '<span class="filter_classes" title="' + data + '">' + data + '</span>';
        }
      }
    }, {
      'width': '10%',
      'targets': 3,
      'bSortable': false,
      'data': 'features',
      'sType': 'sortOff',
      'className': 'node-feature',
      'defaultContent': 'N/A',
      'render': function(data, type, full) {
        if (data.length > 1) {
          var dataContent = '';
          for (var i in data) {
            dataContent += "<span class='filter_features'>" + data[i] + "</span></br>";
          }
          return '<span title="' + data + '" data-content="' + dataContent + '" data-toggle="popover_filter"\
                     style="text-decoration:none;cursor:auto;" data-placement="right">' + data + '</span>';
        } else if (data.length == 1) {
          return '<span class="filter_features" title="' + data + '">' + data + '</span>';
        }
      }
    }, {
      'width': '18%',
      'targets': 4,
      'data': '',
      'sType': 'sortOff',
      'className': 'processors',
      'defaultContent': 'N/A',
      'render': function(data, type, full) {
        return '<span title="' + full['available_processors'] + '/' + full['configured_processors'] + '">' +
          full['available_processors'] + '/' + full['configured_processors'] + '</span>';
      }
    }, {
      'width': '7%',
      'targets': 5,
      'data': 'jobs',
      'sType': 'sortOff',
      'className': 'jobs-count',
      'defaultContent': 'N/A',
      'render': function(data, type, full) {
        return '<span title="' + data + '">' + data + '</span>';
      }
    }, {
      'width': '10%',
      'targets': 6,
      'data': 'processor_utilization_percentage',
      'sType': 'sortOff',
      'className': 'cpu-utilization',
      'defaultContent': 'N/A',
    }, {
      'width': '10%',
      'targets': 7,
      'data': 'memory_utilization_percentage',
      'sType': 'sortOff',
      'className': 'memory-utilization',
      'defaultContent': 'N/A',
    }],
  };

  function setPageSize() {
    var pageSize = localStorage.getItem("nodePageSize");
    if (pageSize) {
      if ($('#viewPerPage').length) {
        $('#viewPerPage').val(pageSize);
      }
      return pageSize;
    }
  }

  function createPaginator() {
    $('.datagrid-container .paginator-container').bootpag({
      total: 1, // total pages
      maxVisible: 7,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      pageNumber = num;
      loadData(true);
    });
  }

  function loadData(cb) {
    var pageSize = $('#viewPerPage').val();
    var offset = (pageNumber - 1) * pageSize;
    var url = '/node_table/?max=' + pageSize + '&offset=' + pageNumber + getSearchParams() +
      '&sort=' + JSON.stringify(sort);
      api.get(url).done(function(data) {
        totalCount = data.count;
        $('#totalResourcesCnt').text(totalCount + ' results returned');
        table.clear();
        table.rows.add(data.results);
        table.draw();
        pushSearchParams(data.query_set);

        updatePaginator(totalCount, pageSize, pageNumber);
        createPopover();
        addColorsForStatusColumn();
        if ($.isFunction(cb)) {
          cb();
        }
      })
      .fail(function(result) {
        if (/^\s*<!DOCTYPE html>/.test(result.responseText)) {
          if (/Viewpoint Login/.test(result.responseText)) {
            window.location = '/logout?expired=1';
            return;
          }
        }
        table.clear();
        table.draw();
        updatePaginator(0, 1, 1);
      });
  }

  function createEventHandlers() {
    $('body').on('click', '.filter_features', function() {
      $('.filter_features span').trigger('click');
      $('#searchCat').val('Features');
      $('#searchBox').val($(this).text());
      $('#searchBtn').trigger('click');
    });
    $('body').on('click', '.filter_classes', function() {
      $('.filter_class span').trigger('click');
      $('#searchCat').val('Class');
      $('#searchBox').val($(this).text());
      $('#searchBtn').trigger('click');
    });
    $('#viewPerPage').change(function() {
      pageNumber = 1;
      loadData();
    });
    $("input.to-number-inf").each(function() {
      if ($(this).val() == "0") {
        $(this).val("∞");
      }
    });
    $("input.to-number-inf").on("blur", function() {
      if ($(this).val() == '' || $(this).val() == '-1') {
        $(this).val("∞");
      }
    });
    $("input.number-from").on("blur", function() {
      if ($(this).val() == '') {
        $(this).val("0");
      }
    });
    $(".number-input ").on("keypress keyup blur", function(event) {
      if ((event.which > 31 && event.which < 48 || event.which > 57)) {
        event.preventDefault();
      }
    });
    $('body').on('change', '#viewPerPage', function() {
      localStorage.setItem("nodePageSize", $(this).val());
    });
    $(document).on('click', '#node-table th:not(.sorting_disabled)', function() {
      var colId = $(this).attr('id');
      sort = {};
      sortParams[colId] = -sortParams[colId];
      sort[colId] = sortParams[colId];
      loadData(true);
    });
    $('#searchBtn').click(function() {
      pageNumber = 1;
      if (!$('#searchCat').val()) {
        return;
      }
      var findBy = $('#searchCat').val().trim();
      var value = $('#searchBox').val().trim();
      if (findBy && value) {
        $('#searchCat option:selected').prop('disabled', true).addClass('optiondisabled');
        searchParams[findBy] = value;
        loadData(true);
      }
    });
    $('#nodeList_filterBtn').click(function() {
      if (checkData()) {
        pageNumber = 1;
        searchParams.resourceState = $('#resourceState').val();
        searchParams.procsFrom = $('#procsFrom').val();
        searchParams.procsTo = $('#procsTo').val();
        searchParams.jobsFrom = $('#jobsFrom').val();
        searchParams.jobsTo = $('#jobsTo').val();
        searchParams.proc_utilFrom = $('#proc_utilFrom').val();
        searchParams.proc_utilTo = $('#proc_utilTo').val();
        searchParams.memory_utilFrom = $('#memory_utilFrom').val();
        searchParams.memory_utilTo = $('#memory_utilTo').val();
        loadData(true);
      }
    });
    $('#nodelist_clearAllBtn').click(function() {
      $('input[type=text]').val('');
      $('input.to-number-inf').trigger('blur');
      $('input.number-from').trigger('blur');
      $('#searchCat').val("").attr('selected', 'selected');
      $('#resourceState').val('').attr('selected', 'selected');
      $('#opt1').removeClass('optiondisabled');
      $('#opt2').removeClass('optiondisabled');
      $('#opt3').removeClass('optiondisabled');
      $('#opt1').removeAttr('disabled');
      $('#opt2').removeAttr('disabled');
      $('#opt3').removeAttr('disabled');
      $('.search-values').remove();
      $('#totalResourcesCnt').text('');
      searchParams = {};
      loadData(true);
    });
  }

  function pushSearchParams(data) {
    $('.search-values').remove();

    if (data.id) {
      showSearchOptions('NodeID', data.id);
      searchParams.NodeID = data.id;
      $('#opt1').prop('disabled', true).addClass('optiondisabled');
      delete data.id;
    }
    if (data.classes) {
      showSearchOptions('Class', data.classes);
      searchParams.Class = data.classes;
      $('#opt2').prop('disabled', true).addClass('optiondisabled');
      delete data.classes;
    }
    if (data.features) {
      showSearchOptions('Features', data.features);
      searchParams.Features = data.features;
      $('#opt3').prop('disabled', true).addClass('optiondisabled');
      delete data.features;
    }
    for (var i in data) {
      $('#' + i).val(data[i]);
      searchParams[i] = data[i];
    }
    $('input.to-number-inf').trigger('blur');
    $('input.number-from').trigger('blur');
  }

  function getURLParameterByName(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
      results = regex.exec(location.search);
    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
  }

  function getSearchParams() {
    var params = $.extend(true, {}, searchParams);
    for (var i in params) {
      if (!params[i]) {
        delete params[i];
      }
    }
    return $.isEmptyObject(params) ? '' : '&' + $.param(params);
  }

  function showSearchOptions(findBy, value) {
    var displayedOption = findBy;
    var element = $('<span></span>')
      .addClass('search-values filter_' + findBy.toLowerCase())
      .append(
        $('<input></input>')
        .attr('type', 'text')
        .addClass('form-control')
        .prop('disabled', true)
        .val(displayedOption + ':' + value)
      )
      .append(
        $('<span></span>')
        .addClass('glyphicon glyphicon-remove remove-search-val')
        .click(function(e) {
          searchParams[findBy] = '';
          $(this).parent().remove();
          $('#searchCat option[value="' + findBy + '"]').prop('disabled', false).removeClass('optiondisabled');
          $('#searchBox').val('');
          loadData(true);
        })
      );
    $('#searchValues').append(element);
  }

  function createPopover() {
    $("[data-toggle=popover_filter]").popover({
      html: true,
      template: '<div class="popover" role="tooltip"><div class="arrow"></div>' +
        '<div class="popover-content"></div></div>'
    }).on("mouseenter", function() {
      var _this = this;
      $(this).popover("show");
      $(this).siblings(".popover").on("mouseleave", function() {
        $(_this).popover('hide');
      });
    }).on("mouseleave", function() {
      var _this = this;
      setTimeout(function() {
        if (!$(".popover:hover").length) {
          $(_this).popover("hide");
        }
      }, 100);
    });
  }

  function refreshController() {
    setRefreshInterval();
    changeIntervalHandler();
    $('#resourceListRefresh').click(function() {
      loadData();
    });
    $('#selectIntervals').change(changeIntervalHandler);

    function setRefreshInterval() {
      var refreshInterval = localStorage.getItem('nodesRefreshInterval');
      if (refreshInterval) {
        $('#selectIntervals').val(refreshInterval);
      }
    }

    function changeIntervalHandler() {
      var $selectIntervals = $('#selectIntervals');
      var value = $selectIntervals.val();
      if (!value) {
        $selectIntervals.val("15000");
        value = 15000;
      }
      if (value === 'OFF') {
        Repeater.stop();
      } else {
        Repeater.setTimeout(value);
      }
      localStorage.setItem("nodesRefreshInterval", value);
    }

  }

  function initFilter() {
    // Init filters (In case user returned by "Return to previous search" link)
    for (var pName in searchParams) {
      var param = getURLParameterByName(pName);
      if (param && param != '-1') {
        $('#' + pName).val(param);
        searchParams[pName] = param;
        $('#' + pName).val(param);
      }
    }
  }

  function addColorsForStatusColumn(state) {
    $.each($('td.node-state'), function() {
      var state = $(this).find('span').text();
      var color = getColorByNodeState(state);
      var darkColor = getColorByNodeState(state, true);
      $(this).css({'background': 'linear-gradient(to top, ' + darkColor + ', ' + color + ')'});
    });
  }

  $(function() {
    disablecopypaste();
    createEventHandlers();
    Repeater = new Viewpoint.Repeater({
      timeout: $('#selectIntervals').val(),
      fn: [loadData]
    });
    refreshController();
    setPageSize();
    api = $('.node-table-container').loader(new Viewpoint.API()).data('api');
    table = $('#node-table').DataTable(dataTableOpts);
    $.fn.dataTableExt.oSort["sortOff-desc"] = function(x, y) {
      return 0;
    };
    $.fn.dataTableExt.oSort["sortOff-asc"] = function(x, y) {
      return 0;
    };
    initFilter();
    createPaginator();
    loadData();

    // Clear browser url
    var i = window.location.href.indexOf('?');
    var url = window.location.href.substr(0, i);
    window.history.pushState('', document.title, url);
  });

})(jQuery);
