/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($, window) {
  'use strict';

  if (typeof $ === 'undefined')
    throw new Error('This script requires jQuery');

  if (typeof async === 'undefined')
    throw new Error('This script requires Async.js');

  var Viewpoint = window.Viewpoint || {};

  var Repeater = function(options) {
    if (!options.fn && !Array.isArray(options.fn) && !options.fn.length) {
      throw new Error('No function to run passed!');
    }

    this._err = options.onError || function (err) {
      if (err) {
        throw new Error(err);
      }
    }

    this._timeout = options.timeout || 5000;
    this._enabled = false;
    this._fn = options.fn;
    this._currentDelayedFn;

    var _repeater = this;

    this._checkStatus = function() {
      return _repeater._enabled;
    }

    this._getTimeout = function() {
      return _repeater._timeout;
    }

    this._run = function() {
      return async.whilst(
        _repeater._checkStatus,
        function (callback) {
          _repeater._currentDelayedFn = setTimeout(function(){
            return async.parallel(
            _repeater._fn,
            function(err) {
              callback(err);
            }
            );
          }, _repeater._getTimeout());
        },
        _repeater._err
      );
    }

    this.start = function() {
      _repeater._enabled = true;
      _repeater._run();
    }

    this.stop = function() {
      _repeater._enabled = false;
      clearTimeout(_repeater._currentDelayedFn);
    }

    this.setTimeout = function(timeout) {
      _repeater.stop();
      _repeater._timeout = timeout;
      _repeater.start();
    }

    this.isRunning = function() {
      return _repeater._checkStatus();
    }
  }

  Viewpoint.Repeater = Repeater;

  window.Viewpoint = Viewpoint;
})(jQuery, window);
