/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

/**
 * Patch Bootstrap Validator and bind it to Viewpoint API.
 */

// FIXME: now it seems that it will be better just
// to fork and hack the original Bootstrap Validator :(

// TODO: fork http://1000hz.github.io/bootstrap-validator/, place here and hack it

;(function($) {
  'use strict';

  var AsyncValidatorsKeys = [
    'jobExedir',
    'jobErrorpath',
    'jobOutputpath',
    'remoteDirectory',
    'remoteFile'
  ];

  if (!$.fn.validator)
    throw new Error('This Validator requires Bootstrap Validator (http://1000hz.github.io/bootstrap-validator/)');

  var _super = $.fn.validator.Constructor;

  var api;
  // To validate of unique variable name
  var variables;

  var Validator = function(element, options) {
    api = options.api || $.viewpoint.api;
    variables = options.variables;
    // XXX: it will broke the original Bootstrap Validator
    Validator.INPUT_SELECTOR = _super.INPUT_SELECTOR =
        options.selector || ':input[name]:not([type="submit"], button, [formnovalidate]):enabled:visible, span[name]';
    _super.call(this, element, options);
    $.extend(Validator.VALIDATORS, options.custom);
    Validator.OPTIONS = options;
    this.$element.on('DOMNodeInserted.bs.validator', $.proxy(function(e) {
      if ($(e.target).is(Validator.INPUT_SELECTOR))
        this.validateInput(e);
    }, this));
  };

  Validator.VALIDATORS = {
    'native': function ($el) {
      var el = $el[0];
      return el.checkValidity ? el.checkValidity() : true;
    },
    'match': function ($el) {
      var target = $el.data('match');
      return !$el.val() || $el.val() === $(target).val();
    },
    'minlength': function ($el) {
      var minlength = $el.data('minlength');
      return !$el.val() || $el.val().length >= minlength;
    }
  };
  Validator.DEFAULTS = $.extend(_super.DEFAULTS, { delay: 0 });

  Validator.prototype = $.extend({}, _super.prototype);

  Validator.prototype.constructor = Validator;

  Validator.prototype.clearErrors = function(el) {
    _super.prototype.clearErrors.call(this, el);

    var ui = el.data('select2') && el.data('select2').$container ||
        el.data('popover-anchor') &&  $(el.data('popover-anchor')) ||
        el;

    ui.popover('destroy');

    el.removeData('bs.validator.errors.shown');
  };

  Validator.prototype.destroy = function() {
    this.$element
      .find(this.INPUT_SELECTOR)
      .each(function() {
        $(this)
          .popover('destroy')
          .removeData('bs.validator.errors.shown');
      });

    _super.prototype.destroy.call(this, this.$element);
  };

  function checkErrors($el) {
    var errors   = [];
    var deferred = $.Deferred();
    var options = this.options;

    $el.data('bs.validator.deferred') && $el.data('bs.validator.deferred').reject();
    $el.data('bs.validator.deferred', deferred);

    function getErrorMessage(key) {
      return $el.data(key + '-error') || $el.data('error') || key == 'native' &&
        $el[0].validationMessage || options.errors[key];
    }

    $.when.apply($, $.map(Validator.VALIDATORS, function (validator, key) {
        return $.Deferred(function (def) {
            if ($el.data(key) && AsyncValidatorsKeys.indexOf(key) > -1) {
                return validator.call(this, $el)
                .done(function () {
                  def.resolve();
                })
                .fail(function(error_message) {
                  def.reject({key: key, error_message: error_message});
                });
            }

            if (($el.data(key) || key == 'native') && !validator.call(this, $el)) {
              return def.reject({key: key});
            }

            def.resolve();
          }).promise();
        })
      )
      .fail(function (params) {
        var error = getErrorMessage(params.key);
        if (!~errors.indexOf(error)) {
          errors.push(params.error_message || error);
        }
      })
      .always(function() {
        if (!errors.length && $el.val() && $el.data('remote')) {
          this.defer($el, function () {
            var data = {};
            data[$el.attr('name')] = $el.val();
            $.get($el.data('remote'), data)
              .fail(function (jqXHR, textStatus, error) {
                errors.push(getErrorMessage('remote') || error);
              })
              .always(function () {
                deferred.resolve(errors);
              });
          });
        }
        else {
          deferred.resolve(errors);
        }
      });

    return deferred.promise();
  }
  Validator.prototype.runValidators = function(el) {
    var deferred = $.Deferred();

    if (el.data('bs.validator.deferred'))
        el.data('bs.validator.deferred').reject(); // [1]

    checkErrors.call(this, el)
      .done($.proxy(function(errors) {
        el.data('bs.validator.deferred', deferred);

        var name = el.attr('name');
        var remoteValidation = el.data('remote-validation') ||
          this.$element.data('remote-validation');

        var val;

        if (el.is('[type=checkbox]'))
            val = el.is(':checked');
        else if (el.is('span'))
          val = el.text();
        else if (el.data('durationpicker')) {
          var duration = el.durationpicker('parseDuration');
          if (duration == 0) duration='';
          val = typeof duration == 'object' && duration.fullSeconds ||
            duration;
        } else if (el.attr('type') == 'radio')
          val = el.filter(':checked').val();
        else val = el.val();

        if (val === null)
          val = '';

        if (name && remoteValidation) {
          var url = remoteValidation + '?';

          if ($.isArray(val))
            url += $.map(val, function(v) {
                return name + '=' + v;
              }).join('&');
          else url += name + '=' + val;

          this.defer(el, function() {
            api.get(url)
              .done(function() {
                // the server returned no errors, but we can't ignore the previously reported validator results.
                // the actual argument would be [].concat(errors)
                deferred.resolve(errors);
              })
              .fail(function(jqXHR) {
                if (jqXHR.status != 400) {
                  // Display popover for HTTP 400 only
                  // and behave as if there is no error
                  // in any other case
                  deferred.resolve(errors);
                  return;
                }

                var detail = jqXHR.responseJSON.detail;
                var remoteErrors = $.isArray(detail) ? detail : [detail];
                remoteErrors = errors.concat(remoteErrors);
                deferred.resolve(remoteErrors);
              });
          });
        } else {
          deferred.resolve(errors);
        }
      }, this))
      .fail(function() {
        // It can be rejected by old or by [1]
        deferred.reject();
      });

    return deferred.promise();
  };

  Validator.prototype.showErrors = function(el) {
    _super.prototype.showErrors.call(this, el);

    var html = $('<ul>')
      .addClass('list-unstyled')
      .append($.map(el.data('bs.validator.errors'), function(error) {
        return $('<li>').html(error);
      }));

    var ui = el.data('select2') && el.data('select2').$container ||
        el.data('popover-anchor') &&  $(el.data('popover-anchor')) ||
        el;

    ui
      .popover({
        container: 'body',
        content: html,
        html: true,
        template: '<div class="popover" role="tooltip"><div class="arrow"></div><div class="popover-content"></div></div>',
        trigger: 'hover'
      });
    ui.data('bs.popover').options.content = html;
    if (this.options.showPopover == true || this.options.showPopover == undefined) {
      el.data('bs.validator.errors.shown', true);
    }
  };

  var i = 0; // fields counter
  var totalElements = 0;
  // HACK: this hack to validate form before data sent to server
  Validator.prototype.onSubmit = function (e) {
    i = 0;
    totalElements = this.$element.find(Validator.INPUT_SELECTOR).length;
    if (this.$element.data('resubmit') === false) {
      if (this.isIncomplete() || this.hasErrors()) {
        e.preventDefault();
        this.$element.data('isValid', false);
      } else {
        this.$element.data('isValid', true);
      }
      this.$element.removeData('resubmit');
    } else if (this.$element.data('resubmit') === undefined) {
      // change the resubmit flag in "true" to send the form after all elements will be checked
      this.$element.data('resubmit', true);
      this.validate();
    }
  };

  Validator.prototype.validateAll = function (e) {
    i = 0;
    totalElements = this.$element.find(Validator.INPUT_SELECTOR).length;
    this.validate();
  };

  Validator.prototype.toggleSubmit = function() {
    if(!this.options.disable)
      return;

    var btn = this.options.submitButton || $('button[type="submit"], input[type="submit"]')
      .filter('[form="' + this.$element.attr('id') + '"]')
      .add(this.$element.find('input[type="submit"], button[type="submit"]'));

    var additionalСondition;
    if (this.options.additionalСondition) {
      additionalСondition = this.options.additionalСondition();
    } else {
      additionalСondition = false;
    }

    var disable = this.isIncomplete() || this.hasErrors() || additionalСondition;
    btn.prop('disabled', disable);
    btn.toggleClass('disabled', disable);
    i++;
    if (i == totalElements && !this.hasErrors()) {
      // all elements have been checked, generate event
      var validateCompleteEvent = $.Event('validationCompleted');
      this.$element.trigger(validateCompleteEvent);
    }
    if (i == totalElements && this.hasErrors()) {
      var validateErrEvent = $.Event('validationErrors');
      this.$element.trigger(validateErrEvent);
    }
  };

  Validator.prototype.validateInput = function(e) {
    if (!$(e.target).is(Validator.INPUT_SELECTOR))
      return;

    // FIXME: checkAdditionalFields is a kind of quick-and-dirty solution for cross-dependant validators.
    // This should be refactored in scope of IRIS-1393
    if (this.options.checkAdditionalFields) {
      var elements = [];
      elements = this.options.checkAdditionalFields(e);
      if (!elements.length) {
        _super.prototype.validateInput.call(this, e);
        return;
      }
      i++;
      for (var el in elements) {
        _super.prototype.validateInput.call(this, elements[el]);
      }
      return;
    }
    _super.prototype.validateInput.call(this, e);
  };

  var validateJobName = function ($el) {
    var name = $el.val();
    if (name.length === 0) {
      return true;
    }
    if (/^[A-Za-z-_\d]+$/.test(name)) {
      return true;
    }
    return false;
  };

  var validateTotalCores = function ($el) {
    var status = $el.val();
    if (status || $('#cpupernode-visible-totalamount').prop('checked')) {
      return true;
    }
    return false;
  };

  var validateTotalNodes = function ($el) {
    var status = $el.val();
    if (status || $('#cpupernode-visible-nodeswithcount').prop('checked')) {
      return true;
    }
    return false;
  };

  var validateCoresPerNode = function ($el) {
    var status = $el.val();
    if (status || $('#cpupernode-visible-nodeswithcount').prop('checked')) {
      return true;
    }
    return false;
  };

  var validateUniqueVariable = function ($el) {
    var allVariables = [];
    // here is assumed that we have a list of variables to only from the create job page
    if (variables) {
      allVariables = allVariables.concat(variables);
    } else if (Validator.OPTIONS.widgetName != 'jobEnvVariables') {
      // add values from job env variables to validate custom widgets (create template page)
      var jobEnvVariables = Object.keys(JSON.parse($('#jobEnvVariables-defaultValue').val() || '{}') || {});
      allVariables = allVariables.concat(jobEnvVariables);
    }
    $.each($('.var-name'), function(k,v) {
      if (!$($el).is(v) && $(v).val()) {
        allVariables.push($(v).val());
      }
    });
    if ($.inArray($el.val(), allVariables) == -1) {
      return true;
    } else {
      return false;
    }
  };

  var validateUniqueValueInDialog = function ($el) {
    var allValues = [];
    $.each($('.variable-key'), function(k,v) {
      if (!$($el).is(v)) {
        allValues.push($(v).val());
      }
    });
    if ($.inArray($el.val(), allValues) == -1) {
      return true;
    } else {
      return false;
    }
  };

  var validateVariableName = function ($el) {
    // do not validate if field is empty
    if (!$el.val()) {
      return true;
    }
    //Rules:
    //firstly: any alphanumeric character ('a'..'z', 'A'..'Z') or underscore ('_')
    //then (repeated 0..n times): any alphanumeric character, underscore or digit
    var pattern = new RegExp('^[a-z_]{1}[a-z0-9_]*$', 'i');
    if (!pattern.test($el.val())) {
      return false;
    }
    return true;
  };

  var validateDefaultValue = function ($el) {
    if ($el.val() == '' || $.isEmptyObject(JSON.parse($el.val()).map)) {
      $($el.data('popover-anchor')).addClass('error');
      return false;
    }
    $($el.data('popover-anchor')).removeClass('error');
    return true;
  };

  var validateDataManagement = function ($el) {
    return $.Deferred(function(def){
      api.head('/api/directory/' + $el.val())
        .done(function(){
          def.resolve();
          })
        .fail(function(data){
          var status = data.status;
          if (status === 404) {
          def.reject('Directory does not exist');
          } else if (status === 403) {
            def.reject('You do not have permission to access this directory');
          } else {
            def.reject('Path is not valid');
          }
        });
      }).promise();
  };

  var validateRemoteFile = function ($el) {
    return $.Deferred(function(def){
      if ($el.data('required-field')) {
        api.get('/api/fsobject/' + $el.val())
          .done(function(data){
            if (data.object_type == 'FILE') {
              def.resolve();
            } else if (data.object_type == 'DIRECTORY') {
              def.reject('It is a directory, file is needed');
            }
          })
          .fail(function(data){
            var status = data.status;
            if (status === 404) {
              def.reject('File does not exist');
            } else if (status === 403) {
              def.reject('You do not have permission to access this directory');
            } else {
              def.reject('Path is not valid');
            }
          });
      } else {
        def.resolve();
      }
    }).promise();
  };

  var validateRequiredField = function ($el) {
    return !!$el.val();
  };

  var validateUrlField = function ($el) {
    // https://10.2.185.68:9443 - true
    return /^(https?:\/\/)([\da-z\.-]+)\:([\d]{1,5})$/.test($el.val());
  };

  function createValidator(type) {
    return function($el) {
      var $formGroup = $el.closest('.form-group');
      var $checkboxes = $formGroup.find('input[type="' + type + '"]');
      $checkboxes.data('bs.validator.errors', []);
      if (!$checkboxes.length) {
        return true;
      }

      var uncheckedExists = false;
      $($checkboxes).each(function(i, e) {
        if (!$checkboxes.is(':checked')) {
          uncheckedExists = true;
          return false;
        }
      });

      return !uncheckedExists;
    };
  }

  var validateRequiredCheckbox = createValidator('checkbox');

  var validateJobShell = function ($el) {
    var value = $el.val();
    if (!value.length) {
      return true;
    }

    var pattern = /^(([a-zA-Z0-9/]+)(@[a-zA-Z0-9.]+)?(,|$))*$/;
    return pattern.test(value);
  };

  var validateArrayDiapason = function ($el) {
    if (!$el.val()) {
      return true;
    }
    if (!$('#arrays-start').val()) {
      return true;
    }
    var max = parseInt($el.val(), 10);
    var min = parseInt($('#arrays-start').val(), 10);
    if (!_.isNumber(min)) {
      return true;
    }
    if (!_.isNumber(max)) {
      return true;
    }
    return (min < max);
  }

  var validateSliderWidget = function ($el) {
    if (!$el.val()) {
      $el.attr('data-content', 'Field is required');
      return false;
    }

    var initVal = $el.closest('.slider-values').find('.initialval');
    var step = $el.closest('.slider-values').find('.stepval');
    var maxVal = $el.closest('.slider-values').find('.maxval');
    var minVal = $el.closest('.slider-values').find('.minval');

    if ($el.is(initVal)) {
      if (parseInt(initVal.val()) > parseInt(maxVal.val())) {
        $el.attr('data-content', 'The initial value is greater than the maximum');
        return false;
      } else if (parseInt(initVal.val()) < parseInt(minVal.val())) {
        $el.attr('data-content', 'The initial value is less than the minimum');
        return false;
      }
    } else if ($el.is(minVal)) {
      if (parseInt(minVal.val()) > parseInt(maxVal.val())) {
        $el.attr('data-content', 'The minimum value is greater than the maximum');
        return false;
      } else if (parseInt(minVal.val()) == parseInt(maxVal.val())) {
        $el.attr('data-content', 'Minimum and maximum values are the same');
        return false;
      }
    } else if ($el.is(maxVal)) {
      if (parseInt(minVal.val()) > parseInt(maxVal.val())) {
        $el.attr('data-content', 'The maximum value is less than the minimum');
        return false;
      } else if (parseInt(minVal.val()) == parseInt(maxVal.val())) {
        $el.attr('data-content', 'Minimum and maximum values are the same');
        return false;
      }
    }
    return true;
  };

  var Plugin = function(option) {
    return this.each(function() {
      var el = $(this);
      // [KN]: note that the initial validator class doesn't escape the error text
      // (see getErrorMessage() function)

      var customOptions = {
        custom: {
          jobName: validateJobName,
          jobShell: validateJobShell,
          jobExedir: validateDataManagement,
          jobErrorpath: validateDataManagement,
          jobOutputpath: validateDataManagement,
          uniqueVariable: validateUniqueVariable,
          variableName: validateVariableName,
          defaultValue: validateDefaultValue,
          totalCores: validateTotalCores,
          totalNodes: validateTotalNodes,
          coresPerNode: validateCoresPerNode,
          remoteDirectory: validateDataManagement,
          remoteFile: validateRemoteFile,
          requiredField: validateRequiredField,
          urlWithPort: validateUrlField,
          requiredCheckbox: validateRequiredCheckbox,
          uniqueValueInDialog: validateUniqueValueInDialog,
          sliderWidget: validateSliderWidget,
          arrayDiapason: validateArrayDiapason
        },
        errors: {
          jobName: 'Name allows alphanumeric, "-", "_" characters',
          jobShell: 'Value should have following format: [&lt;path' +
          '&gt;[@&lt;host&gt;][,&lt;path&gt;[@&lt;host&gt;],...]',
          jobExedir: 'Directory does not exist',
          jobErrorpath: 'Directory does not exist',
          jobOutputpath: 'Directory does not exist',
          uniqueVariable: 'Variable name should be unique',
          variableName: 'Variable name should start from any alphabetic character or underscore and contain any alphanumeric character or underscore',
          defaultValue: 'Default value may not be blank',
          totalCores: '"Total Cores" must have default value',
          totalNodes: '"Total Nodes" must have default value',
          coresPerNode: '"Cores Per Node" must have default value',
          remoteDirectory: 'Directory does not exist',
          remoteFile: 'File does not exist',
          requiredField: 'Field is required',
          urlWithPort: 'Value should be correct Url with port number',
          requiredCheckbox: 'Field is required',
          requiredRadio: 'Field is required',
          uniqueValueInDialog: 'Value should be unique',
          sliderWidget: 'Check your values',
          arrayDiapason: 'End Index Default Value should be larger than Start Index Default Value'
        }
      };

      var options = $.extend({}, Validator.DEFAULTS, customOptions, el.data(),
        typeof option == 'object' && option);
      var data = el.data('bs.validator');

      if (!data && option == 'destroy')
       return;
      if (!data)
        el.data('bs.validator', (data = new Validator(this, options)));
      if (typeof option == 'string')
        data[option]();
    });
  };

  var old = $.fn.validator;

  $.fn.validator = Plugin;
  $.fn.validator.Constructor = Validator;

  $.fn.validator.noConflict = function() {
    $.fn.validator = old;
    return this;
  };

  // Bootstrap Data API support for 'validator2'
  $(window).on('load', function() {
    $('form[data-toggle="validator2"]').each(function() {
      var form = $(this);
      Plugin.call(form, form.data());
    });
  });
})(jQuery);
