/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */
;
(function($) {
  'use strict';

  var Viewpoint = window.Viewpoint || {};

  var api;
  var table;
  var Repeater;
  var refreshTimer;
  var totalCount = 0;
  var pageNumber = 1;
  var searchParams = {
    'job_name': '',
    'job_custom_name': '',
    'user_name': '',
    'queue_status': '',
    'job_type': '',
    'job_start_datetime': '',
    'job_end_datetime': '',
    'wallclock_start': '',
    'wallclock_end': ''
  };
  Viewpoint.searchParams = searchParams;
  window.Viewpoint = Viewpoint;

  var sort = {"job_submit_datetime" : -1};
  var dataTableOpts = {
      dom: 'tlp',
      bFilter: false,
      bInfo: false,
      bPaginate: false,
      order: [[ 4, 'desc' ]],
      'columnDefs': [{
          'targets': 0,
          'data': 'job_id',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'job-id-cell',
          'render': function(data, type, full) {
            return createJobIdColumn (data, type, full);
          }
        }, {
          'targets': 1,
          'width': '15%',
          'data': 'job_custom_name',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'job-name',
          'render': function(data, type, full) {
            if (!data || data == '') {
              return '<span title="None">None</span>';
            } else {
              return '<span title="' + data + '">' + data + '</span>';
            }
          }
        }, {
          'targets': 2,
          'data': 'user_name',
          'className': 'submitter',
          'sType': 'sortOff',
          'defaultContent': 'N/A'
        }, {
          'targets': 3,
          'data': 'job_start_datetime',
          'sType': 'sortOff',
          'className': 'job-start-date',
          'render': function(data, type, full) {
            return createDateColumn(data, type, full);
          }
        }, {
          'targets': 4,
          'data': 'job_submit_datetime',
          'sType': 'sortOff',
          'className': 'submit-date',
          'render': function(data, type, full) {
            return createDateColumn(data, type, full);
          }
        }, {
          'targets': 5,
          'data': 'queue_status',
          'title': 'Queue Status',
          'sType': 'sortOff',
          'className': 'job-status',
          'render': function(data, type, full) {
            return createQueueStatusColumn(data, type, full);
          }
        }, {
          'targets': 6,
          'data': 'processor_count',
          'sType': 'sortOff',
          'className': 'processor-count',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            return createCoresColumn(data, type, full);
          }
        }, {
          'targets': 7,
          'data': 'node_count',
          'className': 'node-count',
          'sType': 'sortOff',
          'defaultContent': 'N/A',
        }, {
          'targets': 8,
          'data': 'wallclock_seconds',
          'sType': 'sortOff',
          'className': 'wallclock',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            return createWallclockColumn (data, type, full);
          }
        }
      ],
  };

  $(document).on('click', '#workload-table th', function(){
    var colId = $(this).attr('id');
    sort = {};
    sortingState[colId] = -sortingState[colId];
    sort[colId] = sortingState[colId];
    loadData();
  });

  function createDateColumn (data, type, full) {
    if (data) {
      var localTime = moment.utc(data.replace(' UTC', '')).toDate();
      return moment(localTime).format('YYYY-MM-DD HH:mm:ss');
    } else return 'N/A';
  }

  function createWallclockColumn (data, type, full) {
    return $('<div></div>')
      .append(
        $('<span></span>')
          .attr('title', 'elapsed time: ' + full.elapsed_time)
          .text(data)
      ).html();
  }

  function createQueueStatusColumn (data, type, full) {
    var queueStatusTitle = window.QueueStatusTooltips[full.queue_status] || '';
    return '<span title="' + queueStatusTitle + '">' +
              full.queue_status +
           '</span>';
  }

  function createCoresColumn (data, type, full) {
    var jobStates = ['STARTING', 'ACTIVE', 'COMPLETED', 'VACATED', 'CANCELED', 'REMOVED', 'RUNNING'];
    if (full.dedicate_all_processors && ($.inArray(full.job_state, jobStates) == -1)) {
      return '<span title="This job has not been scheduled yet. ' +
             'Requested core count is being estimated based on total amount of requested nodes.">~' + data + '</span>';
    } else {
      return '<span>' + data + '</span>';
    }
  }

  function createJobIdColumn(data, type, full) {
    var link;
    var tab = isNodeDetails() ? 'nodedetails' : 'workload';
    var params = jQuery.extend(true, {}, searchParams);
    params.jobname = params.job_name;
    delete params.job_name;
    for (var i in params) {
      if (!params[i]) {
        delete params[i];
      }
    }
    if ($.viewpoint.user.permissions.job_details) {
      link = $('<a></a>')
        .attr('href', '/jobdetails/?job_id=' +
          data + '&job_name=' + data + '&tab=' + tab +
          '&' + $.param(params) + '&path_previous=' + $.url('path', window.location.href)
        )
        .attr('title', data)
        .addClass('job-id-link')
        .text(data);
    } else {
      link = $('<span></span>').text(data);
    }

    // TODO refactor this
    if (full.is_array_master) {
      link.addClass('is-array');
    }

    // create popup menu only if user has 'Job Edit Admin' permission or job is of the user
    if ($.viewpoint.user.name == full.user_name || $.viewpoint.user.permissions.job_edit_admin) {
      var popupDiv = $('<div></div>')
          .addClass('menu-btn-container')
          .append($('<span></span>')
            .addClass('glyphicon glyphicon-align-justify trigger')
          );
      var popupContent = $('<div></div>').addClass('content hide');
      if ($.inArray(full.queue_status, ['BLOCKED', 'DEFERRED', 'NOTQUEUED', 'IDLE', 'UNKNOWN', 'ELIGIBLE']) > -1 && $.inArray('user', full.holds) == -1) {
        popupContent.append(
          $('<a></a>')
            .addClass('hold')
            .attr('href', '#')
            .attr('data-job-id', data)
            .text('Hold')
        )
        .append(
          $('<a></a>')
            .addClass('cancel')
            .attr('href', '#')
            .attr('data-job-id', data)
            .text('Cancel')
        );
        return $('<div></div>').append(link.add(popupDiv.append(popupContent))).html();
      }
      if ($.inArray('user', full.holds) > -1) {
        popupContent.append(
          $('<a></a>')
            .addClass('release')
            .attr('href', '#')
            .attr('data-job-id', data)
            .text('Release')
        )
        .append(
          $('<a></a>')
            .addClass('cancel')
            .attr('href', '#')
            .attr('data-job-id', data)
            .text('Cancel')
        );
        return $('<div></div>').append(link.add(popupDiv.append(popupContent))).html();
      }
      if ($.inArray(full.queue_status, ['RUNNING','SUSPENDED','DEFERRED','BLOCKED','NOTQUEUED','STAGING','STARTING', 'ELIGIBLE', 'ACTIVE']) > -1) {
        popupContent.append(
          $('<a></a>')
            .addClass('cancel')
            .attr('href', '#')
            .attr('data-job-id', data)
            .text('Cancel')
          );
        return $('<div></div>').append(link.add(popupDiv.append(popupContent))).html();
      }
      return $('<div></div>').append(link).html();
    } else {
      return $('<div></div>').append(link).html();
    }
  }

  function initJobArrayRows() {
    var names = $('.is-array');
    $.each(names, function(k, v) {
      $(v).parent().parent().addClass('master-job').attr('data-jobName', $(v).text());
    });
  }

  function createPopupMenu() {
    $('.popover').remove();
    $('.trigger').popover('destroy');
    $('.trigger').popover({
      html: true,
      container: 'body',
      animation: false,
      content: function() {
        return $(this).parent().find('.content').html();
      },
      placement: 'right'
    });
    $('.trigger').on('click', '.job-id-cell', function(e) {
      $(this).popover('show');
      e.stopPropagation();
    }).popover();
    // Appending context-menu button
    $('.job-id-cell div').hide();
    $(document).on('mouseenter', '.job-id-cell', function() {
      $(this).children('div').show();
    });
    $(document).on('mouseleave', '.job-id-cell', function() {
      $('.job-id-cell div').hide();
    });
    var mouseleave = function () {
      setTimeout(function () {
        if ($('.popover:hover').length === 0) {
          $('.trigger').popover('hide');
        }
      }, 250);
    };
    $(document).on('mouseleave', '#workload-table tr', mouseleave);
    $(document).on('mouseleave', '.popover', mouseleave);
  }

  function enableSubjobsWidget() {
    $('.master-job').each(function(i, e) {
        $(e).jobArrayWidget(true);
    });
  }

  function loadData(cb) {
    var pageSize = $('#viewPerPage').val();
    var offset = (pageNumber - 1) * pageSize;
    // save current search parameters for restoring them when we come back from job details page
    window.workloadSeachParams = searchParams;
    var url = '/api/workload-jobs/?page_size=' +
      pageSize + '&offset=' + offset + '&sort=' + JSON.stringify(sort) +
      getSearchParams();
    if (isNodeDetails()) {
      url += '&node=' + $('#node_details_name_value').text();
      url += '&job_state=RUNNING';
      url += '&job_type=all';
      url += '&match-subjobs=true';
    }
    api.get(url).done(function(data) {
      totalCount = data.count;
      table.clear();
      table.rows.add(data.results);
      table.draw();
      updatePaginator(totalCount, pageSize, pageNumber);
      createPopupMenu();
      initJobArrayRows();
      enableSubjobsWidget();
      $('#totalJobsCnt').text(totalCount + " " + gettext('results returned'));
      if (!$('.modal').is(':visible')) {
        if ($.isFunction(cb)) {
          cb();
        }
      }
      addColorsForQueueStatus();
    });
  }

  function addColorsForQueueStatus() {
    $.each($('td.job-status'), function(){
      var queueStatus = $(this).find('span').text();
      if (queueStatus == 'COMPLETED')
        $(this).addClass('completed-state');
      if (queueStatus == 'ACTIVE')
        $(this).addClass('active-state');
      if (queueStatus == 'BLOCKED')
        $(this).addClass('blocked-state');
      if (queueStatus == 'ELIGIBLE')
        $(this).addClass('eligible-state');
    });
  }

  function showSearchOptions(findBy, value, text) {
    var displayedOption = text;
    var element = $('<span></span>')
      .addClass('search-values')
      .append(
        $('<input></input>')
          .attr('type', 'text')
          .addClass('form-control')
          .prop('disabled', true)
          .val(displayedOption + ':' + value)
      )
      .append(
        $('<span></span>')
          .addClass('glyphicon glyphicon-remove remove-search-val')
          .click(function(e) {
            searchParams[findBy] = '';
            $(this).parent().remove();
            $('#searchCat option[value="' + findBy + '"]').prop('disabled', false).removeClass('optiondisabled');
            $('#searchBox').val('');
            loadData();
          })
      );
    $('#searchValues').append(element);
  }

  function getSearchParams() {
    if (!searchParams) {
      return '';
    }
    var params = [];
    $.each(searchParams, function(param, value) {
      if (value) {
        params.push(param + '=' + value);
      }
    });
    return params.length ? '&' + params.join('&') : '';
  }

  function createPaginator() {
    $('.datagrid-container .paginator-container').bootpag({
        total: 1, // total pages
        maxVisible: 7,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
    }).on('page', function(event, num) {
      pageNumber = num;
      loadData();
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    totalRows = parseInt(totalRows);
    rowsPerPage = parseInt(rowsPerPage);
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('.datagrid-container .paginator-container').bootpag({
        total: total, // total pages
        page: currentRow,
        maxVisible: 7,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
    });
  }

  function createEventHandlers() {
    $('#viewPerPage').change(function() {
      pageNumber = 1;
      loadData();
      if (isNodeDetails()) {
        localStorage.setItem("nodedetailsPageSize", $(this).val());
      } else if ($('.dashboard-page').length > 0) {
        localStorage.setItem("dashboardPageSize", $(this).val());
      } else {
        localStorage.setItem("workloadPageSize", $(this).val());
      }
    });

    $('body').on('click', function(e) {
      if (typeof $(e.target).data('original-title') == 'undefined' && !$(e.target).parents().is('.popover.in')) {
        $('[data-original-title]').popover('hide');
      }
    });

    $('#searchBtn').click(function() {
      var findBy = $('#searchCat').val() != 'select' && $('#searchCat').val().trim();
      var value = $('#searchBox').val().trim();
      if (findBy && value || $('#searchCat').hasClass('search-select')) {
        var select = $('#searchCat');
        if (!select.hasClass('search-select')) { // select has this class only on home page
          $('#searchCat option:selected').prop('disabled', true).addClass('optiondisabled');
          $('#searchCat .init-option').prop('selected', true);
          showSearchOptions(findBy, value, $('#searchCat option:selected').text());
          $('#searchCat').val("select").attr('selected', 'selected');
          $('#searchBox').val('');
        } else { // clear search parameters if we on home page
          searchParams.job_name = '';
          searchParams.job_custom_name = '';
          searchParams.user_name = '';
        }
        searchParams[findBy] = value;
        pageNumber = 1;
        loadData();
      }
    });

    $("#searchBox").keypress(function(event) {
      if (event.which == 13) {
        $('#searchBtn').click();
      }
    });

    $('#dateTimeRange').change(function() {
      searchParams.job_start_datetime = '';
      searchParams.job_end_datetime = '';
      searchParams.wallclock_start = '';
      searchParams.wallclock_end = '';
      if ($(this).val() == 'startdate') {
        $('.start-date').show();
        $('#startDate').datetimepicker({ language: 'pt-BR' });
        var currentDate = new Date();
        $('#startDate').data('datetimepicker').setDate(new Date(Date.UTC(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate(), 0, 0)));
        $('#endDate').datetimepicker({ language: 'pt-BR' });
        $('#endDate').datetimepicker({ language: 'pt-BR' });
        $('.wall-clock').hide().find('input').val('');
      } else if ($(this).val() == 'wallclock') {
        $('.start-date').hide().find('input').val('');
        $('.wall-clock').show();
      } else {
        $('.start-date').hide().find('input').val('');
        $('.wall-clock').hide().find('input').val('');
      }
    });

    $('#workload_filterBtn').click(function() {
      if (checkFilterData()) {
        searchParams.queue_status = $('#queueStatus').val();
        searchParams.job_type = $('#jobType').val();

        var startDate = $('#txtStartDate').val();
        var endDate = $('#txtEndDate').val();
        if (moment(startDate).isValid())
          searchParams.job_start_datetime = moment(startDate).utc().format('YYYY-MM-DD HH:mm:ss') + ' UTC';
        if (moment(endDate).isValid())
          searchParams.job_end_datetime = moment(endDate).utc().format('YYYY-MM-DD HH:mm:ss') + ' UTC';

        searchParams.wallclock_start = getWallclockStart() || '';
        searchParams.wallclock_end = getWallclockEnd() || '';

        if (!isEmptySearchParams()) {
          pageNumber = 1;
          loadData();
        }
      }
    });

    $('#workload_clearAllBtn').click(function() {
      $('.search-filter-section select, .search-filter-section input').val('');
      $('.search-values').remove();
      $('#opt1').removeClass('optiondisabled');
      $('#opt2').removeClass('optiondisabled');
      $('#opt3').removeClass('optiondisabled');
      $('#opt1').removeAttr('disabled');
      $('#opt2').removeAttr('disabled');
      $('#opt3').removeAttr('disabled');
      $('#searchCat').val("select").attr('selected', 'selected');
      $('.start-date').hide().find('input').val('');
      $('.wall-clock').hide().find('input').val('');
      for (var param in searchParams) searchParams[param] = '';
      pageNumber = 1;
      loadData();
    });

    $(document).on('click', '.popover-content .hold', changeStatus);
    $(document).on('click', '.popover-content .cancel', changeStatus);
    $(document).on('click', '.popover-content .release', changeStatus);

    function getWallclockStart() {
      var day = parseInt($('#day').val()) || 0;
      var hr = parseInt($('#hr').val()) || 0;
      var min = parseInt($('#min').val()) || 0;
      var sec = parseInt($('#sec').val()) || 0;
      return day * 86400 + hr * 3600 + min * 60 + sec;
    }

    function getWallclockEnd() {
      var day = parseInt($('#today').val()) || 0;
      var hr = parseInt($('#tohr').val()) || 0;
      var min = parseInt($('#tomin').val()) || 0;
      var sec = parseInt($('#tosec').val()) || 0;
      return day * 86400 + hr * 3600 + min * 60 + sec;
    }

    function isEmptySearchParams() {
      for (var key in searchParams) {
        if (searchParams[key]) {
          return false;
        }
      }
      return true;
    }
  }

  function changeStatus() {
    var jobId = $(this).data('job-id');
    var status = $(this).attr('class');
    if (status == 'release') {
      status = 'unhold';
    }
    $.ajax({
      url: '/modifyjob/',
      type: 'GET',
      data: {
        jobId: jobId,
        changeStatus: status
      }
    })
    .done(function(msg) {
      if (msg.indexOf('ERROR:') == 0) {
          $.notify(msg, {
            type: 'danger',
            onClose: function() {}
          });
      } else {
          msg = '\"' + status + '\" Request was sent to server.';
          $.notify(msg, {
            type: 'info',
            onClose: function() {}
          });
      }
    });
    $('.popover').remove();
  }

  function workloadWidgetExists() {
    return $('#jobpanel').length;
  }

  function resourceWidgetExists() {
    return $('#resourcepanel').length;
  }

  function cpuMemoryWidgetExists() {
    return $('#systemresourcespanel').length;
  }

  function refreshController() {
    // Auto refresh
    setRefreshInterval();
    changeIntervalHandler();
    $('#selectIntervals').change(changeIntervalHandler);
    $('#jobTypeDialogBtn').click(Repeater.stop);
    // refresh when on icon click
    $('#workloadRefresh').click(function() {
      loadData();
      // Reload Job Summary chart
      if (workloadWidgetExists()) {
        drawJobSumChart();
      }
      if (resourceWidgetExists()) {
        drawResSumChart();
      }
      if (cpuMemoryWidgetExists()) {
        drawCpuMemoryChart();
      }
    });

    function changeIntervalHandler() {
      var $selectIntervals = $('#selectIntervals');
      var value = $selectIntervals.val();

      if (!value) {
        $selectIntervals.val("15000");
        value = 15000;
      }

      if (value === 'OFF') {
        Repeater.stop();
      } else {
        Repeater.setTimeout(value);
      }
      if (isNodeDetails()) {
        localStorage.setItem("nodeDetailsRefreshInterval", value);
      } else {
        localStorage.setItem("jobsRefreshInterval", value);
      }
    }

    function setRefreshInterval() {
      if (isNodeDetails()) {
        var refreshInterval = localStorage.getItem("nodeDetailsRefreshInterval");
      } else {
        var refreshInterval = localStorage.getItem("jobsRefreshInterval");
      }
      if (refreshInterval) {
        $('#selectIntervals').val(refreshInterval);
      }
    }

    $('body').on('hide.bs.modal', '#job-type-dialog', changeIntervalHandler);
  }

  function setPageSize() {
    if (isNodeDetails()) {
      $('#viewPerPage').val(localStorage.getItem("nodedetailsPageSize") || '5');
      $('#workload-table').addClass('node-details');
    } else if ($('.dashboard-page').length > 0) {
      $('#viewPerPage').val(localStorage.getItem("dashboardPageSize") || '10');
    } else {
      $('#viewPerPage').val(localStorage.getItem("workloadPageSize") || '20');
    }
  }

  function isNodeDetails() {
    return ($('.form-nodeDetail').length > 0);
  }

  // TODO: refactor this, use $.url('?<parameter>')
  function getURLParameterByName(name) {
    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
        results = regex.exec(location.search);
    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
  }

  function setOldSearchParams() {
    $('#queueStatus').val(searchParams.queue_status);
    $('#jobType').val(searchParams.job_type);
    if (searchParams.job_name) {
      showSearchOptions('job_name', searchParams.job_name, 'Job ID');
      // Show search options on home page
      $('.dashboard-page #searchBox').val(searchParams.job_name);
      $('#searchCat.search-select').val('job_name');
    }
    if (searchParams.user_name) {
      showSearchOptions('user_name', searchParams.user_name, 'Submitter ID');
      // Show search options on home page
      $('.dashboard-page #searchBox').val(searchParams.user_name);
      $('#searchCat.search-select').val('user_name');
    }
    if (searchParams.job_custom_name) {
      showSearchOptions('job_custom_name', searchParams.job_custom_name, 'Job Name');
      // Show search options on home page
      $('.dashboard-page #searchBox').val(searchParams.job_custom_name);
      $('#searchCat.search-select').val('job_custom_name');
    }
    if (searchParams.job_start_datetime || searchParams.job_end_datetime) {
      $('#dateTimeRange').val('startdate');
      $('#txtStartDate').val(searchParams.job_start_datetime.replace(' UTC', ''));
      $('#txtEndDate').val(searchParams.job_end_datetime.replace(' UTC', ''));
      $('.start-date').show();
    } else if (searchParams.wallclock_start || searchParams.wallclock_end) {
      $('.wall-clock').show();
      $('#day').val(moment.duration(searchParams.wallclock_start, 'seconds').days());
      $('#hr').val(moment.duration(searchParams.wallclock_start, 'seconds').hours());
      $('#min').val(moment.duration(searchParams.wallclock_start, 'seconds').minutes());
      $('#sec').val(moment.duration(searchParams.wallclock_start, 'seconds').seconds());

      $('#today').val(moment.duration(searchParams.wallclock_end, 'seconds').days());
      $('#tohr').val(moment.duration(searchParams.wallclock_end, 'seconds').hours());
      $('#tomin').val(moment.duration(searchParams.wallclock_end, 'seconds').minutes());
      $('#tosec').val(moment.duration(searchParams.wallclock_end, 'seconds').seconds());
    }
  }

  $(document).ready(function() {

    $('#searchCat').val('select');
    $('#searchBox').val('');
    $('#queueStatus').val('');
    $('#jobType').val('');
    $('#dateTimeRange').val('');

    // Default Page Size for Workload and NodeDetails are differ each other
    setPageSize();
    api = $('.workload-table-container').loader(new Viewpoint.API()).data('api');
    table = $('#workload-table').DataTable(dataTableOpts);

    $.fn.dataTableExt.oSort["sortOff-desc"] = function (x, y) {
      return 0;
    };
    $.fn.dataTableExt.oSort["sortOff-asc"] = function (x, y) {
      return 0;
    };

    createPaginator();

    var fnArray = [];
    fnArray.push(loadData);
    if (!isNodeDetails()) {
      if (workloadWidgetExists()) {
        fnArray.push(drawJobSumChart);
      }
      if (resourceWidgetExists()) {
        fnArray.push(drawResSumChart);
      }
      if (cpuMemoryWidgetExists()) {
        fnArray.push(drawCpuMemoryChart);
      }
    }
    Repeater = new Viewpoint.Repeater({
      timeout: $('#selectIntervals').val(),
      fn: fnArray
    });
    refreshController();
    createEventHandlers();

    var restoredSearchParams = {
      'jobname': '',
      'job_custom_name': '',
      'user_name': '',
      'queue_status': '',
      'job_type': '',
      'job_start_datetime': '',
      'job_end_datetime': '',
      'wallclock_start': '',
      'wallclock_end': ''
    };

    var needRestoreFilter = false;
    if ($.url('?filters')) {
      searchParams = JSON.parse(decodeURIComponent($.url('?filters')));
      needRestoreFilter = true;
    } else {
      for (var i in restoredSearchParams) {
        var param = getURLParameterByName(i);
        if (param) {
          needRestoreFilter = true;
          if (i == 'jobname') {
            searchParams['job_name'] = param;
          } else {
            searchParams[i] = param;
          }
        }
      }
    }
    if (needRestoreFilter) {
      setOldSearchParams();
      // Need to clear url
      var i = window.location.href.indexOf('?');
      var url = window.location.href.substr(0, i);
      window.history.pushState('', document.title, url);
    }

    loadData();
  });

})(jQuery);

