/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;
(function($, Handlebars) {
  'use strict';

  var customWidgets = [];
  var lastSectionId = 0;
  var lastWidgetId = 0;
  var avaiableFeatures = [];

  var templateUrl = null;
  var templateHistoryUrl = null;
  var templateVersion = null;
  var lastSavedTmplt = {};
  var templateType = '';
  var descriptionInitialized = false;

  function CustomWidgetConfig(position, type, label, variableName, defaultValue, visible, editable, required, order) {
    this.label = label || "";
    this.tooltip = this.label;
    this.variable_name = variableName || "";
    this.default_value = defaultValue || "";
    this.visible = visible;
    this.editable = editable;
    this.properties = [
      {
        name: "id",
        value: "custom"
      },
      {
        name: "position",
        value: position
      },
      {
        name: "type",
        value: type
      },
      {
        name: "required",
        value: required.toString()
      },
      {
        name: "order",
        value: order
      }
    ];
  }

  function CustomWidget(id, position) {
    this.id = id;
    this.position = position;
  }

  function generateSectionId() {
    lastSectionId++;
    return lastSectionId;
  }

  function generateWidgetId() {
    lastWidgetId++;
    return "widget-" + lastWidgetId;
  }

  function getLastAddedWidget() {
    return customWidgets[customWidgets.length - 1];
  }

  $(window).on('beforeunload', function() {
    if (JSON.stringify(serializeTemplate()) !== JSON.stringify(lastSavedTmplt)) {
      return 'You have unsaved data!';
    }
  });

  function widgetIsEnabled(id) {
    return $('#' + id + '-enable').prop('checked');
  }

  $(document).ready(function() {
    $("#template-description").on('instanceReady.ckeditor', function() {
      descriptionInitialized = true;
    });

    $('form[data-toggle="validator2"]').submit(function(e) {
      e.preventDefault();
    });

    $(document).on('shown.bs.modal', '#map-editor', function(e) {
      if (!$(e.target).find('input[type="radio"]').is(':checked'))
        $(e.target).find('input[type="radio"]').first().prop('checked', true);
    });

    $(document).on('hidden.bs.modal', '#script-builder', validateScript);

    $(document).on('change', '#script-editable, #script-visible', validateScript);

    var customWidgetTypeChangeHandler = function() {
      $(this).closest('.draftWidget').find('.default-value').val('');
      renderSerializedCustomWidgets(serializeCustomWidgets());
      $('form[data-toggle="validator2"]').validator('validate');
    };

    var simpleWidget = Handlebars.compile($('#simple-widget').html());
    var openDirWidget = Handlebars.compile($('#open-dir-widget').html());
    var withoutDefaultWidget = Handlebars.compile($('#without-default-widget').html());
    var numCoresWidget = Handlebars.compile($('#num-cores-widget').html());
    var priority = Handlebars.compile($('#priority-widget').html());
    var scriptWidget = Handlebars.compile($('#script-widget').html());
    var jobArraysWidget = Handlebars.compile($('#jobArrays-widget').html());
    var yesNoWidget = Handlebars.compile($('#yesno-widget').html());
    var customWidget = Handlebars.compile($('#custom-widget').html());
    var customDatetimeWidget = Handlebars.compile($('#custom-datetime-widget').html());
    var sliderWidgetHtml = Handlebars.compile($('#slider-widget').html());
    var moabTemplateWidget = Handlebars.compile($('#moab-template-widget').html());
    var datetimeWidget = Handlebars.compile($('#datetime-widget').html());
    var oneColumnSection = Handlebars.compile($('#oneColumn-section').html());
    var twoColumnSection = Handlebars.compile($('#twoColumn-section').html());
    var genericResourcesWidget = Handlebars.compile($('#generic-resource-widget').html());
    var jobEnvVariablesWidget = Handlebars.compile($('#job-env-variables-widget').html());
    var featureTagsWidget = Handlebars.compile($('#feature-tags-widget').html());
    var mapWidget = Handlebars.compile($('#map-widget').html());
    var listWidget = Handlebars.compile($('#list-widget').html());
    var memoryWidget = Handlebars.compile($('#memory-widget').html());

    var defaultsSectionHtml = "";
    defaultsSectionHtml += simpleWidget({
      id: "name",
      label: "Name",
      variableName: "NAME",
      defaultValue: "",
      editable: true,
      visible: true,
      isJobName: true,
      enable: true
    });
    defaultsSectionHtml += jobArraysWidget({
      id: 'arrays',
      label: 'Job Arrays',
      minVariableName: 'STARTINDEX',
      minDefaultValue: '0',
      maxVariableName: 'ENDINDEX',
      maxDefaultValue: '1',
      editable: false,
      visible: false,
      enable: false
    });
    defaultsSectionHtml += scriptWidget({
      id: "script",
      label: "Submission Script",
      variableName: "SCRIPT",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: true
    });

    $('#defaults-section').append(defaultsSectionHtml);

    $('#show-script-builder-btn').click(function() {
      var payload = {
        text: $('#script-defaultValue').val(),
        tags: getAvaiableTags(),
        envVariables: getEnvVariables(),
        moabEnvVariables: JSON.parse($('#moabenvironmentvariables-defaultValue').val()) &&
              widgetIsEnabled('moabenvironmentvariables'),
        moabArrEnvVariables: $('#arrays-enable').prop('checked') && widgetIsEnabled('arrays')
      };

      $.viewpoint.modal.show('/script-builder', payload, {keyboard: false, backdrop: 'static'})
          .done(function(payload) {
            $('#script-defaultValue').val(payload);
          });

      function getEnvVariables() {
        if (!widgetIsEnabled('jobEnvVariables'))
          return [];

        var variables = JSON.parse($('#jobEnvVariables-defaultValue').val() || null);
        var result = [];

        if (variables) {
          $.each(variables, function(key, value) {
            result.push(key);
          });
        }

        return result;
      }

      function getAvaiableTags() {
        var result = [];
        $(serializeTemplate().widgets).each(function(index, widget) {
          var widgetId = getProperty(widget, 'id');
          var tag = {};
          var variable = getProperty(widget, 'variable_name');
          var tooltip = getProperty(widget, 'tooltip');
          // TODO add a check for compliance with the rules for naming variables instead of /\w+/
          if (/\w+/.test(variable)) {
            tag.variable = variable;
            tag.tooltip = tooltip;
            tag.enabled = true;
            if (widgetId === 'custom') {
              tag.isCustom = true;
              result.push(tag);
              return true;
            }
            if (widgetId === 'arrays') {
              result.push({
                variable: getProperty(widget, 'start-variable'),
                tooltip: VariablesTooltips.startindex,
                isCustom: false,
                enabled: widgetIsEnabled('arrays')
              });
              result.push({
                variable: getProperty(widget, 'end-variable'),
                tooltip: VariablesTooltips.endindex,
                isCustom: false,
                enabled: widgetIsEnabled('arrays')
              });
              return true;
            }
            if (widgetId === 'jobEnvVariables') {
              return true;
            }
            if (!widgetIsEnabled(widgetId)) {
              tag.enabled = false;
              tag.tooltip = 'Note: this widget is currently disabled. please check the "Enabled" box on it to make ' +
                  'the variable available to the script builder';
            }
            tag.isCustom = false;
            result.push(tag);
          }
        });
        return result;
      }
    });

    function loadDataFromServerToSelect(element, endpoint, defaultValue){
      var api = new Viewpoint.API();
      api.get('/api/' + endpoint + '/?page_size=9999').done(function(data, textStatus, jqXHR) {
        var callback = function(i, e) {
          return $('<option/>').text(e);
        };
        var options = $(data.results).sort().map(callback).get();
        $(element).append(options);
        if (defaultValue) {
          $(element).val(defaultValue);
        }
      });
    }

    var timeManagementSectionHtml = "";
    timeManagementSectionHtml += datetimeWidget({
      id: "duration",
      label: "Duration",
      inputClass: 'duration-input',
      variableName: "DURATION",
      defaultValue: "",
      editable: true,
      visible: true,
      enable: true
    });
    timeManagementSectionHtml += datetimeWidget({
      id: "eligibledate",
      inputClass: 'duration-input',
      label: "Delay Start By",
      variableName: "ELIGIBLEDATE",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });

    $('#timeManagement-section').append(timeManagementSectionHtml);

    var credentialsLeftSectionHtml = "";

    credentialsLeftSectionHtml += withoutDefaultWidget({
      id: "account",
      label: "Account",
      variableName: "ACCOUNT",
      editable: true,
      visible: false,
      enable: false
    });

    credentialsLeftSectionHtml += withoutDefaultWidget({
      id: "destinationQueue",
      label: "Queue / Class",
      variableName: "DESTINATIONQUEUE",
      editable: true,
      visible: false,
      enable: false
    });

    $('#credentialsLeft-section').append(credentialsLeftSectionHtml);

    var credentialsRightSectionHtml = "";

    credentialsRightSectionHtml += withoutDefaultWidget({
      id: "qos",
      label: "Quality of Service",
      variableName: "QOS",
      editable: true,
      visible: false,
      enable: false
    });

    $('#credentialsRight-section').append(credentialsRightSectionHtml);

    var resourcesLeftSectionHtml = "";
    resourcesLeftSectionHtml += numCoresWidget({
      id: "cpupernode",
      label: "Number of Cores",
      variableName: "NUMOFCORES",
      editable: true,
      enable: true,
      totalamount: true,
      nodeswithcount: true,
      malleablecorecount: true,
      isTotalCores: true,
      isTotalNodes: true,
      isCoresPerNode: true,
      isMinimumCores: true,
      isMaximumCores: true
    });

    $('#resourcesLeft-section').append(resourcesLeftSectionHtml);

    var resourcesRightSectionHtml = "";
    resourcesRightSectionHtml += memoryWidget({
      id: "memorypernode",
      label: "Memory (GB)",
      variableName: "MEMORYAMOUNT",
      editable: true,
      enable: true,
      elements: [
        { groupId: 'totalmemory', groupLabel: 'Total Memory', visible: 'true', editable: 'true',
          options: [{label:'Default Value', value: '0.50', optionId: 'totalmemory'}]
        },
        { groupId: 'memorypercore', groupLabel: 'Memory per Core', visible: 'true', editable: 'true',
          options: [{label:'Default Value', value: '0.50', optionId: 'memorypercore'}]
        },
      ]
    });

    resourcesRightSectionHtml += listWidget({
      id: "architecture",
      label: "Architecture",
      variableName: "ARCHITECTURE",
      defaultValue: "",
      editable: true,
      visible: true,
      enable: true
    });

    $('#resourcesRight-section').append(resourcesRightSectionHtml);

    var dataManagementSectionHtml = "";
    dataManagementSectionHtml += openDirWidget({
      id: "exedir",
      label: "Execution Path",
      variableName: "EXECUTIONDIRECTORY",
      defaultValue: "",
      editable: true,
      visible: false,
      enable: false
    });
    dataManagementSectionHtml += openDirWidget({
      id: "outputpath",
      label: "Output Path",
      variableName: "OUTPUTPATH",
      defaultValue: "",
      editable: true,
      visible: false,
      enable: false,
      useExecutionPath: true
    });
    dataManagementSectionHtml += openDirWidget({
      id: "errorpath",
      label: "Error Path",
      variableName: "ERRORPATH",
      defaultValue: "",
      editable: true,
      visible: false,
      enable: false,
      useExecutionPath: true
    });

    $('#dataManagement-section').append(dataManagementSectionHtml);

    var otherLeftSectionHtml = "";
    otherLeftSectionHtml += yesNoWidget({
      id: "join",
      label: "Merge Streams",
      variableName: "JOIN",
      editable: false,
      visible: false,
      enable: false,
      selectedValue: "no"
    });
    otherLeftSectionHtml += yesNoWidget({
      id: "hold",
      label: "Hold Job",
      variableName: "HOLD",
      editable: false,
      visible: false,
      enable: false,
      selectedValue: "no"
    });
    otherLeftSectionHtml += simpleWidget({
      id: "shell",
      label: "Script Shell",
      variableName: "SHELL",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false,
      isShell: true,
    });
    otherLeftSectionHtml += genericResourcesWidget({
      id: "genericresources",
      label: "Generic Resources",
      variableName: "GENERICRESOURCES",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });
    otherLeftSectionHtml += jobEnvVariablesWidget({
      id: "jobEnvVariables",
      variableName: "Job Environment Variables",
      label: "Job Env Variables",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });
    otherLeftSectionHtml += listWidget({
      id: "operatingSystem",
      label: "Operating System",
      variableName: "OPERATINGSYSTEM",
      defaultValue: "FIRST",
      editable: false,
      visible: false,
      enable: false
    });

    $('#otherLeft-section').append(otherLeftSectionHtml);

    $('#open-generic-resource').click(function() {
      var payload = getPayload($('#genericresources-defaultValue').val());
      $.viewpoint.modal.show('/generic-resources', payload)
        .done(function(payload) {
          renderGresTable(payload);
        });
    });

    $('#open-job-env-variables-editor').click(function() {
      $('form[data-toggle="validator2"]').validator('destroy');
      var widgetDefaultValue = $('#jobEnvVariables-defaultValue').val() || null;
      $.viewpoint.modal.show('/map-editor', {
        data: JSON.parse(widgetDefaultValue),
        validator: {
          div: '.variables-list',   // container which will be use validator
          widgetName: 'jobEnvVariables',
          inputs: validatorConfig.selector,  // selector of input fields which need to validate
          // this function prepares the input fields. The implication is that the inputs are inside "wrapper"
          prepareInputs : function(wrapper) {
            wrapper.find('.variable-key')
              .attr('data-unique-variable', true)
              .attr('data-variable-name', true)
              .attr('data-required-field', true)
              .addClass('var-name');
            wrapper.find('.variable-value')
              .attr('data-required-field', true);
          }
        }
      })
      .done(function(payload) {
        // Close all validation messages which can leave after dialog closed
        $('.popover').remove();

        $('#jobEnvVariables-defaultValue').val(JSON.stringify(payload.map));
        if (payload.map && payload.defaultValue) {
          renderMapTable($('#job-env-variables-table').find('tbody'), payload);
        }
        $('form[data-toggle="validator2"]').validator(validatorConfig);
      });
    });

    function getPayload(gresStr) {
      var payload = [];
      var genResArr = gresStr.split('\n');
      $.each(genResArr, function(i, val) {
        var genResource = {};
        var res = val.split(':');
        if (res[0] && res[1] && res[2]) {
          genResource.name = res[0];
          genResource.count = res[1];
          genResource.type = res[2];
          payload[i] = genResource;
        }
      });
      return payload;
    }

    function renderGresTable(data) {
      $('#gres-table tbody').empty();
      if (data && data.length) {
        var resources = '';
        $.each(data, function(i, row) {
          resources += row.name + ':' + row.count + ':' + row.type + '\n';
          $('#gres-table tbody')
            .append(
              $('<tr></tr>')
                .append(
                  $('<td></td>').text(row.name)
                )
                .append(
                  $('<td></td>').text(row.count)
                )
                .append(
                  $('<td></td>').text(row.type == 'job' ? 'Per Job' : 'Per Core')
                )
            );
        });
        $('#genericresources-defaultValue').val(resources);
      } else {
        $('#genericresources-defaultValue').val('');
        $('#gres-table tbody')
          .append(
            $('<tr></tr>')
              .append(
                $('<td></td>')
                  .addClass('table-no-data')
                  .attr('colspan', '3')
                  .text('No required generic resources')
              )
          );
      }
    }

    function renderMapTable($tbody, data) {
      var isEnvVars = ($tbody.selector.indexOf('job-env-variables-table') > -1);
      $tbody.empty();
      var map;
      var defaultValue;
      if (data) {
        map = data.map || null;
        defaultValue = data.defaultValue || [];
      } else {
        map = null;
      }

      if (!$.isEmptyObject(map)) {
        $.each(map, function(key, value) {
          var $isDefault = $.inArray(key, defaultValue) != -1 ?
          ($('<span></span>')
            .addClass('glyphicon glyphicon-ok')
            .addClass('pull-right')) :
          '';

          var keyTd = $('<td></td>').text(key);
          var valTd = $('<td></td>').text(value).append($isDefault);

          // This changing of order made cause order differs in JobEnv Variables and Custom-widgets
          if (isEnvVars) {
            $tbody
              .append(
                $('<tr></tr>')
                  .append(keyTd)
                  .append(valTd)
            );
          } else {
            $tbody
              .append(
                $('<tr></tr>')
                  .append(valTd)
                  .append(keyTd)
            );
          }
        });
      } else {
        $tbody
          .append(
            $('<tr></tr>')
              .append(
                $('<td></td>')
                  .addClass('table-no-data')
                  .attr('colspan', '3')
                  .text('No environment variables set')
              )
          );
      }
    }

    var otherRightSectionHtml = "";

    otherRightSectionHtml += yesNoWidget({
      id: "moabenvironmentvariables",
      label: "Moab Environment Variables",
      variableName: "MOABENVIRONMENTVARIABLES",
      editable: false,
      visible: false,
      enable: true,
      selectedValue: "yes"
    });
    otherRightSectionHtml += withoutDefaultWidget({
      id: "emailoptions",
      label: "Email Options",
      variableName: "EMAILOPTIONS",
      editable: false,
      visible: false,
      enable: false
    });
    otherRightSectionHtml += withoutDefaultWidget({
      id: "maillist",
      label: "Email Notification List",
      variableName: "MAILLIST",
      editable: false,
      visible: false,
      enable: false
    });
    otherRightSectionHtml += priority({
      id: "priority",
      label: "User Priority",
      variableName: "USERPRIORITY",
      defaultValue: "0",
      editable: false,
      visible: false,
      enable: false
    });
    otherRightSectionHtml += featureTagsWidget({
      id: "features-requested",
      type: "featuretags",
      label: "Requested Features",
      variableName: "FEATURESREQUESTED",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });
    otherRightSectionHtml += featureTagsWidget({
      id: "features-excluded",
      label: "Excluded Features",
      variableName: "FEATURESEXCLUDED",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });
    otherRightSectionHtml += moabTemplateWidget({
      id: "moabtemplates",
      label: "Moab Job Template",
      variableName: "MOABTEMPLATE",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });

    $('#otherRight-section').append(otherRightSectionHtml);

    var nodePoliciesLeftSectionHtml = "";

    nodePoliciesLeftSectionHtml += listWidget({
      id: 'nodes-requested-policy',
      label: 'Node Requested Policy',
      variableName: 'NODESREQUESTEDPOLICY',
      defaultValue: 'EXACTSET',
      editable: false,
      visible: false,
      enable: false,
      title: 'Node default policy',
      elements: [
        {label: 'Exact Set', value: 'EXACTSET', title: 'An exact set policy means all the nodes in the specified ' +
            'node list must be selected for the job'},
        {label: 'Superset', value: 'SUPERSET', title: 'A superset policy means the node list is the only ' +
            'source of nodes that should be considered for running the job. If the job can\'t find the necessary ' +
            'resources in the superset node list it should not run. No other nodes should be considered in ' +
            'allocating the job'},
        {label: 'Subset', value: 'SUBSET', title: 'A subset policy means the specified node list is used first to ' +
            'select nodes for the job. If the job requires more nodes than are in the subset node list, they will be ' +
            'obtained from elsewhere if possible. If the job does not require all of the nodes in the subset node ' +
            'list, it will use only the ones it needs.'},
      ]
    });

    nodePoliciesLeftSectionHtml += featureTagsWidget({
      id: "nodes-requested",
      label: "Nodes Requested List",
      variableName: "NODESREQUESTED",
      defaultValue: "",
      editable: false,
      visible: false,
      enable: false
    });

    $('#node-policy-left-section').append(nodePoliciesLeftSectionHtml);

    var nodePoliciesRightSectionHtml = "";

    nodePoliciesRightSectionHtml += listWidget({
      id: 'node-access-policy',
      label: 'Node Access Policy',
      variableName: 'NODEACCESSPOLICY',
      defaultValue: 'SHARED',
      editable: false,
      visible: false,
      enable: false,
      title: 'Node access policy',
      elements: [
        {label: 'Shared', value: 'SHARED', title: 'Tasks from any combination of jobs may use available resources'},
        {label: 'Single Job', value: 'SINGLEJOB', title: 'Only tasks from a single job may use the node\'s resources'},
        {label: 'Single Task', value: 'SINGLETASK', title: 'Only a single task from a single job may run on the node'},
        {label: 'Single User', value: 'SINGLEUSER', title: 'Tasks from any jobs owned by the same user may use available resources'},
        {label: 'Unique User', value: 'UNIQUEUSER', title: 'Any number of tasks from a single job may allocate resources ' +
       'from a node but only if the user has no other jobs running on that node. Unique user limits the number of jobs ' +
        'a single user can run on a node, allowing other users to run jobs with the remaining resources'}
      ]
    });

    nodePoliciesRightSectionHtml += listWidget({
      id: 'node-allocation-policy',
      label: 'Node Allocation Policy',
      variableName: 'NODEALLOCATIONPOLICY',
      defaultValue: 'FIRSTAVAILABLE',
      editable: false,
      visible: false,
      enable: false,
      title: 'Node allocation policy',
      elements: [
        {label: 'First Available', value: 'FIRSTAVAILABLE', title: 'Simple first come, first served algorithm ' +
        'where nodes are allocated in the order they are presented by the resource manager. ' +
        'This is a very simple, and very fast algorithm'},
        {label: 'Last Available', value: 'LASTAVAILABLE', title: 'Nodes are allocated in descending order that ' +
        'they are presented by the resource manager, or the reverse of First Available policy'},
        {label: 'Minimum Configured Resources', value: 'MINRESOURCE', title: 'Prioritizes nodes according to ' +
        'the configured memory resources on each node. Those nodes with the fewest configured memory ' +
        'resources, that still meet the job\'s resource constraints, are selected'},
        {label: 'CPU Load', value: 'CPULOAD', title: 'Nodes are selected that have the maximum amount of available, ' +
        'unused CPU power (<#of CPU\'s> - <CPU load>). CPU Load is a good algorithm for timesharing node systems ' +
        'and applies to jobs starting immediately. For the purpose of future reservations, ' +
        'the Minimum Configured Resources algorithm is used'},
        {label: 'Contiguous', value: 'CONTIGUOUS', title: 'Allocates nodes in contiguous (linear) blocks as ' +
        'required by the Compaq RMS system'},
        {label: 'Maximum Balance', value: 'MAXBALANCE', title: 'Attempts to allocate the most balanced set of ' +
        'nodes possible to a job. In most cases, but not all, the metric for balance of the nodes is node ' +
        'procspeed. Thus, if possible, nodes with identical procspeeds are allocated to the job. If identical ' +
        'procspeed nodes cannot be found, the algorithm allocates the set of nodes with the minimum node ' +
        'procspeed span or range'}
      ]
    });

    $('#node-policy-right-section').append(nodePoliciesRightSectionHtml);


    var dynamicSection = $("#dynamic-section");

    // Activate datepickers
    $('.date-input').datetimepicker({format: 'YYYY-MM-DD hh:mm'});
    $('.duration-input').durationpicker({
      readonly: true
    });
    var durationChanged = function(evt) {
      var duration = $(evt.target).durationpicker('parseDuration');
      $(evt.target).parent().tooltip({
        placement: 'bottom'
      });
      if (duration) {
        $(evt.target).parent()
          .attr('data-original-title', getFriendlyDuration(duration))
          .tooltip('fixTitle');
      } else {
        $(evt.target).parent()
          .attr('data-original-title', '')
          .tooltip('fixTitle');
      }
    };
    $('.duration-input').on('hide.durationpicker', durationChanged);
    $('.duration-input').change(durationChanged);

    $('.duration-input, .time-input').parent()
      .find('.glyphicon-calendar')
      .removeClass('glyphicon-calendar')
      .addClass('glyphicon-time');

    loadTemplateOrCreateNew();

    $("#addButton-left").click(function() {
      var id = generateWidgetId();
      renderCustomWidget(new CustomWidgetConfig(
          "left",
          "textbox",
          "",
          "",
          "",
          true,
          true,
          true
      ), id);
      customWidgets.push(new CustomWidget(id, "left"));
    });

    $("#addButton-right").click(function() {
      var id = generateWidgetId();
      renderCustomWidget(new CustomWidgetConfig(
          "right",
          "textbox",
          "",
          "",
          "",
          true,
          true,
          true
      ), id);
      customWidgets.push(new CustomWidget(id, "right"));
    });

    $("#addButton-center").click(function() {
      var id = generateWidgetId();
      renderCustomWidget(new CustomWidgetConfig(
          "center",
          "textbox",
          "",
          "",
          "",
          true,
          true,
          true
      ), id);
      customWidgets.push(new CustomWidget(id, "center"));
    });

    var btnClicked = $('#saveTmpltButton');

    $('#saveTmpltButton').click(function(e) {
      btnClicked = $(this);
      $('form[data-toggle="validator2"]').validator('validateAll');
    });

    $('#saveAndCloseTmpltButton').click(function(e) {
      btnClicked = $(this);
      $('form[data-toggle="validator2"]').validator('validateAll');
    });

    // run validator if user change any value of slider widget
    // all fields is related to each other in the slider widget
    $('body').on('change', '.slider-values input', function() {
      $('form[data-toggle="validator2"]').validator('validate');
    });

    var checkAdditionalFields = function(e) {
      var elements = [];
      // FIXME: refactor this
      if (e.target.id == 'cpupernode-visible-totalamount') {
        e.target = $('#cpupernode-totalcores');
        elements.push(e);
      }
      if (e.target.id == 'cpupernode-visible-nodeswithcount') {
        var e1 = jQuery.extend(true, {}, e);
        e.target = $('#cpupernode-totalnodes');
        elements.push(e);
        e1.target = $('#cpupernode-corespernode');
        elements.push(e1);
      }
      return elements;
    };

    var validatorConfig = {
      api: $('.buttons-container').data('api'),
      selector: '.var-name, #name-defaultValue, #cpupernode-visible-totalamount, #cpupernode-corespernode,' +
          '#cpupernode-totalcores, #cpupernode-totalnodes, #cpupernode-visible-nodeswithcount, #shell-defaultValue,' +
          '[data-api-label="Label"], .default-value, .variable-value, .slider-values input, #arrays-start, #arrays-end',
      checkAdditionalFields: checkAdditionalFields,
      showPopover: true,
      submitButton: $('#saveTmpltButton, #saveAndCloseTmpltButton'),
    };

    $('form[data-toggle="validator2"]').validator(validatorConfig);

    $('body').on('validationCompleted', 'form[data-toggle="validator2"]', function(e) {
      if (btnClicked.val() == 'Save template') {
        saveTemplate.call(this, function(data) {
          templateUrl = data.url;
          if (data.history_url) {
            $('.show-history-btn').show();
            templateHistoryUrl = data.history_url;
          } else {
            $('.show-history-btn').hide();
            templateHistoryUrl = null;
          }
          lastSavedTmplt = serializeTemplate();
          window.history.pushState(lastSavedTmplt, '', '/job_templates/template_builder?url=' + data.url);
          $('#exportTmpltLink').attr('href', templateUrl + '?format=template');
          $('#version-alert').hide();
          $.notify('Template has been saved', {
            type: 'success',
            delay: 3000
          });
        });
      } else if (btnClicked.val() == 'Save and Close') {
        saveTemplate.call(this, function(data) {
          templateUrl = data.url;
          lastSavedTmplt = serializeTemplate();
          window.history.pushState(lastSavedTmplt, '', 'job_templates/template_builder?url=' + data.url);
          $('#version-alert').hide();
          $.notify('Template has been saved', {
            type: 'success',
            delay: 3000,
            onClose: function () {
              window.location.href = '/templates';
            }
          });
        });
      }
    });

    $(window).scroll(function(){
      $('.buttons-container').css('top', '');
      if ($('.buttons-container').offset().top >$('footer').offset().top - $('.buttons-container').height()) {
        $('.buttons-container').offset({top:$('footer').offset().top - $('.buttons-container').height() - 10});
      }
    });

    $("#exportTmpltButton").click(function() {
      var message = 'You have unsaved data! You must save this template before continuing.';
      if (Object.keys(lastSavedTmplt).length === 0) {
        message = 'Nothing to export. This template has not been saved yet.';
      }

      if (JSON.stringify(serializeTemplate()) !== JSON.stringify(lastSavedTmplt)) {
        $.notify(message, {
          type: 'warning',
          delay: 3000
        });
        return;
      }
      var api = $('#builder-body').data('api');
      api.head($('#exportTmpltLink').attr('href'))
        .done(function(){
          $('#exportTmpltLink')[0].click();
        })
        .fail(function(jqXHR){
          if (jqXHR.status === 404) {
            $.notify('We detected the unsynchronized content, please try to refresh the page',
              { type: 'warning' });
          } else {
            $.notify(jqXHR.statusText,
              { type: 'warning' });
          }
        });
    });

    $(document).on('click', '.date .glyphicon, .date .glyphicon',
        function(event) {
          var container = $(event.target).closest('.date')[0];
          if ($(container).find('input').data('DateTimePicker')) {
            $(container).find('input').data('DateTimePicker').show();
          }
        });

    $('body').on('paste', '#name-defaultValue', function(e) {
      var element = this;
      setTimeout(function () {
        var text = $(element).val().replace(/\s/g, "_");
        $(element).val(text);
        $('form[data-toggle="validator2"]').validator('validate');
      }, 50);
    });

    $('body').on('keypress', '#name-defaultValue', function(event) {
      if (event.which === 32) {
        var $input = $('#name-defaultValue');
        $input.val($input.val() + '_');
        return false;
      }
    });

    var stopPropagationTriggers = ['#description-section-name', '#basic-section-name', '#advanced-section-name',
     '#custom-section-name', '.description-section-visible'];

    (function() {
      for (var i = 0; i < stopPropagationTriggers.length; i++) {
        $(stopPropagationTriggers[i]).click(function(event){
          event.stopPropagation();
        });
      }
    })();

    $(".custom-widget-type").change(customWidgetTypeChangeHandler);

    $("#template-name").click(function() {
      $("#template-name-input").val($("#template-name").text());
      $('#rename-modal').modal('show');
      $('#template-name').removeClass('error');
    });

    $(document).click(function (e) {
      if (typeof $(e.target).data('original-title') == 'undefined' &&
       !$(e.target).parents().is('.popover.in') && !$(e.target).is('.glyphicon-cog')) {
       if ($('.popover').hasClass('in')) {
          $('[data-toggle="touchspin-settings"]').popover('hide');
       }
      }
    });

    $(document).on('shown.bs.popover', '.touchspin-settings', function(event){
      var id = $(this).closest('.input-group').find('input').attr('id');
      var popoverId = $(this).attr('aria-describedby');
      $('.popover:not([id="' + popoverId + '"])').popover('hide');
      $('.popover[id="' + popoverId + '"]').data('input-id', id);
      $('.popover[id="' + popoverId + '"] .min').val($('#' + id).data('min')).floatInput();
      $('.popover[id="' + popoverId + '"] .max').val($('#' + id).data('max')).floatInput();
      $('.popover[id="' + popoverId + '"] .step').val($('#' + id).data('step')).floatInput();
    });

    $(document).on('click', '.touchspin-settings-apply', function(){
      var $popover = $(this).closest('.popover'),
          $btnGroup = $(this).closest('.form-group'),
          id = $popover.data('inputId'),
          $input = $('#' + id),
          value = $input.val(),
          $min = $popover.find('.min'),
          min = $min.val(),
          $max = $popover.find('.max'),
          max = $max.val(),
          $step = $popover.find('.step'),
          step = $step.val(),
          $errorMessage = $popover.find('.error-message'),
          $minCores = $('#cpupernode-minimumcores'),
          $maxCores = $('#cpupernode-maximumcores'),
          addError = function($el) {
            $el.closest('.form-group').addClass('has-error');
            $btnGroup.addClass('has-error');
            $popover.find('.help-block').show();
          },
          removeError = function($el) {
            $el.closest('.form-group').removeClass('has-error');
          };

      if (step.length && step != 0) {
        removeError($step);
      } else {
        $errorMessage.text("Step value cannot be empty.");
        if (step == 0) {
          $errorMessage.text("Step value cannot be null.");
        }
        addError($step);
        return;
      }

      if ((min.length && min != 0) && (max.length && max != 0)) {
        if ((max - min) % step == 0) {
          removeError($max);
        } else {
          $errorMessage.text("(max - min) should be divisible by step value.");
          addError($max);
          return;
        }
        if (parseFloat(min) < parseFloat(max)) {
          removeError($min);
        } else {
          $errorMessage.text("Min value cannot be greater than max value.");
          addError($min);
          return;
        }
      } else {
        removeError($min);
        removeError($max);
        if (min == 0) {
          $errorMessage.text("Min value cannot be null.");
          addError($min);
        }
        if (!min.length) {
          $errorMessage.text("Min value cannot be empty.");
          addError($min);
        }
        if (max == 0) {
          $errorMessage.text("Max value cannot be null.");
          addError($max);
        }
        if (!max.length) {
          $errorMessage.text("Max value cannot be empty.");
          addError($max);
        }
        return;
      }

      $('#' + id).data('min', min);
      $('#' + id).data('max', max);
      $('#' + id).data('step', step);

      if (parseFloat($maxCores.data('min')) <= parseFloat($minCores.data('min'))) {
        $maxCores.data('min', parseFloat($minCores.data('min')) + parseFloat($maxCores.data('step')));
      }

      if (parseFloat($minCores.data('max')) >= parseFloat($maxCores.data('max'))) {
        $minCores.data('max', parseFloat($maxCores.data('max')) - parseFloat($minCores.data('step')));
      }

      $(this).closest('.popover').popover('hide');

      $input.trigger("touchspin.updatesettings", {
        max: $input.data('max'),
        min: $input.data('min'),
        step: $input.data('step')
      });

      if ((max - value) % step != 0) {
        if (parseFloat(value) < parseFloat(max) && parseFloat(value) > parseFloat(min)) {
          var stepCount = Math.round((value - min) / step);
          $input.val(parseFloat(min) + parseFloat(step * stepCount));
        }
      }

    });

    $('body').on('click', '.show-history-btn', function(e) {
      var payload = {
        url: templateHistoryUrl,
        version: templateVersion,
      };

      $.viewpoint.modal.show('/template-history', payload)
        .done(function(payload) {

        });
    });

    $('#rename-modal').on('shown.bs.modal', function() {
      $("#template-name-input").select().focus();
    });

    $("#save-template-name").click(function() {
      $("#rename-modal").find("form").submit();
    });

    $("#rename-modal").find("form").submit(function(event) {
      event.preventDefault();

      if ($('#template-name-input').val().trim() === '') {
        $.notify('Template name may not be blank', { type: 'warning' });
        return;
      }

      $("#template-name").text($("#template-name-input").val());
      $('#rename-modal').modal('hide');
    });

    //Functions

    function addRequirementsForTouchspin() {
      $('#memorypernode-totalmemory-defaultValue, #memorypernode-memorypercore-defaultValue').attr({
        'data-min': 0.25,
        'data-max': 65535,
        'data-step': 0.25
      });
      $('#cpupernode-totalcores, #cpupernode-totalnodes, #cpupernode-corespernode').attr({
        'data-min': 1,
        'data-max': 65535,
        'data-step': 1
      });
      $('#cpupernode-minimumcores, #cpupernode-maximumcores').attr({
        'data-max': 65535,
        'data-step': 1
      });
      $('#cpupernode-minimumcores').data('min', 1);
      $('#cpupernode-maximumcores').data('min', 2);
    }

    function addTouchSpin() {
      $('#memorypernode-totalmemory-defaultValue, #memorypernode-memorypercore-defaultValue')
        .TouchSpin({
          verticalbuttons: true,
          step: $(this).data('step'),
          decimals: 2,
          min: $(this).data('min'),
          max: $(this).data('max'),
          postfix: '<span class="glyphicon glyphicon-cog"></span>',
          postfix_extraclass: "btn btn-default touchspin-settings",
          forcestepdivisibility : 'none'
        })
        .change(function(el){
          if ($(el.target).val() == '') {
            $(el.target).val('0.25');
          }
          roundValueForTouchspin(this);
        });

      var spinnerChangeHandler = function(el) {
        if ($(el.target).val() == '')
          $(el.target).val('0');
        roundValueForTouchspin(this);
      }

      var roundValueForTouchspin = function(el) {
        var max = $(el).data('max'),
            min = $(el).data('min'),
            step = $(el).data('step'),
            value = $(el).val();

        if ((max - value) % step != 0) {
          if (parseFloat(value) < parseFloat(max) && parseFloat(value) > parseFloat(min)) {
            var stepCount = Math.round((value - min) / step);
            $(el).val(parseFloat(min) + parseFloat(step * stepCount));
          }
        }
      }

      var minAndMaxSpinnerChangeHandler = function(el) {
        roundValueForTouchspin(this);
        var $minCores = $('#cpupernode-minimumcores'),
            $maxCores = $('#cpupernode-maximumcores');

        if ($(this).val() == '') {
          if ($(this).attr('id') == 'cpupernode-minimumcores') {
            $(this).val('1');
          } else {
            $(this).val('2');
          }
        } else {
          if ($(this).attr('id') == 'cpupernode-maximumcores' && parseInt($(this).val()) < 2) {
            $(this).val('2');
          }
          if ($(this).attr('id') == 'cpupernode-minimumcores' && parseInt($(this).val()) < 1) {
            $(this).val('1');
          }
        }
        if (parseInt($minCores.val()) >= parseInt($maxCores.val())) {
          if ($(this).attr('id') == 'cpupernode-minimumcores') {
            $maxCores.val(parseInt($minCores.val()) + parseInt($maxCores.data('step')));
          } else {
            $minCores.val(parseInt($maxCores.val()) - parseInt($minCores.data('step')));
          }
          $('#cpupernode-maximumcores, #cpupernode-minimumcores').trigger("touchspin.updatesettings");
        }
        if (parseInt($maxCores.val()) <= parseInt($minCores.val())) {
          if ($(this).attr('id') == 'cpupernode-minimumcores') {
            $maxCores.val(parseInt($minCores.val()) + parseInt($maxCores.data('step')));
          } else {
            $minCores.val(parseInt($maxCores.val()) - parseInt($minCores.data('step')));
          }
          $('#cpupernode-maximumcores, #cpupernode-minimumcores').trigger("touchspin.updatesettings");
        }
      }

      $("#priority-defaultValue").TouchSpin({
        verticalbuttons: true,
        min: -1024,
        max: 0
      }).change(spinnerChangeHandler);

      $('#cpupernode-totalcores, #cpupernode-totalnodes, #cpupernode-corespernode').TouchSpin({
        verticalbuttons: true,
        min: $(this).data('min'),
        max: $(this).data('max'),
        step: $(this).data('step'),
        postfix: '<span class="glyphicon glyphicon-cog"></span>',
        postfix_extraclass: "btn btn-default touchspin-settings",
        forcestepdivisibility : 'none'
      }).change(spinnerChangeHandler);

      $('#cpupernode-minimumcores, #cpupernode-maximumcores').TouchSpin({
        verticalbuttons: true,
        min: $(this).data('min'),
        max: $(this).data('max'),
        step: $(this).data('step'),
        postfix: '<span class="glyphicon glyphicon-cog"></span>',
        postfix_extraclass: "btn btn-default touchspin-settings",
        forcestepdivisibility : 'none'
      }).change(minAndMaxSpinnerChangeHandler);

      $('.touchspin-settings').attr({
        'data-toggle': 'touchspin-settings',
      });

      $('[data-toggle="touchspin-settings"]').popover({
        html: true,
        container: '.touchspin-settings-container',
        placement: 'right',
        content: function() {
            return $('.touchspin-settings-popover').html();
          }
      });
    }

    function loadTemplateOrCreateNew() {
      // apiGetTemplate - api in different container with attribute data-show-error="false"
      // which disables standart Loader error messaging
      var apiGetTemplate = $('#template').data('api');
      addRequirementsForTouchspin();
      CKEDITOR.addCss('body { margin: 15px; }');
      CKEDITOR.addCss('.cke_editable h1 { margin: 8px 0px; font-weight: 700; }');
      CKEDITOR.addCss('.cke_editable p { margin-top: 0; margin-bottom: 10px; }');
      $('#template-description').ckeditor({
        toolbar: [
          {name: 'basicstyles', items: ['Bold', 'Italic', 'Strike', '-', 'RemoveFormat']},
          {name: 'styles', items: ['Styles', 'Format']},
          {name: 'paragraph', items: ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', '-', 'Blockquote']},
          {name: 'links', items: ['Link', 'Unlink']},
          {name: 'insert', items: ['Table', 'HorizontalRule', 'SpecialChar', 'Image']}
        ]
      });

      $("#arrays-start").TouchSpin({
        verticalbuttons: true,
        max: 10000000
      });
      $("#arrays-end").TouchSpin({
        verticalbuttons: true,
        max: 10000000
      });
      $('#arrays-enable').parent().hide();

      function switchOnOffArrayWidget(isEnabled) {
        var enableCheckbox = $('#arrays-enable');
        $('#arrays-visible').prop('disabled', !isEnabled);
        enableCheckbox.prop('checked', isEnabled);
        if (isEnabled) {
          enableCheckbox.closest('.draftWidget').show();
        } else {
          enableCheckbox.closest('.draftWidget').hide();
        }
      }
      switchOnOffArrayWidget(false);

      function formatJobType(jobType) {
        if (!jobType.id) {
          return jobType.text;
        }
        var typeIcon = window.getJobTypeIcon(jobType.id);
        return typeIcon + jobType.text;
      }

      $('#template-type').select2({
        width: '53%',
        minimumResultsForSearch: -1,
        escapeMarkup: function(m) {
          return m;
        },
        templateResult: formatJobType,
        templateSelection: formatJobType
      }).change(function() {
        switchOnOffArrayWidget(false);
        var val = $(this).val();

        switch (val) {
          case 'array': switchOnOffArrayWidget(true); break;
        }
        setTimeout(function() {
          $('form[data-toggle="validator2"]').validator('validate');
        }, 50);
      });

      if (urlHasTemplateUrl()) {
        templateUrl = getTemplateUrl();
        $('#exportTmpltLink').attr('href', templateUrl + '?format=template');

        $('#template > div > div > div > h1').text('Edit Application Template');
        apiGetTemplate.get(templateUrl)
          .done(function(data, textStatus, jqXHR) {
            if (data.owner != $.viewpoint.user.name &&
              !$.viewpoint.user.permissions.templates_admin) {
              $.notify('You do not have permissions to edit this template. You will be redirected to Dashboard in a moment',
                {
                  type: 'warning',
                  onClose: function() {
                    window.location.href = "/dashboard";
                  }
                });
              return;
            }

            renderTemplate(data);
            addTouchSpin();
            renderCPUDefaults();
            renderMemoryDefaults();
            lastSavedTmplt = serializeTemplate();
            $('#builder-body').show();
            activateTemplateType();
            activateMoabTemplatesWidgets();
            $.fn.addArrows('footer');
          })
          .fail(function(jqXHR, textStatus, errorThrown) {
            if (errorThrown == 'NOT FOUND') {
              //var tid = templateUrl.substring(templateUrl.lastIndexOf('/') + 1);
              $.notify('<p>Requested version of template outdated or not found.</p>',
                { delay: 0, type: 'warning' });
            } else {
              var message = textStatus || errorThrown;
              $.notify('<h1>Server said:</h1><p>' + message + '</p>',
                { delay: 0, type: 'warning' });
            }
            $.notify('You will be redirected to Dashboard in a moment', {
              type: 'info',
              onClose: function() {
                window.location.href = "/dashboard";
              }
            });
          });
      } else {
        $('button.show-history-btn').hide();
        $('#template > div > div > div > h1').text('Create Application Template');
        activateFeaturesWidgets();
        activateNodeRequestedWidget();
        var widgets = serializeTemplate().widgets;
        $.each(widgets, function(i, widget) {
          addVisibleEditableLogic(getProperty(widget, 'id'));
        });

        $("#template-name").text("Untitled Application Template");
        activatePermissionWidgets();
        renderGresTable();
        renderCPUDefaults();
        renderMemoryDefaults();
        $("#cpupernode-visible-default-totalamount").prop('checked', true);
        $("#memorypernode-visible-default-memorypercore").prop('checked', true);
        renderMapTable($('#job-env-variables-table').find('tbody'));
        $('#builder-body').show();
        $.fn.addArrows('footer');
        $('#cpupernode-totalcores').val(1);
        $('#cpupernode-totalnodes').val(1);
        $('#cpupernode-corespernode').val(1);
        $('#cpupernode-minimumcores').val(1);
        $('#cpupernode-maximumcores').val(2);
        activateTemplateType();
        activateMoabTemplatesWidgets();
        loadDataFromServerToSelect('#architecture-defaultValue', 'architectures', false);
        loadDataFromServerToSelect('#operatingSystem-defaultValue', 'operating-systems', false);
        addTouchSpin();
      }
    }

    function renderCPUDefaults() {
      $("#cpupernode-visible-default-totalamount").click(function() {
        var checked = $(this).is(':checked');
        if (checked) {
          $("#cpupernode-visible-default-nodeswithcount").removeAttr('checked');
          $("#cpupernode-visible-default-malleablecorecount").removeAttr('checked');
        } else {
          $("#cpupernode-visible-default-nodeswithcount").prop('checked', true);
        }
      });
      $("#cpupernode-visible-default-nodeswithcount").click(function() {
        var checked = $(this).is(':checked');
        if (checked) {
          $("#cpupernode-visible-default-totalamount").removeAttr('checked');
          $("#cpupernode-visible-default-malleablecorecount").removeAttr('checked');
        } else {
          $("#cpupernode-visible-default-totalamount").prop('checked', true);
        }
      });
      $("#cpupernode-visible-default-malleablecorecount").click(function() {
        var checked = $(this).is(':checked');
        if (checked) {
          $("#cpupernode-visible-default-nodeswithcount").removeAttr('checked');
          $("#cpupernode-visible-default-totalamount").removeAttr('checked');
        } else {
          $("#cpupernode-visible-default-totalamount").prop('checked', true);
        }
      });
      var toggleDefaults = function() {
        if ($('#cpupernode-enable').is(':checked')) {
          var checked = $('#cpupernode-visible-totalamount').is(':checked') ||
          $("#cpupernode-visible-nodeswithcount").is(':checked') ||
          $("#cpupernode-visible-malleablecorecount").is(':checked');
          if (checked) {
            $("#cpupernode-visible-default-totalamount").parent().hide();
            $("#cpupernode-visible-default-nodeswithcount").parent().hide();
            $("#cpupernode-visible-default-malleablecorecount").parent().hide();
          } else {
            $("#cpupernode-visible-default-totalamount").parent().show();
            $("#cpupernode-visible-default-nodeswithcount").parent().show();
            $("#cpupernode-visible-default-malleablecorecount").parent().show();
          }
        }
      };
      $('#cpupernode-visible-totalamount, #cpupernode-visible-nodeswithcount, #cpupernode-visible-malleablecorecount').click(toggleDefaults);
      toggleDefaults();
    }

    function renderMemoryDefaults() {
      $("#memorypernode-visible-default-totalmemory").click(function() {
        var checked = $(this).is(':checked');
        if (checked) {
          $("#memorypernode-visible-default-memorypercore").removeAttr('checked');
        } else {
          $("#memorypernode-visible-default-memorypercore").prop('checked', true);
        }
      });
      $("#memorypernode-visible-default-memorypercore").click(function() {
        var checked = $(this).is(':checked');
        if (checked) {
          $("#memorypernode-visible-default-totalmemory").removeAttr('checked');
        } else {
          $("#memorypernode-visible-default-totalmemory").prop('checked', true);
        }
      });
      var toggleDefaults = function() {
        if ($('#memorypernode-enable').is(':checked')) {
          var checked = $('#memorypernode-visible-totalmemory').is(':checked') ||
          $("#memorypernode-visible-memorypercore").is(':checked');
          if (checked) {
            $("#memorypernode-visible-default-totalmemory").parent().hide();
            $("#memorypernode-visible-default-memorypercore").parent().hide();
          } else {
            $("#memorypernode-visible-default-totalmemory").parent().show();
            $("#memorypernode-visible-default-memorypercore").parent().show();
          }
        }
      };
      $('#memorypernode-visible-totalmemory, #memorypernode-visible-memorypercore').click(toggleDefaults);
      toggleDefaults();
    }

    function renderCustomWidget(widget, widgetId) {
      if (customWidgets.length === 0) {
        if (getProperty(widget, "position") === "center") {
          dynamicSection.append(oneColumnSection({id: lastSectionId}));
          $("#section-" + lastSectionId).append(getCustomWidgetHtml(widgetId, widget));
        } else {
          dynamicSection.append(twoColumnSection({id: lastSectionId}));
          $("#section-" + lastSectionId + "-" + getProperty(widget, "position")).append(getCustomWidgetHtml(widgetId, widget));
        }
      } else {
        if (((getProperty(widget, "position") === "left" || getProperty(widget, "position") === "right") && getLastAddedWidget().position === "center") || (getProperty(widget, "position") === "center" && getLastAddedWidget().position !== "center")) {
          generateSectionId();
          if (getProperty(widget, "position") === "center") {
            dynamicSection.append(oneColumnSection({id: lastSectionId}));
            $("#section-" + lastSectionId).append(getCustomWidgetHtml(widgetId, widget));
          } else {
            dynamicSection.append(twoColumnSection({id: lastSectionId}));
            $("#section-" + lastSectionId + "-" + getProperty(widget, "position")).append(getCustomWidgetHtml(widgetId, widget));
          }
        } else {
          if (getProperty(widget, "position") === "center") {
            $("#section-" + lastSectionId).append(getCustomWidgetHtml(widgetId, widget));
          } else {
            $("#section-" + lastSectionId + "-" + getProperty(widget, "position")).append(getCustomWidgetHtml(widgetId, widget));
          }
        }
      }

      var $widget = $('#' + widgetId);

      $widget.find(".custom-widget-type").change(customWidgetTypeChangeHandler);
      $widget.find(".custom-widget-type").click(function() {
        // Clear validation error-messages
        $('.popover').remove();
      });

      $widget.find('.date-input').datetimepicker({format: 'YYYY-MM-DD hh:mm'});

      if (getProperty(widget, 'type') === 'number')
        $widget.find('.default-value').TouchSpin({verticalbuttons: true, min: -2147483648, max: 2147483647 });

      $widget.find(".button-delete").click(function() {
        var id = $(this).attr('data-id');
        $("#" + id).remove();
        for (var i = 0; i < customWidgets.length; i++) {
          if (customWidgets[i].id === id) {
            customWidgets.splice(i, 1);
          }
        }
        renderSerializedCustomWidgets(serializeCustomWidgets());
        $('.alert').remove();
        $('.popover').remove();
        $('form[data-toggle="validator2"]').validator('validate');
      });

      $widget.find(".button-edit").click(function() {
        $('form[data-toggle="validator2"]').validator('destroy');
        var id = $(this).attr('data-id');
        var $defaultValue = $('#' + id + '-defaultValue');
        var type = $('#' + id + '-type').val();
        var widgetConfig = JSON.parse($defaultValue.val() || null);
        var dialogConfig = {
          withDefault: true,
          title: 'Widget Configuration',
          valueColumnName: 'Label (item text)',
          keyColumnName: 'Value (item value)',
          validator: {
            div: '.variables-list',   // container which will be use validator
            inputs: '.variable-key, .variable-value',  // selector of input fields which need to validate
            // this function prepares the input fields. The implication is that the inputs are inside "wrapper"
            prepareInputs : function(wrapper) {
              wrapper.find('.variable-key')
                .attr('data-unique-value-in-dialog', true)
                .attr('data-required-field', true);
              wrapper.find('.variable-value')
                .attr('data-required-field', true);
            }
          }
        };
        if (widgetConfig) {
          dialogConfig.data = widgetConfig.map;
          dialogConfig.defaultValue = widgetConfig.defaultValue;
        }
        if ($.inArray(type, ['checkbox', 'combolist']) != -1) {
          dialogConfig.multiple = true;
        }

        // Clear validation error-messages
        $('.popover').remove();

        $.viewpoint.modal.show('/map-editor', dialogConfig)
          .done(function(payload) {
            $defaultValue.val(JSON.stringify(payload));
            var $tbody = $('#' + id + '-table').find('tbody');
            renderMapTable($tbody, payload);
            $('form[data-toggle="validator2"]').validator(validatorConfig);
            $('form[data-toggle="validator2"]').validator('validate');
          });
      });

      function getCustomWidgetHtml(id, widgetConfig) {
        var templateConfig = createTemplateObject(id, widgetConfig);
        var widgetType = getProperty(widgetConfig, "type");

        switch(widgetType) {
          case "datetime": return customDatetimeWidget(templateConfig);
          case "radio": return getMapWidgetHtml(templateConfig);
          case "checkbox": return getMapWidgetHtml(templateConfig);
          case "list": return getMapWidgetHtml(templateConfig);
          case "combolist": return getMapWidgetHtml(templateConfig);
          case "slider": return getSliderWidget(templateConfig);
          case "remotefile": return getWidgetWithoutDefault(templateConfig);
          case "remotedirectory": return getWidgetWithoutDefault(templateConfig);
          case "file-upload": return getWidgetWithoutDefault(templateConfig);
          default: return customWidget(templateConfig);
        }
      }

      function getWidgetWithoutDefault(templateConfig) {
        templateConfig.hideDefault = true;
        templateConfig.visible = "checked";
        templateConfig.editable = "checked";
        return customWidget(templateConfig);
      }

      function getSliderWidget(templateConfig) {
        if (templateConfig.defaultValue) {
          var config = JSON.parse(templateConfig.defaultValue);
          templateConfig.min = config.min || '0';
          templateConfig.max = config.max || '1';
          templateConfig.step = config.step || '1';
          templateConfig.svalue = config.svalue || '0';
        }
        return sliderWidgetHtml(templateConfig);
      }

      function getMapWidgetHtml(templateConfig) {
        templateConfig.validateDefaultValue = true;
        if (templateConfig.type == 'radio') {
          templateConfig.required = '';
          templateConfig.requiredHidden = true;
        }
        var $template = $(mapWidget(templateConfig));

        var $tbody = $template.find('#' + templateConfig.id + '-table').find('tbody');
        var widgetConfig = JSON.parse(templateConfig.defaultValue || null);

        renderMapTable($tbody, widgetConfig);
        return $template;
      }

      function createTemplateObject(id, widgetConfig) {
        return {
          id: id,
          label: getProperty(widgetConfig, "label"),
          variableName: getProperty(widgetConfig, "variable_name"),
          defaultValue: getProperty(widgetConfig, "default_value"),
          visible: getProperty(widgetConfig, "visible") ? "checked" : "",
          editable: getProperty(widgetConfig, "editable") ? "checked" : "",
          required: 'true' == getProperty(widgetConfig, "required") ? "checked" : "",
          position: getProperty(widgetConfig, "position"),
          type: getProperty(widgetConfig, "type")
        };
      }

      addVisibleEditableLogic(widgetId);

      var type = $widget.find(".custom-widget-type").val();
      if (type == 'slider') {
        $widget.find('.minval').TouchSpin({verticalbuttons: true, min: 0, max: 1000000});
        $widget.find('.maxval').TouchSpin({verticalbuttons: true, min: 1, max: 1000000});
        $widget.find('.stepval').TouchSpin({verticalbuttons: true, min: 1, max: 1000000});
        $widget.find('.initialval').TouchSpin({
          verticalbuttons: true,
          min: 0, max: 1000000,
          forcestepdivisibility: 'floor',
          step: parseInt($widget.find('.stepval').val()),
        });

        $widget.find('.stepval, .maxval, .minval').on('input change', function() {
          var stepValue = parseInt($widget.find('.stepval').val());
          var minValue = parseInt($widget.find('.minval').val());
          var maxValue = parseInt($widget.find('.maxval').val());
          $widget.find('.initialval').trigger('touchspin.updatesettings',
              {min: minValue, max: maxValue, step: stepValue});
          $widget.find('.stepval').trigger('touchspin.updatesettings',
              {max: maxValue});
        });
      }
    }

    function renderSerializedCustomWidgets(widgets) {
      dynamicSection.empty();
      customWidgets = [];
      var sortedWidgets = _(widgets).sortBy(function(e) { return parseInt(getProperty(e, 'order')) || 0; }).value();
      $.each(sortedWidgets, function(index, element) {
        var id = generateWidgetId();
        renderCustomWidget(element, id);
        customWidgets.push(new CustomWidget(id, getProperty(element, "position")));
      });
    }


    function renderTemplate(config) {
      templateHistoryUrl = config.history_url;
      templateVersion = config.version;
      renderWidgets(config.widgets);
      activateFeaturesWidgets(config.widgets);
      activateNodeRequestedWidget(config.widgets);
      renderMetaInformation(config);
      renderSectionNames(config.sections);
      renderVersionAlert(config);
      activatePermissionWidgets(config.permissions);

      templateType = config.type;
      if ($('#template-type option').length > 0) {
        $('#template-type').select2('val', (config.type));
      }
      $('#version-description').val(config.version_description);
      if (!config.history_url) $('.show-history-btn').hide();

      // open the template history dialog if the user previously clicked restore in the dialog
      if ($.url("?showHistoryDialog") === 'true')
        $('.show-history-btn').trigger('click');

      function renderVersionAlert(template) {
        if (!template.current) {
          $('button.show-history-btn').hide();
          var api = new Viewpoint.API();
          api.get(template.history_url + '?current=True').done(function(currentTemplate) {
            $('#version-alert .alert')
              .append(
                $('<strong>Warning! </strong>')
              )
              .append(' You are viewing an old version of this template.')
              .append(
                $('<div></div>')
                  .addClass('row')
                  .append(
                    $('<div></div>')
                      .addClass('col-xs-3')
                      .append(
                        $('<a></a>')
                          .attr('href', '/job_templates/template_builder?url=' + currentTemplate.results[0].url)
                          .text('View Current Version')
                          .addClass('btn btn-default')
                      )
                  )
                  .append(
                    $('<div></div>')
                      .addClass('col-xs-3')
                      .append(
                        $('<a></a>')
                          .addClass('restore btn btn-default')
                          .attr('href', template.url)
                          .text('Restore this Version')
                          .click(function(e) {
                            e.preventDefault();
                            var url = $(this).attr('href');
                            $.viewpoint.confirm('Restore Operation', 'You are sure that you want to restore this ' +
                                'version of the template?')
                            .done(function(result) {
                              if (result) {
                                var api = $('#builder-body').data('api');
                                var serializedTemplate = serializeTemplate();
                                var message = 'Template has been restored';
                                if (JSON.stringify(serializedTemplate) !== JSON.stringify(lastSavedTmplt)) {
                                  // user has unsaved changes, we must to save them (IRIS-1218)
                                  message = 'Template has been restored from this version. Changes made while ' +
                                    'viewing this version have been stored in the restore';
                                  lastSavedTmplt = serializedTemplate;
                                  api.post(templateHistoryUrl, serializedTemplate).done(function(data) {
                                    $.notify(message, {
                                      type: 'success',
                                      delay: 3000,
                                      onClose: function () {
                                        window.location.href = '/job_templates/template_builder?url=' + data.url;
                                      }
                                    });
                                  });
                                } else {
                                  // in this case we do a copy of the template on the server
                                  api.put(url)
                                    .done(function(data) {
                                      $.notify(message, {
                                        type: 'success',
                                        delay: 3000,
                                        onClose: function () {
                                          window.location.href = '/job_templates/template_builder?url=' + data.url;
                                        }
                                      });
                                    });
                                }
                              }
                            });
                            return false;
                          })
                      )
                  )
                  .append(
                    $('<div></div>')
                      .addClass('col-xs-3')
                      .append(
                        $('<a></a>')
                          .addClass('show-history-btn btn btn-default')
                          .attr('href', '#')
                          .text('Show History')
                      )
                  )
              )
              .show();
            });
        }
      }

      function renderSectionNames(sections) {
        $(sections).each(function(i, e) {
          $('#' + e.name + '-section-name').val(e.value);
        });
      }

      function renderMetaInformation(config) {
        $("#template-name").text(config.name);

        if (config.description) {
          if (descriptionInitialized) {
            $("#template-description").val(config.description.text);
          } else {
            $("#template-description").on('instanceReady.ckeditor', function() {
              $(this).val(config.description.text);
            });
          }
          $('#description-section-visible').prop('checked', config.description.visible);
        }

        if (config.published) {
          $('input[name=publish]').filter('[value=published]').prop('checked', true);
        } else {
          $('input[name=publish]').filter('[value=unpublished]').prop('checked', true);
        }
      }

      function renderWidgets(widgets) {
        var customWidgets = [];
        $.each(widgets, function(i, e) {
          if (getProperty(e, "id") === "custom") {
            customWidgets.push(e);
          } else {
            renderFreeFormWidget(e);
          }
        });
        renderSerializedCustomWidgets(customWidgets);
      }

      function renderFreeFormWidget(widget) {
        var id = getProperty(widget, "id");
        var defaultValue = getProperty(widget, "default_value");
        var label = getProperty(widget, "label");
        var visible = getProperty(widget, "visible");
        var editable = getProperty(widget, "editable");
        var enable = getProperty(widget, "enable") || String(visible);

        setDefaultValue(id, defaultValue);
        setLabel(id, label);
        setEnableValue(id, enable);
        setCheckboxValue("#" + id + "-visible", visible);
        setCheckboxValue("#" + id + "-editable", editable);

        if (id === "arrays") {
          $("#arrays-start").val(getProperty(widget, "start-value"));
          $("#arrays-end").val(getProperty(widget, "end-value"));
        }

        var useExecutionPath = '';
        if (id === 'errorpath') {
          useExecutionPath = getProperty(widget, "useExecutionPath") || 'true';
          setCheckboxValue("#errorpath-useExecutionPath", useExecutionPath);
          if (useExecutionPath === 'true' || enable === 'true')
            setEnableValue(id, 'true');
        } else if (id === 'outputpath') {
          useExecutionPath = getProperty(widget, "useExecutionPath") || 'true';
          setCheckboxValue("#outputpath-useExecutionPath", useExecutionPath);
          if (useExecutionPath === 'true' || enable === 'true')
            setEnableValue(id, 'true');
        }

        if (id === "cpupernode") {
          var totalamount = getProperty(widget, "totalamount") || 'true',
              nodeswithcount = getProperty(widget, "nodeswithcount") || 'true',
              malleablecorecount = getProperty(widget, "malleablecorecount") || 'true',
              totalamountEditable = getProperty(widget, "totalamount_editable") || 'true',
              nodeswithcountEditable = getProperty(widget, "nodeswithcount_editable") || 'true',
              malleablecorecountEditable = getProperty(widget, "malleablecorecount_editable") || 'true',
              totalamountVisibleDefault = getProperty(widget, "totalamount_visible_default") || 'true',
              nodeswithcountVisibleDefault = getProperty(widget, "nodeswithcount_visible_default") || 'false',
              malleablecorecountVisibleDefault = getProperty(widget, "malleablecorecount_visible_default") || 'false',
              totalCoresDefault = getProperty(widget, "totalcores_default") || 1,
              totalNodesDefault = getProperty(widget, "totalnodes_default") || 1,
              coresPerNodeDefault = getProperty(widget, "corespernode_default") || 1,
              minimumCores = getProperty(widget, "minimumcores_default") || 1,
              maximumCores = getProperty(widget, "maximumcores_default") || 2,
              touchspinSettings = ['totalCores', 'totalNodes', 'coresPerNode', 'minimumCores', 'maximumCores'];

          $.each(touchspinSettings, function(){
            var min = 0;
            if (this == 'minimumCores') {
              min = 1;
            } else if (this == 'maximumCores') {
              min = 2;
            }
            $('#cpupernode-' + this.toLowerCase()).data('min', (getProperty(widget, this + 'Min') || min));
            $('#cpupernode-' + this.toLowerCase()).data('max', (getProperty(widget, this + 'Max') || 65535));
            $('#cpupernode-' + this.toLowerCase()).data('step', (getProperty(widget, this + 'Step') || 1));
          });

          totalamount = totalamount.toLowerCase() === 'true';
          nodeswithcount = nodeswithcount.toLowerCase() === 'true';
          malleablecorecount = malleablecorecount.toLowerCase() === 'true';
          totalamountEditable = totalamountEditable.toLowerCase() === 'true';
          nodeswithcountEditable = nodeswithcountEditable.toLowerCase() === 'true';
          malleablecorecountEditable = malleablecorecountEditable.toLowerCase() === 'true';
          setCheckboxValue("#cpupernode-visible-totalamount", totalamount);
          setCheckboxValue("#cpupernode-visible-nodeswithcount", nodeswithcount);
          setCheckboxValue("#cpupernode-visible-malleablecorecount", malleablecorecount);
          setCheckboxValue("#cpupernode-editable-totalamount", totalamountEditable);
          setCheckboxValue("#cpupernode-editable-nodeswithcount", nodeswithcountEditable);
          setCheckboxValue("#cpupernode-editable-malleablecorecount", malleablecorecountEditable);
          setCheckboxValue("#cpupernode-visible-default-totalamount", totalamountVisibleDefault);
          setCheckboxValue("#cpupernode-visible-default-nodeswithcount", nodeswithcountVisibleDefault);
          setCheckboxValue("#cpupernode-visible-default-malleablecorecount", malleablecorecountVisibleDefault);
          setEnableValue('cpupernode', getProperty(widget, "enable") || 'true');

          $('#cpupernode-totalcores').val(totalCoresDefault);
          $('#cpupernode-totalnodes').val(totalNodesDefault);
          $('#cpupernode-corespernode').val(coresPerNodeDefault);
          $('#cpupernode-minimumcores').val(minimumCores);
          $('#cpupernode-maximumcores').val(maximumCores);
        }

        if (id === "memorypernode") {
          var totalmemory = getProperty(widget, "totalmemory") || 'true',
              memorypercore = getProperty(widget, "memorypercore") || 'true',
              totalmemoryVisibleDefault = getProperty(widget, "totalmemory_visible_default") || 'true',
              memorypercoreVisibleDefault = getProperty(widget, "memorypercore_visible_default") || 'false',
              totalMemoryDefault = getProperty(widget, "totalmemory_default") || 0.50,
              memoryPerCoreDefault = getProperty(widget, "memorypercore_default") || 0.50,
              totalmemoryEditable = getProperty(widget, "totalmemory_editable") || 'true',
              memorypercoreEditable = getProperty(widget, "memorypercore_editable") || 'true',
              touchspinSettings = ['totalMemory', 'memoryPerCore'];

          $.each(touchspinSettings, function(){
            $('#memorypernode-' + this.toLowerCase() + '-defaultValue').data('min', (getProperty(widget, this + 'Min') || 0.25));
            $('#memorypernode-' + this.toLowerCase() + '-defaultValue').data('max', (getProperty(widget, this + 'Max') || 65535));
            $('#memorypernode-' + this.toLowerCase() + '-defaultValue').data('step', (getProperty(widget, this + 'Step') || 0.25));
          });

          totalmemory = totalmemory.toLowerCase() === 'true';
          memorypercore = memorypercore.toLowerCase() === 'true';
          totalmemoryEditable = totalmemoryEditable.toLowerCase() === 'true';
          memorypercoreEditable = memorypercoreEditable.toLowerCase() === 'true';
          setCheckboxValue("#memorypernode-visible-totalmemory", totalmemory);
          setCheckboxValue("#memorypernode-visible-memorypercore", memorypercore);
          setCheckboxValue("#memorypernode-visible-default-totalmemory", totalmemoryVisibleDefault);
          setCheckboxValue("#memorypernode-visible-default-memorypercore", memorypercoreVisibleDefault);
          setCheckboxValue("#memorypernode-editable-totalmemory", totalmemoryEditable);
          setCheckboxValue("#memorypernode-editable-memorypercore", memorypercoreEditable);
          setEnableValue('memorypernode', getProperty(widget, "enable") || 'true');

          $('#memorypernode-totalmemory-defaultValue').val(totalMemoryDefault);
          $('#memorypernode-memorypercore-defaultValue').val(memoryPerCoreDefault);
        }

        function setLabel(id, label) {
          $("#" + id + "-label").val(label);
        }

        function setDefaultValue(id, defaultValue) {
          if (id === 'genericresources') {
              var data = getPayload(defaultValue);
              renderGresTable(data);
          }
          if (id === 'jobEnvVariables') {
              $("#" + id + "-defaultValue").val(defaultValue);
              renderMapTable($('#job-env-variables-table').find('tbody'), {map: JSON.parse(defaultValue || null)});
          }
          if (id === 'architecture') {
            loadDataFromServerToSelect('#architecture-defaultValue', 'architectures', defaultValue);
            return;
          }
          if (id === 'operatingSystem') {
            loadDataFromServerToSelect('#operatingSystem-defaultValue', 'operating-systems', defaultValue);
            return;
          }
          if (id === 'duration' || id === 'eligibledate') {
            $("#" + id + "-defaultValue").durationpicker('setFullSeconds', defaultValue);
          } else if (typeof defaultValue === 'boolean') {
            $("#" + id + "-defaultValue option").each(function() {
              $(this).prop('selected', $(this).val() === defaultValue.toString());
            });
          } else {
            $("#" + id + "-defaultValue").val(defaultValue);
          }
        }

        function setEnableValue(id, enable) {
          if (enable !== undefined || enable !== null) {
            setCheckboxValue("#" + id + "-enable", enable === "true");
          }
        }

        function setCheckboxValue(selector, value) {
          if (value == 'false' || value == 'False') {
            value = false;
          }
          $(selector).prop("checked", value);
        }

        addVisibleEditableLogic(id);
      }
    }
    $('[data-toggle="tooltip"]').tooltip();
  });

  function serializeTemplate() {
    var sectionNames = ['defaults', 'timeManagement', 'credentials', 'resources', 'dataManagement', 'custom',
        'basic', 'advanced', 'description'];

    var result = {
      name: $("#template-name").text(),
      version_description: $('#version-description').val(),
      type: $('#template-type').select2('val'),
      description: {
        text: $("#template-description").val(),
        visible: $('#description-section-visible').is(':checked')
      },
      published: $('input[name=publish]:checked').val() === "published",
      widgets: [],
      sections: serializeSections(sectionNames),
      permissions: serializePermissions()
    };

    result.widgets.push(serializeFreeFormWidget("name", VariablesTooltips.name));
    result.widgets.push(serializeFreeFormWidget("duration", VariablesTooltips.duration));
    result.widgets.push(serializeFreeFormWidget("arrays"));
    result.widgets.push(serializeFreeFormWidget("eligibledate", VariablesTooltips.eligibledate));
    result.widgets.push(serializeFreeFormWidget("priority", VariablesTooltips.priority));
    result.widgets.push(serializeFreeFormWidget("script"));
    result.widgets.push(serializeFreeFormWidget("account", VariablesTooltips.account));
    result.widgets.push(serializeFreeFormWidget("destinationQueue", VariablesTooltips.destinationQueue));
    result.widgets.push(serializeFreeFormWidget("qos", VariablesTooltips.qos));
    result.widgets.push(serializeFreeFormWidget("cpupernode", VariablesTooltips.cpuNodes));
    result.widgets.push(serializeFreeFormWidget("memorypernode", VariablesTooltips.memorypernode));
    result.widgets.push(serializeFreeFormWidget("architecture", VariablesTooltips.architecture));
    result.widgets.push(serializeFreeFormWidget("exedir", VariablesTooltips.exedir));
    result.widgets.push(serializeFreeFormWidget("errorpath", VariablesTooltips.errorpath));
    result.widgets.push(serializeFreeFormWidget("outputpath", VariablesTooltips.outputpath));
    result.widgets.push(serializeFreeFormWidget("join", VariablesTooltips.join));
    result.widgets.push(serializeFreeFormWidget("hold", VariablesTooltips.hold));
    result.widgets.push(serializeFreeFormWidget("shell", VariablesTooltips.shell));
    result.widgets.push(serializeFreeFormWidget("moabenvironmentvariables", VariablesTooltips.moabenvironmentvariables));
    result.widgets.push(serializeFreeFormWidget("maillist", VariablesTooltips.maillist));
    result.widgets.push(serializeFreeFormWidget("emailoptions", VariablesTooltips.emailoptions));
    result.widgets.push(serializeFreeFormWidget("genericresources", VariablesTooltips.genericresources));
    result.widgets.push(serializeFreeFormWidget("jobEnvVariables"));
    result.widgets.push(serializeFreeFormWidget("moabtemplates", VariablesTooltips.moabtemplates));
    result.widgets.push(serializeFreeFormWidget("node-access-policy"));
    result.widgets.push(serializeFreeFormWidget("node-allocation-policy"));
    result.widgets.push(serializeFreeFormWidget("nodes-requested-policy"));
    result.widgets.push(serializeFreeFormWidget("operatingSystem"));
    result.widgets.push(serializeSelect2Widget("features-requested", VariablesTooltips.featuresRequested));
    result.widgets.push(serializeSelect2Widget("features-excluded", VariablesTooltips.featuresExcluded));
    result.widgets.push(serializeSelect2Widget("nodes-requested"));

    result.widgets = result.widgets.concat(serializeCustomWidgets());

    return result;

    function serializeSelect2Widget(id, tooltip) {
      var defVal = $('#' + id + '-defaultValue').select2('val');
      var result = {
        default_value: defVal ? JSON.stringify(defVal) : "",
        label: $("#" + id + "-label").val() || "",
        variable_name: $("#" + id + "-variableName").val() || "",
        visible: getVisibleValueForVariable(id),
        editable: getEditableValueForVariable(id),
        properties: [
          {
            name: "id",
            value: id
          }
        ]
      };
      if (tooltip) {
        result.tooltip = tooltip;
      }

      var enable = getEnableValueForVariable(id);
      result.properties.push({
        name: "enable",
        value: enable
      });

      return result;
    }

    function serializeFreeFormWidget(id, tooltip) {
      var result = {
        default_value: $("#" + id + "-defaultValue").val() || "",
        label: $("#" + id + "-label").val() || "",
        variable_name: $("#" + id + "-variableName").val() || "",
        visible: getVisibleValueForVariable(id),
        editable: getEditableValueForVariable(id),
        properties: [
          {
            name: "id",
            value: id
          }
        ]
      };
      if (tooltip) {
        result.tooltip = tooltip;
      }

      if (id === 'duration' || id === 'eligibledate') {
        var duration = $('#' + id + '-defaultValue').durationpicker('parseDuration');
        if (duration.valid) {
          result.default_value = duration.fullSeconds;
        } else {
          result.default_value = 0;
        }
      }

      if (id === 'errorpath' || id === 'outputpath') {
        result.properties.push({
          name: "useExecutionPath",
          value: $("#" + id + "-useExecutionPath").is(":checked")
        });
      }

      if (id === 'cpupernode') {
        var totalVisible = $("#" + id + "-visible-totalamount").is(":checked"),
            withCoresVisible = $("#" + id + "-visible-nodeswithcount").is(":checked"),
            malleableCoreCountVisible = $("#" + id + "-visible-malleablecorecount").is(":checked"),
            totalEditable = $("#" + id + "-editable-totalamount").is(":checked"),
            withCoresEditable = $("#" + id + "-editable-nodeswithcount").is(":checked"),
            malleableCoreCountEditable = $("#" + id + "-editable-malleablecorecount").is(":checked"),
            totalVisibleDefault = $("#" + id + "-visible-default-totalamount").is(":checked"),
            withCoresVisibleDefault = $("#" + id + "-visible-default-nodeswithcount").is(":checked"),
            malleableCoreCountVisibleDefault = $("#" + id + "-visible-default-malleablecorecount").is(":checked"),
            totalCoresDefault = $('#cpupernode-totalcores').val(),
            totalNodesDefault = $('#cpupernode-totalnodes').val(),
            coresPerNodeDefault = $('#cpupernode-corespernode').val(),
            minimumCoresDefault = $('#cpupernode-minimumcores').val(),
            maximumCoresDefault = $('#cpupernode-maximumcores').val(),
            touchspinSettings = {};

        $.each([
          {
            'name': 'totalCores',
            'value': '#cpupernode-totalcores'
          },
          {
            'name': 'totalNodes',
            'value': '#cpupernode-totalnodes'
          },
          {
            'name': 'coresPerNode',
            'value': '#cpupernode-corespernode'
          },
          {
            'name': 'minimumCores',
            'value': '#cpupernode-minimumcores'
          },
          {
            'name': 'maximumCores',
            'value': '#cpupernode-maximumcores'
          }
        ], function(){
          var property = this;
          touchspinSettings[property.name] = {};
          $.each(['min', 'max', 'step'], function(){
            touchspinSettings[property.name][this] = $(property.value).data(this);
          });
        });

        var propertiesArray = [
          {
            name: "totalamount",
            value: totalVisible
          },
          {
            name: "nodeswithcount",
            value: withCoresVisible
          },
          {
            name: "malleablecorecount",
            value: malleableCoreCountVisible
          },
          {
            name: "totalamount_editable",
            value: totalEditable
          },
          {
            name: "nodeswithcount_editable",
            value: withCoresEditable
          },
          {
            name: "malleablecorecount_editable",
            value: malleableCoreCountEditable
          },
          {
            name: "totalamount_visible_default",
            value: totalVisibleDefault
          },
          {
            name: "nodeswithcount_visible_default",
            value: withCoresVisibleDefault
          },
          {
            name: "malleablecorecount_visible_default",
            value: malleableCoreCountVisibleDefault
          },
          {
            name: "totalcores_default",
            value: totalCoresDefault
          },
          {
            name: "totalnodes_default",
            value: totalNodesDefault
          },
          {
            name: "corespernode_default",
            value: coresPerNodeDefault
          },
          {
            name: "minimumcores_default",
            value: minimumCoresDefault
          },
          {
            name: "maximumcores_default",
            value: maximumCoresDefault
          }
        ];

        $.each(Object.keys(touchspinSettings), function(){
          propertiesArray.push({'name': this + 'Min', 'value': touchspinSettings[this].min});
          propertiesArray.push({'name': this + 'Max', 'value': touchspinSettings[this].max});
          propertiesArray.push({'name': this + 'Step', 'value': touchspinSettings[this].step});
        });

        result.visible = totalVisible || withCoresVisible || malleableCoreCountVisible;

        $.each(propertiesArray, function(){
          result.properties.push(this);
        });
      }

      if (id === 'memorypernode') {
        var totalVisible = $("#" + id + "-visible-totalmemory").is(":checked"),
            withCoresVisible = $("#" + id + "-visible-memorypercore").is(":checked"),
            totalEditable = $("#" + id + "-editable-totalmemory").is(":checked"),
            withCoresEditable = $("#" + id + "-editable-memorypercore").is(":checked"),
            totalVisibleDefault = $("#" + id + "-visible-default-totalmemory").is(":checked"),
            withCoresVisibleDefault = $("#" + id + "-visible-default-memorypercore").is(":checked"),
            totalMemoryDefault = $('#memorypernode-totalmemory-defaultValue').val(),
            memoryPerCoreDefault = $('#memorypernode-memorypercore-defaultValue').val(),
            touchspinSettings = {};

        $.each([
          {
            'name': 'totalMemory',
            'value': '#memorypernode-totalmemory-defaultValue'
          },
          {
            'name': 'memoryPerCore',
            'value': '#memorypernode-memorypercore-defaultValue'
          }
        ], function(){
          var property = this;
          touchspinSettings[property.name] = {};
          $.each(['min', 'max', 'step'], function(){
            touchspinSettings[property.name][this] = $(property.value).data(this);
          });
        });

        var propertiesArray = [
          {
            name: "totalmemory",
            value: totalVisible
          },
          {
            name: "memorypercore",
            value: withCoresVisible
          },
          {
            name: "totalmemory_visible_default",
            value: totalVisibleDefault
          },
          {
            name: "memorypercore_visible_default",
            value: withCoresVisibleDefault
          },
          {
            name: "totalmemory_default",
            value: totalMemoryDefault
          },
          {
            name: "memorypercore_default",
            value: memoryPerCoreDefault
          },
          {
            name: "totalmemory_editable",
            value: totalEditable
          },
          {
            name: "memorypercore_editable",
            value: withCoresEditable
          }
        ];

        $.each(Object.keys(touchspinSettings), function(){
          propertiesArray.push({'name': this + 'Min', 'value': touchspinSettings[this].min});
          propertiesArray.push({'name': this + 'Max', 'value': touchspinSettings[this].max});
          propertiesArray.push({'name': this + 'Step', 'value': touchspinSettings[this].step});
        });

        result.visible = totalVisible || withCoresVisible;

        $.each(propertiesArray, function(){
          result.properties.push(this);
        });

      }

      var enable = getEnableValueForVariable(id);
      if (enable) {
        result.properties.push({
          name: "enable",
          value: enable
        });
      }

      if (id === "arrays") {
        result.default_value = "";
        result.properties.push({
          name: "start-value",
          value: $("#arrays-start").val()
        });
        result.properties.push({
          name: "end-value",
          value: $("#arrays-end").val()
        });

        result.variable_name = 'arrays';
        result.properties.push({
          name: 'start-variable',
          value: $("#arrays-min-variableName").val()
        });
        result.properties.push({
          name: 'end-variable',
          value: $("#arrays-max-variableName").val()
        });
      }

      if (id === 'moabtemplates') {
        var defaultValue = '';
        $('#moabtemplates-ids option:selected').each(function() {
          defaultValue += $(this).text() + ';';
        });
        result.default_value = defaultValue;
      }
      return result;
    }

    function getVisibleValueForVariable(variable) {
      return $("#" + variable + "-visible").is(":checked");
    }

    function getEditableValueForVariable(variable) {
      return $("#" + variable + "-editable").is(":checked");
    }

    function getEnableValueForVariable(variable) {
      var enableElement = $("#" + variable + "-enable");
      if (enableElement.length > 0) {
        return enableElement.is(":checked").toString();
      }
      return null;
    }
  }

  function getSliderDefaultValue(id) {
    var minValue = $("#" + id + "-min").val();
    var maxValue = $("#" + id + "-max").val();
    var stepValue = $("#" + id + "-step").val();
    var sliderValue = $("#" + id + "-svalue").val();
    var config = {
      min: minValue,
      max: maxValue,
      step: stepValue,
      svalue: sliderValue
    };
    return JSON.stringify(config);
  }

  function serializeCustomWidgets() {
    var result = [];
    $.each(customWidgets, function(index, value) {
      var id = value.id;
      var defaultValue = $("#" + id + "-defaultValue").val();
      var widgetType = $('#' + id + '-type').val();
      if (widgetType == 'slider') {
        defaultValue = getSliderDefaultValue(id);
      }
      result.push(new CustomWidgetConfig(
        value.position,
        $("#" + id + "-type").val(),
        $("#" + id + "-label").val(),
        $("#" + id + "-variableName").val(),
        defaultValue,
        $("#" + id + "-visible").is(':checked'),
        $("#" + id + "-editable").is(':checked'),
        $("#" + id + "-required").is(':checked'),
        index
      ));
    });
    return result;
  }

  function getProperty(object, property) {
    if (!(property in object)) {
      for (var i = 0; i < object.properties.length; i++) {
        if (object.properties[i].name === property) {
          return object.properties[i].value;
        }
      }
      return null;
    }
    return object[property];
  }

  function urlHasTemplateUrl() {
    return $.url("?url") !== null;
  }

  function getTemplateUrl() {
    return $.url("?url");
  }

  var collapsibleDivs = ['.basic-collapsible-div', '.advanced-collapsible-div', '.custom-collapsible-div',
    '.description-collapsible-div', '.node-policy-collapsible-div'];
  var collapseTriggers = ['.basic-collapse', '.advanced-collapse', '.custom-collapse', '.description-collapse',
      '.node-policy-collapse'];

  function addCollapseListener(trigger, collapsibleDiv) {
    $(trigger).click(function() {
      if (!$(collapsibleDiv).hasClass('collapsing')) {
        $(collapsibleDiv).collapse('toggle');
        var $trigger = $('span' + trigger);
        if ($trigger.hasClass('glyphicon-chevron-down')) {
          $trigger
            .removeClass('glyphicon-chevron-down')
            .addClass('glyphicon-chevron-right');
        } else {
          $trigger
            .removeClass('glyphicon-chevron-right')
            .addClass('glyphicon-chevron-down');
        }
      }
    });
  }

  (function() {
    for (var i = 0; i < collapseTriggers.length; i++) {
      addCollapseListener(collapseTriggers[i], collapsibleDivs[i]);
    }
  })();

  function addVisibleEditableLogic(id) {
    var visibleBox = $('#' + id + '-visible');
    var editableBox = $('#' + id + '-editable');
    var enableBox = $('#' + id + '-enable');
    var requiredBox = $('#' + id + '-required');

    if (enableBox.length && visibleBox.length && editableBox.length) {
      visibleBox.prop('disabled', !enableBox.is(':checked'));
      enableBox.click(function() {
        var checked = !!($(this).is(':checked'));
        visibleBox.prop('disabled', !checked).prop('checked', false);
        editableBox.prop('disabled', !visibleBox.is(':checked')).prop('checked', false);
        requiredBox.prop('disabled', !checked);
      });
    }

    if (visibleBox.length && editableBox.length) {
      editableBox.prop('disabled', !visibleBox.is(':checked'));
      visibleBox.click(function() {
        var checked = !!($(this).is(':checked'));
        editableBox.prop('disabled', !checked).prop('checked', checked);
        requiredBox.prop('disabled', !checked).prop('checked', checked);
      });
    }

    if (editableBox.length && requiredBox.length) {
      requiredBox.prop('disabled', !editableBox.is(':checked'));
      editableBox.click(function() {
        var checked = !!($(this).is(':checked'));
        requiredBox.prop('disabled', !checked).prop('checked', checked);
      });
    }

    if (id === 'outputpath' || id === 'errorpath') {
      var useExecPath = $('#' + id + '-useExecutionPath');
      useExecPath.prop('disabled', !enableBox.is(':checked'));
      enableBox.click(function() {
        var checked = !!($(this).is(':checked'));
        $(useExecPath).prop('disabled', !checked);
      });
    }

    // TODO: refactoring the code for  widget "Number of Cores"
    if (id === 'cpupernode') {
      $('#cpupernode-visible-totalamount, #cpupernode-visible-nodeswithcount, #cpupernode-visible-malleablecorecount,' +
        ' #cpupernode-editable-totalamount, #cpupernode-editable-nodeswithcount, #cpupernode-editable-malleablecorecount')
          .prop('disabled', !enableBox.is(':checked'));
      enableBox.click(function() {
        var checked = !!($(this).is(':checked'));
        $('#cpupernode-visible-totalamount, #cpupernode-visible-nodeswithcount, #cpupernode-visible-malleablecorecount').prop('disabled', !checked);
        $('#cpupernode-editable-totalamount, #cpupernode-editable-nodeswithcount, #cpupernode-editable-malleablecorecount').prop('disabled', true);
        if (!checked) {
          $("#cpupernode-visible-default-totalamount, #cpupernode-visible-default-nodeswithcount, #cpupernode-visible-default-malleablecorecount").parent().hide();
          $('#cpupernode-visible-totalamount, #cpupernode-visible-nodeswithcount, #cpupernode-visible-malleablecorecount,' +
            ' #cpupernode-editable-totalamount, #cpupernode-editable-nodeswithcount, #cpupernode-editable-malleablecorecount')
              .prop('checked', false);
        } else if (!$('#cpupernode-visible-totalamount').is(':checked') &&
            !$('#cpupernode-visible-nodeswithcount').is(':checked') &&
            !$('#cpupernode-visible-malleablecorecount').is(':checked')) {
          $("#cpupernode-visible-default-totalamount, #cpupernode-visible-default-nodeswithcount, #cpupernode-visible-default-malleablecorecount").parent().show();
        }
      });
      $('#cpupernode-visible-totalamount, #cpupernode-visible-nodeswithcount, #cpupernode-visible-malleablecorecount').click(function(){
        var checked = $(this).is(':checked');
        $('#' + this.id.replace('visible', 'editable')).prop('disabled', !checked).prop('checked', checked);
      });
    }

    if (id === 'memorypernode') {
      $('#memorypernode-visible-totalmemory, #memorypernode-visible-memorypercore, ' +
        '#memorypernode-editable-totalmemory, #memorypernode-editable-memorypercore')
          .prop('disabled', !enableBox.is(':checked'));
      enableBox.click(function() {
        var checked = !!($(this).is(':checked'));
        $('#memorypernode-visible-totalmemory, #memorypernode-visible-memorypercore').prop('disabled', !checked);
        $('#memorypernode-editable-totalmemory, #memorypernode-editable-memorypercore').prop('disabled', true);
        if (!checked) {
          $("#memorypernode-visible-default-totalmemory, #memorypernode-visible-default-memorypercore").parent().hide();
          $('#memorypernode-visible-totalmemory, #memorypernode-visible-memorypercore, ' +
           '#memorypernode-editable-totalmemory, #memorypernode-editable-memorypercore').prop('checked', false);
        } else if (!$('#memorypernode-visible-totalmemory').is(':checked') &&
            !$('#memorypernode-visible-memorypercore').is(':checked')) {
          $("#memorypernode-visible-default-totalmemory, #memorypernode-visible-default-memorypercore").parent().show();
        }
      });
      $('#memorypernode-visible-totalmemory, #memorypernode-visible-memorypercore').click(function(){
        var checked = $(this).is(':checked');
        $('#' + this.id.replace('visible', 'editable')).prop('disabled', !checked).prop('checked', checked);
      });
    }
  }

  function activateFeaturesWidgets(widgets) {
    var featuresRequestedWidget = $('#features-requested-defaultValue');
    var featuresExcludedWidget = $('#features-excluded-defaultValue');

    featuresRequestedWidget.select2({
      width: '100%',
      placeholder: 'None',
      tags: true,
      closeOnSelect: true,
      tokenSeparators: [' '],
      templateSelection: function(e) {
        var isValid = $.inArray(e.text, avaiableFeatures) != -1;

        return $('<span/>')
          .text(e.text)
          .addClass(isValid ? 'valid-tag' : 'invalid-tag');
      }
    });

    featuresExcludedWidget.select2({
      width: '100%',
      placeholder: 'None',
      tags: true,
      closeOnSelect: true,
      tokenSeparators: [' '],
      templateSelection: function(e) {
        var isValid = $.inArray(e.text, avaiableFeatures) != -1;

        return $('<span/>')
          .text(e.text)
          .addClass(isValid ? 'valid-tag' : 'invalid-tag');
      }
    });

    if (widgets) {
      addDefaultValue('features-requested', featuresRequestedWidget);
      addDefaultValue('features-excluded', featuresExcludedWidget);
    } else {
      $.viewpoint.api.get('/api/features/?page_size=999')
        .done(function(data, textStatus, jqXHR) {
          $.each(data.results, function (index, value) {
            appendOption(index, value, featuresRequestedWidget);
            appendOption(index, value, featuresExcludedWidget);
          });
          avaiableFeatures = data.results;
        });
    }

    function addDefaultValue(id, widget) {
      var featureWidget = getWidgetById(id, widgets);
      $.viewpoint.api.get('/api/features/?page_size=999')
        .done(function(data, textStatus, jqXHR) {
          $.each(data.results, function (index, value) { appendOption(index, value, widget) });
          avaiableFeatures = data.results;
          if (featureWidget && featureWidget.default_value) {
            widget.select2('val', JSON.parse(featureWidget.default_value));
          }
        });
    }

    function appendOption(i, val, widget) {
      widget.append(
        $('<option></option>')
          .attr('value', val)
          .text(val)
      );
    }
  }

  function activateNodeRequestedWidget(widgets) {
    var widget = $('#nodes-requested-defaultValue');
    var pageSize = 25;

    widget.select2({
      width: '100%',
      placeholder: 'None',
      tags: true,
      tokenSeparators: [' '],
      minimumInputLength: 1,
      templateSelection: function(e) {
        var api = new Viewpoint.API();
        var $el = $('<span/>').text(e.text);

        api.get('/api/nodes/' + e.text + '/')
            .done(function() {
              $el.addClass('valid-tag');
            })
            .fail(function() {
              $el.addClass('invalid-tag');
            });

        return $el;
      },
      ajax: {
        url: "/api/nodes",
        dataType: 'json',
        delay: 250,
        data: function(params) {
          return {
            name__icontains: params.term,
            page: params.page,
            page_size: pageSize,
            ordering: 'name'
          };
        },
        processResults: function(data, params) {
          params.page = params.page || 1;

          return {
            results: $.map(data.results, function(item) {
              return {
                text: item.name,
                id: item.name
              };
            }),
            pagination: {
              more: (params.page * pageSize) < data.count
            }
          };
        }
      }
    });

    if (widgets && getWidgetById('nodes-requested', widgets)) {
      var nodeWidget = getWidgetById('nodes-requested', widgets);
      var defaultValue = JSON.parse(nodeWidget.default_value || null);
      $.each(defaultValue || [], appendOption);
      widget.select2('val', defaultValue);
    }

    function appendOption(i, val) {
      widget.append(
        $('<option></option>')
          .attr('value', val)
          .text(val)
      );
    }
  }

  function getWidgetById(id, widgets) {
    for (var i = 0; i < widgets.length; i++) {
      if (getProperty(widgets[i], 'id') === id) {
        return widgets[i];
      }
    }
    return null;
  }

  function activateMoabTemplatesWidgets() {
    var api = $('#builder-body').data('api');
    var widget = $('#moabtemplates-ids');
    widget.select2({
      width: '100%',
      allowClear: true,
      placeholder: 'None',
      minimumResultsForSearch: -1
    });

    api.get('/api/moab-templates/').done(function(data) {
      $.each(data.results, function(_index, v) {
        $('#moabtemplates-ids').append($("<option value='" + v.id + "'>" + v.id + "</option>"));
      });
      var defaultValue = $('#moabtemplates-defaultValue').val();
      if (defaultValue) {
        var values = defaultValue.split(';');
        widget.val(values).trigger('change');
      }
    });
  }

  function activateTemplateType() {
    var api = $('#builder-body').data('api');
    api.get('/api/template-types/')
      .done(function(data) {
        var perms = $.viewpoint.user.permissions;
        $.each(data.results, function(_i, type) {
          if (type.id == 'nitro' && !perms.nitro_templates)
            return;
          if (type.id == 'remote_viz' && !perms.remote_visualization_templates)
            return;
          $('#template-type').append(
            '<option value="' + type.id + '">' + type.name + '</option>'
          );
        });
        if (urlHasTemplateUrl()) {
          $('#template-type').select2('val', templateType);
        } else {
          $('#template-type').select2('val', 'regular');
        }
      });
  }

  function activatePermissionWidgets(permissions) {
    var api = $('#permissions-container').data('api');

    $('#permissions-user').select2({
      width: '100%',
      minimumInputLength: 1,
      placeholder: 'None',
      tags: true,
      selectOnClose: true,
      closeOnSelect: true,
      tokenSeparators: [' '],
      templateSelection: function(e) {
        return renderSelect2Tag('users', e);
      }
    });

    $('#permissions-group').select2({
      width: '100%',
      minimumInputLength: 1,
      placeholder: 'None',
      tags: true,
      selectOnClose: true,
      closeOnSelect: true,
      tokenSeparators: [' '],
      templateSelection: function(e) {
        return renderSelect2Tag('groups', e);
      }
    });

    $('#permissions-account').select2({
      width: '100%',
      minimumInputLength: 1,
      placeholder: 'None',
      tags: true,
      selectOnClose: true,
      closeOnSelect: true,
      tokenSeparators: [' '],
      templateSelection: function(e) {
        return renderSelect2Tag('credentials/accounts', e);
      }
    });

    if (permissions) {
      var permissionValues = {user: [], group: [], account: []};

      $(permissions).each(function(i, e) {
        permissionValues[e.type].push(e.name);
      });
    }

    api.get('/api/users/').done(function(data, textStatus, jqXHR) {
      var users = addAllIfDoNotExist(data.results);
      var usersOptions = $(users).map(optionGenerateCallback).get();
      var $userPermissions = $('#permissions-user');

      $userPermissions.append(usersOptions);
      if (permissions) {
        $userPermissions.select2('val', permissionValues.user);
        lastSavedTmplt = serializeTemplate();
      } else {
        $userPermissions.select2('val', 'ALL');
      }
    });

    api.get('/api/groups/').done(function(data, textStatus, jqXHR) {
      var groups = addAllIfDoNotExist(data.results);
      var groupsOptions = $(groups).map(optionGenerateCallback).get();
      var $groupPermissions = $('#permissions-group');

      $groupPermissions.append(groupsOptions);
      if (permissions) {
        $groupPermissions.select2('val', permissionValues.group);
        lastSavedTmplt = serializeTemplate();
      } else {
        $groupPermissions.select2('val', 'ALL');
      }
    });

    api.get('/api/credentials/accounts/').done(function(data, textStatus, jqXHR) {
      var accounts = addAllIfDoNotExist(data.results);
      var accountsOptions = $(accounts).map(optionGenerateCallback).get();
      var $accountPermissions = $('#permissions-account');

      $accountPermissions.append(accountsOptions);
      if (permissions) {
        $accountPermissions.select2('val', permissionValues.account);
        lastSavedTmplt = serializeTemplate();
      } else {
        $accountPermissions.select2('val', 'ALL');
      }
    });

    function renderSelect2Tag(credential, e) {
      var $rendered = $('<span/>').text(e.text);

      if (e.text === 'ALL') {
        return $('<span/>')
            .text(e.text)
            .addClass('valid-tag');
      }

      if (credential == 'credentials/accounts') {
        api.head('/api/' + credential + '/?name=' + e.text)
          .done(function() {
            addValidIcon($rendered);
          })
          .fail(function() {
            addInvalidIcon($rendered);
          });
      } else {
        api.head('/api/' + credential + '/' + e.text + '/')
          .done(function() {
            addValidIcon($rendered);
          })
          .fail(function() {
            addInvalidIcon($rendered);
          });
      }
      return $rendered;
    }

    function optionGenerateCallback(i, e) {
      return $('<option/>').text(e);
    }

    function addAllIfDoNotExist(array) {
      var result = _.clone(array);
      if ($.inArray('ALL', result) === -1) {
        result.push('ALL');
      }
      return result;
    }

    function addValidIcon($selector) {
      $selector.addClass('valid-tag');
    }

    function addInvalidIcon($selector) {
      $selector.addClass('invalid-tag');
    }
  }

  function serializeSections(sections) {
    var result = [];

    $(sections).each(function(i, e) {
      result.push({
        name: e,
        value: $('#' + e + '-section-name').val()
      });
    });

    return result;
  }

  function saveTemplate(successCallback) {
    if (validatePermissions() &&
      validateScript() && validateFeatures() && validateNodesRequestedPolicy()) {
      var serializedTemplate = serializeTemplate();

      var api = $(this).parent().loader(new Viewpoint.API(), {404: fail404Callback}).data('api');
      $(this).parent().css('min-height', '0px');

      if (templateHistoryUrl) {
        api.post(templateHistoryUrl, serializedTemplate).done(successCallback);
      } else if (templateUrl) {
        api.put(templateUrl, serializedTemplate).done(successCallback);
      } else {
        api.post('/api/templates/', serializedTemplate).done(successCallback);
      }
    } else {
      $.notify('Please correct the error in the form', {type: 'danger'});
    }

    function fail404Callback(jqXHR) {
      if (jqXHR.status == 404) {
        $.viewpoint.confirm('Template not found', 'We detected the unsynchronized content. ' +
              'Do you want  save it as a new template?')
          .done(function(result) {
            if (result) {
              api.post('/api/templates/', serializedTemplate).done(successCallback);
            }
          });
      }
    }

  }


  function validatePermissions() {
    return $('#permissions-container').find('.invalid-tag').length === 0;
  }

  function validateFeatures() {
    $('#features-requested-defaultValue-group').removeClass('has-error');
    $('#features-excluded-defaultValue-group').removeClass('has-error');
    var result = _.intersection($('#features-requested-defaultValue').val(), $('#features-excluded-defaultValue').val());
    if (result && result.length) {
      $('#features-requested-defaultValue-group').addClass('has-error');
      $('#features-excluded-defaultValue-group').addClass('has-error');
      $.notify('Requested and Excluded features can not have common values', {type: 'warning'});
      return false;
    }
    return $('#features-requested-defaultValue-group').find('.invalid-tag').length === 0 &&
     $('#features-excluded-defaultValue-group').find('.invalid-tag').length === 0;
  }

  function validateNodesRequestedPolicy() {
    return $('#nodes-requested-defaultValue-group').find('.invalid-tag').length === 0;
  }

  function validateScript() {
    var val = $('#script-defaultValue').val().trim();
    var editable = $('#script-editable').is(':checked');

    if (!editable && !val) {
      $('#script-defaultValue').parent().parent().addClass('has-error');
      $.notify('You should type script body or make it editable', { type: 'warning' });
      return false;
    }
    $('#script-defaultValue').parent().parent().removeClass('has-error');
    return true;
  }

  function serializePermissions() {
    var results = [];
    var permissionTypes = ['user', 'group', 'account'];

    $(permissionTypes).each(function(i, permissionType) {
      var values = $('#permissions-' + permissionType).val() || [];
      var valuesAsPermissions = $(values).map(function(i, e) {
        return {name: e, type: permissionType};
      }).get();
      results = results.concat(valuesAsPermissions);
    });

    return results;
  }

  // TODO: function already exists in job_create
  function getFriendlyDuration(duration) {
    var title = '';
    if (duration.weeks) {
      title += duration.weeks + 'week';
      if (duration.weeks > 1) {
        title += 's';
      }
    }
    if (duration.days) {
      if (title) {
        title += ', ';
      }
      title += duration.days + 'day';
      if (duration.days > 1) {
        title += 's';
      }
    }
    if (duration.hours) {
      if (title) {
        title += ', ';
      }
      title += duration.hours + 'hour';
      if (duration.hours > 1) {
        title += 's';
      }
    }
    if (duration.minutes) {
      if (title) {
        title += ', ';
      }
      title += duration.minutes + 'minute';
      if (duration.minutes > 1) {
        title += 's';
      }
    }
    if (duration.seconds) {
      if (title) {
        title += ', ';
      }
      title += duration.seconds + 'second';
      if (duration.seconds > 1) {
        title += 's';
      }
    }
    return title;
  }

})(jQuery, Handlebars);
