/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */
;(function () {
  'use strict';
    var elementsPerPage = 10;
    var currentViewClass = 'grid-group-item';
    var $jobTypeDialog = $('#job-type-dialog');

    $(function() {
      $jobTypeDialog.one('shown.bs.modal', function () {
        showSelectJobDialog();
      });

      var $input = $jobTypeDialog.find('.search-container').find('select');
      $input.select2();
      var $searchBtn = $jobTypeDialog.find('.show-search-box-btn');
      $searchBtn.loader(new Viewpoint.API());

      var perms = $.viewpoint.user.permissions;
      $searchBtn.data('api')
          .get('/api/templates/?published=True&page_size=999')
          .done(function(data) {
            var listOfTemplates = $(data.results)
                .map(function(i, e) {
                  var item = e.name;
                  // Check if user can create Nitro Job
                  if (e.type == 'nitro' && !perms.nitro_jobs_create) {
                    return null;
                  }
                  if (e.type == 'remote_viz' && !perms.remote_visualization_jobs_create) {
                    return null;
                  }
                  return item;
                }).get();

            $input.select2({
              width: '100%',
              tags: true,
              data: listOfTemplates,
              dropdownParent: '#job-type-dialog',
              allowClear: true,
              placeholder: 'None',
              minimumInputLength: 1
            });

            $input.select2('val', null);
            // remove title which contain template name after initialisation
            $('.select2-selection__rendered').attr('title','');
            $input.on('select2:select', function(e) {
              loadData(renderData, 1, elementsPerPage, e.params.data.text);
            });
            $input.on('select2:unselect', function(e) {
              loadData(renderData, 1, elementsPerPage, null);
            });
          });

      $searchBtn.data('api')
          .get('/api/template-types/')
          .done(function(data) {
            var perms = $.viewpoint.user.permissions;
            $.each(data.results, function (_i, type) {
              if (type.id == 'nitro' && !perms.nitro_jobs_create) {
                return;
              }
              if (type.id == 'remote_viz' && !perms.remote_visualization_jobs_create) {
                return;
              }
              $('#template-type').append(
                '<option value="' + type.id + '">' + type.name + '</option>'
              );
            });
          });

      $('#template-type').select2({
        minimumResultsForSearch: -1,
        width: '160px',
        escapeMarkup: function(m) {
          return m;
        },
        templateResult: formatJobType,
        templateSelection: formatJobType
      });
      $jobTypeDialog.find('.filter select').select2({
        minimumResultsForSearch: -1,
        width: '160px',
      });
      $('#template-type').on('select2:select', function(e) {
        loadData(renderData, 1, elementsPerPage);
      });
      $jobTypeDialog.find('.filter select').on('select2:select', function(e) {
        localStorage.setItem('selectAppTmplOwnerFilter', $(this).val());
        loadData(renderData, 1, elementsPerPage);
      });
    });

    $jobTypeDialog.find('.show-search-box-btn').click(function(e) {
      var $searchContainer = $jobTypeDialog.find('.search-container');
      var $searchBtn = $jobTypeDialog.find('.show-search-box-btn');
      var $filterContainer = $jobTypeDialog.find('.filter');

      if ($searchContainer.is(':hidden')) {
        $filterContainer.find('select').select2('close');
        $filterContainer.hide();
        $searchContainer
          .show()
          .animate({width: '180px'}, 500, function() {
            $searchContainer.find('select').select2('open');
          });
        $searchBtn.addClass('active');
      } else {
        $searchContainer.find('select').select2('close');
        $searchContainer.hide();
        $searchContainer.removeAttr('style');
        $filterContainer
          .show()
          .animate({}, 500, function() {});
        $searchBtn.removeClass('active');
      }
    });

    $('#list-view-Btn').click(function (event) {
      event.preventDefault();
      currentViewClass = 'list-group-item';
      $jobTypeDialog.find('#templates .item').addClass('list-group-item');
      $jobTypeDialog.find('#templates .item').removeClass('grid-group-item');
      $jobTypeDialog.find('#grid-view-Btn')
          .removeClass('selected-view')
          .removeClass('active');
      $(this)
          .addClass('selected-view')
          .addClass('active');
    });

    $('#grid-view-Btn').click(function (event) {
      event.preventDefault();
      currentViewClass = 'grid-group-item';
      $jobTypeDialog.find('#templates .item').removeClass('list-group-item');
      $jobTypeDialog.find('#templates .item').addClass('grid-group-item');
      $jobTypeDialog.find('#list-view-Btn')
          .removeClass('selected-view')
          .removeClass('active');
      $(this)
          .addClass('selected-view')
          .addClass('active');
    });

  function formatJobType(jobType) {
    if (!jobType.id) {
      return jobType.text;
    }
    var typeIcon = window.getJobTypeIcon(jobType.id);
    return typeIcon + jobType.text;
  }

  function loadData(fn, num, elementsPerPage, templateName) {
    $jobTypeDialog.find('.modal-content').loader(new Viewpoint.API());
    var api = $jobTypeDialog.find('.modal-content').data('api');
    var url = '/api/templates/?published=True&page_size=' + elementsPerPage + '&page=' + num;

    var type = $('#template-type').select2('val');
    var filterByOwner = $jobTypeDialog.find('.filter select').select2('val');

    if (type) {
      url += '&type=' + type;
    }
    var name = $jobTypeDialog.find('.search-container').find('select').select2('val');
    if (templateName) {
      url += '&name__icontains='+ templateName;
    } else if (typeof templateName === 'undefined' && name) {
      url += '&name__icontains='+ name;
    }
    if (filterByOwner) {
      switch (filterByOwner) {
        case 'shared':
          url += '&owner__ne='+ $.viewpoint.user.name;
          break;
        case 'owned':
          url += '&owner='+ $.viewpoint.user.name;
          break;
      }
    }
    var perms = $.viewpoint.user.permissions;
    api.get(url)
      .done(function (data, textStatus, jqXHR) {
        var count = data.count;
        data.results = $(data.results)
          .map(function(i, e) {
            if (e.type == 'nitro' && !perms.nitro_jobs_create) {
              --count;
              return null;
            }
            if (e.type == 'remote_viz' && !perms.remote_visualization_jobs_create) {
              --count;
              return null;
            }
            return e;
          }).get();

        $jobTypeDialog.find('#templates').empty();
        fn(data);
        updatePaginator(count, elementsPerPage, num);
      })
      .fail(function (jqXHR, textStatus, errorThrown) {
        console.error(jqXHR.responseJSON.detail);
      });
  }

  function renderData(data) {
    var strings = [];
    for (var i = 0; i < data.results.length; ++i) {
      var typeIcon = window.getJobTypeIcon(data.results[i].type);
      strings.push(
        '<div class="item  col-xs-4 col-lg-4 ' + currentViewClass + '">',
          '<div class="type-item" data-template-url="' + data.results[i].url + ' "title="' + data.results[i].name +'">',
            '<div class="caption">',
              '<div class="file-name-container">',
                '<div class="jobtype-container">' + typeIcon + '</div>',
                '<div class="file-name">',
                  data.results[i].name,
                '</div>',
              '</div>',
              '<span class="file-date">',
                formatDate(data.results[i].date),
              '</span>',
              '<span class="file-owner">',
                data.results[i].owner,
              '</span>',
            '</div>',
          '</div>',
        '</div>');
    }
    $jobTypeDialog.find('#templates').append(strings.join(''));
    selectFileListener();
  }

  function formatDate(date) {
    var d = new Date(date);
    return d.getFullYear() + "/" + (d.getMonth() + 1) + "/" + d.getDate();
  }

  function selectFileListener() {
    $('#job-type-dialog #templates .item .type-item').click(function (event) {
      $('#job-type-dialog #templates .item .type-item').removeClass('selected-file');
      $(this).addClass('selected-file');
      // get search parameters
      var searchParams = _.pick(window.workloadSeachParams, _.identity);
      var path = "/create_job?path_previous=" + $.url('path', window.location.href);
      if (searchParams && _.size(searchParams))
        path += ('&filters=' + JSON.stringify(searchParams));
      path += "&t=" + $(this).attr("data-template-url");
      window.location.href = path;
    });
  }

  function removeAllTemplates() {
    $jobTypeDialog.find('#templates').empty();
  }

  function showSelectJobDialog() {
    removeAllTemplates();
    createPaginator();
    $jobTypeDialog.find('.filter select').select2('val', localStorage.getItem('selectAppTmplOwnerFilter') || 'all');
    loadData(renderData, 1, elementsPerPage);
  }

  function createPaginator() {
    $jobTypeDialog.find('.paginator-container').bootpag({
      total: 1,     // visible pagination
      maxVisible: 7,
      leaps: true,         // next/prev leaps through maxVisible
      firstLastUse: true
    }).on('page', function(event, num) {
      loadData(renderData, num, elementsPerPage);
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $jobTypeDialog.find('.paginator-container').bootpag({
      total: total, // total pages
      page: currentRow            // default page
    }).show();
  }

  $('.count-entries select').on('change', function (e) {
    var option = $("option:selected", this);
    elementsPerPage = this.value;
    loadData(renderData, 1, elementsPerPage);
  });

})();
