// Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

/*
    * Jobs.js used in dashboard, job details, workload list
    * Created Date: 11/18/2014
    * Created By: Abhinavr
    * Version History:
    * 11/18/2014: [Abhinavr] Moved all the Jobs specific methods/functions from main.js to Jobs.js
*/
var persistedSortingState = {
        'colname': null,
        'sortState': -1,
        'isEnabled': true
    };
function checkFilterData() {
    "use strict";
    var priorityFrom, priorityTo, rankFrom, rankTo, StartDate, EndDate, re, eDate, sDate, wallclockStart, wallclockStartDay, wallclockStartHr, wallclockStartMin, wallclockStartSec, wallclockEnd, wallclockEndDay, wallclockEndHr, wallclockEndMin, wallclockEndSec;
    priorityFrom = 0;
    priorityFrom = parseInt($('#priorityFrom').val(), 10);
    priorityTo = 0;
    priorityTo = parseInt($('#priorityTo').val(), 10);
    if (priorityFrom !== 0 && priorityFrom > priorityTo && priorityTo !== 0) {
        alert(gettext('Priority: Please enter correct Range'));
        return false;
    }
    rankFrom = 0;
    rankFrom = parseInt($('#rankFrom').val(), 10);
    rankTo = 0;
    rankTo = parseInt($('#rankTo').val(), 10);
    if (rankFrom !== 0 && rankFrom > rankTo && rankTo !== 0) {
        alert(gettext('Rank: Please enter correct Range'));
        return false;
    }
    StartDate = $('#txtStartDate').val() || '';
    EndDate = $('#txtEndDate').val() || '';
    re = /-/gi;
    StartDate = StartDate.replace(re, '/');
    EndDate = EndDate.replace(re, '/');
    eDate = new Date(EndDate);
    sDate = new Date(StartDate);
    if (StartDate !== '' && EndDate !== '' && sDate > eDate) {
        alert(gettext('Please ensure that the End Date is greater than or equal to the Start Date.'));
        return false;
    }
    wallclockStart = 0;
    wallclockStartDay = 0;
    wallclockStartDay = parseInt($('#wallclockStart #day').val(), 10);
    wallclockStartHr = 0;
    wallclockStartHr = parseInt($('#wallclockStart #hr').val(), 10);
    wallclockStartMin = 0;
    wallclockStartMin = parseInt($('#wallclockStart #min').val(), 10);
    wallclockStartSec = 0;
    wallclockStartSec = parseInt($('#wallclockStart #sec').val(), 10);
    wallclockEnd = 0;
    wallclockEndDay = 0;
    wallclockEndDay = parseInt($('#wallclockEnd #today').val(), 10);
    wallclockEndHr = 0;
    wallclockEndHr = parseInt($('#wallclockEnd #tohr').val(), 10);
    wallclockEndMin = 0;
    wallclockEndMin = parseInt($('#wallclockEnd #tomin').val(), 10);
    wallclockEndSec = 0;
    wallclockEndSec = parseInt($('#wallclockEnd #tosec').val(), 10);
    wallclockStart = parseInt(wallclockStartDay, 10) * 86400 + parseInt(wallclockStartHr, 10) * 3600 + parseInt(wallclockStartMin, 10) * 60 + parseInt(wallclockStartSec, 10);
    wallclockEnd = parseInt(wallclockEndDay, 10) * 86400 + parseInt(wallclockEndHr, 10) * 3600 + parseInt(wallclockEndMin, 10) * 60 + parseInt(wallclockEndSec, 10);
    if (wallclockStart !== 0 && wallclockStart > wallclockEnd && wallclockEnd !== 0) {
        alert(gettext("Please enter valid wallclock parameter: 'From' value must be equal to or less than the 'To' value."));
        return false;
    }
    return true;
}

function onSort(colName) {
    "use strict";
    var colId = $(colName).attr('id');
    persistedSortingState.colname = colId;
    $('#pageNumber').val(1);
    switch (colId) {
    case 'job_name':
        sortingState.jobid = -sortingState.jobid;
        persistedSortingState.sortState = sortingState.jobid;
        break;
    case 'user_name':
        sortingState.jobname = -sortingState.jobname;
        persistedSortingState.sortState = sortingState.jobname;
        break;
    case 'job_custom_name':
        sortingState.jobcustomname = -sortingState.jobcustomname;
        persistedSortingState.sortState = sortingState.jobcustomname;
        break;
    case 'priority':
        sortingState.priority = -sortingState.priority;
        persistedSortingState.sortState = sortingState.priority;
        break;
    case 'job_start_datetime':
        sortingState.job_start_datetime = -sortingState.job_start_datetime;
        persistedSortingState.sortState = sortingState.job_start_datetime;
        break;
    case 'job_submit_datetime':
        sortingState.job_submit_datetime = -sortingState.job_submit_datetime;
        persistedSortingState.sortState = sortingState.job_submit_datetime;
        break;
    case 'job_state':
        sortingState.jobstatus = -sortingState.jobstatus;
        persistedSortingState.sortState = sortingState.jobstatus;
        break;
    case 'rank':
        sortingState.rank = -sortingState.rank;
        persistedSortingState.sortState = sortingState.rank;
        break;
    case 'wallclock_seconds':
        sortingState.wallclock = -sortingState.wallclock;
        persistedSortingState.sortState = sortingState.wallclock;
        break;
    case 'processor_count':
        sortingState.processor_count = -sortingState.processor_count;
        persistedSortingState.sortState = sortingState.processor_count;
        break;
    case 'node_count':
        sortingState.node_count = -sortingState.node_count;
        persistedSortingState.sortState = sortingState.node_count;
        break;
    default:
        break;
    }
    onPageClick();
    setupSortClass(persistedSortingState);
}


function drawResSumChart(cb) {
  $.getJSON('/rpc/getresourcesummary/', function (data) {
    if (!data.messages) {
      var totalStateCnt = 0;
      var chartData = {};
      chartData.type = 'donut';
      chartData.columns = [];
      for (var i in data) {
        totalStateCnt += Number(data[i].count);
      }

      var resourcesTitle = totalStateCnt + ' Node';
      if (totalStateCnt == 0) {
        resourcesTitle = 'No Data';
      } else if (totalStateCnt > 1) {
        resourcesTitle += 's';
      }
      $('#countNodes').text(resourcesTitle);

      var colorScheme = [];
      for (var i in data) {
        colorScheme.push(getColorByNodeState(data[i].state));
        chartData.columns.push([
          data[i].state,
          data[i].count
        ]);
      }

      var selectNodePie = function(state, newPage) {
        var url = '/nodelist/?resourceState=' + state;
        if (newPage) {
          window.open(url);
        } else {
          window.location.href = url;
        }
      };

      chartData.onclick = function (d) {
        selectNodePie(d.id);
      };

      // Draw the data
      var chart = c3.generate({
        bindto: '#resourcepanelbody',
        size: {
          height: 130
        },
        data: chartData,
        legend: {
          position: 'right',
          item: {
            onclick: function (id) {
              selectNodePie(id);
            }
          }
        },
        donut: {
          width: 25,
          label: {
            format: function (value, ratio, id) {
              return value;
            }
          }
        },
        tooltip: {
          format: {
            value: function (value, ratio, id) {
              var pctFormat = d3.format('%');
              return value + ' (' + pctFormat(ratio) + ')';
            }
          }
        },
        padding: {
          bottom: 15
        },
        color: {
          pattern: colorScheme
        }
      });

      setTimeout(function() {
        correctNodesChartLabels();

        d3.selectAll('#resourcepanelbody .c3-legend-item')
          .attr('transform', function() {
            return "translate(20, 0)";
          });

        // for middle click
        $("#resourcepanelbody .c3-chart-arc").mousedown(function(e) {
          e.preventDefault();
          var self = $(e.target);
          if (e.which == 2) {
            var status = d3.selectAll(self).data()[0].data.id;
            selectNodePie(status, true);
          }
        });
      }, 100);
    }
    if ($.isFunction(cb)) {
      cb();
    }
  });
}

function formatDate(dateVal){
  var str = dateVal.replace(/-/g,'/');
  var date = moment.utc(str, 'YYYY/MM/DD H:s').toDate();
  if (moment(date).isValid()) {
    return moment(date).format('MMM DD, H:ss');
  }
}

function drawCpuMemoryChart(cb) {
    $.getJSON('/rpc/getsystemutilization/', function (jsonData) {

      var xArray = [];
      var processor_array= ['CPU'];
      var memory_array= ['Memory'];
      if (!jsonData.messages) {
        if (jsonData == '') {
          $('#chart_div').empty();
          $('#chart_div').append("<div class='noChData'>No Data Found</div>");
        }	else {
        for (var i in jsonData) {
          var timestamp_datetime =jsonData[i].timestamp_datetime.slice(0,15);
          var time = formatDate(timestamp_datetime);
          var processor_dedication = Math.floor(Number(jsonData[i].processor_dedication_percentage));
          var memory_dedication = Math.floor(Number(jsonData[i].memory_dedication_percentage));
          xArray.push(time);
          processor_array.push(processor_dedication)
          memory_array.push(memory_dedication)
        }

        var pctFormat = d3.format('%');
        var chart = c3.generate({
          // CPU: Dedicated System Resources
          bindto: '#chart_div',
          data: {
            columns: [
              memory_array,
              processor_array
            ],
            types: {
              Memory: 'line',
              CPU: 'area'
            }
          },
          point: {
            r: 1
          },
          legend: {
            show: false
          },
          axis: {
            x: {
              type: 'category',
              categories: xArray,
              show: false,
            },
            y: {
              show: false,
              max: 100,
              min: 0,
            }
          },
          grid: {
            y: {
              lines: [
                { value: 0, text: '0', position: 'start' },
                { value: 50, text: '50', position: 'start' },
                { value: 100, text: '100', position: 'start' },
              ]
            }
          },
          padding: {
            bottom: -20
          },
          color: {
            pattern: ["#00457e", "#ccc6ae"]
          },
          tooltip: {
            contents: function (d, defaultTitleFormat, defaultValueFormat, color) {
              if (d[1].value > 0) {
                return this.getTooltipContent(d, defaultTitleFormat, defaultValueFormat, color);
              } else {
                return '<div class="no-data-tooltip">' +
                  '<div class="no-data-tooltip-header">' + xArray[d[0].x] + '</div>' +
                  '<div class="no-data-tooltip-content">' +
                  '0 memory dedicated to jobs usually means that <br> users are not requesting memory when they submit jobs.' +
                '</div></div>';
              }
            },
            format: {
              title: function (i) {
                return xArray[i];
              },
              value: function (value, ratio, id) {
                return pctFormat(value / 100);
              }
            }
          }
        });
      }
    } else {
      $('#chart_div').empty();
      $('#chart_div').append("<div class='noChData'>" + jsonData.messages + "</div>");
    }
    if ($.isFunction(cb)) {
      cb();
    }
  });
}

function windowWasResized() {
  correctJobsChartLabels();
  correctNodesChartLabels();
}

function correctNodesChartLabels() {
  d3.selectAll('#resourcepanelbody .c3-chart-arc>text')
    .attr('transform', function(d, i) {
      var r = 40,
          a = (d.startAngle + d.endAngle) / 2 - (Math.PI / 2);
      return 'translate(' + (Math.cos(a) * r) + "," + (Math.sin(a) * r) + ')';
    });
}

function correctJobsChartLabels() {
  d3.selectAll('#jobpanelbody .c3-chart-arc>text')
    .attr('transform', function(d, i) {
      var r = 40,
          a = (d.startAngle + d.endAngle) / 2 - (Math.PI / 2);
      return 'translate(' + (Math.cos(a) * r) + "," + (Math.sin(a) * r) + ')';
    });
}

function drawJobSumChart(cb) {
  $.getJSON('/rpc/getjobsummary/?format=json', function (data) {
    if (!data.messages) {
      var totalStateCnt = 0;
      var completedCnt = 0;
      var chartData = {};
      chartData.type = 'donut';
      chartData.columns = [];

      for (var i in data) {
        if (data[i].queue_status.toLowerCase() !== 'completed') {
          totalStateCnt += Number(data[i].count);
        } else {
          completedCnt += Number(data[i].count);
        }
      }

      var colorScheme = [];
      var singleBar = false;
      if (totalStateCnt == 0 && completedCnt > 0) {
        $('#countJobs').text('All jobs in a completed state');
        chartData.columns.push([
          'COMPLETED',
          completedCnt
        ]);
        colorScheme.push(getColorByQueueStatus('completed'));
        singleBar = true;
      } else {
        var jobsTitle = totalStateCnt + ' Job';
        if (totalStateCnt == 0) {
          jobsTitle = 'No Jobs';
        } else if (totalStateCnt > 1) {
          jobsTitle += 's';
        }
        $('#countJobs').text(jobsTitle);

        var piesCount = 0;
        for (var i in data) {
          if (data[i].queue_status.toLowerCase() !== 'completed') {
            piesCount++;
            colorScheme.push(getColorByQueueStatus(data[i].queue_status));
            chartData.columns.push([
              data[i].queue_status,
              data[i].count
            ]);
          }
        }
        singleBar = (piesCount == 1);
      }

      var selectJobPie = function(status, newPage) {
        var url = '/workload/?queue_status=' + status.toUpperCase();
        if (newPage) {
          window.open(url);
        } else {
          window.location.href = url;
        }
      };

      chartData.onclick = function (d) {
        selectJobPie(d.id);
      };

      // Draw the data
      var chart = c3.generate({
        bindto: '#jobpanelbody',
        size: {
          height: 130
        },
        data: chartData,
        legend: {
          position: 'right',
          item: {
            onclick: function (id) {
              selectJobPie(id);
            }
          }
        },
        donut: {
          width: 25,
          label: {
            format: function (value, ratio, id) {
              return value;
            }
          }
        },
        tooltip: {
          format: {
            value: function (value, ratio, id) {
              var pctFormat = d3.format('%');
              return value + ' (' + pctFormat(ratio) + ')';
            }
          }
        },
        padding: {
          bottom: 15
        },
        color: {
          pattern: colorScheme
        }
      });

      setTimeout(function() {
        correctJobsChartLabels();

        d3.selectAll('#jobpanelbody .c3-legend-item')
          .attr('transform', function() {
            return "translate(20, 0)";
          });

        // for middle click
        $("#jobpanelbody .c3-chart-arc").mousedown(function(e) {
          e.preventDefault();
          var self = $(e.target);
          if (e.which == 2) {
            var status = d3.selectAll(self).data()[0].data.id;
            selectJobPie(status, true);
          }
        });
      }, 100);
    } else {

    }
    if ($.isFunction(cb)) {
      cb();
    }
  });
}

function validate_user_priority(ctrl, event) {
    "use strict";
    var keycode = event.keyCode || event.charCode;
    if (keycode === 8 || keycode === 46 || keycode === 9 || keycode === 45) {
    } else if (!(keycode >= 48 && keycode <= 57)) {
        event.preventDefault();
    }
}
function chkData(ctrl) {
    "use strict";
    var reg, x, msg;
    x = document.getElementById(ctrl);
    if (!x) {
        return true;
    }
    x = x.value;
    if (ctrl === 'qos_txtBox') {
        reg = /^[A-Za-z0-9]+$/;
        msg = gettext('Quality Of Service should have alphanumeric characters only');
    } else {
        reg = /^[\-]?[0-9]+$/;
        if (ctrl === 'job_priority') {
            msg = gettext('User Job Priority should have numeric characters only');
        } else {
            msg = gettext('System Priority should have numeric characters only');
        }
    }
    if (!x.match(reg) && x !== '') {
        alert(msg);
        return false;
    }
    return true;
}
function validate_QOS_OnKeyPress(Id, obj, KeyEvent) {
    "use strict";
    var input, keycode, evt;
    input = document.getElementById(Id).value;
    evt = KeyEvent || window.event;
    keycode = evt.which || evt.keyCode;
    if (keycode === 8 || keycode === 46 || keycode === 9) {
    } else if (!(keycode >= 48 && keycode <= 57) && !(keycode >= 65 && keycode <= 90) && !(keycode >= 97 && keycode <= 122)) {
        evt.preventDefault();
    }
}
function validate_user_priority_OnKeyUp(ctrl, max, min, e) {
    "use strict";
    var input, evt;
    input = parseInt(document.getElementById(ctrl).value, 10);
    evt = e || window.event;
    if (input < max || input > min) {
        alert(gettext('Job User Priority:Value should be between {0} to {1}').replace("{0}", -1024).replace("{1}", 1024));
        document.getElementById(ctrl).value = 0;
        evt.preventDefault();
        return false;
    }
    chkData('job_priority');
    return true;
}

function validateJobPriority(event) {
    "use strict";
    var chCode = ('charCode' in event) ? event.charCode : event.keyCode;
    if (chCode > 0 && // check that key code exists
            chCode != 32 && chCode != 45 && // allow space and dash
            !(chCode >= 48 && chCode <= 57)// allow 0-9
            ) {
        event.preventDefault();
    }
}

function disableWorkloadContentOnPageLoad() {
    "use strict";
    $('.form-control').prop('readonly', 'true');
    $('.filter').prop('disabled', true);
    $('#searchCat').prop('disabled', true);
    $('#queueStatus').prop('disabled', true);
    $('#dateTimeRange').prop('disabled', true);
    $('#searchBtn').prop('readonly', 'true');
    $('#workloadRefresh').prop('readonly', 'true');
    $('#searchCategory').prop('disabled', true);
    $('#jobType').prop('disabled', true);
    $('#search_btn').attr('disabled', 'disabled');
}
function enableWorkloadContentAfterPageLoad() {
    "use strict";
    $('.form-control').removeAttr('readonly');
    $('.filter').prop('disabled', false);
    $('#searchCat').prop('disabled', false);
    $('#queueStatus').prop('disabled', false);
    $('#dateTimeRange').prop('disabled', false);
    $('#searchBtn').removeAttr('readonly');
    $('#workloadRefresh').prop('disable', false);
    $('#searchCategory').prop('disabled', false);
    $('#jobType').prop('disabled', false);
    $('#search_btn').removeAttr('disabled');
}
function durationContent() {
    "use strict";
    var mainInput, days, hrs, min, sec;
    mainInput = $('#durInput');
    days = parseInt($('.popover #dInput').val(), 10);
    hrs = parseInt($('.popover #hInput').val(), 10);
    min = parseInt($('.popover #mInput').val(), 10);
    sec = parseInt($('.popover #sInput').val(), 10);
    if (days < 10) {
        days = '0' + days;
    } else if (isNaN(days)) {
        days = '00';
    }
    if (hrs < 10) {
        hrs = '0' + hrs;
    } else if (isNaN(hrs)) {
        hrs = '00';
    }
    if (min < 10) {
        min = '0' + min;
    } else if (isNaN(min)) {
        min = '00';
    }
    if (sec < 10) {
        sec = '0' + sec;
    } else if (isNaN(sec)) {
        sec = '00';
    }
    mainInput.val(days + ':' + hrs + ':' + min + ':' + sec);
    mainInput.trigger('change');
    document.getElementById('durationContent').style.display = 'none';
}

function cancelpopover() {
    "use strict";
    var divDuration = document.getElementById('durationContent');
    divDuration.style.display = 'none';
}
function clearpopover() {
    "use strict";
    document.getElementById('dInput').value = '00';
    document.getElementById('hInput').value = '00';
    document.getElementById('mInput').value = '00';
    document.getElementById('sInput').value = '00';
}
function hideNodes(nodename) {
    "use strict";
    $('#addnode_' + nodename).remove();
    var parentObj = $('#reqNode_' + nodename).parent();
    $('#reqNode_' + nodename).remove();
    parentObj.append($('<span/>').attr('id', 'reqNode_' + nodename).html('+').addClass('add').attr('onclick', 'toggleNodes(this,\'' + nodename + '\',0)'));
    $('#nodeListContainer').trigger('change');
}
function toggleNodes(obj, str, status) {
    "use strict";
    var parentobj;
    if (status === 1) {
        hideNodes(str);
    } else {
        str = escape(str);
        $('#nodeListContainer').append('<span class=\'label label-default\' id=\'addnode_' + str + '\'><a id=\'node_' + str + '\'>' + str + '</a><span class=\'glyphicon glyphicon-remove\' id=\'destroyNode_' + str + '\' onclick=\'hideNodes("' + str + '")\'></span></span>');
        parentobj = $(obj).parent();
        $(obj).remove();
        parentobj.append($('<span/>').attr('id', 'reqNode_' + str).html('-').addClass('add').attr('onclick', 'hideNodes("' + str + '")'));
        $('#nodeListContainer').trigger('change');
    }
}
function hideFeatures(resourcename) {
    "use strict";
    $('#addfeature_' + resourcename).remove();
    var parentObj = $('#reqFeature_' + resourcename).parent();
    $('#reqFeature_' + resourcename).remove();
    parentObj.append($('<span/>').attr('id', 'reqFeature_' + resourcename).html('+').addClass('add').attr('onclick', 'toggleFeatures(this,\'' + resourcename + '\',0)'));
    $('#featuresContainer').trigger('change');
}
function toggleFeatures(obj, str, status) {
    "use strict";
    if (status === 1) {
        hideFeatures(str);
    } else {
        var parentobj;
        $('#featuresContainer').append('<span class=\'label label-default add_features\' id=\'addfeature_' + str + '\'><a  id=\'feature_' + str + '\' title=\'' + str + '\'>' + str + '</a><span class=\'glyphicon glyphicon-remove\' id=\'destroyFeature_' + str + '\' onclick=\'hideFeatures("' + str + '")\'></span></span>');
        parentobj = $(obj).parent();
        $(obj).remove();
        parentobj.append($('<span/>').attr('id', 'reqFeature_' + str).html('-').addClass('add').attr('onclick', 'hideFeatures("' + str + '")'));
        $('#featuresContainer').trigger('change');
    }
}

function durationDay(ctrl, min, max) {
    "use strict";
    var input;
    input = parseInt(document.getElementById(ctrl).value, 10);
    if (input < min || input > max) {
        clearInterval(intervalId);
        document.getElementById(ctrl).value = min;
        return false;
    }
    return true;
}
function incrementDay(id) {
    "use strict";
    var value;
    if (durationDay(id, 0) === true) {
        value = parseInt($('#' + id).val(), 10);
        if (isNaN(value)) {
            $('#' + id).val(0);
        } else {
            $('#' + id).val(value + 1);
        }
    }
}
function durationHour(ctrl, min, max) {
    "use strict";
    var min = 0;
    var max = 23;
    var input = parseInt(document.getElementById(ctrl).value, 10);
    if (input < min || input > max) {
        alert(gettext("Duration hour {0}s:Value should be between {1} - {2}{0}").replace('{0}', "'").replace('{1}', 0).replace('{2}', 23).replace('{0}', "'"));
        clearInterval(intervalId);
        document.getElementById(ctrl).value = max;
        return false;
    }
    return true;
}
function incrementHour(id) {
    "use strict";
    if (durationHour(id, 0, 22) === true) {
        var value = parseInt($('#' + id).val(), 10);
        if (isNaN(value)) {
            $('#' + id).val(0);
        } else {
            $('#' + id).val(value + 1);
        }
    }
}
function durationSeconds(ctrl, min, max) {
    "use strict";
    var min = 0;
    var max = 59;
    var input = parseInt(document.getElementById(ctrl).value, 10);
    if (input < min || input > max) {
        alert(gettext("Duration seconds:Value should be between {0} - {1}").replace('{0}', 0).replace('{1}', 59));
        clearInterval(intervalId);
        document.getElementById(ctrl).value = max;
        return false;
    }
    return true;
}
function durationMinutes(ctrl, min, max) {
    "use strict";
    var min = 0;
    var max = 59;
    var input = parseInt(document.getElementById(ctrl).value, 10);
    if (input < min || input > max) {
        alert(gettext("Duration minutes:Value should be between {0} - {1}").replace('{0}', 0).replace('{1}', 59));
        clearInterval(intervalId);
        document.getElementById(ctrl).value = max;
        return false;
    }
    return true;
}
function incrementMinutes(id) {
    "use strict";
    if (durationMinutes(id, 0, 58) === true) {
        var value = parseInt($('#' + id).val(), 10);
        if (isNaN(value)) {
            $('#' + id).val(0);
        } else {
            $('#' + id).val(value + 1);
        }
    }
}
function incrementSeconds(id) {
    "use strict";
    if (durationSeconds(id, 0, 58) === true) {
        var value = parseInt($('#' + id).val(), 10);
        if (isNaN(value)) {
            $('#' + id).val(0);
        } else {
            $('#' + id).val(value + 1);
        }
    }
}
function putEllipses(strVal) {
    "use strict";
    var length = strVal.length;
    if (length > 10) {
        strVal = strVal.substring(0, 10);
        strVal += ' ...';
    }
    return strVal;
}

$(document).click(function (e) {
    "use strict";
    $('.close-popover-link').each(function () {
        if (!$(this).is(e.target) && $(this).has(e.target).length === 0 && $('.popover').has(e.target).length === 0) {
            return;
        }
    });
});

function initCPUStatisticChart(div, jobId) {
  $('.cpu-statistics-settings').hide();
  var granularity = cpuGranularity;
  var diapason = cpuDiapason;

  var api = $('.cpu-statistics.api').data('api');
  var url = '/api/jobs/' + jobId + '/cpu-statistics/' +
    '?granularity=' + granularity +
    '&range=' + diapason +
    '&page_size=10000';
  api.get(url)
    .done(function(data) {
      cpuStatisticReceived(div, data, granularity, diapason);
    })
    .fail(function(error) {
        console.error(error);
    })
    .always(function() {
      $('.cpu-statistics-settings').show();
    });
}

var cpuGranularity = 1;
var cpuDiapason = 60;
var cpuAutoRefresh = true;

function initCPUStatistic() {
  var div = $('#cpu-statistics-container')[0];
  var jobId = $('#job_name').val();
  var reloadTaskId = null;
  var timeout = cpuGranularity;

  if ($('#cpu-statistics-container-unavailable').length) {
      cpuAutoRefresh = false;
  } else {
      initCPUStatisticChart(div, jobId);
      reloadTaskId = window.setInterval(reloadCpuGraph, timeout * 60 * 1000);
  }

  function reloadCpuGraph() {
    if (!$('#scheduler-info-viewer').length) {
      initCPUStatisticChart(div, jobId);
    }
  }

  $('body').on('click','.popover-content .cpu-statistics-refresh', function(){
    reloadCpuGraph();
    $("[data-toggle=cpu-statistics-controls]").popover('hide');
  });

  $('body').on('change', '.popover-content .cpu-statistics-granularity', function(){
    timeout = parseInt($(this).val());
    cpuGranularity = $('.popover-content .cpu-statistics-granularity').val();
    if ($('.popover-content #graph-refresh').is(':checked')) {
      clearInterval(reloadTaskId);
      reloadTaskId = window.setInterval(reloadCpuGraph, timeout * 60 * 1000);
    }
  });

  $('body').on('change', '.popover-content .cpu-statistics-diapason', function(){
      cpuDiapason = $('.popover-content .cpu-statistics-diapason').val();
  });

  $('.cpu-statistics-config').on('shown.bs.popover', function () {
    $('[data-toggle="tooltip"]').tooltip();
    $('.popover-content .cpu-statistics-granularity').val(cpuGranularity);
    $('.popover-content .cpu-statistics-diapason').val(cpuDiapason);
    $('.popover-content #graph-refresh').prop('checked', cpuAutoRefresh);
  })

  $('body').on('show.bs.modal', '#file-viewer', function(){
    clearInterval(reloadTaskId);
  });

  $('body').on('hide.bs.modal', '#file-viewer', function(){
    if (cpuAutoRefresh){
      reloadTaskId = window.setInterval(reloadCpuGraph, timeout * 60 * 1000);
    }
  });

  $('body').on('change', '.popover-content #graph-refresh', function(){
    if (this.checked) {
    cpuAutoRefresh = true;
    reloadTaskId = window.setInterval(reloadCpuGraph, timeout * 60 * 1000);
    }
    else {
      cpuAutoRefresh = false;
      clearInterval(reloadTaskId);
      reloadTaskId = null;
    }
  });
}

function cpuStatisticReceived(div, data, granularity, diapason) {
  if (data.count == 0) {
    $('#cpu-statistics-container').text('No utilization metrics found for this job');
    $('#cpu-statistics-container').addClass('unavailable');
    $('.cpu-statistics').css('height', 100);
    $('.cpu-statistics').css('width', 480);
    return;
  }
  $('#cpu-statistics-container').removeClass('unavailable');

  data = data.results;

  var chartData = {};
  chartData.type = 'donut';
  chartData.columns = [];

  var stamps = ['x'];
  var dedicatedCpu = ['Dedicated'];
  var utilizedCpu = ['Utilized'];

  var max = 0;
  $.each(data, function(_i, val) {
    var m = Math.max(val.effective_dedicated_processors, val.effective_utilized_processors);
    if (max < m) {
      max = m;
    }
  });
  max = Math.round(100 * max) * 0.01; // Throwing out the fractions which are less than 0.01
  max = Math.ceil(max);
  if (Math.abs(max) < 1e-3) {
    max = 1.0;
  }

  // data - real data array
  // pctData - percents data array found on 100%=pct100
  //

  var xLabel = function(date) {
    var time = date.getHours() + '';
    if (time.length == 1) {
      time = '0' + time;
    }
    var minutes = date.getUTCMinutes() + '';
    if (minutes.length == 1) {
      minutes = '0' + minutes;
    }
    time += ':' + minutes;
    return time;
  }

  var min_x, max_x;
  for (var i=0; i<data.length; i++) {
    var momDate = moment(data[i].timestamp_datetime, "YYYY-MM-DD HH-mm-ss ZZ");
    var date = momDate.toDate();

    // Math.round(num * 100) / 100 - algorithm is used for
    // rounding: 63.33333333 -> 63.33
    var pctDedicated = Math.round(100 * data[i].effective_dedicated_processors) / 100;
    var pctUtilized = Math.round(100 * data[i].effective_utilized_processors) / 100;
    stamps.push(date);
    dedicatedCpu.push(pctDedicated);
    utilizedCpu.push(pctUtilized);
    /*if (i == 0) {
      min_x = time;
    }
    if (i == data.length - 1) {
      max_x = time;
    }*/
  }

  var yLines = [{
    value: 0,
    text: '0',
    position: 'start'
  }];
  var format = d3.format(".2f");
  var yStep = max / 4;
  for (var i=1; i<=4; i++) {
    yLines.push({
      value: yStep*i,
      text: format(yStep*i),
      position: 'start'
    });
  }

  var xLines = [];
  // For too many points (> 100) do not show grid lines
  if (stamps.length < 100) {
    for (var x in stamps) {
      if (stamps[x] != 'x') {
        xLines.push({
          value: stamps[x]
        });
      }
    }
  }

  var p1 = stamps[1],
      p2 = stamps[stamps.length - 1];
  var minDiff = Math.abs(p2.getTime() - p1.getTime()) / 60000;
  var xTicks = 13;
  if (minDiff + 1 < xTicks) {
    xTicks = minDiff + 1;
  }

  var chartData = {};
  chartData.x = 'x';
  chartData.columns = [
    stamps,
    dedicatedCpu,
    utilizedCpu
  ];
  chartData.names = {
    Dedicated: 'Dedicated CPUs',
    Utilized: 'Utilized CPUs'
  };
  chartData.types = {
    Dedicated: 'area-step',
    Utilized: 'area-step'
  };

  var chart = c3.generate({
    // Chart on Details
    bindto: '#cpu-statistics-container',
    size: {
      height: 220,
      width: 470
    },
    data: chartData,
    point: {
      r: 0
    },
    axis: {
      x: {
        show: true,
        type: 'timeseries',
        tick: {
          count: xTicks,
          format: function (d) {
            return xLabel(d);
          }
        }
      },
      y: {
        show: false,
        max: max,
        min: 0,
        label: 'CPU Count'
      }
    },
    grid: {
      x: {
        lines: xLines
      },
      y: {
        lines: yLines
      }
    },
    padding: {
      left: 20,
      top: 20,
      right: 20
    },
    tooltip: {
      format: {
        title: function (d) {
          return 'Time: ' + xLabel(d);
        }
      }
    }
  });

  $('.cpu-statistics').css('height', 220);

  // make y-grid-lines thin
  setTimeout(function() {
    d3.selectAll('.cpu-statistics .c3-ygrid-lines .c3-ygrid-line')
      .attr('transform', function(d, i) {
        return 'translate(0.5,0.5)';
      });
    d3.selectAll('.cpu-statistics .c3-xgrid-lines .c3-xgrid-line')
      .attr('transform', function(d, i) {
        return 'translate(0.5,0.5)';
      });
  }, 300);
}

function handleTopMessages() {
    $.each(['.alert-success', '.alert-danger'], function(_i, selector) {
        var $jq = $(selector);
        if ($jq.is(":visible")) {
            var elm = $jq[0];
            if ($(elm).innerHeight() > 80) {
                $jq.css('overflow-y', 'scroll');
                $jq.css('height', '80px');
            }
        }
    });
}
