/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */
;(function($) {
  'use strict';
  var $openFile = $('#open-file');
  $openFile.find('.open-file-container').loader(new Viewpoint.API());
  var api = $openFile.find('.open-file-container').data('api');

  var selectedElementUrl = null;
  var currentPath = null;
  var currentPage = 1;
  var currentOrder = 'display_name';
  var currentOrderDirection = '';
  var currentFilterField = 'display_name';
  var currentFilterValue;
  var itemSelected = 'folder';

  $(function() {
    var initConfig = $.viewpoint.modal.payload();
    var initPath = initConfig.path;
    var startPath = initConfig.startPath;
    var initTitle = initConfig.title;
    var renderDate = window.renderDate;
    var renderSize = window.renderSize;
    var fileName = initConfig.fileName;

    $.viewpoint.modal.payload({ result: initPath });

    $openFile.find('.select-btn').prop('disabled', true);

    if (initTitle) {
      $('#open-file .modal-title').text(initTitle);
    }

    if (!initConfig.showFiles) {
      $openFile.find('.new-file-btn').hide();
      $openFile.find('.open-file-container').addClass('disabled-files');
    }
    if (!initConfig.showDirectories) {
      $openFile.find('.open-file-container').addClass('disabled-folders');
    }

    var fileTemplate = Handlebars.compile($openFile.find('#file-template').html());
    var folderTemplate = Handlebars.compile($openFile.find('#folder-template').html());
    var breadcrumbTemplate = Handlebars.compile($openFile.find('#breadcrumb-template').html());

    createPaginator();
    loadInitPath();

    activateFooterHandlers();
    $('.new-folder-btn').click(newFolderClicked);
    $('.new-file-btn').click(newFileClicked);
    $('.folder-create-btn').click(function() {
      $('.new-folder-name-box').find('form').submit();
    });
    $('.new-folder-name-box').find('form').submit(createFolderCallback);
    $('.file-create-btn').click(function() {
      $('.new-file-name-box').find('form').submit();
    });
    $('.new-file-name-box').find('form').submit(createFileCallback);
    $('.home-link-container').click(function(){
      loadHomePath();
    });

    $('#open-file #viewPerPage').change(function() {
      currentPage = 1;
      loadData();
    });

    $('.open-file-clear-filter').click(function() {
      $('#filter-open-file').val('');
      setFilter();
    });

    $('.open-file-container .order-by-name, .open-file-container .order-by-date').click(changeOrder);
    $('.open-file-container .order-by-type, .open-file-container .order-by-size').click(changeOrder);
    $('.open-file-container .filter-btn').click(setFilter);
    $('#filter-open-file').on('keypress', function(event) {
      if (event.which === 13)
        setFilter();
    });

    function activateFooterHandlers() {
      $openFile.find('.close-btn').click(function() {
        $.viewpoint.modal.dismiss({ result: initPath });
      });
      $openFile.find('.select-btn').click(function() {
        var path = '';
        if (selectedElementUrl) {
          path = decodeURI(getAbsolutePathFromUrl(selectedElementUrl));
        } else {
          path = decodeURI(getAbsolutePathFromUrl(getCurrentPath()));
        }
        if (initConfig.forbidEmptyPath && path == '') {
          $.notify('Please select correct path', {
            type: 'warning'
          });
        } else {
          if (fileName) {
            if (itemSelected == 'folder') {
              api.head('/api/fsobject/' + path + '/' + fileName)
                .done(function(){
                  $.viewpoint.confirm('File "' + fileName + '" already exists', 'Do you want to overwrite it?').done(function(result){
                    if (result) {
                      $.viewpoint.modal.dismiss({
                        fileSelected: true,
                        result: path
                      });
                    }
                  });
                })
                .fail(function(){
                  $.viewpoint.modal.dismiss({
                    fileSelected: true,
                    result: path
                  });
                });
            } else if (itemSelected == 'file') {
              $.viewpoint.confirm('Warning', 'Do you want to overwrite existing file?').done(function(result){
                if (result) {
                  $.viewpoint.modal.dismiss({
                    fileSelected: true,
                    result: path,
                    overwrite: true
                  });
                }
              });
            }
          } else {
            $.viewpoint.modal.dismiss({
              fileSelected: true,
              result: path
            });
          }
        }
      });
    }

    function getAbsolutePathFromUrl(url) {
      var result = "";
      var nextFolder = 'folder';

      // http://192.168.53.121:8080/api/directory/<dir_name>//
      //                            <1>     <2>         <3>
      var index = 3;

      while (nextFolder !== '') {
        nextFolder = $.url(index, url);
        if (nextFolder !== '') {
          result += '/' + nextFolder;
        }
        index++;
      }

      return result;
    }

    function loadHomePath() {
      loadData('/api/directory/~/');
    }

    function loadInitPath() {
      var path = initPath;
      if (startPath) {
        path = startPath;
      }
      path = '/api/directory/' + encodeURI(path);
      api.head(path)
        .done(function(){
          loadData(path);
        })
        .fail(function(){
          loadData('/api/directory/~/');
        });
    }

    function createPaginator() {
      $('#open-file .paginator-container').bootpag({
        total: 10,
        maxVisible: 5,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
      }).on('page', function(event, num) {
        // reload
        currentPage = num;
        loadData();
      });
    }

    function updatePaginator(data) {
      if (data.next || data.previous) {
        var pageSize = $('#open-file #viewPerPage').val();
        pageSize = pageSize || 100;
        var pagesCount = Math.ceil(data.count / pageSize);
        if (pagesCount == 0) { pagesCount = 1; }
        $('#open-file .paginator-container').bootpag({
          total: pagesCount, // total pages
          page: currentPage, // default page
          next: 'next',
          prev: 'prev'
        }).show();
        $('#open-file .paginator-container').show();
      } else {
        $('#open-file .paginator-container').hide();
      }
    }

    function renderFileList(data) {
      selectedElementUrl = null;
      var $listGroup = $openFile.find('.files-list');
      $listGroup.empty();
      $(data).each(function(index, e) {
        var fileListElement = fileListElementFactory(e);
        if (fileListElement) {
          $listGroup.append(fileListElement);
        }
      });
      $listGroup.find('.file-item, .folder-item')
          .click(clickElementHandler)
          .hover(function() {
            if ($(this).find('.confirm-deleting').is(':hidden')) {
              $(this).find('.glyphicon-trash').show();
            }
          }, function() {
            $(this).find('.glyphicon-trash').hide();
          });
      $listGroup.find('.glyphicon-trash').click(showConfirm);
      $listGroup.find('.confirm-deleting').click(deleteFolderClicked);
    }

    function fileListElementFactory(config) {
      config.extension = '';
      if (/.*\.(.*)/.test(config.display_name))
        config.extension = config.display_name.match(/.*\.(.*)/)[1];
      if (config.object_type === "DIRECTORY") {
        return folderTemplate(config);
      }
      if (config.object_type === "FILE") {
        return fileTemplate(config);
      }
      return null;
    }

    function renderBreadcrumbs(path) {
      var $breadcrumbs = $openFile.find('.breadcrumb');
      $breadcrumbs.empty();
      var root = $.url('protocol', path) + '://' + $.url('domain', path) + ":" + $.url('port', path) +
        '/' + $.url(1, path) + '/' + $.url(2, path);

      $breadcrumbs.append(breadcrumbTemplate({
        name: 'Remote File System',
        url: root,
        title: 'Remote File System'
      }));

      var nextFolder = 'folder';
      var index = 3;
      var currentUrl = String(root);
      while (nextFolder !== '') {
        nextFolder = $.url(index, path);
        currentUrl += '/' + nextFolder;
        if (nextFolder !== '') {
          var folderName = decodeURIComponent(nextFolder);
          if (folderName.trim() === '') {
            folderName = '" "';
          }
          $breadcrumbs.append(breadcrumbTemplate({
            name: folderName,
            url: currentUrl + '/',
            title: folderName
          }));
          currentPath = currentUrl + '/';
        }
        index++;
      }

      $breadcrumbs.find('.folder').click(clickBreadcrumbHandler);

      $('.breadcrumb-container').data('plugin_rcrumbs', null); // destroy rcrumbs plugin
      $('.breadcrumb-container').rcrumbs({windowResize: false});
      if ($('.breadcrumb-container ul li').not('.show').length)
        $breadcrumbs.prepend(
          $(breadcrumbTemplate({
            name: '...',
            url: '',
            title: ''
          })).addClass('left-ellipsis show'));
      else
        $('.breadcrumb-container .left-ellipsis').remove();
    }

    var clickBreadcrumbHandler = function(event) {
      event.preventDefault();
      var path = $(this).attr('data-url');
      if (path) {
        currentPage = 1;
        loadData(path);
      }
    };

    var clickElementHandler = function(event) {
      event.preventDefault();

      if ($(event.target).is('.glyphicon-trash')) {
        return;
      }

      if ($(this).hasClass('folder-item')) {
        itemSelected = 'folder';
      } else if ($(this).hasClass('file-item')) {
        itemSelected = 'file';
      }

      if ($(this).hasClass('active') && $(this).hasClass('folder-item')) {
        var path = $(this).attr('data-url');
        currentPage = 1;
        loadData(path);
      } else {
        selectedElementUrl = $(this).attr('data-url');
        $openFile.find('.select-btn').prop('disabled', false);
        // If pickFilesOnly==true user cannot select folder
        if ($(this).hasClass('folder-item') && initConfig.pickFilesOnly) {
          $openFile.find('.select-btn').prop('disabled', true);
        }
        // If showFiles==false user cannot select file (but files are visible)
        if ($(this).hasClass('file-item') && !initConfig.showFiles) {
          $openFile.find('.select-btn').prop('disabled', true);
          return;
        }
        if ($(this).hasClass('folder-item') && !initConfig.showDirectories) {
          return;
        }
        $(this).parent().find('a').removeClass('active');
        $(this).addClass('active');
      }
    };

    function getCurrentPath() {
      return currentPath + '/';
    }

    function setFilter(event) {
      currentFilterValue = $('#filter-open-file').val();
      currentPage = 1;
      loadData();
    }

    function changeOrder(event) {
      var orderField = '';
      if ($(event.target).hasClass('order-by-name')) {
        orderField = 'display_name';
      }
      if ($(event.target).hasClass('order-by-date')) {
        orderField = 'date_modified';
      }
      if ($(event.target).hasClass('order-by-type')) {
        orderField = 'extension';
      }
      if ($(event.target).hasClass('order-by-size')) {
        orderField = 'size_bytes';
      }
      if (!orderField) {
        return;
      }

      if (currentOrder == orderField) {
        currentOrderDirection = (currentOrderDirection == '-') ? '': '-';
      } else {
        currentOrderDirection = '';
        currentOrder = orderField;
      }

      $('.order-by-date .order-marker').hide();
      $('.order-by-name .order-marker').hide();
      $('.order-by-type .order-marker').hide();
      $('.order-by-size .order-marker').hide();
      switch (orderField) {
        case 'display_name':
          $('.order-by-name .order-marker').show();
          break;
        case 'date_modified':
          $('.order-by-date .order-marker').show();
          break;
        case 'extension':
          $('.order-by-type .order-marker').show();
          break;
        case 'size_bytes':
          $('.order-by-size .order-marker').show();
          break;
      }

      if (currentOrderDirection == '-') {
        $('.order-marker')
          .removeClass('glyphicon-arrow-up')
          .addClass('glyphicon-arrow-down');
      } else {
        $('.order-marker')
          .removeClass('glyphicon-arrow-down')
          .addClass('glyphicon-arrow-up');
      }

      currentPage = 1;
      loadData();
    }

    function loadData(initUrl, failCallbackPath) {
      $openFile.find('.select-btn').prop('disabled', true);
      var url = initUrl || getCurrentPath();
      var pageSize = $('#open-file #viewPerPage').val();
      pageSize = pageSize || 100;
      url += '?page_size=' + pageSize;
      url += '&page=' + currentPage;
      url += '&ordering=' + currentOrderDirection + currentOrder;
      if (currentFilterValue) {
        url += '&filter-by=' + currentFilterField +
          '&filter-value=' + currentFilterValue;
      }
      api.get(url)
        .done(function(data, textStatus, jqXHR) {
          $.each(data.contents, function(_i, item) {
            item.date = renderDate(item.date_modified);
            item.size = renderSize(item.size_bytes);
          });
          renderFileList(data.contents);
          renderBreadcrumbs(data.current_directory.url);
          updatePaginator(data);
          $.viewpoint.modal.payload({
            result: getAbsolutePathFromUrl(data.current_directory.url)
          });
          if (!initConfig.pickFilesOnly) {
            $openFile.find('.select-btn').prop('disabled', false);
          }
          var $filterInput = $('#filter-open-file').val();
          if ($filterInput != '') {
            $('.open-file-message-container').show();
            $('.open-file-message').text('Filtering by "' + $filterInput + '" returned ' + data.count + ' results');
          } else {
            $('.open-file-message-container').hide();
          }
        })
        .fail(function() {
          if (failCallbackPath) {
            loadData(failCallbackPath);
          }
        });
    }

    function deleteFolderClicked(event) {
      event.stopPropagation();
      var trash = $(event.target);
      var a = $(event.target).closest('.list-group-item');
      var url = a.attr('data-url');
      api.delete(url)
        .done(function(data, textStatus, jqXHR) {
          selectedElementUrl = null;
          trash.closest('a').slideUp(600, function() {
            trash.closest('a').remove();
          });
          setTimeout(loadData, 200);
        });
    }

    function showConfirm(event) {
      event.stopPropagation();
      var $deleteBtn = $(event.target);
      var $confirmBtn = $deleteBtn.next();

      $deleteBtn.hide();
      $confirmBtn.show();

      var timeout = null;

      $confirmBtn.hover(function(){
        if (timeout) {
          clearTimeout(timeout);
        }
      }, function() {
        timeout = setTimeout(function() {
          $confirmBtn.slideUp(600, function() {
            $confirmBtn.hide();
          });
        }, 2000);
      });
    }

    function newFileClicked(event) {
      $('.new-file-name-box').toggle();
      $('.new-folder-name-box').hide();
      $('.new-file-name-box .file-name')
        .val('')
        .focus();
    }

    function createFile(fileName) {
      var url = getCurrentPath().replace("directory", "file") + encodeURI(fileName);
        api.post(url, {"content" : ""})
          .done(function(data, textStatus, jqXHR) {
            $('.new-file-name-box').hide();
            loadData();
          });
    }

    function createFileCallback(event) {
      event.preventDefault();

      var fileName = $('.new-file-name-box .file-name').val();

      if (fileName) {
        var url = getCurrentPath().replace("directory", "fsobject") + encodeURI(fileName);
        api.head(url)
        .done(function(){
          api.get(url).done(function(data){
            if (data.object_type == 'FILE') {
              $.viewpoint.confirm('File already exists','Are you sure you want to overwrite it? ').done(function(result){
                if (result) {
                  createFile(fileName);
                }
              });
            } else {
              createFile(fileName);
            }
          });
        })
        .fail(function(){
          createFile(fileName);
        });
      }
    }

    function newFolderClicked(event) {
      $('.new-folder-name-box').toggle();
      $('.new-file-name-box').hide();
      $('.new-folder-name-box .folder-name')
        .val('')
        .focus();
    }

    function createFolderCallback(event) {
      event.preventDefault();

      var folderName = $('.new-folder-name-box .folder-name').val();

      if (folderName) {
        var url = getCurrentPath() + folderName + '/';
        api.post(url)
          .done(function(data, textStatus, jqXHR) {
            $('.new-folder-name-box').hide();
            currentPage = 1;
            loadData();
          });
      }
    }
  });
})(jQuery);
