/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

/**
 * Scheduler Info Viewer
 * Load messages from nodes about job

 * Usage:
 $('#show-modal').click(function(e) {
    $.viewpoint.modal.show('/scheduler-info-viewer', {
      jobId: '<Job Id>',
  })
 */

;
(function() {
  'use strict';

  var $modalWindow;
  var $warningContainer;
  var $warningSection;
  var $blockContainer;
  var $blockSection;
  var $schedulerContainer;
  var $schedulerSection;
  var initConfig;
  var warningApi;
  var blockApi;
  var schedulerApi;

  $(function() {
    $modalWindow = $('#scheduler-info-viewer');
    initConfig = $.viewpoint.modal.payload();

    $warningContainer = $modalWindow.find('.warnings-bar');
    $warningContainer.loader(new Viewpoint.API());
    $warningSection = $warningContainer.find('.warning-messages');
    warningApi = $warningContainer.data('api');

    $blockContainer = $modalWindow.find('.blocks-bar');
    $blockContainer.loader(new Viewpoint.API());
    $blockSection = $blockContainer.find('.block-reasons');
    blockApi = $blockContainer.data('api');

    $schedulerContainer = $modalWindow.find('.scheduler-bar');
    $schedulerContainer.loader(new Viewpoint.API());
    $schedulerSection = $schedulerContainer.find('.scheduler');
    schedulerApi = $schedulerContainer.data('api');

    $('.block-show-all').click(function(e) {
      showSection(initConfig.jobId, $blockSection);
    });

    $('.warning-show-all').click(function(e) {
      showSection(initConfig.jobId, $warningSection);
    });

    $('.scheduler-show-all').click(function(e) {
      showSection(initConfig.jobId, $schedulerSection);
    });

    showSection(initConfig.jobId, $schedulerSection);
  });

  function showSection(jobId, section) {
    var link = section.parent().find('div > span');
    var $table = section.find('table');
    if (link.hasClass('glyphicon-chevron-right')) {
      link.removeClass('glyphicon-chevron-right');
      link.addClass('glyphicon-chevron-down');
      if (section == $warningSection) {
        loadWarning(jobId);
      }
      if (section == $blockSection) {
        loadBlock(jobId);
      }
      if (section == $schedulerSection) {
        loadScheduler(jobId);
      }
      section.collapse('toggle');
    } else {
      link.removeClass('glyphicon-chevron-down');
      link.toggleClass('glyphicon-chevron-right');
      section.collapse('toggle');
      section.find('p').remove();
      $table.DataTable().destroy();
      $table.empty();
    }
  }

  function loadWarning(jobId) {
    warningApi.get('/api/jobs/' + jobId + '/warnings/')
      .done(function(data) {
        renderWarning(data);
      })
      .fail($.proxy(renderNoContentMessage, null, $warningSection, 'No warning messages found'));
  }

  function loadBlock(jobId) {
    blockApi.get('/api/jobs/' + jobId + '/blocks/')
      .done(function(data) {
        renderBlock(data);
      })
      .fail($.proxy(renderNoContentMessage, null, $blockSection, 'No block reasons found'));
  }

  function loadScheduler(jobId) {
    schedulerApi.get('/api/jobs/' + jobId + '/scheduler/')
      .done(function(data) {
        renderNodeAnalysis(data);
      })
      .fail($.proxy(renderNoContentMessage, null, $schedulerSection, 'Scheduling analysis returned no data at this point'));
  }

  function renderWarning(data) {
    var dataSet = [];
    $.each(data.results, function(k, v) {
      dataSet.push([v]);
    });
    $('.warnings-table').DataTable({
      data: dataSet,
      'dom': 'iftlp',
      'paging': dataSet.length > 10 ? true : false,
      'oLanguage': {
        'sEmptyTable' : 'No warning messages found'
      },
      columns: [
        { title: 'Message' },
      ]
    });
  }

  function renderBlock(data) {
    var dataSet = [];
    $.each(data.results, function(k, v) {
      if (v.message) {
        dataSet.push([v.type || 'N/A', v.message]);
      }
    });
    $('.reasons-table').DataTable({
      data: dataSet,
      'dom': 'iftlp',
      'paging': dataSet.length > 10 ? true : false,
      'oLanguage': {
        'sEmptyTable' : 'No block reasons found'
      },
      columns: [
        { "width": "30%" , title: 'Block Reason' },
        { title: 'Message' },
      ]
    });
  }

  function renderNodeAnalysis(data) {
    var dataSet = [];
    $.each(data.results, function(k, v) {
      dataSet.push([v.name, v.message]);
    });
    $('.scheduler-table').DataTable({
      data: dataSet,
      'dom': 'iftlp',
      'paging':   dataSet.length > 10 ? true : false,
      'oLanguage': {
        'sEmptyTable' : 'Scheduling analysis returned no data at this point'
      },
      columns: [
        { "width": "30%" , title: 'Node Name' },
        { title: 'Message' },
      ]
    });
  }

  var renderNoContentMessage = function(container, message) {
    container.find('table')
      .DataTable({
        data: [],
        'dom': 'iftlp',
        'paging': false,
        'oLanguage': {
          'sEmptyTable' : message
        },
        columns: [
          { title: 'Message' },
        ]
    });
  };

})();
