/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

    var editor;
    var text;
    var textFile;
    var tags;
    var envVariables;
    var defaultValue;
    var payload;
    var $scriptBuilder;
    var api;
    var breadcrumbTemplate;
    var folderTemplate;
    var fileTemplate;
    var tooltipsForBody = {};
    var moabEnvVariablesList = [
      {name: 'MOAB_ACCOUNT', description: 'Account Name'},
      {name: 'MOAB_BATCH', description: 'Variable set if a batch job (non-interactive)'},
      {name: 'MOAB_CLASS', description: 'Class Name'},
      {name: 'MOAB_DEPEND', description: 'Job dependency string'},
      {name: 'MOAB_GROUP', description: 'Group Name'},
      {name: 'MOAB_JOBID', description: 'Job ID'},
      {name: 'MOAB_JOBNAME', description: 'Job Name'},
      {name: 'MOAB_MACHINE', description: 'Name of the machine (ie. Destination RM) that the job is running on'},
      {name: 'MOAB_NODECOUNT', description: 'Number of nodes allocated to job'},
      {name: 'MOAB_NODELIST', description: 'Comma-separated list of nodes (listed singly with no ppn info)'},
      {name: 'MOAB_PARTITION', description: 'Partition name the job is running in. If grid job, cluster scheduler\'s name'},
      {name: 'MOAB_PROCCOUNT', description: 'Number of processors allocated to job'},
      {name: 'MOAB_QOS', description: 'QOS Name'},
      {name: 'MOAB_SUBMITDIR', description: 'Job submission directory'},
      {name: 'MOAB_TASKMAP', description: 'Node list with procs per node listed. Format: <nodename>.<procs>'},
      {name: 'MOAB_USER', description: 'User Name'}
    ];
    var moabArrEnvVariablesList = [
      {name: 'MOAB_JOBARRAYINDEX', description: 'Job index into array'},
      {name: 'MOAB_JOBARRAYRANGE', description: 'Job array count'},
      {name: 'MOAB_JOBINDEX', description: 'Index in job array'},
    ];
    var warningTooltipTemplate =
      '<div class="tooltip warntooltip" role="tooltip"><div class="tooltip-arrow"></div>' +
      '<div class="tooltip-inner"></div></div>';
    var currentPage = 1;
    var currentOrder = 'display_name';
    var currentOrderDirection = '';
    var currentFilterField = 'display_name';
    var currentFilterValue;

    var renderDate = window.renderDate;
    var renderSize = window.renderSize;

    function hideRevertBtn() {
      if (!defaultValue) {
        $scriptBuilder.find('.revert-to-default-btn').hide();
      }
    }

    function escapeRegExp(str) {
      return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
    }

    // This function returns regex which chooses words to highlight by the next rules:
    // 1) first symbol is '%' , next alphanumeric symbols, last is '%' sign
    // 2) before selected string can be backslash or double backslash
    // 3) if before variable one backslash then not highlight variable
    // See more in IRIS-1340
    function getRegex(keywords) {
      $.each(keywords, function(key, value) {
        keywords[key] = '%' + escapeRegExp(value) + '%';
      });
      return '([\\\\]{2})(' + keywords.join('|') + ')|(' + keywords.join('|') +')|([\\\\]{1}' +
                        keywords.join('|') + ')';
    }

    function getEnvRegexp() {
      var moabEnvTags = [];
      if (payload.moabEnvVariables) {
        $.each(moabEnvVariablesList, function(i, tag) {
          moabEnvTags.push('\\$' + tag.name);
        });
      }
      if (payload.moabArrEnvVariables) {
        $.each(moabArrEnvVariablesList, function(i, tag) {
          moabEnvTags.push('\\$' + tag.name);
        });
      }
      return moabEnvTags.join('|') || null;
    }

    function createEditor(data) {
      var tagsArr = [];
      $.each(data, function(k, v) {
        if (v.enabled)
          tagsArr.push(data[k].variable);
      });


    define("Tooltips", [], function(require, exports, module) {
      "use strict";

      var dom = require("ace/lib/dom");
      var oop = require("ace/lib/oop");
      var event = require("ace/lib/event");
      var Range = require("ace/range").Range;
      var Tooltip = require("ace/tooltip").Tooltip;

      function TokenTooltip(editor, tooltipsList) {
        if (editor.tokenTooltip)
          return;
        Tooltip.call(this, editor.container);
        editor.tokenTooltip = this;
        this.editor = editor;
        this.tooltips = tooltipsList;
        this.update = this.update.bind(this);
        this.onMouseMove = this.onMouseMove.bind(this);
        this.onMouseOut = this.onMouseOut.bind(this);
        this.offsetLeft = $('#script-builder-editor').offset().left;
        this.offsetTop = $('#script-builder-editor').offset().top - $(window).scrollTop();
        event.addListener(editor.renderer.scroller, "mousemove", this.onMouseMove);
        event.addListener(editor.renderer.content, "mouseout", this.onMouseOut);
      }

      oop.inherits(TokenTooltip, Tooltip);

      (function() {
        this.token = {};
        this.range = new Range();

        this.update = function() {
          this.$timer = null;
          var r = this.editor.renderer;
          if (this.lastT - (r.timeStamp || 0) > 1000) {
            r.timeStamp = this.lastT;
            this.maxHeight = window.innerHeight;
            this.maxWidth = window.innerWidth;
          }

          var canvasPos = r.scroller.getBoundingClientRect();
          var offset = (this.x + r.scrollLeft - canvasPos.left - r.$padding) / r.characterWidth;
          var row = Math.floor((this.y + r.scrollTop - canvasPos.top) / r.lineHeight);
          var col = Math.round(offset);
          var screenPos = {row: row, column: col, side: offset - col > 0 ? 1 : -1};
          var session = this.editor.session;
          var docPos = session.screenToDocumentPosition(screenPos.row, screenPos.column);
          var token = session.getTokenAt(docPos.row, docPos.column + 1);
          if (!token && !session.getLine(docPos.row)) {
            token = {
              type: "",
              value: "",
              state: session.bgTokenizer.getState(0)
            };
          }
          if (!token) {
            session.removeMarker(this.marker);
            this.hide();
            return;
          }
          if (token.type == "keyword") {
            this.width = this.getWidth();
            this.height = this.getHeight();
            this.tokenText = token.type;
            this.show(null, this.x, this.y);
            this.token = token;
            session.removeMarker(this.marker);
            this.range = new Range(docPos.row, token.start, docPos.row, token.start + token.value.length);
            var variable = this.editor.session.getTextRange(this.range);
            var description = this.tooltips[variable.replace(/[\$%]/g, '')];
            if (description) {
              this.setText(description);
              this.marker = session.addMarker(this.range, "ace_bracket", "text");
              this.setClassName('editor-body-tooltip');
            } else {
              this.hide();
              session.removeMarker(this.marker);
            }
          } else {
            this.hide();
            session.removeMarker(this.marker);
          }
        };

        this.onMouseMove = function(e) {
          this.x = e.clientX;
          this.y = e.clientY;
          if (this.isOpen) {
            this.lastT = e.timeStamp;
            this.setPosition(this.x, this.y);
          }
          if (!this.$timer)
            this.$timer = setTimeout(this.update, 100);
        };

        this.onMouseOut = function(e) {
          if (e && e.currentTarget.contains(e.relatedTarget))
            return;
          this.hide();
          this.editor.session.removeMarker(this.marker);
          this.$timer = clearTimeout(this.$timer);
        };

         this.setPosition = function(x, y) {
           if (this.offsetLeft == 0) {
             this.offsetLeft = $('#script-builder-editor').offset().left;
           }
           if (this.offsetTop == 0) {
             this.offsetTop = $('#script-builder-editor').offset().top - $(window).scrollTop();
           }
           Tooltip.prototype.setPosition.call(this, x - this.offsetLeft, y - this.offsetTop + 85);
         };

         this.destroy = function() {
           this.onMouseOut();
           event.removeListener(this.editor.renderer.scroller, "mousemove", this.onMouseMove);
           event.removeListener(this.editor.renderer.content, "mouseout", this.onMouseOut);
           delete this.editor.tokenTooltip;
         };

         this.resize = function() {
           this.offsetLeft = $('#script-builder-editor').offset().left;
           this.offsetTop = $('#script-builder-editor').offset().top - $(window).scrollTop();
         };

      }).call(TokenTooltip.prototype);
      exports.TokenTooltip = TokenTooltip;
    });


      define('DynamicRules', [], function(require, exports, module) {
        'use strict';
        var oop = require('ace/lib/oop');
        var TextHighlightRules = require('ace/mode/text_highlight_rules').TextHighlightRules;
        var DynHighlightRules = function() {
          this.$rules = {
            'start' : [
              {
                token : ['text', 'keyword', 'keyword', 'text'],
                regex : getRegex(tagsArr.concat(envVariables))
              },
              {
                token : ['text', 'constant', 'constant', 'text'],
                regex : '([\\\\]{2})(%\\w+%)|(%\\w+%)|([\\\\]{1}%\\w+%)'
              },
              {
                token : 'keyword',
                regex : getEnvRegexp()
              }
            ]
            };
          this.normalizeRules();
        };
        oop.inherits(DynHighlightRules, TextHighlightRules);
        exports.DynHighlightRules = DynHighlightRules;
      });
      editor = ace.edit('script-builder-editor');
      var TextMode = require('ace/mode/text').Mode;
      var util = require('ace/autocomplete/util');
      var autocomplete = require('ace/autocomplete');
      // This is a workaround to autocomplete with "%" sign (IRIS-2245)
      autocomplete.Autocomplete.prototype.gatherCompletions = function(editor, callback) {
        var session = editor.getSession();
        var pos = editor.getCursorPosition();

        var line = session.getLine(pos.row);
        var prefix = util.retrievePrecedingIdentifier(line, pos.column, /[a-zA-Z_0-9\-\u00A2-\uFFFF%\$]/);

        this.base = session.doc.createAnchor(pos.row, pos.column - prefix.length);
        this.base.$insertRight = true;

        var matches = [];
        var total = editor.completers.length;
        editor.completers.forEach(function(completer, i) {
            completer.getCompletions(editor, session, pos, prefix, function(err, results) {
                if (!err)
                    matches = matches.concat(results);
                var pos = editor.getCursorPosition();
                var line = session.getLine(pos.row);
                callback(null, {
                    prefix: util.retrievePrecedingIdentifier(line, pos.column, /[a-zA-Z_0-9\-\u00A2-\uFFFF%\$]/),
                    matches: matches,
                    finished: (--total === 0)
                });
            });
        });
        return true;
      };

      var dynamicMode = new TextMode();
      dynamicMode.HighlightRules = require('DynamicRules').DynHighlightRules;
      editor.session.setMode(dynamicMode);
      var TokenTooltip;
      require(["Tooltips"], function (tooltip) {
        TokenTooltip = tooltip.TokenTooltip;
      });
      editor.tokenTooltip = new TokenTooltip(editor, tooltipsForBody);

      editor.completers = [];
      editor.completers.push(getTagsCompleter(tagsArr));
      if (envVariables.length)
        editor.completers.push(getTagsCompleter(envVariables));
      if (payload.moabEnvVariables)
        editor.completers.push(getEnvCompleter(moabEnvVariablesList));
      if (payload.moabArrEnvVariables)
        editor.completers.push(getEnvCompleter(moabArrEnvVariablesList));
      editor.setOptions({
        enableBasicAutocompletion: true,
        enableSnippets: true,
        enableLiveAutocompletion: false
      });
      editor.session.bgTokenizer.start(0);
      // to disable the display of messages to the console
      editor.$blockScrolling = Infinity;
      editorListener(editor, data);
      resizeFormListener(editor);
      if (text) {
        editor.setValue(text);
      }
      editor.setReadOnly(payload.editable);
      editor.getSession().getSelection().selectionLead.setPosition(0,0);
      editor.session.selection.clearSelection();
      delete define.modules.DynamicRules;
      delete define.payloads.DynamicRules;

      // autocompleter for environment variables
      function getEnvCompleter(variables) {
        return {
          getCompletions: function(editor, session, pos, prefix, callback) {
            callback(null, variables.map(function(word) {
              var line = session.getLine(pos.row);
              var prefix = util.retrievePrecedingIdentifier(line, pos.column, null);
              return {
                caption: '$' + word.name,
                value: '$' + word.name,
                meta: 'Variable',
              };
            }));
          }
        };
      }

      // Creating autocompleter of tags, which can react on % sign
      function getTagsCompleter(tags) {
        return {
          identifierRegexps: /[a-zA-Z_0-9\-\u00A2-\uFFFF]/,
          getCompletions: function(editor, session, pos, prefix, callback) {
            callback(null, tags.map(function(word) {
              var line = session.getLine(pos.row);
              var prefix = util.retrievePrecedingIdentifier(line, pos.column, null);
              return {
                caption: '%' + word + '%',
                value: '%' + word +'%',
                meta: 'Variable',
              };
            }));

          }
        };
      }

    }

    function deleteVerticalLineInEditor() {
      $('.ace_print-margin').removeClass('ace_print-margin');
    }

    function resizeFormListener(editor) {
      $('#script-builder .resize').click(function() {
        if ($('#script-builder .glyphicon-resize-full').length) {
          $('body').css('padding-right', '');
          $('#script-builder').css('padding-right', '');
          $('#script-builder > .modal-dialog').addClass('full-window');
          $('#script-builder .resize > span')
            .removeClass('glyphicon-resize-full')
            .addClass('glyphicon-resize-small');
          editor.resize();
          editor.tokenTooltip.resize();
          deleteVerticalLineInEditor();
        } else {
          $('#script-builder > .modal-dialog').removeClass('full-window');
          $('#script-builder .resize > span')
            .removeClass('glyphicon-resize-small')
            .addClass('glyphicon-resize-full');
          editor.resize();
          editor.tokenTooltip.resize();
          deleteVerticalLineInEditor();
        }
        $('.editor-left').toggleClass('col-md-8').toggleClass('col-md-10');
        $('.editor-right').toggleClass('col-md-4').toggleClass('col-md-2');
      });
    }

    function editorListener(editor, data) {
      var timer = null;
      // Not select text after drop
      editor.container.addEventListener('drop', function (e) {
        editor.session.selection.clearSelection();
      });
      editor.getSession().on('change', function(e) {
        if (timer) {
          clearTimeout(timer);
          timer = null;
        }
        timer = setTimeout(function() {
          calculateTags(editor, data);
        }, 600);
      });
      clearTimeout(timer);
    }

    function getTagsFromEditor(text) {

      var pushVarToArray = function(key, value) {
        if (match[value]) {
          variables.push(match[value]);
          return false;
        }
      };

      var variablesRegexp = new RegExp('([\\\\]{2})(%\\w+%)|(%\\w+%)|([\\\\]{1}%\\w+%)','g');
      var variables = [];
      var match = variablesRegexp.exec(text);

      while (match !== null) {
        $.each([2,3], pushVarToArray);
        match = variablesRegexp.exec(text);
      }
      return variables;
    }

    function calculateTags(editor, data) {
      var text = editor.getValue().toString();
      var variables = getTagsFromEditor(text);
      for (var i in data) {
        var el = getHtmlTagElement(data[i].variable, '.tag-container .user-tag');
        var counterRegexp = new RegExp('%' + data[i].variable + '%','g');
        if ($.inArray('%' + data[i].variable + '%', variables) != -1) {
          var count = variables.toString().match(counterRegexp).length;
          el.addClass('used-tag');
          el.children().last().text(count);
        } else {
          el.removeClass('used-tag');
          el.children().last().text('');
        }
      }

      $(envVariables).each(function(i, variable) {
        var el = getHtmlTagElement(variable, '.tag-container .user-tag');
        var counterRegexp = new RegExp('%' + variable + '%','g');
        if ($.inArray('%' + variable + '%', variables) != -1) {
          var count = variables.toString().match(counterRegexp).length;
          el.addClass('used-tag');
          el.children().last().text(count);
        } else {
          el.removeClass('used-tag');
          el.children().last().text('');
        }
      });
      if (payload.moabEnvVariables) {
        calculateEnvTags(moabEnvVariablesList);
      }
      if (payload.moabArrEnvVariables) {
        calculateEnvTags(moabArrEnvVariablesList);
      }

      function getHtmlTagElement(name, elSelector) {
        var element = $(elSelector).filter(function() {
          if ($(this).text() == name && !$(this).hasClass('disabled')) {
            return $(this);
          }
        }).parent();
        return element;
      }

      function calculateEnvTags(list) {
        for (var i in list) {
          var reg = new RegExp('\\$' + list[i].name, 'g');
          var el = getHtmlTagElement(list[i].name, '.tag-container .env-tag');
          var match = text.match(reg);
          var count = match ? match.length : 0;
          if (count) {
            el.addClass('used-tag');
            el.children().last().text(count);
          } else {
            el.removeClass('used-tag');
            el.children().last().text('');
          }
        }
      }
    }

    function displayTagsList(data) {
      var fn = function(e) {
        var tagName = '%' + $(this).text() + '%';
        e.originalEvent.dataTransfer.setData('text', tagName);
      };
      var fnEnv = function(e) {
        var tagName = '$' + $(this).text();
        e.originalEvent.dataTransfer.setData('text', tagName);
      };
      data.sort(
          function(a,b) {
          if (a.variable < b.variable)
            return -1;
          if (a.variable > b.variable)
            return 1;
          return 0;
        });
      var moabVariables = $('<div></div>').append('<p>Moab Inputs</p>');
      var customVariables = $('<div></div>').append('<p>Custom Inputs</p>');
      for (var tag in data) {
        var span = $('<span class="user-tag"></span>')
          .text(data[tag].variable.replace(/%/g, ''));
        if (data[tag].enabled) {
          span.attr('draggable', 'true').on('dragstart', fn);
        } else {
          span.addClass('disabled');
        }
        if (data[tag].tooltip) {
          tooltipsForBody[data[tag].variable] = data[tag].tooltip;
          span.attr('title', data[tag].tooltip);
        }
        if (data[tag].isCustom) {
          customVariables.append(
            $('<div class="tag"></div>')
            .append(span)
            .append('<span class="counter"></span>')
          );
        } else {
          moabVariables.append(
            $('<div class="tag"></div>')
            .append(span)
            .append('<span class="counter"></span>')
          );
        }
      }
      $('.tag-container').append(moabVariables);
      if (customVariables.find('div').length > 0) {
        $('.tag-container').append(customVariables);
      }
      if (payload.moabEnvVariables) {
        $('.tag-container').append(getMoabEnvSection());
      }
      if (payload.moabArrEnvVariables) {
        $('.tag-container').append(getMoabArrEnvSection());
      }
      if (envVariables && envVariables.length) {
        var envVariablesContainer = $('<div></div>').append('<p>Job Env Variables</p>');
        $(envVariables).each(function(index, variable) {
          var span = $('<span class="user-tag"></span>')
            .attr('draggable', 'true')
            .text(variable)
            .on('dragstart', fn);
          envVariablesContainer.append(
            $('<div class="tag"></div>')
              .append(span)
              .append('<span class="counter"></span>')
          );
        });
        $('.tag-container').append(envVariablesContainer);
      }

      function getMoabEnvSection() {
        var section = $('<div></div>').append('<p>Moab Env Variables</p>');
        $.each(moabEnvVariablesList, function(i, tag) {
          var span = $('<span class="env-tag"></span>')
            .attr('draggable', 'true')
            .text(tag.name)
            .on('dragstart', fnEnv);
          if (tag.description) {
            tooltipsForBody[tag.name] = tag.description;
            span.attr('title', tag.description);
          }
          section.append(
            $('<div class="tag"></div>')
            .append(span)
            .append('<span class="counter"></span>')
          );
        });
        return section;
      }

      function getMoabArrEnvSection() {
        var section = $('<div></div>').append('<p>Moab Array Env Variables</p>');
        $.each(moabArrEnvVariablesList, function(i, tag) {
          var span = $('<span class="env-tag"></span>')
            .attr('draggable', 'true')
            .text(tag.name)
            .on('dragstart', fnEnv);
          if (tag.description) {
            tooltipsForBody[tag.name] = tag.description;
            span.attr('title', tag.description);
          }
          section.append(
            $('<div class="tag"></div>')
            .append(span)
            .append('<span class="counter"></span>')
          );
        });
        return section;
      }

    }

  function renderComponents(data) {
    data = data.filter(
      function (el) {
        return el.variable !== 'SCRIPT';
      }
    );
    createEditor(data);
    displayTagsList(data);
    if (payload.editable) {
      $('#dropdown-menu').prop('disabled', true);
      $('.done-btn').prop('disabled', true);
    }
  }

  function createScript() {
    $('.tag-container').empty();
    renderComponents(tags);
  }

  function showRemoteImport(event) {
    event.preventDefault();

    $scriptBuilder.find('.script-builder').hide();
    $scriptBuilder.find('.editor-controls').hide();
    $scriptBuilder.find('.local-import').hide();
    $scriptBuilder.find('.local-import-controls').hide();
    $scriptBuilder.find('.export-controls').hide();
    $scriptBuilder.find('.file-browser').show();
    $scriptBuilder.find('.file-browser').removeClass('export');
    $scriptBuilder.find('.file-browser-controls').show();

    loadHomeDir();
  }

  function loadHomeDir() {
    $('.paginator-space').hide();
    var pageSize = $('#viewPerPage').val();
    pageSize = pageSize || 100;
    var url = '/api/directory/~?page=' + currentPage +
      '&page_size=' + pageSize +
      '&ordering=' + currentOrderDirection + currentOrder;
    if (currentFilterValue) {
      url += '&filter-by=' + currentFilterField +
        '&filter-value=' + currentFilterValue;
    }
    api.get(url)
      .done(function(data, textStatus, jqXHR) {
        $.each(data.contents, function(_i, item) {
          item.date = window.renderDate(item.date_modified);
          item.size = renderSize(item.size_bytes);
        });
        renderBreadcrumb(data.current_directory.url);
        setExportPath(getAbsolutePathFromUrl(data.current_directory.url));
        renderFileList(data.contents);
        updatePaginator(data);
        $('.paginator-space').show();
      })
      .fail(function(jqXHR, textStatus, errorThrown) {
        console.error(jqXHR.responseJSON.detail);
      });
  }

  function showEditor() {
    $scriptBuilder.find('.local-import').hide();
    $scriptBuilder.find('.local-import-controls').hide();
    $scriptBuilder.find('.file-browser').hide();
    $scriptBuilder.find('.file-browser').removeClass('export');
    $scriptBuilder.find('.file-browser-controls').hide();
    $scriptBuilder.find('.export-controls').hide();

    editor.getSession().getSelection().selectionLead.setPosition(0,0);
    editor.focus();
    $scriptBuilder.find('.script-builder').show();
    $scriptBuilder.find('.editor-controls').show();
  }

  function showExport() {
    var str = editor.getValue().toString().trim();
    if (!str) {
      $.notify("Script body is empty",
        { type: 'warning' }
      );
      return;
    }
    // the maxFileSize is initialized in the base.html
    if (maxFileSize > 0 && str.length > maxFileSize) {
      $.notify('The script is too large for upload to remote file system', { type: 'warning' });
      return;
    }
    $scriptBuilder.find('.local-import').hide();
    $scriptBuilder.find('.local-import-controls').hide();
    $scriptBuilder.find('.file-browser-controls').hide();
    $scriptBuilder.find('.script-builder').hide();
    $scriptBuilder.find('.editor-controls').hide();

    $scriptBuilder.find('.file-browser').show();
    $scriptBuilder.find('.file-browser').addClass('export');
    $scriptBuilder.find('.export-controls').show();

    $scriptBuilder.find('.export-file-name').val('');

    loadHomeDir();
  }

  function closeDialogClicked() {
    if ($scriptBuilder.find('.script-builder').is(':visible')) {
      destroyEditor();
      $.viewpoint.modal.dismiss(text);
    } else {
      showEditor();
    }
  }

  $('.home-link-container').click(function(){
    loadHomeDir();
  });

  $(window).unbind('keydown');
  $(window).keydown(function(event) {
    if (event.which === 27 && $('div').is('#script-builder')) {
      var scriptValue = $('#script').val() !== undefined ? $('#script').val() : $('#script-defaultValue').val();
      if (scriptValue != editor.getValue()) {
        $.viewpoint.confirm('Script Builder is going to close','Are you sure you want to close the script builder? ' +
         'Doing so will lose all changes since your last save.').done(function(result){
          if (result) {
            closeDialogClicked();
          }
        });
      } else {
        if (isRequestCompleted) {
          closeDialogClicked();
        }
      }
    }
  });

  //TODO Fix it!
  var isRequestCompleted = true;
  $.ajaxSetup({
    beforeSend: function(){
      isRequestCompleted = false;
    },
    complete: function() {
      isRequestCompleted = true;
    }
  });

  function setExportPath(path) {
    var visiblePath = decodeURIComponent(path);
    var re1 = new RegExp('\/\\s$');
    var re2 = new RegExp('\/\\s\/', 'g');
    visiblePath = visiblePath.replace(re1, '/" "');
    visiblePath = visiblePath.replace(re2, '/" "/');
    $scriptBuilder.find('.export-path')
      .attr('data-url', path)
      .val(visiblePath);
  }

  function fileListElementFactory(config) {
    config.extension = '';
    if (/.*\.(.*)/.test(config.display_name))
      config.extension = config.display_name.match(/.*\.(.*)/)[1];
    if (config.object_type == "DIRECTORY") {
      return folderTemplate(config);
    }
    if (config.object_type == "FILE") {
      return fileTemplate(config);
    }
  }

  function showLocalImport(event) {
    event.preventDefault();

    $scriptBuilder.find('.local-import-preview').empty();
    $scriptBuilder.find('.local-import-preview')
      .append($('<div class="preview-title">Preview Area</div>'))
      .append(
            $('<textarea></textarea>')
              .addClass('preview-textarea')
              .attr('readonly', 'readonly')
          );
    $scriptBuilder.find('.selected-file').text('Browse');

    $scriptBuilder.find('.script-builder').hide();
    $scriptBuilder.find('.editor-controls').hide();
    $scriptBuilder.find('.file-browser').hide();
    $scriptBuilder.find('.file-browser').removeClass('export');
    $scriptBuilder.find('.export-controls').hide();
    $scriptBuilder.find('.file-browser-controls').hide();
    $scriptBuilder.find('.local-import').show();
    $scriptBuilder.find('.local-import-controls').show();
  }

  var clickFolderHandler = function(event) {
    event.preventDefault();
    if ($(event.target).hasClass('delete-container') ||
      $(event.target).hasClass('glyphicon-trash')) {
      return;
    }
    currentPage = 1;
    var pageSize = $('#viewPerPage').val();
    pageSize = pageSize || 100;
    var path = $(this).attr('data-url');
    path += '?page=' + currentPage +
      '&page_size=' + pageSize +
      '&ordering=' + currentOrderDirection + currentOrder;
    if (currentFilterValue) {
      path += '&filter-by=' + currentFilterField +
        '&filter-value=' + currentFilterValue;
    }
    setExportPath(getAbsolutePathFromUrl(path));
    $scriptBuilder.find('.export-file-name').val('');

    api.get(path)
      .done(function(data, textStatus, jqXHR) {
        $.each(data.contents, function(_i, item) {
          item.date = window.renderDate(item.date_modified);
          item.size = renderSize(item.size_bytes);
        });
        renderBreadcrumb(data.current_directory.url);
        renderFileList(data.contents);
        updatePaginator(data);
      })
      .fail(function(jqXHR, textStatus, errorThrown) {
        console.error(jqXHR.responseJSON.detail);
      });
  };

  var selectedFileUrl = null;
  var selectedFileSize = 0;

  var clickFileHandler = function(event) {
    event.preventDefault();

    selectedFileUrl = $(this).attr('data-url');
    selectedFileSize = $(this).attr('data-size');
    $(this).parent().find('a').removeClass('active');
    $(this).addClass('active');
    $scriptBuilder.find('.export-file-name').val($(this).attr('data-file-name'));
  };

  function renderFileList(data) {
    var $listGroup = $scriptBuilder.find('#file-list .list-group');
    $listGroup.empty();
    $(data).each(function(index, e) {
      $listGroup.append(fileListElementFactory(e));
    });
    $listGroup.find('.folder-item, .file')
      .hover(function() {
        if ($(this).find('.confirm-deleting').is(':hidden')) {
          $(this).find('.glyphicon-trash').show();
        }
      }, function() {
        $(this).find('.glyphicon-trash').hide();
      });
    $listGroup.find('.glyphicon-trash').click(showConfirm);
    $listGroup.find('.confirm-deleting').click(deleteFolderClicked);
    $listGroup.find('.file').click(clickFileHandler);
    $listGroup.find('.folder-item').click(clickFolderHandler);

    selectedFileUrl = null;
    selectedFileSize = 0;
  }

  function showConfirm(event) {
    event.stopPropagation();
    var $deleteBtn = $(event.target);
    var $confirmBtn = $deleteBtn.next();
    $deleteBtn.hide();
    $confirmBtn.show();

    var timeout = null;

    $confirmBtn.hover(function(){
      if (timeout) {
        clearTimeout(timeout);
      }
    }, function() {
      timeout = setTimeout(function() {
        $confirmBtn.slideUp(600, function() {
          $confirmBtn.hide();
        });
      }, 2000);
    });
  }

  function renderBreadcrumb(path) {
    var $breadcrumbs = $scriptBuilder.find('.breadcrumb');
    $breadcrumbs.empty();
    var root = $.url('protocol', path) + '://' + $.url('domain', path) + ":" +
      $.url('port', path) + "/" + $.url(1, path) + "/" + $.url(2, path) + "/";
    $breadcrumbs.append(breadcrumbTemplate({
      name: 'Remote File System',
      url: root
    }));

    var nextFolder = 'folder';
    var index = 3;
    var currentUrl = String(root);
    while (nextFolder !== '') {
      nextFolder = $.url(index, path);
      currentUrl += '/' + nextFolder;
      if (nextFolder !== '') {
        var folderName = decodeURIComponent(nextFolder);
        if (folderName.trim() === '') {
          folderName = '" "';
        }
        $breadcrumbs.append(breadcrumbTemplate({
            name: folderName,
            url: currentUrl
          }
        ));
      }
      index++;
    }

    $breadcrumbs.find('.folder').click(clickFolderHandler);
  }

  function handleFileSelect(evt) {
    var files = evt.target.files;
    var f = files[0];

    var $preview = $scriptBuilder.find('.local-import-preview');
    var reader = new FileReader();

    reader.onloadstart = (function() {
      return function(e) {
        $('#script-builder .local-import-controls .btn, #script-builder .close-dialog-btn')
          .addClass('disabled')
          .prop('disabled', true);
      };
    })(f);

    reader.onloadend = (function() {
      return function(e) {
        $('#script-builder .local-import-controls .btn, #script-builder .close-dialog-btn')
          .removeClass('disabled')
          .prop('disabled', false);
      };
    })(f);

    reader.onload = (function() {
      return function(e) {
        textFile = e.target.result;
        $scriptBuilder.find('.selected-file').text(f.name);
        $preview.empty();
        $preview.append(
            $('<textarea></textarea>')
                .addClass('preview-textarea')
                .val(textFile.length > 10000 ? textFile.slice(0, 10000) +
                 '...\n(This file is too big for a full preview)' : textFile)
                .attr('readonly', 'readonly')
        );
      };
    })(f);

    reader.readAsText(f);
  }

  function getAbsolutePathFromUrl(url) {
    var result = "";
    var nextFolder = 'folder';

    // http://192.168.53.121:8080/api/directory/<dir_name>//
    //                            <1>     <2>         <3>
    var index = 3;

    while (nextFolder !== '') {
      nextFolder = $.url(index, url);
      if (nextFolder !== '') {
        result += '/' + nextFolder;
      }
      index++;
    }

    return result;
  }

  function newFolderClicked(event) {
    $('.new-folder-name-box').toggle();
    $('.new-file-name-box').hide();
    $('.new-folder-name-box .folder-name')
        .val('')
        .focus();
  }

  function newFileClicked(event) {
    $('.new-file-name-box').toggle();
    $('.new-folder-name-box').hide();
    $('.new-file-name-box .file-name')
      .val('')
      .focus();
  }

  function getCurrentPath() {
    var li = $('#script-builder ol.breadcrumb li').last();
    return li.find('a').attr('data-url') + '/';
  }

  function createFolderCallback(event) {
    event.preventDefault();

    var folderName = $('.new-folder-name-box .folder-name').val();
    if (folderName) {
      var url = getCurrentPath() + folderName + '/';
      api.post(url)
        .done(function(data, textStatus, jqXHR) {
          $('.new-folder-name-box').hide();
          renderCurrentFolder();
        });
    }
  }

  function createFile(fileName) {
    var url = getCurrentPath().replace("directory", "file") + encodeURI(fileName);
      api.post(url, {"content" : ""})
        .done(function(data, textStatus, jqXHR) {
          $('.new-file-name-box').hide();
          renderCurrentFolder();
        });
  }

  function createFileCallback(event) {
    event.preventDefault();

    var fileName = $('.new-file-name-box .file-name').val();

    if (fileName) {
      var url = getCurrentPath().replace("directory", "fsobject") + encodeURI(fileName);
      api.head(url)
        .done(function(){
          api.get(url).done(function(data){
            if (data.object_type == 'FILE') {
              $.viewpoint.confirm('File already exists','Are you sure you want to overwrite it? ').done(function(result){
                if (result) {
                  createFile(fileName);
                }
              });
            } else {
              createFile(fileName);
            }
          });
        })
        .fail(function(){
          createFile(fileName);
        });
    }
  }

  function deleteFolderClicked(event) {
    event.stopPropagation();
    var trash = $(event.target);
    var a = $(event.target).closest('.list-group-item');
    var url = a.attr('data-url');
    api.delete(url)
      .done(function() {
        trash.closest('a').slideUp(600, function() {
          trash.closest('a').remove();
        });
      });
  }

  function renderCurrentFolder() {
    var pageSize = $('#viewPerPage').val();
    pageSize = pageSize || 100;
    var path = getCurrentPath();
    path += '?page=' + currentPage +
            '&page_size=' + pageSize +
            '&ordering=' + currentOrderDirection + currentOrder;
    if (currentFilterValue) {
      path += '&filter-by=' + currentFilterField +
        '&filter-value=' + currentFilterValue;
    }
    api.get(path)
      .done(function(data, textStatus, jqXHR) {
        $.each(data.contents, function(_i, item) {
          item.date = window.renderDate(item.date_modified);
          item.size = renderSize(item.size_bytes);
        });
        renderFileList(data.contents);
        updatePaginator(data);
        var $filterInput = $('#filter-by-input').val();
        if ($filterInput != '') {
          $('.filter-message-container').show();
          $('#script-builder .file-browser').addClass('active-filter');
          $('#script-builder .full-window .file-browser #file-list').addClass('active-filter');
          $('.filter-message').text('Filtering by "' + $filterInput + '" returned ' + data.count + ' results');
        } else {
          $('#script-builder .file-browser').removeClass('active-filter');
          $('#script-builder .full-window .file-browser #file-list').removeClass('active-filter');
          $('.filter-message-container').hide();
        }
      });
  }

  function destroyEditor() {
    $('.ace_autocomplete').remove();
    editor.destroy();
    var el = editor.container;
    el.parentNode.removeChild(el);
    editor.tokenTooltip.destroy();
  }

  function doneClicked() {
    destroyEditor();
    $.viewpoint.modal.dismiss(editor.getValue());
  }

  function cancelClicked() {
    destroyEditor();
    $.viewpoint.modal.dismiss(text);
  }

  function selectLocalClicked() {
    $('#script-builder .local-import-controls .btn, #script-builder .close-dialog-btn')
      .addClass('disabled')
      .prop('disabled', true);
    // This is a workaround for IRIS-2155
    setTimeout(function() {
      if (textFile) {
        editor.setValue(textFile);
        showEditor();
        $('#script-builder .local-import-controls .btn, #script-builder .close-dialog-btn')
          .removeClass('disabled')
          .prop('disabled', false);
      }
    }, 100);
  }

  function remoteClicked() {
    if ($('.file-browser a.active').length) {
      if (selectedFileSize === 0) {
        $.notify("File is empty", {
          type: 'danger' }
        );
        return;
      }
      api.get(selectedFileUrl, 'text').done(function(data, textStatus, jqXHR) {
        editor.setValue(data);
        showEditor();
      }).fail(function(data, textStatus, jqXHR) {
        var status = data.status;
        if (status === 406) {
          $.notify("File is empty or malformed", {
            type: 'danger' }
          );
        } else {
          textStatus = (status === 0) ? 'Permission denied ' : textStatus || 'Error';
          $.notify(textStatus, {
            type: 'danger' }
          );
        }
      });
    } else {
      $.notify("You must select a file", { type: 'danger' });
    }
  }

  function exportClicked() {
    var path = $scriptBuilder.find('.export-path').attr('data-url');
    var text = editor.getValue();
    var fileName = $scriptBuilder.find('.export-file-name').val();
    if (fileName) {
      api.head('/api/fsobject' + path + '/' + fileName + '/').done(function(data){
        $.viewpoint.confirm('File already exists','Are you sure you want to overwrite it? ').done(function(result){
          if (result) {
            api.post('/api/file' + path + '/' + fileName + '/', {content: text})
              .done(function(data, textStatus, jqXHR) {
                showEditor();
              });
          }
        });
      })
      .fail(function(){
        api.post('/api/file' + path + '/' + fileName + '/', {content: text})
          .done(function(data, textStatus, jqXHR) {
            showEditor();
          });
      });
    } else {
      $.notify("The file name must be set", { type: 'danger' });
    }
  }

  function checkPermissionsForExportImport() {
    api.head('/api/directory/~/')
      .done()
      .fail(function(data) {
        var status = data.status;
        if (status === 403) {
          var msg = 'Access denied: We\'re sorry but you cannot access this area of ' +
            'the portal because your Viewpoint user does not have a system account';
          $scriptBuilder.find('.show-export-btn').prop('disabled', true);
          $scriptBuilder.find('.show-export-btn-container')
            .tooltip({
              title: msg,
              template: warningTooltipTemplate
            });
          $scriptBuilder.find('.show-remote-import-btn')
            .addClass('disabled')
            .off('click', showRemoteImport)
            .on('click', function(e) { e.stopPropagation(); })
            .tooltip({
              title: msg,
              template: warningTooltipTemplate
            });
        }
      });
  }

  function createPaginator() {
    $('.paginator-container').bootpag({
      total: 10,
      maxVisible: 4,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      // reload
      currentPage = num;
      renderCurrentFolder();
    });
  }

  function updatePaginator(data) {
    if (data.next || data.previous) {
      var pageSize = $('#viewPerPage').val();
      pageSize = pageSize || 100;
      var pagesCount = Math.ceil(data.count / pageSize);
      if (pagesCount == 0) { pagesCount = 1; }
      $('.paginator-container').bootpag({
        total: pagesCount, // total pages
        page: currentPage,  // default page
        next: 'next',
        prev: 'prev'
      }).show();
      $('.paginator-container').show();
    } else {
      $('.paginator-container').hide();
    }
  }

  function setFilter(event) {
    currentFilterValue = $('#filter-by-input').val();
    currentPage = 1;
    renderCurrentFolder();
  }

  function changeOrder(event) {
    var orderField = '';
    if ($(event.target).hasClass('order-by-name')) {
      orderField = 'display_name';
    }
    if ($(event.target).hasClass('order-by-date')) {
      orderField = 'date_modified';
    }
    if ($(event.target).hasClass('order-by-type')) {
      orderField = 'extension';
    }
    if ($(event.target).hasClass('order-by-size')) {
      orderField = 'size_bytes';
    }
    if (!orderField) {
      return;
    }

    if (currentOrder == orderField) {
      currentOrderDirection = (currentOrderDirection == '-') ? '': '-';
    } else {
      currentOrderDirection = '';
      currentOrder = orderField;
    }

    $('.order-by-date .order-marker').hide();
    $('.order-by-name .order-marker').hide();
    $('.order-by-type .order-marker').hide();
    $('.order-by-size .order-marker').hide();
    switch (orderField) {
      case 'display_name':
        $('.order-by-name .order-marker').show();
        break;
      case 'date_modified':
        $('.order-by-date .order-marker').show();
        break;
      case 'extension':
        $('.order-by-type .order-marker').show();
        break;
      case 'size_bytes':
        $('.order-by-size .order-marker').show();
        break;
    }

    if (currentOrderDirection == '-') {
      $('.order-marker')
        .removeClass('glyphicon-arrow-up')
        .addClass('glyphicon-arrow-down');
    } else {
      $('.order-marker')
        .removeClass('glyphicon-arrow-down')
        .addClass('glyphicon-arrow-up');
    }

    currentPage = 1;
    renderCurrentFolder();
  }

  $(function() {
    var $loader = $('#script-builder .modal-content');
    $loader.loader(new Viewpoint.API());
    api = $loader.data('api');
    $scriptBuilder = $('#script-builder');

    fileTemplate = Handlebars.compile($('#file-template').html());
    folderTemplate = Handlebars.compile($('#folder-template').html());
    breadcrumbTemplate = Handlebars.compile($('#breadcrumb-template').html());

    payload = $.viewpoint.modal.payload();
    text = payload.text;
    tags = payload.tags;
    envVariables = payload.envVariables;
    defaultValue = payload.defaultValue;

    $.viewpoint.modal.payload(text || ' ');

    createScript();
    showEditor();
    hideRevertBtn();

    $(window).resize(function() {
      var $resizeBtn = $('#script-builder').find('.resize');

      if ($(window).width() < 1024) {
        $resizeBtn.hide();
        if (0 == $('#script-builder').find('.glyphicon-resize-full').length) {
          $resizeBtn.click();
        }
        return;
      }

      if ($(window).width() >= 1024 && $resizeBtn.is(':hidden')) {
        $resizeBtn.show();
      }
    });

    $scriptBuilder.find('.local-import-files').on('change', handleFileSelect);
    $scriptBuilder.find('.done-btn').click(doneClicked);
    $scriptBuilder.find('.cancel-btn').click(cancelClicked);

    $scriptBuilder.find('.show-remote-import-btn').on('click', showRemoteImport);
    $scriptBuilder.find('.return-to-editor-btn').click(showEditor);
    $scriptBuilder.find('.show-local-import-btn').click(showLocalImport);
    $scriptBuilder.find('.show-export-btn').click(showExport);
    $('.close-dialog-btn').click(closeDialogClicked);

    checkPermissionsForExportImport();

    $scriptBuilder.find('.export-btn').click(exportClicked);
    $scriptBuilder.find('.revert-to-default-btn').click(function() {
      editor.setValue(defaultValue || '');
      editor.getSession().getSelection().selectionLead.setPosition(0,0);
      editor.session.selection.clearSelection();
    });
    $scriptBuilder.find('.select-remote-file-btn').click(remoteClicked);
    $scriptBuilder.find('.select-local-file-btn').click(selectLocalClicked);

    $('.new-folder-btn').click(newFolderClicked);
    $('.new-file-btn').click(newFileClicked);
    $('.folder-create-btn').click(function() {
      $('.new-folder-name-box').find('form').submit();
    });
    $('.new-folder-name-box').find('form').submit(createFolderCallback);
    $('.file-create-btn').click(function() {
      $('.new-file-name-box').find('form').submit();
    });
    $('.new-file-name-box').find('form').submit(createFileCallback);

    $('#script-builder').on('shown.bs.modal', function() {
      editor.focus();
    });

    createPaginator();
    $('#viewPerPage').change(function() {
      currentPage = 1;
      renderCurrentFolder();
    });

    $('.clear-filter').click(function() {
      $('#filter-by-input').val('');
      setFilter();
    });

    $('.file-browser .order-by-name, .file-browser .order-by-date').click(changeOrder);
    $('.file-browser .order-by-type, .file-browser .order-by-size').click(changeOrder);
    $('.file-browser .filter-btn').click(setFilter);
    $('#filter-by-input').on('keypress', function(event) {
      if (event.which === 13)
        setFilter();
    })
  });
})(jQuery);
