/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  $(function() {
    pageLoaded();
  });
  var totalCount = 0;
  var currentOrder = '0.asc';
  var filter = {};
  var pageNumber = 1;
  var table;
  var opts;
  var Repeater;
  var sort = {
    'colId': "'name'",
    'sortMethod': 'sortBy'
  }
  function pageLoaded() {
    setPageSize();
    createDataTable();

    Repeater = new Viewpoint.Repeater({
      timeout: $('#selectIntervals').val(),
      fn: [loadData]
    });

    $.fn.dataTableExt.oSort["sortOff-desc"] = function (x, y) {
      return 0;
    };
    $.fn.dataTableExt.oSort["sortOff-asc"] = function (x, y) {
      return 0;
    };

    $('#session-name-checkbox').click(onOffNameFilter);
    $('#job-id-checkbox').click(onOffJobIdFilter);
    $('#session-login-checkbox').click(onOffLoginFilter);

    $(document).on('click', '.session-connect', function(event) {
      event.preventDefault();
      connectSession(this);
    });
    $(document).on('click', '.session-disconnect', function(event) {
      event.preventDefault();
      disconnectSession(this);
    });
    $(document).on('click', '.session-terminate', function(event) {
      event.preventDefault();
      terminateSession(this);
    });
    $(document).on('click','#data-table tbody tr',function(e) {
      if (e.target.tagName.toLowerCase() != 'a') {
        $(this).toggleClass('active-session');
      }
    });
    $('#viewPerPage').change(function(){
      pageNumber = 1;
      if ($('#grid-view-btn').hasClass('selected-view')) {
        renderGridView()
      } else {
        loadData();
      }
    })
    $(document).on('click', '#createSessionDialog', function(event) {
      $('#command-input').val('');
      $('#resolution').val('1024x768');
      $('#create-session-dialog').modal('show');
    });
    $(document).on('click', '#sessionRefresh', function(event) {
      if ($('#grid-view-btn').hasClass('selected-view')) {
        renderGridView();
      } else {
        loadData();
      }
    });

    $('body').on('hide.bs.modal', '.modal', function() {
      if ($('#selectIntervals').is(':visible') && $('#selectIntervals').val() != 'OFF') {
        changeIntervalHandler();
      }
    });

    $('body').on('show.bs.modal', '.modal', Repeater.stop);

    $('#selectIntervals').change(changeIntervalHandler);
    setRefreshInterval();
    changeIntervalHandler();

    $(document).on('click', '#apply-filter-btn', function(event) {
      var table = $('#data-table').DataTable();
      var pageSize = $('#viewPerPage').val();
      if ($('#session-name-checkbox').is(":checked")) {
       filter['name'] = $('#session-name').val();
      } else {
       delete filter.name;
      }
      if ($('#job-id-checkbox').is(":checked")) {
        filter['moabJobId'] = $('#job-id').val();
      } else {
        delete filter.moabJobId;
      }
      if ($('#session-login-checkbox').is(":checked")) {
        filter['login'] = $('#session-login').val();
      } else {
        delete filter.login;
      }
      if ($('#grid-view-btn').hasClass('selected-view')) {
        renderGridView();
      } else {
        loadData();
      }
    });
    $(document).on('click', '#reset-btn', function(event) {
      filter = {};
      resetFilters();
      if ($('#grid-view-btn').hasClass('selected-view')) {
        renderGridView();
      } else {
        loadData();
      }
    });

    $('#viewPerPage').on('change', function() {
      var pageSize = $('#viewPerPage').val();
      var table = $('#data-table').dataTable();
      var settings = table.fnSettings();
      settings._iDisplayLength = parseInt(pageSize, 10);
      table.fnDraw();
      updatePaginator(totalCount, pageSize, 1);
      localStorage.setItem("sessionsPageSize", $(this).val());
    });

    $('#create-session-dialog').on('hidden.bs.modal', function () {
      $('#command-input').popover('destroy');
      $('.command-container').removeClass('has-error');
    });

    $('#create-session-dialog').on('shown.bs.modal', function () {
      $('#command-input').focus();
    });

    $(document).on('click', '#start-new-session', function(event) {
      event.preventDefault();
      $('#create-session-dialog .create-session').submit();
    });
    $('#create-session-dialog .create-session').submit(createSession);

    $(document).on('click', '.session-table th', function(){
      var colId = $(this).attr('id');
      sort['colId'] = '"' + colId + '"';
      if ($(this).attr('aria-sort') == 'ascending') {
        sort['sortMethod'] = 'sortBy';
      } else if ($(this).attr('aria-sort') == 'descending') {
        sort['sortMethod'] = 'sortReverse';
      }
      loadData();
    });

    function isValidForm() {
      if ($('#command-input').val() == '') {
        $('#command-input').popover('show');
        $('.command-container').addClass('has-error');
        return false;
      } else {
        $('#command-input').popover('destroy');
        $('.command-container').removeClass('has-error');
        return true;
      }
    }

    function createSession(event) {
      event.preventDefault();
      if (isValidForm()) {
        var command = $('#command-input').val();
        var resolution = $('#resolution').val();
        var opts = {'command':command,'geometry':resolution, 'name': 'Viewpoint Admin Session'};
        $('#create-session-dialog').modal('hide');
        $('.datagrid-container').data('api').post('/api/fastx-sessions/', opts).done(function() {
          if ($('#grid-view-btn').hasClass('selected-view')) {
            renderGridView();
          } else {
            loadData();
          }
        });
      }
      return;
    }

    resetFilters();
    createPaginator();
    $('[data-toggle="command-error"]').popover();
    $('#grid-view-btn').click(function(){
      $(this).blur();
      if (!$(this).hasClass('selected-view')) {
        $('.session-table').hide();
        $('.session-table-grid').show();
        $(this).addClass('selected-view');
        $('#list-view-btn').removeClass('selected-view');
        renderGridView();
        $('.intervalDiv').hide();
        $('#order-Btn').show();
        $('.order-field-displayed').text($('[data-sort=' + sort['colId'] + ']').text());
        if (sort['sortMethod'] == 'sortBy') {
          $('.order-marker')
            .removeClass('glyphicon-arrow-down')
            .addClass('glyphicon-arrow-up');
        } else if (sort['sortMethod'] == 'sortReverse') {
          $('.order-marker')
            .removeClass('glyphicon-arrow-up')
            .addClass('glyphicon-arrow-down');
        }
        Repeater.stop();
      }
    })
    $('#list-view-btn').click(function(){
      $(this).blur();
      if (!$(this).hasClass('selected-view')) {
        $('.session-table').show();
        $(this).addClass('selected-view');
        $('#grid-view-btn').removeClass('selected-view');
        $('.session-table-grid').hide();
        $('.intervalDiv').show();
        $('#order-Btn').hide();
        loadData(function(){
          $('#data-table th').removeClass('sorting_desc sorting_asc');
          $('#' + sort['colId'].substring(1, sort['colId'].length - 1)).addClass(sort['sortMethod'] == 'sortBy' ? 'sorting_asc' : 'sorting_desc');
          changeIntervalHandler();
        });
      }
    })

    $('.sort-order .dropdown-menu li a').click(function() {
      var colId = $(this).data('sort');
      if (sort['colId'].substring(1, sort['colId'].length - 1) != colId) {
        sort['colId'] = '"' + colId + '"';
        sort['sortMethod'] = 'sortBy';
        $('.order-marker')
          .removeClass('glyphicon-arrow-down')
          .addClass('glyphicon-arrow-up');
      } else {
        if ($('.order-marker').hasClass('glyphicon-arrow-down')) {
          $('.order-marker')
            .removeClass('glyphicon-arrow-down')
            .addClass('glyphicon-arrow-up');
          sort['sortMethod'] = 'sortBy';
        } else if ($('.order-marker').hasClass('glyphicon-arrow-up')) {
          $('.order-marker')
            .removeClass('glyphicon-arrow-up')
            .addClass('glyphicon-arrow-down');
          sort['sortMethod'] = 'sortReverse';
        }
      }
      $('.order-field-displayed').text($(this).text());
      renderGridView();
    });

    $('body').on('click','.grid-view-disabled, .screenshot', function() {
      var $container = $(this).closest('.grid-view');
      var $img = $container.children('img');
      var $span = $container.children('span');
      var id = $img.data('id') || $span.data('id');
      $(this).loader(new Viewpoint.API());
      var api = $(this).data('api');
      var url = '/api/fastx-sessions/' + id + '/screenshot/';
      api.get(url).done(function(data){
        if (data.result && data.result.src) {
          if ($img.length) {
            $img.attr('src', data.result.src);
          } else {
            $span.remove();
            $container.prepend('<img src="' + data.result.src + '" data-id="' + id + '" class="screenshot">');
          }
        } else {
          if ($img.length) {
            $img.remove();
            $container.prepend('<span class="grid-view-disabled" data-id="' + id + '">Screenshot is not currently available</span>');
          }
          if (data.status_code == 404) {
            $.notify('The session you tried to reference has been terminated, refreshing grid view...', {
              type: 'success'
            });
            renderGridView();
          }
        }
      });
    });
    renderGridView();
  }

  function renderGridView(){
    var pageSize = $('#viewPerPage').val();
    var api = $('.datagrid-container').data('api');
    var url = '/api/fastx-sessions/?load_screenshot=true&page_size=' + pageSize + '&page=' + pageNumber + '&sort=' + JSON.stringify(sort) + '&filter=' + JSON.stringify(filter);
    api.get(url).done(function(data){
      if (data.count) {
        $('.session-table-grid').html('');
        var totalCount = data.count;
        var pageCount = data.results.length;
        var gridBody = [];
        for (var i = 0; i < pageCount; i++) {
          var sessionGridView = [];
          var session = data.results[i];
          var image = session.screenshot;
          var title = '';
          var screenshot;
          if (session.params) {
            title += session.params.name ? 'Session Name: ' + session.params.name + '\n' : 'Session Name: N/A\n';
            title += session.params.moabJobId ? 'Job ID: ' + session.params.moabJobId + '\n' : 'Job ID: N/A\n';
          }
          title += session.login ? 'Submitter ID: ' + session.login + '\n' : 'Submitter ID: N/A\n';
          title += session.started ? 'Started: ' + renderDate(session.started) + '\n' : 'Started: N/A\n';
          title += session.lastConnect ? 'Last Connect: ' + renderDate(session.lastConnect) + '\n' : 'Last Connect: N/A\n';
          title += session.lastDisconnect ? 'Last Disconnect: ' + renderDate(session.lastDisconnect) + '\n' : 'Last Disconnect: N/A\n';
          var links = createGridPopupMenu(session);
          if (image && image.result && image.result.src) {
            screenshot = '<img src="' + image.result.src + '" data-id="' + session.id + '" class="screenshot">' + '<div class="session-name">' + session.params.name + '</div>';
          } else {
            screenshot = '<span class="grid-view-disabled" data-id="' + session.id + '">Screenshot is not currently available</span>' + '<div class="session-name">' + session.params.name + '</div>';
          }
          sessionGridView.push(
            '<div class="grid-view col-xs-3" title="' + title + '">',
              screenshot,
              createActionButtons(session),
            '</div>'
          );
          gridBody.push(sessionGridView.join(''));
        }
        $('.session-table-grid').append(gridBody.join(''));
        createPopupMenu();
        updatePaginator(totalCount, pageSize, pageNumber);
    } else {
      $('.session-table-grid').html('<div class="no-data-available">No data available in table</div>');
    }
    })
  }

  function changeIntervalHandler() {
    var $selectIntervals = $('#selectIntervals');
    var value = $selectIntervals.val();

    if (!value) {
      $selectIntervals.val("15000");
      value = 15000;
    }

    if (value === 'OFF') {
      Repeater.stop();
    } else {
      if ($('#selectIntervals').is(':visible')) {
        Repeater.setTimeout(value);
      }
    }

    localStorage.setItem("sessionRefreshInterval", value);
  }

  function setRefreshInterval() {
    var refreshInterval = localStorage.getItem("sessionRefreshInterval");
    if (refreshInterval) {
      $('#selectIntervals').val(refreshInterval);
    }
  }

  function setPageSize() {
    var pageSize = localStorage.getItem("sessionsPageSize");
    if (pageSize) {
      $('#viewPerPage').val(pageSize);
    }
  }

  function createPopupMenu() {
    $('.popover').remove();
    $('.trigger').popover('destroy');
    $('.trigger').popover({

      html: true,
      container: 'body',
      animation: false,
      content: function() {
        return $(this).parent().find('.content').html();
      },
      placement: 'left'
    });
    $('.trigger').off('click').on('click', function(e) {
      $('.trigger').popover('hide');
      $(this).popover('show');
      e.stopPropagation();
    }).popover();
    // Appending context-menu button
    $('.name-cell div').hide();
    $(document)
      .off('mouseenter', '.name-cell')
      .on('mouseenter', '.name-cell', function() {
        $(this).children('.menu-btn-container').show();
      })
      .off('mouseleave', '.name-cell')
      .on('mouseleave', '.name-cell', function() {
        $('.name-cell div').hide();
        mouseleave();
      });
    $(document)
      .off('mouseenter', '.grid-view')
      .on('mouseenter', '.grid-view', function() {
        $(this).children('.action-buttons').show();
      })
      .off('mouseleave', '.grid-view')
      .on('mouseleave', '.grid-view', function() {
        $(this).children('.action-buttons').hide();
        mouseleave();
      });
    var mouseleave = function() {
      setTimeout(function () {
        if ($('.popover:hover').length === 0) {
          $('.trigger').popover('hide');
        }
      }, 250);
    };
    $(document)
      .off('mouseleave', '.popover')
      .on('mouseleave', '.popover', mouseleave)
      .off('mouseleave', '#data-table tr')
      .on('mouseleave', '#data-table tr', mouseleave);
    $('.trigger').on('show.bs.popover', popoverShow);
    $('.trigger').on('shown.bs.popover', popoverShown);
    $('body').on('click', '.popover', function(){
      $(this).remove();
    });
  }

  function popoverShow(event) {
    var count = $('#data-table tbody tr.active-session').length;
    if (count == 1) {
      $('#data-table tbody tr.active-session').removeClass('active-session');
    }
  }

  function popoverShown(event) {
    var item = $(event.target).closest('tr')[0];
    if (!$(item).hasClass('active-session')) {
      $('.popover-content').removeClass('active');
    } else {
      $('.popover-content').addClass('active');
    }
  }

  function getSelectedUrls() {
    var urls = $('#data-table tbody tr.active-session').map(function() {
      return $(this).attr('data-url');
    });
    return urls;
  }

  function createGridPopupMenu(data) {
    var links = '<a href=' + data.url + ' class="session-connect">Connect</a>';
    links += '<a href=' + data.url + ' class="session-terminate">Terminate</a>';
    if (data.clients > 0) {
      links += '<a href=' + data.url + ' class="session-disconnect">Disconnect</a>';
    }
    return links;
  }

  function createActionButtons(data) {
    var $linksContainer = $('<div>'),
        $actionButtons= $('<div>',{class: 'action-buttons'});
    $actionButtons.appendTo($linksContainer);
    $('<a>', {
      href: data.url,
      title: 'Connect to this session',
      class: 'session-connect',
    }).appendTo($actionButtons).append('<span class="glyphicon glyphicon-play-circle"></span>').appendTo($actionButtons);
    if (data.clients > 0) {
      $('<a>', {
        href: data.url,
        title: 'Disconnect all users from this session',
        class: 'session-disconnect',
      }).appendTo($actionButtons).append('<span class="glyphicon glyphicon-remove-circle"></span>').appendTo($actionButtons);
    }
    $('<a>', {
      href: data.url,
      title: 'Terminate this session',
      class: 'session-terminate',
    }).appendTo($actionButtons).append('<span class="glyphicon glyphicon-trash"></span>').appendTo($actionButtons);
    if (data.params && data.params.moabJobId) {
      $('<a>', {
        href: '/jobdetails/?job_name=' + data.params.moabJobId + '&job_id=' + data.params.moabJobId,
        title: 'Go to the job details page',
        class: 'session-job-details',
      }).appendTo($actionButtons).append('<span class="glyphicon glyphicon-circle-arrow-right"></span>').appendTo($actionButtons);
    }
    return $linksContainer.html();
  }

  function createNameColumn(data, type, full) {
    var links = '<a href=' + data.url + ' class="session-connect">Connect</a>';
    links += '<a href=' + data.url + ' class="session-terminate">Terminate</a>';
    links += '<a href="#" class="data-url" data-name="' + ("params" in data && "name" in data.params ? data.params.name : "") + '"></a>';
    if (data.clients > 0) {
      links += '<a href=' + data.url + ' class="session-disconnect">Disconnect</a>';
    }
    var name;
    if ('params' in data && 'name' in data.params) {
      name = data.params.name.length > 30 ? data.params.name.substring(0,30) + '...' : data.params.name;
    } else {
      name = 'N/A';
    }

    var block = $('<span>' + name + '</span>').add(
        $('<div class="menu-btn-container">')
            .append('<span class="glyphicon glyphicon-align-justify trigger">')
            .append(
            $('<div>')
                .addClass('content hide')
                .append(links)
        )
    );
    return block[0].outerHTML + block[1].outerHTML;
  }

  function createJobIdColumn(data, type, full) {
    if (data.params && data.params.moabJobId) {
      return '<a href="/jobdetails/?job_name=' + data.params.moabJobId + '&job_id=' +
          data.params.moabJobId +'">' + data.params.moabJobId + '</a>';
    } else {
      return 'N/A';
    }
  }

  function addRowTable(data) {
    var table = $('#data-table').DataTable();
    table.row.add(data);
    table.draw();
    totalCount += 1;
    var pageSize = $('#viewPerPage').val();
    updatePaginator(totalCount, pageSize, 1);
  }

  function onOffNameFilter() {
    if ($('#session-name-checkbox').is(":checked")) {
      $('#session-name').val('').attr('disabled', false);
    } else {
      $('#session-name').val('- Session Name -').attr('disabled', true);
    }
  }

  function onOffJobIdFilter() {
    if ($('#job-id-checkbox').is(":checked")) {
      $('#job-id').val('').attr('disabled', false);
    } else {
      $('#job-id').val('- Job Id -').attr('disabled', true);
    }
  }

  function onOffLoginFilter() {
    if ($('#session-login-checkbox').is(":checked")) {
      $('#session-login').val('').attr('disabled', false);
    } else {
      $('#session-login').val('- Submitter Id -').attr('disabled', true);
    }
  }

  function connectSession(item) {
    var url = $(item).attr('href') + '?loadConnectionUrl=false';
    var api = $('#grid-view-btn').hasClass('selected-view') ? $('#datagrid-connection-api-container').data('api') : $('.datagrid-container').data('api');

    function connect(url) {
      var api = $('#datagrid-connection-api-container').data('api');
      api.get(url + '?loadConnectionUrl=true')
        .done(
        function(data) {
          if (data && 'connectionUrl' in data) {
            var win = window.open(data.connectionUrl, '_blank');
          }
        })
        .fail(function(jqXHR, textStatus, errorThrown) {
          if (jqXHR.status == 503) {
            $.viewpoint.confirm('Session was not ready',
              'Session was not ready, it might still be initializing. Do you want to retry now?')
              .done(function(result) {
                if (result) {
                  connect(url);
                }
              });          
          } else if (jqXHR.status == 404) {
            if ($('#grid-view-btn').hasClass('selected-view')) {
              $.notify('The session you tried to reference has been terminated, refreshing grid view...', {
                type: 'success'
              });
              renderGridView();
            } else {
              $.notify('Session not found. This might occur if the session has been externally cancelled while we were trying to connect to it.', {
                type: 'warning'
              });
            }
          } else {
            var message = textStatus || errorThrown;
            if (console && console.debug) {
              console.debug(message);
            }
            $.notify('We apologize, but there seems to be an error in the communication with our Remote Visualization Services. Please try again later or contact support.', {
              type: 'warning'
            });
          }
        });
    }

    api.get(url).done(function(data) {
      if (data) {
        var session = data;
        var clients = session.clients || 0;
        if (clients !== 0) {
          $.viewpoint.confirm('Disconnect users',
            'This session currently has active users connected to it. Do you want to disconnect all other users and take over the session?')
            .done(function(result) {
              if (result) {
                api.post(session.commands, {command: 'disconnect'}).done(function() {
                  connect(session.url);
                });
              }
            });
        } else {
          connect(session.url);
        }
      } else {
        $.notify('Session does not exist', {
          type: 'danger'
        });
      }
    })
    .fail(function(jqXHR, textStatus, errorThrown){
      if (jqXHR.status == 404 && $('#grid-view-btn').hasClass('selected-view')) {
        $.notify('The session you tried to reference has been terminated, refreshing grid view...', {
          type: 'success'
        });
        renderGridView();
      }   
    });
  }

  function disconnectSession(item) {
    var url = $(item).attr('href');
    var urls = getSelectedUrls();
    var command = {'command':'disconnect'};
    if ($.inArray(url, urls) > -1) {
      $.viewpoint.confirm('Disconnect users', 'Are you sure you want to disconnect all other users currently accessing the session?').done(function(result) {
        if (result) {
          $.each(urls, function(key, url) {
            postCommand(url, command);
          });
        }
      });
    } else {
      $.viewpoint.confirm('Disconnect users', 'Are you sure you want to disconnect all other users currently accessing the session?').done(function(result) {
        if (result) {
          postCommand(url, command);
        }
      });
    }

    function postCommand(url, command) {
      var api = $('#grid-view-btn').hasClass('selected-view') ? $('#datagrid-connection-api-container').data('api') : $('.datagrid-container').data('api');
      api.post(url + 'commands/', command).done(function() {
        var row = $('#data-table tbody').find('[data-url="' + url + '"]');
        row.removeClass('active-session');
        row.find('.session-disconnect').remove();
      })
      .fail(function(jqXHR, textStatus, errorThrown){
        if (jqXHR.status == 404 && $('#grid-view-btn').hasClass('selected-view')) {
          $.notify('The session you tried to reference has been terminated, refreshing grid view...', {
            type: 'success'
          });
          renderGridView();
        }  
      });
    }
  }

  function terminateSession(item) {
    var url = $(item).attr('href');
    var urls = getSelectedUrls();
    var table = $('#data-table').DataTable();
    var row =  $('#data-table').find('[data-url="' + url + '"]');
    var api = $('#grid-view-btn').hasClass('selected-view') ? $('#datagrid-connection-api-container').data('api') : $('.datagrid-container').data('api');
    if ($.inArray(url, urls) > -1) {
      $.viewpoint.confirm('Terminate Session', 'Are you sure?').done(function(result) {
        if (result) {
          $.each(urls, function(key, url) {
            var table = $('#data-table').DataTable();
            var row =  $('#data-table').find('[data-url="' + url + '"]');
            $('.datagrid-container').data('api').delete(url).done(function(){
              row.find('td:first-child .trigger').popover('destroy');
              table
                .row(row)
                .remove()
                .draw();
              totalCount -= 1;
              var pageSize = $('#viewPerPage').val();
              updatePaginator(totalCount, pageSize, 1);
            });
          });
        }
      });
    } else {
      $.viewpoint.confirm('Terminate Session', 'Are you sure?').done(function(result) {
        if (result) {
          api.delete(url).done(function(){
            pageNumber = 1;
            if ($('#grid-view-btn').hasClass('selected-view')) {
              renderGridView();
            } else {
              loadData();
            }
          })
          .fail(function(jqXHR, textStatus, errorThrown){
            if (jqXHR.status == 404 && $('#grid-view-btn').hasClass('selected-view')) {
              $.notify('The session you tried to reference has been terminated, refreshing grid view...', {
                type: 'success'
              });
              renderGridView();
            }  
          });
        }
      });
    }
  }

  function createDataTable() {
    var pageSize = $('#viewPerPage').val();
    opts = {
      dom: 'tlp',
      bFilter: true,
      bInfo: false,
      bDestroy:true,
      bAutoWidth: false,
      order: [[ currentOrder.split('.')[0], currentOrder.split('.')[1] ]],
      'columnDefs': [{
          'targets': 0,
          'data': null,
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'name-cell',
          'render': function(data, type, full) {
            return createNameColumn(data, type, full);
          }
        }, {
          'targets': 1,
          'data': null,
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'job-id-cell',
          'render': function(data, type, full) {
            return createJobIdColumn(data, type, full);
          }
        }, {
          'targets': 2,
          'data': 'login',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
        }, {
          'targets': 3,
          'data': 'started',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'date-cell',
          'render': renderDate
        }, {
          'targets': 4,
          'data': 'lastConnect',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'date-cell',
          'render': renderDate
        }, {
          'targets': 5,
          'data': 'lastDisconnect',
          'defaultContent': 'N/A',
          'sType': 'sortOff',
          'className': 'date-cell',
          'render': renderDate
        },
      ],
      'initComplete': function(settings, json) {
        createPopupMenu();
      },
      'fnCreatedRow': function(nRow, aData, iDataIndex) {
        $(nRow).attr('data-url', aData.url);
      },
      "pageLength": parseInt(pageSize, 10)
    };

    var $table = $('#data-table');
    table = $table.DataTable(opts);
    $table.on('draw.dt', function() {
      createPopupMenu();
    });
    $table.on('order.dt', function (e) {
      var order = $table.DataTable().order()[0][0] + '.' +
        $table.DataTable().order()[0][1];
      if (order != currentOrder) {
        $('#application-templates .paginator-container').bootpag({
          page: 1,
          next: 'next',
          prev: 'prev'
        });
        currentOrder = order;
      }
    });
  }

  function loadData(cb) {
    var pageSize = $('#viewPerPage').val();
    var url = '/api/fastx-sessions/?page_size=' + pageSize + '&page=' + pageNumber + '&sort=' + JSON.stringify(sort) + '&filter=' + JSON.stringify(filter);
    $('.datagrid-container').data('api').get(url).done(function(data) {
      totalCount = data.count;
      table.clear();
      table.rows.add(data.results);
      table.draw();
      updatePaginator(totalCount, pageSize, pageNumber);
      createPopupMenu();
      if ($.isFunction(cb)) {
        cb();
      }
    }).fail(function(){
      opts.data = {};
      $('#data-table').DataTable(opts);
    });
  }

  function renderDate(data, type, full) {
    if (data) {
       var localTime = moment.utc(data).toDate();
       return moment(localTime).format('YYYY-MM-DD HH:mm:ss');
    } else {
      return 'N/A';
    }
  }

  function createPaginator() {
    $('#sessions .paginator-container').bootpag({
        total: 1, // total pages
        maxVisible: 7,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
    }).on('page', function(event, num) {
        pageNumber = num;
        if ($('#grid-view-btn').hasClass('selected-view')) {
          renderGridView();
        } else {
          loadData();
        }
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    totalRows = parseInt(totalRows);
    rowsPerPage = parseInt(rowsPerPage);
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('#sessions .paginator-container').bootpag({
        total: total, // total pages
        page: currentRow,
        maxVisible: 7,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
    }).show();
  }

  function resetFilters() {
    $('#session-name')
      .prop('disabled', true)
      .val('- Session Name -');
    $('#session-name-checkbox').prop('checked', false);

    $('#job-id')
      .prop('disabled', true)
      .val('- Job Id -');
    $('#job-id-checkbox').prop('checked', false);

    $('#session-login')
      .prop('disabled', true)
      .val('- Submitter Id -');
    $('#session-login-checkbox').prop('checked', false);
  }
})(jQuery);
