/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

// -- How to use --
// To add widget to row of table just add this code:
//  $('#row_selector').jobArrayWidget();

;(function($) {
  'use strict';
$.fn.jobArrayWidget = function(inTable) {
  var jobName = $(this).data('jobname');
  var rowId = createId(jobName);
  var masterJobRow = $(this);
  var elementsPerPage = 24;
  var nodeName = $('#node_details_name_value').text();

  var api = new Viewpoint.API();

  function createId(name) {
    return 'job-' + name.toString().replace(/[!"\[\]{}%^&*:@~#';/.<>\\|`\s]/g, '').toLowerCase();
  }

  // creating map of css classes by job status
  // job_status : css_class
  
  var classesMap = {};
    classesMap['REMOVED'] = 'removed';
    classesMap['SUSPENDED'] = 'suspended';
    classesMap['COMPLETED'] = 'completed';
    classesMap['FAILED'] = 'failed';
    classesMap['IDLE'] = 'idle';
    classesMap['BLOCKED'] = 'blocked';
    classesMap['HOLD'] = 'hold';
    classesMap['RUNNING'] = 'running';
    classesMap['NONE'] = 'none';
    classesMap['DEFERRED'] = 'deferred';
    classesMap['UNKNOWN'] = 'unknown';
    classesMap['VACATED'] = 'vacated';
    classesMap['NOTQUEUED'] = 'notqueued';
    classesMap['STAGING'] = 'staging';
    classesMap['STARTING'] = 'starting';



  function getClassByStatus(status) {
    if (classesMap[status.toUpperCase()]) {
        return classesMap[status.toUpperCase()]
    }
    return 'unknown-state';
  }

  function isJobDetails() {
    return ($('.form-jobDetail').length > 0);
  }
  function isNodeDetails() {
    return ($('.form-nodeDetail').length > 0);
  }

  // creating table with subjobs of master job
  function createJobTable(data) {
    var Viewpoint = window.Viewpoint || {};
    var params = {};
    if (Viewpoint.searchParams) {
      params = $.extend(true, {}, Viewpoint.searchParams);
      params.jobname = params.job_name;
      delete params.job_name;
      for (var i in params) {
        if (!params[i]) {
          delete params[i];
        }
      }
    }

    var tab = '';
    if (isJobDetails()) {
      tab = 'jobdetails';
    } else {
      tab = isNodeDetails() ? 'nodedetails' : 'workload';
    }
    var countRows = Math.ceil(data.length / 4);
    var countColumns = 4;
    var table = $('<table />');
    for(var rowNum = 0; rowNum < countRows; ++rowNum) {
      var row = $('<tr/>');
      for (var clmn = 0; clmn < countColumns; ++clmn) {
        if (data[rowNum+countRows*clmn]) {
          var link =
            $('<a/>')
              .attr('href', '/jobdetails/?job_name=' +
                data[rowNum+countRows*clmn].name +
                '&job_status=UNKNOWN&job_id=UNKNOWN&tab=' + tab +
                '&' + $.param(params))
              .text(data[rowNum+countRows*clmn].name);
          if (!$.viewpoint.user.permissions.job_details) {
            link = $('<span/>')
              .text(data[rowNum+countRows*clmn].name);
          }
          row.append(
            $('<td />')
              .addClass(getClassByStatus(data[rowNum+countRows*clmn].state))
              .append(link)
          );
        }
      }
      table.append(row);
    }
    return table;
  }

  function createFilterLinks() {
    var filters = [
      { name: 'All Subjobs', class: 'current-page-info'},
      { name: 'Idle', class: 'idle' },
      { name: 'Running', class: 'running' },
      { name: 'Blocked', class: 'blocked' },
      { name: 'Hold', class: 'hold' },
      { name: 'Removed', class: 'removed' },
      { name: 'Completed', class: 'completed' }
    ];

    var filterLinks = $.map(filters, function(n) {
      return '<li><a href="#" class="' + n.class + '">' + '<span data-state="' +
        n.name.replace(' ', '').toLowerCase() + '">' +
        n.name + ' </span><span class="count">n/a</span></a></li>';
    });

    return filterLinks.join('');
  }

  var selectedStatus = null;

  function createPaginator() {
    $('#' + rowId + ' div:last').append('<div class="paginator-container"></div>');
    $('#' + rowId + ' .paginator-container').bootpag({
      total: 1,     // visible pagination
      maxVisible: 10,
      leaps: true,         // next/prev leaps through maxVisible
      firstLastUse: true
    }).on('page', function(event, num) {
      $('#' + rowId + ' table').remove();
      loadData(renderData, num, elementsPerPage, selectedStatus);
      showLoading();
      disableWidget();
      $(this).hide();
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('#' + rowId + ' .paginator-container').bootpag({
      total: total, // total pages
      page: currentRow            // default page
    }).show();
  }

  // function for loading data from server
  function loadData(fn, num, elementsPerPage, status) {
    var url = '/api/jobs/' + jobName + '/subjobs/?page_size=' + elementsPerPage + '&page=' + num;
    if (isNodeDetails()) {
      url += '&node=' + nodeName + '&state=RUNNING';
    }
    if (status) {
      url += '&state=' + status.toUpperCase();
    }
    api.get(url)
      .done(function(data, textStatus, jqXHR) {
        fn(data.results);
        updatePaginator(data.count, elementsPerPage, num);
        if (isNodeDetails()) {
          $('.navbar-text').text('Running Subjobs - ' + data.count);
        } else {
          loadFilterCounts();
        }
        hideLoading();
        enableWidget();
      })
      .fail(function(jqXHR, textStatus, errorThrown) {
        hideLoading();
        $.notify('Server error', {type: 'danger'});
        console.error(jqXHR.responseJSON.detail);
      });
  }

  function renderData(data){
    $('#' + rowId + ' div:last').append(createJobTable(data));
  }

  function renderMainForm() {
    if (inTable == true) {
      masterJobRow.after(
        $('<tr />')
        .addClass('job-arr-widget')
        .append(
        $('<td />')
          .attr('colspan', '9')
          .addClass('hidden-widget')
            .append(
            $('<div />')
            .addClass('collapse')
            .attr('id', rowId)
            .append(
              '<nav class="navbar navbar-default tool-box-container">' +
              '<ul class="nav navbar-nav">' +
                '<li class="allJobs"><p class="navbar-text">Job Array</p></li>' +
              '</ul>' +
              '<ul class="nav navbar-nav navbar-right">' +
                createFilterLinks() +
              '</ul>' +
              '</nav>'
            )
            .append($('<div class="navbar-default widget-content"><div/>'))
          )
        )
      );
    } else {
      masterJobRow.find('.subjobs-body').html(
        $('<div class="job-arr-widget hidden-widget">')
          .append(
          $('<div />')
            .attr('id', rowId)
            .append(
              '<nav class="navbar navbar-default tool-box-container">' +
                '<ul class="nav navbar-nav">' +
                  '<li class="allJobs"><p class="navbar-text">Subjobs</p></li>' +
                '</ul>' +
                '<ul class="nav navbar-nav navbar-right">' +
                  createFilterLinks() +
                '</ul>' +
              '</nav>'
            )
            .append($('<div class="navbar navbar-default widget-content"><div/>'))
          )
      );
    }
    $('.current-page-info').parent().addClass('active');
    if (isNodeDetails()) {
      $('.navbar-text').text('Running Subjobs');
      $('.job-arr-widget .navbar-right').hide()
    } else {
      loadFilterCounts();
      addFilterHandlers();
    }
  }

  function loadFilterCounts() {
    var container;
    if (inTable == true) {
      container = masterJobRow.next();
    } else {
      container = masterJobRow;
    }

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.current-page-info span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=IDLE&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.idle span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=RUNNING&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.running span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=BLOCKED&node=' + nodeName)
      .done(function (data, textStatus, jqXHR) {
        container.find('.blocked span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=HOLD&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.hold span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=REMOVED&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.removed span.count').text(data.count);
      });

    api.get('/api/jobs/' + jobName + '/subjobs/?page_size=1&state=COMPLETED&node=' + nodeName)
      .done(function(data, textStatus, jqXHR) {
        container.find('.completed span.count').text(data.count);
      });
  }

  function addFilterHandlers() {
    $('.job-arr-widget ul.navbar-right a').click(function(event) {
      event.preventDefault();

      $('#' + rowId + ' table').remove();
      var status = $(this).children(":first").attr('data-state');
      if (status == 'allsubjobs') {
        status = null;
      }
      selectedStatus = status;
      showLoading();
      disableWidget();
      loadData(renderData, 1, elementsPerPage, status);
      masterJobRow.next().find('nav.navbar li').removeClass('active');
      masterJobRow.find('nav.navbar li').removeClass('active');
      $(this).parent().addClass('active');
    });
  }

  function createLoadingIcon() {
    $('#' + rowId + ' div:last')
      .append('<img src="../static/legacy/images/spinner.gif" class="spinner-wheel" />');
  }

  function showLoading() {
    var $loading = $('#' + rowId + ' .spinner-wheel');
      if ($loading.length > 0) {
        $loading.show();
      } else {
        createLoadingIcon();
    }
  }

  function hideLoading() {
    $('#' + rowId + ' .spinner-wheel').hide();
  }

  //creating icon for collapse
  $(this).find('td:first').prepend(
    $('<span />')
      .addClass('glyphicon glyphicon-chevron-right widget-trigger')
      .css({'cursor':'pointer'})
  );

  function enableWidget() {
    $('#' + rowId + ' *').removeClass('not-active');
  }

  function disableWidget() {
    $('#' + rowId + ' *').addClass('not-active');
  }

  function showWidget() {
    var needRender = false;
    if (inTable == true) {
      needRender = (masterJobRow.next('.job-arr-widget').length == 0);
    } else {
      needRender = (masterJobRow.find('.job-arr-widget').length == 0);
    }
    if (needRender) {
      renderMainForm();
      showLoading();
      loadData(renderData, 1, elementsPerPage);
      createPaginator();
      disableWidget();
    }
    if (inTable == true) {
      $('[data-jobname="' + jobName + '"]').next().removeClass('hidden');
      $('#' + rowId).collapse('toggle');
    } else {
      $('.subjobs-body').collapse('show');
      $('.subjobs-bar .subjobs-title').addClass('expanded');
    }
  }

  function hideWidget() {
    if (inTable == true) {
      $('#'+rowId).collapse('toggle');
      $('[data-jobname="' + jobName + '"]').next().addClass('hidden');
    } else {
      $('.subjobs-body').collapse('hide');
      $('.subjobs-bar .subjobs-title').removeClass('expanded');
    }
  }

  function expandHandler(e) {
    var link = $(e.target);
    if (link.hasClass('glyphicon-chevron-right')) {
      showWidget();
      link.removeClass('glyphicon-chevron-right');
      link.addClass('glyphicon-chevron-down');
    } else {
      hideWidget();
      link.removeClass('glyphicon-chevron-down');
      link.toggleClass('glyphicon-chevron-right');
    }
  }

  //adding listener for icon
  $(this).find('.widget-trigger').on('click', expandHandler);
  $(this).find('.show-all').on('click', expandHandler);

  };
})(jQuery);
