/* Copyright (c) 2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved. */

;(function($) {
  'use strict';

  var initConfig;
  var modalWindow;
  var container;
  var table;
  var api;
  var pageNumber = 1;
  var totalCount;
  var sort = '-version';
  var sortParams = {
    'name': true,
    'version': true,
    'changed_by': true,
    'date': true,
    'version_description': true
  };

  var dataTableOpts = {
      dom: 'iftlp',
      bSortClasses: false,
      bFilter: false,
      bInfo: false,
      bPaginate: false,
      bAutoWidth: false,
      order: [[ 1, 'desc' ]],
      'oLanguage': {
        'sEmptyTable' : 'No historical data'
      },
      'columnDefs': [{
          'targets': 0,
          'data': 'name',
          'sType': 'sortOff',
          'className': 'name-cell',
          'defaultContent': 'N/A',
        }, {
          'targets': 1,
          'data': 'version',
          'sType': 'sortOff',
          'className': 'version-cell',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            return renderVersionColumn(data, type, full);
          }
        }, {
          'targets': 2,
          'data': 'changed_by',
          'sType': 'sortOff',
          'className': 'owner-cell',
          'defaultContent': 'N/A',
        }, {
          'targets': 3,
          'data': 'date',
          'sType': 'sortOff',
          'className': 'date-cell',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            var localTime = moment.utc(data).toDate();
            return moment(localTime).format('YYYY-MM-DD HH:mm:ss');
          }
        }, {
          'targets': 4,
          'data': 'version_description',
          'sType': 'sortOff',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            return renderDescriptionColumn(data, type, full);
          }
        }, {
          'targets': 5,
          'orderable': false,
          'className': 'operations-cell',
          'defaultContent': 'N/A',
          'render': function(data, type, full) {
            return renderOptionColumn(data, type, full);
          }
        }
      ],
      'fnCreatedRow': function(nRow, aData, iDataIndex) {
        // for some reason data('url', url) doesn't work
        $(nRow)
            .attr('data-url', aData.url)
            .attr('data-version', aData.version);
        if (aData.current) {
          $(nRow).attr('data-current', true);
        }
      },
  };

  function getHistory() {
    var regex = new RegExp('templates\/(.*)\/history');
    var historyUrl = $('#template-history').find('tbody').find('tr').first().data('url');
    return historyUrl.match(regex)[1];
  }

  function getSelectedIds() {
    var ids = [];
    $('#template-history tbody tr.active-template').each(function() {
      ids.push($(this).data('version'));
    });
    return ids;
  }

  function createPaginator() {
    $('#template-history .grid-pagination .paginator-container').bootpag({
      total: 1, // total pages
      maxVisible: 7,
      leaps: true,
      firstLastUse: true,
      next: 'next',
      prev: 'prev'
    }).on('page', function(event, num) {
      pageNumber = num;
      loadData();
    });
  }

  function updatePaginator(totalRows, rowsPerPage, currentRow) {
    var totalRows = parseInt(totalRows);
    var rowsPerPage = parseInt(rowsPerPage);
    var total = Math.ceil(totalRows / rowsPerPage);
    total = total === 0 ? 1 : total;
    $('#template-history .grid-pagination .paginator-container').bootpag({
        total: total, // total pages
        page: currentRow,
        maxVisible: 7,
        leaps: true,
        firstLastUse: true,
        next: 'next',
        prev: 'prev'
    });
  }

  function renderDescriptionColumn(data, type, full) {
    return $('<div></div>').append(
      $('<div></div>')
        .addClass('version-description-container')
        .text(data)
    ).append(
      $('<div></div>')
        .addClass('input-group edit-description-container')
        .attr('data-template-url', full.url)
        .append(
          $('<input>')
            .attr('type', 'text')
            .addClass('form-control')
        )
        .append(
          $('<div></div>')
            .addClass('input-group-addon do-editing')
            .append(
              $('<span></span>')
                .addClass('glyphicon glyphicon-ok')
            )
        )
        .append(
          $('<div></div>')
            .addClass('input-group-addon cancel-editing')
            .append(
              $('<span></span>')
                .addClass('glyphicon glyphicon-remove')
            )
        )
        .hide()
    ).html();
  }

  function renderVersionColumn(data, type, full) {
    var text = 'v. ' + data;
    if (full.current) text += ' (CURRENT)';
    return $('<div></div>').append(
      $('<a></a>').attr('href', '/job_templates/template_builder?url=' + full.url).text(text)).html();
  }

  function renderOptionColumn(data, type, full) {
    var restoreLink = $('');
    var exportLink = full.url;
    if ('/' == exportLink.slice(-1)) {
      exportLink = exportLink.slice(0, -1);
    }
    exportLink += '.template';
    if (!full.current)
      restoreLink = $('<a></a>')
        .addClass('restore')
        .attr('href', full.url)
        .append(
          $('<span></span>')
            .addClass('glyphicon glyphicon-share-alt')
            .attr('title', 'Restore this version')
        );

    return $('<div></div>')
      .append(
        restoreLink
      )
      .append(
        $('<a></a>')
          .attr('current', full.current)
          .attr('href', full.url)
          .addClass('delete')
          .append(
            $('<span></span>')
              .addClass('glyphicon glyphicon-remove')
              .attr('title', 'Delete this version')
          )
      )
      .append(
        $('<a></a>')
          .attr('href', exportLink)
          .addClass('export')
          .append(
            $('<span></span>')
              .addClass('glyphicon glyphicon-download')
              .attr('title', 'Export template')
          )
      ).html();
  }

  var renderNoContentMessage = function(container, message) {
    table
      .DataTable({
        data: [],
        'dom': 'iftlp',
        'paging': false,
        'oLanguage': {
          'sEmptyTable' : message
        },
        columns: [
          { title: 'Message' },
        ]
    });
  };

  $(function() {
    $.fn.dataTableExt.oSort["sortOff-asc"] = function(x, y) {
      return 0;
    };
    $.fn.dataTableExt.oSort["sortOff-desc"] = function(x, y) {
      return 0;
    };
    $('#template-history #viewPerPage').val(localStorage.getItem('templateHistoryPageSize') || 10);
    initConfig = $.viewpoint.modal.payload();
    modalWindow = $('#template-history');
    container = modalWindow.find('.history-bar');
    container.loader(new Viewpoint.API());
    api = container.data('api');
    table = container.find('table').DataTable(dataTableOpts);
    createPaginator();
    loadData();

    $('#template-history th').click(function() {
      if ($(this).hasClass('operations-cell'))
        return;

      var colId = $(this).attr('id');
      sort = sortParams[colId] ? colId : '-' + colId;
      sortParams[colId] = !sortParams[colId];
      for (var key in sortParams) {
        if (key == colId) continue;
        sortParams[key] = true;
      }
      loadData();
    });

    $('#template-history #viewPerPage').change(function() {
      pageNumber = 1;
      localStorage.setItem('templateHistoryPageSize', $(this).val());
      loadData();
    });

  });

  function addEvents(){
    $('#template-history a.delete').click(function(event) {
      event.preventDefault();
      event.stopPropagation();
      deleteSingleActiveTemplate();
      var row =  $(this).parent().parent().parent().parent();
      var url = $(this).attr('href');
      var urls = getSelectedUrls();
      if ($.inArray(url, urls) > -1) {
        deleteTemplates();
      } else {
        deleteTemplate.call(this, url);
      }
    });

    $('#template-history a.export').click(function(event) {
      event.preventDefault();
      event.stopPropagation();
      deleteSingleActiveTemplate();
      var url = $(this).attr('href');
      var urls = getSelectedUrls();
      if ($.inArray(url.replace('.template','/'), urls) > -1) {
        exportTemplates();
      } else {
        exportTemplate.call(this, url);
      }
    });

    $('#template-history a.restore').click(function(event) {
      event.preventDefault();
      var url = $(this).attr('href');
      $.viewpoint.confirm('Restore Operation', 'Are you sure you want to restore this ' +
            'version of the template?')
      .done(function(result) {
        if (result) {
          api.put(url)
            .done(function(data) {
              $.notify('Template has been restored', {
                type: 'success',
                delay: 3000,
                onClose: function () {
                  window.location.href = '/job_templates/template_builder?url=' + data.url + '&showHistoryDialog=true';
                }
              });
            });
        }
      });
      return false;
    });

    $('.version-description-container').click(function(event) {
      $(this).hide().next().show();
      $(this).next().find('input').val($(this).text()).focus();
    });

    $('#template-history .cancel-editing').click(function(event) {
      $(this).closest('.edit-description-container').hide().prev().show();
    });

    $('#template-history .do-editing').click(function(event) {
      var container = $(this).closest('.edit-description-container');
      var new_description = $(this).parent().find('input').val();
      api.patch($(container).data('template-url'), {'version_description': new_description})
        .done(function(data) {
          container.prev().text(new_description);
          if (initConfig.version === data.version)
            $('#version-description').val(new_description);
        })
        .always(function() {
          container.hide().prev().show();
        });
    });
    $('#template-history tbody tr').click(selectingHandler);
  }

  function selectingHandler(event) {
    if ($(event.target).is('a')) {
      return;
    }

    var $allTemplates = $('#template-history').find('tbody').find('tr');
    var $selectedTemplates = $allTemplates.filter(function(){
      return $(this).hasClass('active-template');
    });

    //Ctrl or Cmd(Mac) keys handler
    if ((event.ctrlKey && -1 == $.inArray(navigator.platform, ['Macintosh', 'MacIntel'])) || event.metaKey) {
      $(this).toggleClass('active-template');
      resetLastSelectionMarker();
      $(this).addClass('last-selected-template');
      showAndHideRestoreOperation();
      return;
    }

    //Second click on selected template
    if (1 == $selectedTemplates.length && $selectedTemplates.is($(this))) {
      $(this).removeClass('active-template');
      resetLastSelectionMarker();
      showAndHideRestoreOperation();
      return;
    }

    //Shift key handler
    var $lastSelectedTemplate = $allTemplates.filter(function() { return $(this).hasClass('last-selected-template'); });
    if (1 == $lastSelectedTemplate.length && event.shiftKey && !$lastSelectedTemplate.is($(this))) {
      resetSelection();
      var a, b;
      if ($(this).index() < $lastSelectedTemplate.index()) {
        a = $(this);
        b = $lastSelectedTemplate;
      } else {
        a = $lastSelectedTemplate;
        b = $(this);
      }
      $(a).nextUntil(b).addClass('active-template');
      $(this).addClass('active-template');
      $lastSelectedTemplate.addClass('active-template');
      clearTextSelection();
      showAndHideRestoreOperation();
      return;
    }

    //First click on template
    resetSelection();
    $(this).addClass('active-template');
    resetLastSelectionMarker();
    $(this).addClass('last-selected-template');
    showAndHideRestoreOperation();
  }

  function exportTemplate(url) {
    resetSelection();
    resetLastSelectionMarker();
    showAndHideRestoreOperation();
    window.location.href = url;
  }

  function exportTemplates() {
    api.get('/api/tasks/api2.handlers.templates.Export/?data=' + getSelectedIds().join(',') + '&history=' + getHistory())
      .done(function(result) {
        if (result.errors.length) {
          var message = 'Operation was partially successful.<br>The following errors occurred:<ul>';
          $.each(result.errors, function(k, v) {
            message += '<li>' + v + '</li>';
          });
          message += '</ul>';
          $.notify(message, { type: 'success', delay: 0});
        }
        if (result.file) {
          window.location.href = '/api/tasks/api2.handlers.templates.Export/?file=' + encodeURI(result.file);
          // need to change this parameter to show error messages in the future
          $(window).data('unloading', false);
        } else {
          $.notify('Something went wrong. Server did not return a file.', { type: 'success', delay: 0});
        }
      })
  }

  function deleteTemplate(url) {
    resetSelection();
    resetLastSelectionMarker();
    showAndHideRestoreOperation();
    var current = $(this).attr('current');
    var message = 'Doing this will delete this version of the application template. Are you sure you want to delete?';
    if (current)
      message = 'Doing this will revert the application template to the previous version. Are you sure you want to continue?';
    $.viewpoint.confirm('Delete Operation', message).done(function(result) {
      if (result) {
        api.delete(url).done(function(result) {
          pageNumber = 1;
          loadData();
        });
      }
    });
    return false;
  }

  function deleteTemplates() {
    if (totalCount != getSelectedTemplates().length) {
      var current = getSelectedTemplates().filter(function() {
        return true && $(this).data('current') === true;
      });
      var message = 'Doing this will delete this version of the application template. Are you sure you want to delete?';
      if (current.length)
        message = 'Doing this will revert the application template to the previous version. Are you sure you want to continue?';
      $.viewpoint.confirm('Delete Operation', message).done(function(result) {
        if (result) {
          api.post('/api/tasks/', {
            handler: 'api2.handlers.templates.Delete',
            data: {'history': getHistory(), 'versions': getSelectedIds()}
          }).done(function(result) {
            if (result.errors.length) {
              var message = 'Operation was partially successful.<br>The following errors occurred:<ul>';
              $.each(result.errors, function(k, v) {
                message += '<li>' + v + '</li>';
              });
              message += '</ul>';
              $.notify(message, { type: 'success', delay: 0});
            }
            pageNumber = 1;
            loadData();
          });
        }
      });
    } else {
      $.notify('You can not delete all history of template', { type: 'warning', delay: 0});
    }
  }

  function deleteSingleActiveTemplate() {
    var count = $('#template-history tbody tr.active-template').length;
    if (count == 1) {
      $('#template-history tbody tr.active-template').removeClass('active-template');
    }
  }

  function getSelectedUrls() {
    var urls = $('#template-history tbody tr.active-template').map(function() {
      return $(this).data('url');
    });
    return urls;
  }

  function getSelectedTemplates() {
    var $allTemplates = $('#template-history').find('tbody').find('tr');
    var $selectedTemplates = $allTemplates.filter(function(){
      return $(this).hasClass('active-template');
    });
    return $selectedTemplates;
  }

  function clearTextSelection() {
    if (window.getSelection) {
      if (window.getSelection().empty) {  // Chrome
        window.getSelection().empty();
      } else if (window.getSelection().removeAllRanges) {  // Firefox
        window.getSelection().removeAllRanges();
      }
    } else if (document.selection) {  // IE
      document.selection.empty();
    }
  }

  function showAndHideRestoreOperation(count) {
    var $allTemplates = $('#template-history').find('tbody').find('tr');
    var count = $allTemplates.filter(function(){
      return $(this).hasClass('active-template');
    }).length;
    if (count > 1) {
      $('#template-history').find('tbody').find('.operations-cell').find('.restore').show();
      $('#template-history').find('tbody').find('.active-template').find('.operations-cell').find('.restore').hide();
    } else {
      $('#template-history').find('tbody').find('.operations-cell').find('.restore').show();
    }
  }

  function resetSelection() {
    $('#template-history').find('tbody').find('tr').removeClass('active-template');
  }

  function resetLastSelectionMarker() {
    $('#template-history').find('tbody').find('tr').removeClass('last-selected-template');
  }

  function loadData() {
    var pageSize = $('#template-history #viewPerPage').val();
    if (initConfig.url) {
      api.get(initConfig.url + '?page_size=' + pageSize + '&page=' + pageNumber + '&ordering=' + sort)
      .done(function(data) {
        table.clear();
        table.rows.add(data.results);
        table.draw();
        if (data.count === 1)
          $('#template-history .data-table .operations-cell .glyphicon-remove').hide();
        updatePaginator(data.count, pageSize , pageNumber);
        addEvents();
        totalCount = data.count;
      });
    }
  }

})(jQuery);
